# ouf-pica
GBV OUF/Pica project to fetch, and convert Pica+ records to XML

This is a project to
* fetch records from GBV using the OUF tools
* calculate the record status (upsert or delete)
* convert PICA+ to XML
* control the Index Data Harvester (optional)

## Directories
Lots of this project is about finding out how things work, so there will be many
temporary experiments and other outdated files. Here are some of the more
important directories and files. All paths start from the root dir of the project
* README.md: This file
* build_ouf-2.1.5-vzg1.tar.gz: Tarball with the OUF tools, including source
* ouf-2.1.5-vzg1 - same, unpacked.
* scripts: Various scripts
  * profile.sh: Set up some environment variables. Invoke with `. scripts/profile.sh` (note the dot)
  * get-started.sh: Run this script to install OUF binaries for your workspace.
  * ccom: A wrapper around gcc that sets up various include/lib paths. Needed by the makefiles.
  * picapull.pl: A script to read the updsig files produced by `ouf_update` and to produce
    fetch request files for `ouf_fetch`. This does basically what the `ouf_test`does, but
    in a way that can be tweaked to suit our needs.  By default, this script will check for new updsig
    files every 5 seconds (this may be changed in confir/ouf.ini).
  * decode.pl: Converts all of the title files generated by ouf_fetch into XML collections.
    This script will move the title files to rundir/done when finished (unless if run in migrate mode).
    By default, decode.pl will check for new title files every 5 seconds.
  * pica2xmlbulk.pl: Converts a large file of Pica+ records and creates XML collections of 5000.  This
    is useful for migrating large catalogs.
  * picaxml2json.pl: Takes an XML collection and converts it to JSON in conjuction with two XSLT stylesheets.
    The output will work with FOLIO's mod-inventory-update.
* confdir: Configuration files
  * ouf.ini: Configurations for the ouf_update and ouf_fetch binaries.
  * other.ini: Configurations for picapull.pl, decode.pl, and api.pl.
* rundir: All the data we create while running
  * updsig: A place where all updsig files are temporarily stored.
  * processed: This is where picapull.pl moves the updsig files after processing.
  * fetch: The picapull.pl script saves fetch files here.
  * data: This is where ouf_fetch stores title files after reading the fetch files.
  * done: A place to more the title files after decode.pl through with them.
  * xml: This directory stores the XML files created by decode.pl.  This directory should be exposed to the harvester via HTTP.
  * trace: OUF trace files are saved to this place.
  * err: OUF err files can be found here.
  * tmp: The OUF binaries store temporary data here.
* web: The root directory for the OUF API user interface.
* ref_data: Contains scripts and Folio reference data json files.  These are just helper files to help build complex xsl:choose nodes that can be pasted into stylesheets.  The Folio json files may be useful for seeding a demo instance and will guarantee that the conversions will map properly.
* log: The Perl scripts will log output to this directory (unless if run in debug mode).
* harvester: Scripts used by docker for building harvester images
* docker: Scripts used by docker for building ouf images

## Getting started
There is a script that sets things up for you. First you need to set up some
environment variables with
  `. scripts/profile.sh`
Note the dot! After that you can run
  `. scripts/get-started.sh`
This will
  * unpack the tarball
  * compile the ouf tools
  * create the rundir directory structure


## Running it
First of all you need to set up your environment: `. scripts/profile.sh`  (note the dot!)

Then you can start `ouf_update`, `ouf_fetch`, `scripts/picapull.pl` and `scripts/decode.pl`

-- OR --

There are a couple of scripts that will start and stop everything (including setting up the environment):
* `all_start.sh`
* `all_stop.sh`

-- OR -- 

Create docker image from Dockerfile.

From project root directory...
* Build image: `docker build -t ouf .`
* Run container on port 8008: `docker run -td -p 8008:80 ouf`
* To expose the API on port 8009: `sudo docker run -d --name ouf -p 8008:80 -p 8009:3030 --rm ouf`
* To persist data and settings `sudo docker run -d --name ouf -p 3038:80 -p 3039:3030 -v oufrun:/ouf-pica/rundir -v oufconf:/ouf-pica/confdir --rm ouf`

There will be a keepalive script that restarts things if necessary. It can be run from cron.
This may be an overkill, but better safe than sorry.

## Environment variables

* OUF_SLEEP - The amount of seconds to wait between polling of directories for the picapull.pl and decode.pl scripts.  Setting this variable to a higher value will create larger XML collections, which may be useful during migration.  Default: other.ini sleep settings.  
* OUF_MODE - If set to 'migrate', then picapull.pl and decode.pl will run in migrate mode.  This means that logging will be less verbose,  files will be deleted immediately after usage, and most importantly, larger XML collections will be created.  The decode.pl script will not control the harvester in this mode and all cron jobs are disabled (we don't want to delete large collections.)
* OUF_DIR_COUNT - When running in 'migrate' mode, this will determine the number of 'm' directories that are created in the xml/ directory.  Default: other.ini migrate->dir_count setting.
* HARVESTER_URL - This overrides the harvester->url setting in the other.ini file.  If set to nothing (eg. export `HARVESTER_URL=`), the decode.pl will not control the harvester.
* OUF_NOSTART - When running in a container, set this to 1 to disable the immediate start of the OUF scripts.  This will give the operator an opportunity to change default settings before running all scripts.
* OUF_DEBUG - If set to 1, the logs for picapull.pl and decode.pl will be echoed to the console instead of log files.

## Running the harvester

There is a Dockerfile in the `harvester/` directory that will build the harvester, harvester-admin, and load default data.

From the harvester directory...
* Build image: `docker build -t harvester .`
* Run container on port 8080: `docker run -itd --name harvester -p 8080:8080 --rm harvester`
* To perist mysql data: `docker run -itd --name harvester -p 8080:8080 -v mysql:/var/lib/mysql --rm harvester`

## Transformations

The harvester requires transformation steps for it to be useful for this project.  There is a separate repo at https://github.com/indexdata/cbs2folio-transformations which contains two XSLT stylesheets and some test records:
* pica2instance.xml: This is the main transformation step that will be useful for most users
* codes2uuid.xml: This stylesheet will map Pica terms to FOLIO uuids, which will probably be different for each user.  This is where locations and other reference data should be mapped.

NOTE: cbs2folio-transformations is included in this project as a submodule.  Run the following Git commands to activate it:
* `git submodule init`
* `git submodule update`
