/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File        : cb.h
 *  Description : This is the include-file for counted buffer-handling
 *  Author(s)   : Hans van Eck
 *  Creation    : 13-09-1993
 *
 *  History:
 *  13-09-1993 : eck      : creation
 *  17-09-1993 : eck      : Added a lot of interfaces.
 *  21-12-1993 : clemens  : Added CBLength()
 *  17-10-1994 : eck      : Removed errors, added CB*Attribute()-routines
 *  02-12-1994 : HSi      : Modified interface of CB*Attribute routines and
 *                          added CBDelAttribute and CBUpdAttribute.
 *  16-12-1994  : eck     : Modified intefaces of CB*Attribute() routines.
 *                        : The length is implicit now.
 *  06-08-1998  : paul    : CB_SIZE_MAX added
 *  28-04-2000	: eck      : Added CBIsValid()
 *  01-05-2000	: eck      : Added CBGetOptAttribute()
 *
 *========================================================================
 */

#if ! defined(CB_H_INCLUDED)
#define CB_H_INCLUDED

#pragma ident "$Id: cb.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <stddef.h>
#include <limits.h>
#include <pica.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

/**
 * The size of a Counted Buffer is acceptable to the TB-library
 * as long as it is not greater than CB_SIZE_MAX.
 */
#define CB_SIZE_MAX   ((size_t)(UCHAR_MAX + 1) * (size_t)(UCHAR_MAX + 1) - 1)

/*
 * The types should be const declared, so cast are necessary.
 * This will help prevent accidental changes.
 * Unfortunately, though, this causes warnings about an initializers.
 */
typedef unsigned char CBBuf_t;
typedef voidp CBElm_p;


ERROR_TYPE(CBErrno_t);

ERROR_DECLARE(CBE_NOERROR);

extern CBErrno_t CBerrno;

size_t CBLength(const CBBuf_t *CBbuf);

size_t CBSizeLeft(const CBBuf_t *CBbuf, size_t CBsize);

size_t CBElementLength(CBElm_p CBelm);

CBElm_p CBGetElement(CBBuf_t *CBbuf, size_t CBsize, unsigned seqnum);

CBElm_p CBNextElement(CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBelm);

bool CBClear(CBBuf_t *CBbuf, size_t CBsize);

bool CBAppElement(CBBuf_t *CBbuf, size_t CBsize, voidp elmval, size_t elmsiz);

bool CBDelElement(CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBelm);

bool CBUpdElement(CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBelm,
                    voidp elmval, size_t elmlen);

CBElm_p CBGetAttribute(CBBuf_t *CBbuf, size_t CBsize, const char *CBattnam,
                       const char **CBattvalue);

CBElm_p CBNextAttribute(CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBprvname,
                        const char **CBattvalue);

bool CBAppAttribute(CBBuf_t *CBbuf, size_t CBsize, const char *CBattnam,
                    const char *CBattval);

bool CBDelAttribute ( CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBattname );

bool CBUpdAttribute ( CBBuf_t *CBbuf, size_t CBsize, CBElm_p CBattName,
                      const char *CBattValue);

bool CBAppToAttribute(CBBuf_t *CBbuf, size_t CBsize, const char *CBattnam,
                    const char *CBattval);

bool CBIsValid(const CBBuf_t *CBbuf, size_t CBlength);
const char *CBGetOptAttribute(CBBuf_t *CBbuf, size_t CBlength,
				const char *CBattnam);

bool CBGetToggleValue(const char *value, bool *result);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! define(CB_H_INCLUDED) */
