/*
 *  Copyright (c) 1999 by Pica
 *
 *  File	: ckinternals.h
 *  Description	: Internal definitions for cookie header/library
 *  Author(s)	: Bas de Bakker
 *  Creation	: 03-09-1999
 *
 *  History:
 *  03-09-1999	: bas      : creation
 *  05-04-2006	: paul     : removed CK_REQUEST_DELOLDEST
 *
 *  Notes	: What follows are definitions needed by both the cookie
 *    library and the server.  They are NOT to be used in other applications.
 *
 */

#ifndef CKINTERNALS_H_INCLUDED
#define CKINTERNALS_H_INCLUDED

#pragma ident "$Id: ckinternals.h 335412 2016-05-27 11:01:38Z heesakkm $"

typedef struct {
    enum {
	CK_REQUEST_VALIDATE = 'v',
	CK_REQUEST_CREATE = 'c',
	CK_REQUEST_DESTROY = 'd',
	CK_REQUEST_RELEASE = 'r',
	CK_REQUEST_SELECT = 's'
    } type;
    union {
	struct {
	    int seconds;	/* Minimum number of seconds unused. */
	    int total;		/* Max amount of cookies to keep */
	} r;			/* Info for 'r' request. */
    } u;
} ck_request_tp;

typedef struct {
    enum {
	CK_REPLY_PRESENT = 'p',
	CK_REPLY_NOTPRES = 'n',
	CK_REPLY_SELECT = 's',
	CK_REPLY_ERROR = 'e'
    } type;
    union {
	struct {
	    int ncookies;		/* Number of matching cookies found. */
	    const char *cookies;	/* Matching cookies found. */
	} s;				/* Information for 'select' reply. */
    } u;
} ck_reply_tp;

/* Unpack a cookie protocol request to get constituent parameters.  Returns
   FALSE on error.  Puts type in '*type', pointers to cookies (which may point
   into 'request'!) in '*cookies' array, the maximum size of which is given in
   'cks_size'.  Returns the number of cookies received or < 0 on error. */
int
CK_Proto_RequestUnpack(const void *request,
		       int reqsize,
		       ck_request_tp *type,
		       const char **cookies,
		       int cks_size);

/* Create a reply, put pointer to privately owned memory containing reply in
   '*reply', return size of reply or < 0 on error. */
int
CK_Proto_ReplyCreate(void **reply,
		     ck_reply_tp type);

/* Destroy reply created with CK_Proto_ReplyCreate. */
void
CK_Proto_ReplyDestroy(void *reply);

#endif /* CKINTERNALS_H_INCLUDED */
