/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: er.h
 *  Description	: Interface to the ER-library
 *  Author(s)	: Clemens Buys
 *  Creation	: 10-12-1993
 *
 *  History:
 *  10-12-1993	: clemens  : creation
 *  09-03-1994	: eck      : Added ERWarning(), added errcod to ERTrace
 *
 *========================================================================
 */

#if ! defined(ER_H_INCLUDED)
#define ER_H_INCLUDED

#pragma ident "$Id: er.h 368646 2019-07-16 16:25:21Z vaneckh $"

#include <pica.h>

#ifdef __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

ERROR_TYPE(ERCode_t);

ERROR_TYPE(ERErrno_t);

ERROR_DECLARE(ERE_NOERROR);
ERROR_DECLARE(ERE_OPEN);
ERROR_DECLARE(ERE_IO);

extern ERErrno_t ERerrno;

/*PRINTFLIKE3*/
bool ERTrace(ERCode_t errcod, const char *errloc, const char *errcom, ...) __attribute__((format(printf, 3, 4)));

/*PRINTFLIKE3*/
bool ERWarning(ERCode_t errcod, const char *errloc, const char *errcom, ...) __attribute__((format(printf, 3, 4)));

/*PRINTFLIKE3*/
bool ERWrite(ERCode_t errcod, const char *errloc, const char *errcom, ...) __attribute__((format(printf, 3, 4)));

/*PRINTFLIKE3*/
bool ERAbort(ERCode_t errcod, const char *errloc, const char *errcom, ...) __attribute__((format(printf, 3, 4)));

/*PRINTFLIKE3*/
bool ERExit(ERCode_t errcod, const char *errloc, const char *errcom, ...) __attribute__((format(printf, 3, 4)));

#ifdef __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(ER_H_INCLUDED) */
