/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File        : fcv.h
 *  Description : The interface for the FCV-library
 *  Author(s)   : Paul Smit
 *  Creation    : 07-02-1994
 *
 *  History:
 *  07-02-1994  : paul     : creation
 *  11-04-1994  : paul     : #pragma ident removed (TANDEM)
 *  27-04-1994  : paul     : typedefs SynBuf_t and SynBuf_p added
 *  12-05-1994  : paul     : ISTAGSYM added
 *  19-05-1994  : paul     : typedef fsxname_def on TANDEM retrieved from DDL
 *  23-09-1994  : paul     : FCVnormalize and FCVdenormalize copied from nrm.h
 *  08-12-1994  : paul     : FCVdenormalize -> FCVDENORMALIZE (will be imported
 *                           in PASCAL programs)
 *  04-01-1995  : paul     : FCVNRM() added to link it directly into PASCAL
 *  20-02-1995  : paul     : nr of syntax elements and syntax size now of type
 *                           unsigned int, so limited to 64 kB
 *  20-03-1995  : paul     : function indc added
 *  29-03-1995  : paul     : function occx added
 *  02-05-1995  : paul     : template chars moved from 0x80.. to 0x01..
 *  31-07-1995  : paul     : FCV function return type was not defined correctly
 *  11-08-1995  : paul     : NT_OCCURRENCE added
 *  15-08-1995  : paul     : SC_NEWLINE added, since this can be in sourcetable
 *  16-08-1995  : paul     : NT_CODE, NT_TAG_CODE and NT_FATAL added;
 *                           ISTAGMASKSYM, ISSUBFMASKSYM, ISSUBFSYM introduced
 *  25-09-1995  : paul     : conforming to POPC conventions
 *  20-10-1995  : paul     : FCVSortNormTitle and FCVSortPicaTitle added
 *  22-02-1996  : paul     : FCV_MAX_SYNBUF_ELT changed from 4000 to 6000
 *  28-03-1996  : paul     : FCV_MAX_SYNBUF_ELT changed from 6000 to 10750
 *  04-06-1996  : lolke    : done previous action on SUN
 *  11-06-1996  : EZ       : FCVE_UNKNOWN_CHARSET added
 *  05-09-1996  : paul     : Patched FCV_SC_TEMPLATE.
 *  26-02-1997  : paul     : FCVCompileCheck() added
 *  13-10-1997  : paul     : FCV_SC_HTML_OPEN, FCV_SC_HTML_CLOSE and
 *                           FCV_SC_HTML_AMPERSAND added
 *  11-02-1998  : HSi      : Added support for endianess.
 *                           Increased buffer size for diagnostics.
 *  12-03-1998  : paul     : FCVConvert() introduced (must replace FCVNrm())
 *                         : FCVCompileCheck() renamed to FCVStrip()
 *  25-03-1998  : paul     : #define FCV_SC_HTML_OPEN_STR etc. added
 *  16-09-1998  : paul     : FCVSynBuf_t: sb_reverse and sb_tagcount added
 *                           FCVTagCodeConvert() added
 *                           OCCSIZE moved from fcvlocal.h as FCV_OCCSIZE
 *                           FCV_SUBFIELD_NAME_LEN moved from fcvlocal.h
 *                           #define FCV_SYNBUF_LEN(), FCV_SC_SPECIAL() added
 *                           FCVReverse() + FCVTagCodeReverse() added
 *  03-11-1998  : paul     : FCV_MAX_SYNBUF_ELT changed from 10750 to 15000
 *                           FCV_SC_ESCAPE added
 *  23-11-1998  : paul     : FCV_MAX_SYNBUF_ELT changed back from 15000 to
 *                           10750 in case of small memory model on TANDEM
 *  10-12-1998  : paul     : FCV_TAG_PROTECT_OPEN+FCV_TAG_PROTECT_CLOSE moved
 *                           from fcvlocal.h
 *  14-12-1998  : paul     : FCVCharsConvert() moved from fcvlocal.h
 *                           FCVCharsConvert(): interface changed
 *                           FCV_ISTAGSYM() moved to fcv.h
 *  12-01-1999  : OvV      : added FCV_SC_NBSP
 *  26-01-1999  : paul     : FCVSyntacticError_t moved from fcvrev.c
 *                           FCV_SUBFSYM_PROTECT moved from fcvlocal.h
 *  09-06-1999  : BdB   : Add FCV_SC_HTML_PERCENT special char
 *  03-11-1998  : paul  : FCV_MAX_SYNBUF_ELT changed from 15000 to 25000
 *  10-10-2000  : paul  : FCVIsReversible() introduced
 *  10-10-2000  : paul  : FCVTagComparePica(), FCVTagCompareAlphabetical()
 *                        and FCVTagCompare7800Last() introduced
 *  20-11-2000  : paul  : FCV_opt_fetch() introduced
 *  12-06-2001  : paul  : FCVTagFilter() and FCV_TAGFILTER_SEP added
 *  19-07-2001  : paul  : added FCVConvertCallBack() and FCVSwitchHtmlChars()
 *  11-10-2001  : paul  : FCVInterpretEscapes() added
 *  23-10-2001  : paul  : FCVOccurrenceGet() added
 *  31-03-2003  : HvdW  : tandem ifdefs removed
 *  09-04-2003  : paul  : Added FCVConvertAppend() and FCVConvertCS() for
 *                        counted string output
 *  09-04-2003  : paul  : Added FCVReverseAppend() and FCVReverseCS() for
 *                        counted string output
 *                      : Removed parenthesis from defines
 *                        FCV_TAG_PROTECT_OPEN and FCV_TAG_PROTECT_CLOSE
 *  20-05-2003  : paul  : FCV_MAX_SYNBUF_ELT changed from 25000 to 30000
 *  23-07-2003  : paul  : added declaration of FCVerrno
 *  23-10-2003	: paul  : removed FCVErrno parameter from FCVCompileCreate()
 *  30-07-2003  : paul  : added FCVConvertCreate() and FCVReverseCreate()
 *  22-10-2003  : paul  : added member sb_charset to FCVSynBuf_t
 *                      : added typedef FCVCharset_t
 *                      : Moved FCV_SC_TEMPLATE + FCV_SC_SPECIAL to fcvlocal.h
 *  05-12-2003  : paul  : Added FCVCompileCreateOpt()
 *                      : Added FCVCharset()
 *  19-12-2003  : paul  : Added utf8 parameter to FCV_opt_fetch() and
 *                        FCVInterpretEscapes()
 *  21-01-2004  : paul  : Added FCVCharsConvertCreate()
 *  20-05-2003  : paul  : FCV_MAX_SYNBUF_ELT changed from 30000 to 50000
 *  19-03-2004  : paul  : Added FCVNormalizeCS() and FCVDenormalizeCS()
 *  20-05-2003  : paul  : FCV_MAX_SYNBUF_ELT changed from 50000 to 65500
 *  29-11-2004  : paul  : Added FCVCompileCreate2() for PSI
 *  01-12-2004  : paul  : Renamed FCV_OCCSIZE to FCV_OCCLEN_MIN
 *                      : Added FCVTagCodeEnd()
 *  01-01-2005  : paul  : Added FCV_SYNERR_TAG_CONFLICTS_WITH_INVISIBLE_TAG
 *  13-01-2005  : paul  : Added FCVE_PARAM
 *                      : Added FCVPunct*()
 *  02-02-2005  : paul  : Added FCVPrintHtmlChars()
 *  22-03-2005  : paul  : Added FCV_SC_RECORD and FCV_SC_RECORD_STR
 *  23-03-2005  : paul  : Removed FCVCompile(), FCVCompile2() and FCVStrip()
 *                      : Moved syntax structure type def to fcvlocal.h
 *  29-11-2006  : paul  : Added FCVCharsConvertOptionsStrip()
 *  05-02-2010	: paul     : Added FCVPrint() FCVPCatToPPlus() FCVEqual()
 *  27-08-2010	: paul     : CBS-2249 Added FCVtagcodecmp()
 *  09-09-2011	: paul     : CBS-3224 Allow callbacks with userdata in EVAL FCV
 *  25-04-2017  : petersh  : added #define FCV_EMPTY_SUBFIELDS "678"
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(FCV_H_INCLUDED)
#define FCV_H_INCLUDED

#pragma ident "$Id: fcv.h 377898 2020-07-02 12:12:17Z petersh $"

#include <stddef.h>

#include <pica.h>
#include <picatypes.h>
#include <picamisc.h>
#include <cs.h>
#include <pa.h>
#include <pp.h>
#include <nmt.h>
#include <html.h>

typedef const char *(*FCVInclude_f)(const char *, void *);

typedef const char *FCVErrno_t;
ERROR_DECLARE(FCVE_NOERROR);
ERROR_DECLARE(FCVE_NOMEM);
ERROR_DECLARE(FCVE_NOHEAP);
ERROR_DECLARE(FCVE_SYNTAX);
ERROR_DECLARE(FCVE_UNKNOWN_CHARSET);
ERROR_DECLARE(FCVE_UNKNOWN_SOURCE_FORMAT);
ERROR_DECLARE(FCVE_UNKNOWN_TARGET_FORMAT);
ERROR_DECLARE(FCVE_ILLEGAL_SOURCE_TITLE);
ERROR_DECLARE(FCVE_ILLEGAL_NORMAL_TITLE);
ERROR_DECLARE(FCVE_NORMALIZATION_NOT_IMPLEMENTED);
ERROR_DECLARE(FCVE_DENORMALIZATION_NOT_IMPLEMENTED);
ERROR_DECLARE(FCVE_SYNTAX_EMPTY);
ERROR_DECLARE(FCVE_TARGET_EMPTY);
ERROR_DECLARE(FCVE_INTERNAL_ERROR);
ERROR_DECLARE(FCVE_NUMBER_EXPECTED);
ERROR_DECLARE(FCVE_ILLEGAL_OCC);
ERROR_DECLARE(FCVE_TOO_MANY_TAGS);
ERROR_DECLARE(FCVE_ILLEGAL_EXPRESSION);
ERROR_DECLARE(FCVE_ILLEGAL_SYNTAX);
ERROR_DECLARE(FCVE_ILLEGAL_POSITION);
ERROR_DECLARE(FCVE_ILLEGAL_STATEMENT);
ERROR_DECLARE(FCVE_OVERFLOW);
ERROR_DECLARE(FCVE_SORT_ERROR);
ERROR_DECLARE(FCVE_TOO_MANY_STOPS);
ERROR_DECLARE(FCVE_LOAD_SYNTAX);
ERROR_DECLARE(FCVE_TOO_MANY_SYNTAXES);
ERROR_DECLARE(FCVE_PARAM);

extern FCVErrno_t FCVerrno;

/* A SpecialChar can occur in any name or text */
typedef enum {
    /* Template characters */
    FCV_SC_ANY = 0x01, FCV_SC_POSITIVE, FCV_SC_NON_POSITIVE,
    FCV_SC_DIGIT, FCV_SC_NON_DIGIT, FCV_SC_UPPERCASE, FCV_SC_NON_UPPERCASE,
    FCV_SC_LOWERCASE, FCV_SC_NON_LOWERCASE,
    FCV_SC_NEWLINE,
    FCV_SC_ALPHA, FCV_SC_NON_ALPHA,
    FCV_SC_ALPHANUM, FCV_SC_NON_ALPHANUM,

    /* Miscellaneous */
    FCV_SC_QUOTE,

    FCV_SC_HTML_PERCENT = 0x17,
    FCV_SC_HTML_NBSP = 0x18,

    /* Escape character, introducing protection "\escI" */
    FCV_SC_ESCAPE = 0x19,

    /* HTML characters */
    FCV_SC_HTML_OPEN = 0x1a,
    FCV_SC_HTML_CLOSE = 0x1b,
    FCV_SC_HTML_AMPERSAND = 0x1c,

    /* Record, tag and subfield indicators.
     * These values are fixed by the international library community.
     * Don't change them without consent by the rest of the world.
     */
    FCV_SC_RECORD = 0x1d,
    FCV_SC_TAG = 0x1e,
    FCV_SC_SUBFIELD = 0x1f
} SpecialChar_t;
#define FCV_SC_HTML_PERCENT_STR   "\x17"
#define FCV_SC_HTML_NBSP_STR      "\x18"
#define FCV_SC_ESCAPE_STR         "\x19"
#define FCV_SC_HTML_OPEN_STR      "\x1a"
#define FCV_SC_HTML_CLOSE_STR     "\x1b"
#define FCV_SC_HTML_AMPERSAND_STR "\x1c"
#define FCV_SC_RECORD_STR         "\x1d"
#define FCV_SC_TAG_STR            "\x1e"
#define FCV_SC_SUBFIELD_STR       "\x1f"
#define FCV_SC_ANY_STR            "\x01"

// A meta subfield is a subfield representing bool property of a field
#define FCV_SUBFSYM_IS_META(subftyp) \
    ((subftyp) == FCV_SUBFSYM_PROTECT || (subftyp) == FCV_SUBFSYM_FLAG)

/*
 *  Protection used to protect whole tags in edit screens.
 *  FCV_TAG_PROTECT_OPEN is used in cataloguing format;
 *  A subfield with code FCV_SUBFSYM_PROTECT is used in pica format.
 */
#define FCV_TAG_PROTECT_OPEN             FCV_SC_ESCAPE_STR PM_ESC_PROTECT_OPEN
#define FCV_TAG_PROTECT_CLOSE            FCV_SC_ESCAPE_STR PM_ESC_PROTECT_CLOSE
#define FCV_SUBFSYM_PROTECT              '-'

// The code of a subfield that is a yes/no property of its tag
// For example "$!isSelected" marks the tag as being selected
// For example "$!isModified" marks the tag as being modified
#define FCV_SUBFSYM_FLAG		'!'

// Selection of tags used in CCWeb
#define FCV_TAG_SELECT_OPEN		FCV_SC_ESCAPE_STR PM_ESC_SELECT_OPEN
#define FCV_TAG_SELECT_CLOSE		FCV_SC_ESCAPE_STR PM_ESC_SELECT_CLOSE
#define FCV_SUBFFLAG_IS_SELECTED	"isSelected"

// Modification of tags used in CCWeb
#define FCV_TAG_MODIFIED_OPEN		FCV_SC_ESCAPE_STR PM_ESC_MODIFIED_OPEN
#define FCV_TAG_MODIFIED_CLOSE		FCV_SC_ESCAPE_STR PM_ESC_MODIFIED_CLOSE
#define FCV_SUBFFLAG_IS_MODIFIED	"isModified"

// Protection used to protect PPN-expansions in edit+show screens
#define FCV_PPNEXPANSION_PROTECT_OPEN	FCV_SC_ESCAPE_STR PM_ESC_INACCESSIBLE_OPEN
#define FCV_PPNEXPANSION_PROTECT_CLOSE	FCV_SC_ESCAPE_STR PM_ESC_INACCESSIBLE_CLOSE

// Occurrence in the copy leader tag (Pica+: 208@/* Pica3: 70* UNM: e*)
// that will be replaced by an unused occurrence
#define FCV_OCCURRENCE_ANY	"*"

// $% is used to contain the revision-difference type of a tag
// $%A means: added tag; $%D means: deleted tag
// In annotated titles, $%123 means: this tag has been added in revision nr 123
#define FCV_REV_SUBFSYM		'%'
#define FCV_REV_ADDED		"A"
#define FCV_REV_DELETED		"D"

/* this subfields should never be removed during format conversion */
#define FCV_EMPTY_SUBFIELDS "678"

// A non-empty contents-subfield causes CCWeb to NOT silently remove a tag from an edit-screen
// $S $T $U and $x-occurrence aren't contents-subfields, unless they are longer than normal
// If a tag only contains empty contents-subfields and non-contents-subfields then CCWeb will suppress it
#define FCV_IS_CONTENTS_SUBFIELD(sbftyp, sbflen, taglev) \
    (((sbftyp) != PIC_SUBFTYPE_INDICATORS || (sbflen) > strlen(PIC_DEFAULT_INDICATORS)) \
     && ((sbftyp) != PIC_SUBFTYPE_LINKAGE_OCC || (sbflen) > PIC_LINKAGE_OCC_LEN) \
     && (sbftyp) != PIC_SUBFTYPE_SCRIPTCODE	\
     && (taglev != PPLEV_COPY			\
         || (sbftyp) != PIC_XOCC_SBFTYP 	\
         || (sbflen) > PIC_XOCC_LEN)		\
     && (sbftyp) != FCV_SUBFSYM_FLAG)

/* Here's the minimum length of the number of digits in a normalized tag code
 * behind the slash.
 * Examples of codes of normalized tags:
 * ABCD		(occurrence 0)
 * ABCD/01
 * ABCD/100
 * ABCD/32767
 * Note that an occurrence should fit into an int16
 */
#define FCV_OCCLEN_MIN          2

/* The length of a subfield name like $a */
#define FCV_SUBFIELD_NAME_LEN   2

#define FCV_ISTAGSYM(c)     (isalnum(c) || (c) == '@')

#define FCV_ISSUBFSYM(c)     (PIC_SUBFTYPE_OK(c) \
                              || (c) == '<' || (c) == '>' \
                              || (c) == FCV_REV_SUBFSYM \
                              || (c) == FCV_SUBFSYM_FLAG \
                              || (c) == FCV_SUBFSYM_PROTECT)
/*
 * During the transition from charset Pica to charset Utf8, it's necessary to
 * keep track of the charset. CHARSET_ASCII means that none of the string
 * constant in the syntax contains either a Pica-diacritic, or a utf8-grapheme.
 */
typedef enum FCVCharset_t {
    FCV_CHARSET_ASCII,
    FCV_CHARSET_PICA,
    FCV_CHARSET_UTF8
} FCVCharset_t;
    
typedef struct FCVSynBuf_t *FCVSynBuf_p, FCVSynBuf_t;

/*
 * Syntactic errors are generated by FCVReverse() and VALFiltering()
 * if the source record contained an error. These errors can be formatted by
 * VALErrorFormat() and must correspond to the "messages syntactic" rule in
 * the validation table.
 */
typedef enum FCVSyntacticError_t {
    FCV_SYNERR_NONE,
    FCV_SYNERR_SUBF_REQUIRED,
    FCV_SYNERR_TAG_UNKNOWN,
    FCV_SYNERR_TAG_TOO_SHORT,
    FCV_SYNERR_TAG_NO_SPACE,
    FCV_SYNERR_COPYTAG_WITHOUT_COPY,
    FCV_SYNERR_TEXT_DISAPPEARS,
    FCV_SYNERR_TEXT_APPEARS,
    FCV_SYNERR_SUBF_EXPECTED_AFTER,
    FCV_SYNERR_PUNCT_EXPECTED_AFTER,
    FCV_SYNERR_SUBF_EXPECTED_IN_FRONT_OF,
    FCV_SYNERR_PUNCT_EXPECTED_IN_FRONT_OF,
    FCV_SYNERR_TAG_WILL_BE_FILTERED = 100,
    FCV_SYNERR_TAG_WILL_BE_PROTECTED,
    FCV_SYNERR_TAG_CONFLICTS_WITH_INVISIBLE_TAG,
    FCV_SYNERR_PROTECTED_TAG_MODIFIED
} FCVSyntacticError_t;

typedef enum {
    FCVEND_LITTLE, FCVEND_BIG
} FCVEndianness_t;

extern FCVEndianness_t  FCV_int_ttlendian;

#define FCV_DIAG_SIZE           500
extern char     FCVDiagnostic[FCV_DIAG_SIZE];

typedef const char *(*FCVInterpreterNew_f)(const char *, PAHandle_p, cvoidp);
typedef const char *(*FCVInterpreter_f)(HTMLArgList_p);

// Callback function for hyphenating ISBN's
typedef size_t (*FCVHyphenateIsbn_f)(const char *, char *, size_t);
#define FCV_HYPHENATE_ISBN_NOFUNC ((FCVHyphenateIsbn_f)0)

FCVSynBuf_p FCVCompileCreateOpt(const char *options, const char *text_in);
FCVSynBuf_p FCVCompileCreate(const char *text);
void        FCVCompileDestroy(FCVSynBuf_p syn_buf);
FCVSynBuf_p FCVCompileCreate2(const char *options, const char *text_in,
			      FCVInclude_f f, void *arg);

FCVErrno_t FCVNrm(char *srcbuf, size_t srclen,
                  char *tarbuf, size_t *tarlen, size_t tarsiz,
                  FCVSynBuf_p syn_buf);

FCVErrno_t FCVConvert(const char *options, const char *srcbuf, size_t srclen,
                      char *tarbuf, size_t *tarlen, size_t tarsiz,
                      FCVSynBuf_p syn_buf);

FCVErrno_t FCVConvertCallBack(const char *options, const char *srcbuf, 
                              size_t srclen, char *tarbuf, size_t *tarlen, 
                              size_t tarsiz, FCVSynBuf_p syn_buf, 
			      FCVInterpreter_f interpreter);

FCVErrno_t FCVTagCodeConvert(const char *options, const char *srctag,
                             char *tartag, size_t *tarlen, size_t tarsiz,
                             FCVSynBuf_p syn_buf);

const char *FCVTagCodeEnd(const char *options, const char *srctag,
			  FCVSynBuf_p synbuf);

FCVErrno_t FCVConvertAppend(
    const char	*options,
    const char	*srcbuf,
    size_t	srclen,
    CSBuf_p 	tar,
    size_t	extra_tarlen,
    FCVSynBuf_p	synbuf,
    FCVInterpreter_f interpreter);

FCVErrno_t FCVConvertCS(
    const char	*options,
    const char	*srcbuf,
    size_t	srclen,
    CSBuf_p 	tar,
    size_t	max_tarlen,
    FCVSynBuf_p	synbuf,
    FCVInterpreter_f interpreter);

CSBuf_p
FCVConvertCreate(
    const char	*options,
    const char	*srcbuf,
    size_t	srclen,
    size_t	max_tarlen,
    FCVSynBuf_p	synbuf,
    FCVInterpreter_f	interpreter);

CSBuf_p
FCVConvertCreateCS(
        CSBuf_p dest, const char *options, const char *srcbuf, 
	size_t srclen, FCVSynBuf_p synbuf, 
	FCVInterpreterNew_f interpreter_new,
	cvoidp userdata);

// Callbacks+userdata needed for conversion
typedef struct {
    FCVInterpreterNew_f	interpreter_new;
    cvoidp		interpreter_new_userdata;
    FCVInterpreter_f	interpreter;
    FCVHyphenateIsbn_f	hyphenate_isbn;
} FCVConvertCB_t, *FCVConvertCB_p;

void
FCVConvertCBClear(
	FCVConvertCB_p	cb);

CSBuf_p
FCVConvertCB(
	FCVSynBuf_p	synbuf, 
	CSBuf_p		dest,
	const char	*srcbuf,
	long		srclen,
	const char	*options,
	FCVConvertCB_p	cb);

FCVErrno_t
FCVConvertCBOld(
	const char	*options,
	const char	*srcbuf,
	size_t		srclen,
	char		*tarbuf,
	size_t		*tarlen,
	size_t		tarsiz,
	FCVSynBuf_p     synbuf,
	FCVConvertCB_p	cb);

FCVErrno_t FCVReverse(
        const char      *options,
        const char      *srcbuf,
        char            *tarbuf,
        size_t          *tarlen,
        size_t           tarsiz,
        FCVSynBuf_p      synbuf,
        voidp            valerrs);

FCVErrno_t FCVReverseCS(
    const char	*options,
    const char	*srcbuf,
    CSBuf_p 	tar,
    FCVSynBuf_p	synbuf,
    voidp	valerrs);

CSBuf_p
FCVReverseCreate(
    const char	*options,
    const char	*srcbuf,
    FCVSynBuf_p	synbuf,
    voidp	valerrs);

FCVErrno_t FCVReverseAppend(
    const char	*options,
    const char	*srcbuf,
    CSBuf_p 	tar,
    FCVSynBuf_p	synbuf,
    voidp	valerrs);

FCVErrno_t FCVTagCodeReverse(
        const char      *options,
        const char      *srctag,
        char            *tartag,
        size_t          *tarlen,
        size_t          tarsiz,
        FCVSynBuf_p     synbuf);

bool FCVIsReversible(FCVSynBuf_p synbuf);

FCVErrno_t FCVNormalize(char *srcbuf, size_t srclen,
                        char *tarbuf, size_t *tarlen, size_t tarsiz,
                        const char *src_fmt, const char *options);

FCVErrno_t FCVDenormalize(char *srcbuf, size_t srclen,
                          char *tarbuf, size_t *tarlen, size_t tarsiz,
                          const char *tar_fmt, const char *options);

CSBuf_p FCVNormalizeCS(char *srcbuf, size_t srclen, CSBuf_p tar,
		       const char *src_fmt, const char *options);

CSBuf_p FCVDenormalizeCS(char *srcbuf, size_t srclen, CSBuf_p  tar,
			 const char *tar_fmt, const char *options);

FCVErrno_t FCVSortNormTitle(char *buf, size_t len);

FCVErrno_t FCVSortPicaTitle(char *buf, size_t len);

CSBuf_p FCVSortPicaTitleCreate(const char   *buf, size_t len);

CSBuf_p FCVSortNormTitleCreate(const char   *buf, size_t len);

int FCVTagComparePica(const void *t1, const void *t2);

int FCVTagCompareAlphabetical(const void *tag1, const void *tag2);

int FCVTagCompare7800Last(const void *tag1, const void *tag2);

int FCVTagCompareMarc21Cbs(const void *tag1, const void *tag2);

int
FCVtagcodecmp(
        const char      *tagcode1,
        const char      *tagcode2,
        bool             tagcode2_is_mask,
        const char     **tagcode1end,
        const char     **tagcode2end);

FCVCharset_t FCVCharset(FCVSynBuf_p syn_buf);

FCVErrno_t
FCVCharsConvert(
        const char      *options,
        bool             metasbfs_present,
        char            *tarbuf,        /* the buffer containing the target */
        size_t          *tarlen,        /* pointer to new length or NULL */
        size_t           tarsiz);       /* max length of the target */
CSBuf_p
FCVCharsConvertCreate(
        const char	*options,
        bool		metasbfs_present,
	const char	*srcbuf,
	long		srclen);

CSBuf_p
FCVCharsConvertOptionsStrip(
	CSBuf_p 	dest,
	const char	*options);

void
FCVProtectionRemove(
	PPTtl_p              pptitle);
void
FCVMetaSbfsRemove(
	PPTtl_p		pptitle);

void
FCVSwitchHtmlChars(
    char                   *destbuf);
CSBuf_p
FCVPrintHtmlChars(
	bool		html,
	const char	*destbuf);

int
FCVInterpretEscapes(
        bool		utf8,		/* charset=utf8? */
        const char      *name,		/* option to find */
        const char	*valbeg,	/* the begin of the options-value */
        const char	*valend,	/* the end of the options-value */
        char		*target,	/* place to put the value */
        size_t		maxlen);	/* size of the target */

int
FCV_opt_fetch(
        bool		utf8,		/* charset=utf8? */
        const char      *options,	/* the options-string to search */
        const char      *name,		/* option to find */
        char            *target,	/* place to put the value */
        size_t          maxlen);	/* size of the target */

#define FCV_TAGFILTER_SEP    ':'
bool
FCVTagFilter(
        const char      *filtertext,    /* string containing filter */
        char            *ttlbuf);       /* input and output title */

PICOccurrence_t
FCVOccurrenceGet(const char         *tagp);

CSBuf_p
FCVPunctOpenCode(
	const char	*options,
	const char	*tagcode,	/* Pica+ tag code */
	int		sbfcode,	/* Pica+ subfield code */
	FCVSynBuf_p	syntax);	/* the (reversible) syntax */
CSBuf_p
FCVPunctCloseCode(
	const char	*options,
	const char	*tagcode,	/* Pica+ tag code */
	int		sbfcode,	/* Pica+ subfield code */
	FCVSynBuf_p	syntax);	/* the (reversible) syntax */
CSBuf_p
FCVPunctOpenSubf(
	const char	*options,
	const char	*tag,		/* Normalized Pica+ tag */
	const char	*sbfcode,	/* points to subfind inside of tag */
	FCVSynBuf_p	syntax);	/* the (reversible) syntax */
CSBuf_p
FCVPunctCloseSubf(
	const char	*options,
	const char	*tag,		/* Normalized Pica+ tag */
	const char	*sbfcode,	/* points to subfind inside of tag */
	FCVSynBuf_p	syntax);	/* the (reversible) syntax */


CSBuf_p
FCVPrint(
	CSBuf_p		dest,
	FCVSynBuf_p	synbuf,
	const char	*options);

CSBuf_p
FCVPCatToPPlus(
	CSBuf_p		dest,
	FCVSynBuf_p	synbuf,
	const char	*options);

bool
FCVEqual(
	FCVSynBuf_p	synbuf1,
	FCVSynBuf_p	synbuf2,
	const char	*options);

FCVErrno_t
FCVSbfOrder(
        const char      *options,
        const char      *srctag,
        char            *tarbuf,
        size_t          *tarlen,
        size_t          tarsiz,
        FCVSynBuf_p     synbuf);

PPSbf_p
FCVSbfAdd(
        FCVSynBuf_p	synbuf,
	PPTag_p		tag,
	PPSbf_p		newsbf,
	const char	*options);

/* Automatic punctuation */
typedef struct FCVAPTHandle_t	*FCVAPTHandle_p;
void
FCVAPTHandleDestroy(
	FCVAPTHandle_p	handle);
FCVAPTHandle_p
FCVAPTParse(
	const char	*apttxt,
	const char	*options);
const char *FCVAPTDiagnostic(void);
const char *
FCVAPTPunct(
	FCVAPTHandle_p	handle,
	const char	*tag,
	char		begsubf,
	char		endsubf,
	const char	*options);

bool
FCVAPTRemove(
	FCVAPTHandle_p	handle,
	PPElement_p	ppElt,
	const char	*options);

CSBuf_p
FCVAPTPrint(
	FCVAPTHandle_p	handle,
	const char	*options);
CSBuf_p
FCVAPTPrintPunct(
	CSBuf_p 	dest_print,
	const char	*punct,
	const char	*options);

const char *FCV_APTKey(FCVSynBuf_p synbuf);
bool FCV_APTHandleAdd(FCVSynBuf_p synbuf, FCVAPTHandle_p apt_handle);

// Interface of fcvtagrule.c
typedef const struct FCVSynbufElt_t *FCVSynbufElt_cp;
typedef bool (*FCVTagRuleWalk_f)(FCVSynbufElt_cp tagrule, voidp xtra);
bool
FCVTagRuleWalk(
        FCVSynBuf_p		synbuf,
        FCVTagRuleWalk_f	process_tagrule,
        voidp			xtra);

const char *
FCVTagRule2PPTag(
	FCVSynbufElt_cp	tagrule,
	const char	*options);

CSBuf_p
FCVTagRule2CatFmtTag(
	FCVSynbufElt_cp	tagrule,
	CSBuf_p		dest,
	const char	*options);

#define FCV_CATFMT_SBF_ANONYMOUS_NAME	"#"
typedef const struct FCVCatFmtSbf_t	*FCVCatFmtSbf_cp;
typedef const struct FCVCatFmtSbfInfo_t	*FCVCatFmtSbfInfo_cp;
void FCVCatFmtSbfInfoDestroy(FCVCatFmtSbfInfo_cp info);
FCVCatFmtSbfInfo_cp FCVTagRule2CatFmtSbfInfo(FCVSynbufElt_cp tagrule);
FCVSynbufElt_cp FCVCatFmtSbfInfoTagRule(FCVCatFmtSbfInfo_cp info);
NMTHandle_p FCVCatFmtSbfInfoSbfSet(FCVCatFmtSbfInfo_cp info);
PAHandle_p FCVCatFmtSbfInfoOrder(FCVCatFmtSbfInfo_cp info);
bool FCVCatFmtSbfIsRequired(FCVCatFmtSbf_cp catFmtSbf);
bool FCVCatFmtSbfIsRepeatable(FCVCatFmtSbf_cp catFmtSbf);
bool FCVCatFmtSbfConsistsOf1ppSbf(FCVCatFmtSbf_cp catFmtSbf);
const char *FCVCatFmtSbfPPSbfCodes(FCVCatFmtSbf_cp catFmtSbf);
const char *FCVCatFmtSbfCatFmtSbfCodes(FCVCatFmtSbf_cp catFmtSbf);

bool
FCVPPSbfOrderIsTransliterable(
	const char	*sbfOrder);

CSBuf_p
FCVTagRule2PPSbfOrder(
	FCVSynbufElt_cp	tagrule,
	CSBuf_p		dest);

bool
FCVTagRulePunct(
	FCVSynbufElt_cp	tagrule,
	char		sbfcode,
	const char	**open,
	const char	**close);

CSBuf_p
FCVTagRulePrint(
	FCVSynbufElt_cp	tagrule,
	CSBuf_p		dest);

#endif /* ! defined(FCV_H_INCLUDED) */
