/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: fio.h
 *  Description	: The interface to the FIO-library
 *  Author(s)	: Clemens Buys
 *  Creation	: 20-12-1993
 *
 *  History:
 *  20-12-1993	: clemens  : creation
 *  24-02-1994	: HSi	   : added some errorcodes, removed include of pica.h
 *  11-01-1996  : JV       : changed datatype userNumber to char(32)
 *  20-02-1996	: eck      : Changed interfaces to FIOTB* and FIOAC*.
 *  03-07-1997	: HSi	   : Changed returntype of transaction functions from
 *			     void to bool.
 *			     Added FIOTTL-functions.
 *  04-07-1997	: HSi	   : FIOTTLBuf_t -> PICTTLBuf_t
 *  26-09-1997	: HSi	   : Added FIOE_CLOSE.
 *  15-12-1997	: HSi	   : Added FIOTTLUpdate() and FIOTTLDelete().
 *  20-05-1998  : EH       : FIOTitle_t added
 *  28-07-1998  : TF       : fioos.h included
 *  25-08-1998	: BdB	   : Added FIOSignal()
 *  03-09-1998	: BdB	   : Added FIOTTLStoreBulk()
 *  22-10-1998  : TF       : Added (temporary) FIOCBS_, FIODBS_ and  
 *                           FIOILL_ functions 
 *  13-01-1999  : fb       : Updated FIOTitle_t to use sourcecode_t
 *  23-07-1999  : LB       : As sqlddl.h is included no differences 
 *			     anymore for FIOTimestamp and FIODate
 *  22-05-2000  : LB       : FIO_FLIPPER, FIO_LOWEST and FIO_HIGHEST added 
 *  07-06-2000	: eck      : Added error: FIOE_TIMESTAMP
 *  26-06-2000  : LB       : FIO_EMPTY*, FIO_LAST* etc. added 
 *  28-06-2000  : fb       : Corrected the prototype of FIOTTLDatabaseLog.
 *  26-06-2000  : LB       : FIO_EMPTY*, FIO_LAST* set to correct values.
 *  13-02-2001  : EvD      : Added FIOSetSavepoint and
 *                           FIORollbackSavepoint 
 *  19-02-2001	: BdB	: Changed ttl_buflength to int (from int16)
 *  08-05-2001  : evd   : changed FIOTTL_MAXBUFFER from 64k to 4 * 64k
 *  17-01-2002  : evd   : Added FIOE_DUPLICATEROW
 *  15-04-2002  : fb    : Added FIOCancelBatch().
 *  31-03-2003  : HvdW  : tandem ifdefs removed
 *  10-04-2003	: jaap     : Remove include of fioos.h
 *  07-02-2005	: heesakkm : Increased ttl_ppn buffer size from 10 to 12.
 *  17-08-2010  : evd      : cbs-2254 : Added YN_IGNORE to ynflag_v 
 *  29-07-2015	: paul     : CBS-6221 Moved declarations to fiottlpublic.h
 *
 *  Notes	:
 *
 *========================================================================
 */

#if ! defined(FIO_H_INCLUDED)
#define FIO_H_INCLUDED

#pragma ident "$Id: fio.h 339156 2016-11-18 18:26:14Z heesakkm $"

#include <stddef.h>
#include <pica.h>
#include <picatypes.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

#if ! defined (STRDEFINE)
#define STRDEFINE(x)    struct { int16 len; char  val[(x) + 1]; }
#endif
#if ! defined (SIZE_TIMESTAMP)
#define SIZE_TIMESTAMP             26 /* yyyy-mm-dd:hh:mm:ss.msssss */
typedef struct timestamp_t {
    char   val[SIZE_TIMESTAMP +1];
} timestamp_t, *timestamp_p;
#endif
#if ! defined (SIZE_DATE)
#define SIZE_DATE                  10 /* yyyy-mm-dd */  
typedef struct date_t {
    char   val[SIZE_DATE +1];
} date_t, *date_p;
#endif
#if ! defined (YNFLAG)
#define YNFLAG
typedef char ynflag_t;
typedef enum ynflag_v {
    YN_YES             = 'Y',
    YN_NO              = 'N',
    YN_IGNORE	       = 'I'
} ynflag_v;
#endif
#if ! defined (SIZE_SOURCECODE)
#define SIZE_SOURCECODE            10
typedef struct sourcecode_t {
    char   val[SIZE_SOURCECODE +1];
} sourcecode_t, *sourcecode_p;
#endif

typedef timestamp_t FIOTimestamp_t;
typedef date_t      FIODate_t;
typedef struct FIOHandle_t *FIOHandle_p;

#define FIO_FLIPPER '%' 
#define FIO_LOWEST  (' ' - 1)
#define FIO_HIGHEST ('z' + 1)

#define FIO_EMPTY_TIMESTAMP           "1753-01-01 00:00:00.000"
#define FIO_EMPTY_DATETIME            "1753-01-01 00:00"
#define FIO_EMPTY_DATE                "1753-01-01"
#define FIO_EMPTY_FULLTIME            "00:00:00.000"
#define FIO_EMPTY_SHORTTIME           "00:00"

#define FIO_LAST_TIMESTAMP            "9999-12-31 23:59:59.999"
#define FIO_LAST_DATETIME             "9999-12-31 23:59"
#define FIO_LAST_DATE                 "9999-12-31"

#define FIO_EMPTY_STATION_ID          "          "
#define FIO_WILD_STATION_ID           "*         "
#define FIO_EMPTY_LIBID               EMPTY_STATION_ID

#define FIO_DEFAULT_USERGROUP         "000"
#define FIO_DEFAULT_DBSID             "000"


ERROR_TYPE(FIOErrno_t);

ERROR_DECLARE(FIOE_NOERROR);
ERROR_DECLARE(FIOE_LIBINITFAI);		/* Init of library failed */
ERROR_DECLARE(FIOE_NOMEM);		/* Memory allocation failure */
ERROR_DECLARE(FIOE_IO);			/* General IO error */
ERROR_DECLARE(FIOE_IOFATAL);		/* Fatal IO error */
ERROR_DECLARE(FIOE_EOF);		/* EOF reached */
ERROR_DECLARE(FIOE_OVERFLOW);		/* Key or buffer overflow */
ERROR_DECLARE(FIOE_SECURITY);		/* Security violation */
ERROR_DECLARE(FIOE_DUPLICATEKEY);
ERROR_DECLARE(FIOE_ILLEGALKEY);		/* Illegal key in file */
ERROR_DECLARE(FIOE_KEYNOTFOUND);	/* Key not found */
ERROR_DECLARE(FIOE_NOSERVER);
ERROR_DECLARE(FIOE_OPEN);
ERROR_DECLARE(FIOE_CLOSE);
ERROR_DECLARE(FIOE_MAXOPENS);		/* Maximum opens exceeded */
ERROR_DECLARE(FIOE_UNKOPTION);		/* Unknown option */
ERROR_DECLARE(FIOE_INVOPTIONVALUE);	/* Invalid option value */
ERROR_DECLARE(FIOE_INCONSFILE);		/* Inconsistancy in file */
ERROR_DECLARE(FIOE_NOSUPPORT);		/* Operation or option not supported */
ERROR_DECLARE(FIOE_INTERNAL);		/* Internal error */
ERROR_DECLARE(FIOE_TIMESTAMP);		/* Timestamp changed */
ERROR_DECLARE(FIOE_DEADLOCK);
ERROR_DECLARE(FIOE_DUPLICATEROW);
ERROR_DECLARE(FIOE_PARAM);		// Invalid function argument

extern FIOErrno_t FIOerrno;

FIOHandle_p FIOOpen(const char *FIOname, const char *FIOopts);

bool FIOClose(FIOHandle_p FIOhandle);

bool FIOStartTransaction(FIOHandle_p FIOhandle);
bool FIOCommitTransaction(FIOHandle_p FIOhandle);
bool FIORollbackTransaction(FIOHandle_p FIOhandle);
bool FIOSetSavepoint(FIOHandle_p FIOhandle, const char *savepointname);
bool FIORollbackSavepoint(FIOHandle_p FIOhandle, const char *savepointname);
bool FIOCancelBatch(FIOHandle_p FIOHandle);

bool FIOTBRead(FIOHandle_p FIOhandle, const char *FIOopts,
	const char *FIOkey, size_t FIOkeylength,
	char *FIOhit, size_t FIOhitsize, size_t *FIOhitlength,
	char *FIObuf, size_t FIObufsize, size_t *FIObuflength);

bool FIOTBNext(FIOHandle_p FIOhandle, const char *FIOopts,
	char *FIOhit, size_t FIOhitsize, size_t *FIOhitlength,
	char *FIObuf, size_t FIObufsize, size_t *FIObuflength);

bool FIOTBStore(FIOHandle_p FIOhandle, const char *FIOopts,
	const char *FIOkey, size_t FIOkeylength,
	const char *FIObuf, size_t FIObuflength);

bool FIOTBUpdate(FIOHandle_p FIOhandle, const char *FIOopts,
	const char *FIOkey, size_t FIOkeylength,
	const char *FIObuf, size_t FIObuflength);

bool FIOTBDelete(FIOHandle_p FIOhandle, const char *FIOopts,
	const char *FIOkey, size_t FIOkeylength,
	const char *FIObuf, size_t FIObuflength);

/* Install a signal handler, return the old signal handler.  Use NULL to
   de-install.  This routine is in FIO and has a too restrictive interface,
   because the CT-library used to implement FIO on some platforms insists that
   signal handlers should be installed through its routines. */
typedef void (*FIOSignalHandler_tp)(void);
FIOSignalHandler_tp
FIOSignal(int signo, FIOSignalHandler_tp handler);

/*------------------------------------------------------------------------
 * 
 *  Purpose     : temporary declaration of FIOCBS_, FIODBS_ 
 *                and FIOILL_ functions (waiting for a FIO-reform)
 *  Method      :
 *  Author(s)   : Teresita Frizzarin
 *  Creation    : 22-10-1998
 *
 *  History:
 *  22-10-1998  : teresita   : creation
 *
 *  Notes       : copied from Tandem fiocbs.h, fiodbs.h and fioill.h 
 *
 *  Returns     :
 *
 *------------------------------------------------------------------------
 */

/* Stats functions                 */

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(FIO_H_INCLUDED) */
