/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: fmt.h
 *  Description	: The interface for the FMT-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 07-02-1994
 *
 *  History:
 *  07-02-1994	: eck      : creation
 *  09-10-2013	: martinz  : Copied documentation from FMTLIB.DOC.
 *
 *========================================================================
 */

#if ! defined(FMT_H_INCLUDED)
#define FMT_H_INCLUDED

#pragma ident "$Id: fmt.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <stddef.h>

#include <pica.h>
#include <ccv.h>

ERROR_TYPE(FMTErrno_t);

ERROR_DECLARE(FMTE_NOERROR);	/* ok */
ERROR_DECLARE(FMTE_NOMEM); 	/* not enough memory */
ERROR_DECLARE(FMTE_SYNTAX);	/* error in syntax specification */
ERROR_DECLARE(FMTE_TITLE);	/* error in the format of the title */

extern FMTErrno_t FMTerrno;

typedef struct FMTSyntax_t *FMTSyntax_p;

FMTSyntax_p FMTCompile(const char *text, const char *options);

void FMTDestroy(FMTSyntax_p syntax);

size_t FMT(const char *title, FMTSyntax_p syntax, int linelen,
	    char *buffer, size_t bufsize);

size_t FMTSpecial(const char *title, FMTSyntax_p syntax, int linelen,
		    char *buffer, size_t bufsize,
		    const char *(*termfmt)(
			const char	*tagname,
			const char	*tagcontents,
			size_t		taglength,
			size_t		*usedlen)
		    );

#endif /* ! defined(FMT_H_INCLUDED) */
