/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: html.h
 *  Description	: Header for the HTML-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 25-04-1996
 *
 *  History:
 *  25-04-1996	: eck      : creation
 *  07-02-1997	: eck      : Increased HTML_NAMELEN_MAX	to 128
 *  15-08-1997	: eck      : Added argument types in argument lists.
 *			   : FIeld al_name should not be used anymore.
 *  15-05-1998	: BdB	   : Added HTMLNormalToSave()
 *  19-07-2001  : paul     : Added HTML_NOFUNC
 *  31-05-2005  : evd      : Added to HTMLArgList_t lvl_preprocess for 
 *                           recursive call of HTMLPreProcess
 *  23-10-2006	: heesakkm : New interpreter interface: HTMLPreProcess_New().
 *  13-01-2009	: martinz  : Added HTMLXmlSave().
 *  26-10-2009	: paul     : Added HTMLXmlSaveUndo()
 *  13-10-2014  : evd	   : cbs-5848: Added HTMLXmlSaveUndoPRSXml()
 *  13-10-2014  : evd	   : cbs-5848: Added HTMLXmlSavePRSXml()
 *
 *========================================================================
 */

#if ! defined(HTML_H_INCLUDED)
#define HTML_H_INCLUDED

#pragma ident "$Id: html.h 319408 2014-10-17 10:52:15Z heesakkm $"

#include <pica.h>
#include <cs.h>
#include <pa.h>

ERROR_TYPE(HTMLErrno_t);

ERROR_DECLARE(HTMLE_NOERROR);
ERROR_DECLARE(HTMLE_NOMEM);
ERROR_DECLARE(HTMLE_SYNTAX);

extern HTMLErrno_t HTMLerrno;

/* The field-terminator and record-terminator */
#define HTML_FT		'\x01'
#define HTML_RT		'\x02'

#define HTML_NAMELEN_MAX	128

voidp HTMLAlloc(size_t nr_bytes);

char *HTMLStringAlloc(const char *buf, int len);

/* The callback expects a linked-list with the function and its parameters.
 * As an extra service, the way the argument was formed can be inspected.
 */
typedef enum {
    HTMLA_NAME,		/* <%call(name)> */
    HTMLA_NUMBER,	/* <%call(1234)> */
    HTMLA_STRING,	/* <%call("string")> */
    HTMLA_EXPNUM,	/* <%call(%2)> */
    HTMLA_EXPFUNC	/* <%call(%func(...)) */
} HTMLArgtyp_t;

typedef struct HTMLArgList_t *HTMLArgList_p;

struct HTMLArgList_t {
    /* The al_name is for backward compatibility. */
    char		al_name[HTML_NAMELEN_MAX + 1];
    HTMLArgtyp_t	al_type;
    const char		*al_value;
    HTMLArgList_p	al_next;
    int                 lvl_preprocess;
};

#define HTML_NOFUNC	((const char *(*)(HTMLArgList_p)) 0)
int HTMLPreProcess(const char *in_buffer, char *out_buffer, size_t out_size,
		    const char *(*interpreter)(HTMLArgList_p));

#define HTML_NOFUNC_NEW	((const char *(*)(const char *, PAHandle_p, cvoidp)) 0)
CSBuf_p
HTMLPreProcess_New(
	const char	*in_buffer,
	CSBuf_p		out_buffer,
	const char	*(*interpreter)(const char *, PAHandle_p, cvoidp),
	cvoidp		userdata);

int
HTMLNormalToSave(const char *instr, int inlen, int *inused,
		 char *outstr, int outlen);

const char *
HTMLXmlSave(
	const char *contents);

const char *
HTMLXmlSaveN(
	const char	*contents,
	long		length);

const char *
HTMLXmlSaveUndo(
	const char	*contents,
	long		length);

PAHandle_p
HTMLArgs_to_pa(
	HTMLArgList_p	args);

const char *
HTMLXmlSavePRSXml(
	const char *contents);

const char *
HTMLXmlSaveUndoPRSXml(
	const char	*contents,
	long		length);

#endif /* ! defined(HTML_H_INCLUDED) */
