/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File        : idxtypes.h
 *  Description : The interface between the IDX- and FIOIDX-library
 *  Author(s)   : Paul Smit
 *  Creation    : 22-10-1996
 *
 *  History:
 *  22-10-1996  : paul     : creation
 *  16-07-1997  : HSi      : IDX_RECORD_BODY_SIZE depends on platform.
 *  26-09-1997  : HSi      : Added special characters.
 *  18-02-1998  : paul     : IDXKwordNr_t extended from uint8 to uint16
 *  23-02-1998  : paul     : IDXMatAdi_t,IDXIdxType_t,IDXKeyType_t,IDXIktNum_t,
 *                           IDXRecType_t,IDXFamType_t moved from picatypes.h;
 *                           IDX_MATADI_ABSTRACT added
 *                           autipn removed from IDXPhraseRecord_t
 *                           IDX*Record_t contain whole pointer in stead of ipn
 *                           IDX*Key_t introduced
 *                           IDXIdxType_t modified from int16 to int8
 *  17-08-1999	: BdB	   : Add wordlist types and special characters
 *  25-01-2000	: BdB	: Add origipn to IDXIndex_t
 *  17-02-2000	: BdB	: Add IDX*Handle_p for reading indexes
 *  10-04-2002	: paul	: Added IDX_IKTNUM_SELECTION
 *  16-05-2002	: clemens  : Added IDX_IKTNUM_PPN
 *  31-03-2003  : HvdW     : tandem ifdefs removed
 *  15-07-2003  : paul     : Moved pcctypes.h MAT_* defines to IDX_MATADI_*
 *  14-10-2003	: jaap     : Add IDX_IKTNUM_FULLTEXT
 *  23-02-2004  : paul     : Added IDX_IKTNUM_MATADI and IDX_IKTNUM_SUPPLIER
 *                           and IDX_IKTNUM_PUBL_YEAR
 *  24-06-2004  : paul     : Added strlen in IDXIndexSet_t
 *  10-03-2005  : paul     : Added IDX_FAMTYPE_ANY
 *  10-10-2005  : paul     : Added IDX_IKTNUM_CLUSTERNUM
 *  30-05-2005  : paul     : Added IDX_IKTNUM_OLC_SORTSTRING
 *  07-06-2006  : paul     : Added IDXOLC_SORTSTR_TAG_*
 *  10-07-2006	: paul     : don't mention phrase-entry member usortkey anymore
 *                           since it is about to be removed
 *  31-07-2006	: paul     : extended IDX_FAMSORT_LEN_MAX from 32 to 40
 *  02-08-2006	: paul     : undone previous change
 *  18-10-2006	: paul     : redone previous change
 *  02-11-2006	: paul     : IDX_CHWILD_ONE made different from HTML_FT
 *  26-02-2007	: paul     : IDX_CHWILD_ONE, IDX_CHWILD_ANY and IDX_CHWILD_MBE
 *                           changed as a workaround of an ICU library bug
 *  23-05-2007	: paul     : Added iktnums used in search server
 *  29-02-2008	: daalmanf : Made IDP variants of all IPN-dependent types
 *  23-12-2008	: paul     : Added "entry variant selection value" defines
 *  17-06-2009	: paul     : Added IDX_IKTNUM_SOCIAL_DAILY_INSERTS
 *  22-02-2010	: hetty    : Split up IDX_MATADI_PERIODICAL into 
 *			     IDX_MATADI_PERIODICAL_PRINTED and 
 *			     IDX_MATADI_PERIODICAL_ELECTRONIC
 *  15-10-2010	: martinz  : Added IDX_IKTNUM_UTS.
 *  26-03-2013	: martinz  : Added IDXIndexSet_t comments.
 *  04-05-2015  : evd      : cbs-5015: Added IDX_IKTNUM_NORMALISE
 *  05-03-2019	: smitp    : CBS-8675 Increased max index type from 127 to 250
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(IDXTYPES_H_INCLUDED)
#define IDXTYPES_H_INCLUDED

#pragma ident "$Id: idxtypes.h 365572 2019-03-05 07:29:21Z smitp $"

#include <stddef.h>
#include <picatypes.h>
#include <pica.h>
#include <uc.h>
#include <pp.h>


/* Special characters.  Only ONE and ANY can currently be used in
   IDX{Phrase,Family}Read, the fancier ones are only for
   IDXWordList{Create,Match}(). */
/* Any single character (excl \0) */
/* Don't use the same value as is used for HTML_FT */
/* 
 * There is a bug in the ICU library that causes deletion of collation elements
 * of ONE, ANY and MBE, if preceded by a cyrillic character. The only control
 * characters that are not deleted are '\011' through '\015'. Using '\014' may
 * be hazardous because it is already used as the IDXGEN_PERSON_UTF8_SEP
 */
#define IDX_CHWILD_ONE '\011'
/* Any string of 0 or more characters (excluding \0) */
#define IDX_CHWILD_ANY '\012'
/* None or 1 character. */
#define IDX_CHWILD_MBE '\013'
/* Start set of characters */
#define IDX_CHWILD_SET '\004'
/* Start set of characters not to use */
#define IDX_CHWILD_NOSET '\005'
/* Stop set of characters to (not) use */
#define IDX_CHWILD_CLOSE '\006'
/* String containing all special characters.  Useful for various
   routines to match with strchr() and such. */
#define IDX_CHWILD_STR "\011\012\013\004\005\006"
/* Last special character */
#define IDX_CHSPEC_MAX '\017'


typedef char				IDXMatAdi_t;
#define IDX_MATADI_ANY			((IDXMatAdi_t) '*')
#define IDX_MATADI_NONE			((IDXMatAdi_t) ' ')
#define IDX_MATADI_MONOGRAPH		((IDXMatAdi_t) 'B')	/* was M */
#define IDX_MATADI_PERIODICAL_PRINTED	((IDXMatAdi_t) 'T')	/* was P */
#define IDX_MATADI_PERIODICAL_ELECTRONIC    ((IDXMatAdi_t) 'P')
#define IDX_MATADI_ELECTRONIC_RESOURCE 	((IDXMatAdi_t) 'O')	/* ??? was E */
#define IDX_MATADI_ONLINE_CONTENTS      ((IDXMatAdi_t) 'U')	/* ??? was O */
#define IDX_MATADI_ABSTRACT             ((IDXMatAdi_t) 'D')	/* ??? was A */
#define IDX_MATADI_ARTICLE              ((IDXMatAdi_t) 'A')	/* ??? was S */
#define IDX_MATADI_DISSERTATION         ((IDXMatAdi_t) 'Y')
#define IDX_MATADI_OK(c)		(isalnum(c))
#define IDX_MATADI_MASK_OK(c)		(   IDX_MATADI_OK(c)                  \
					 || (c) == IDX_MATADI_ANY)
#define IDX_MATADI_MATCH(c,mask)	(   (c) == (mask)                     \
					 || (mask) == IDX_MATADI_ANY)

typedef byte				IDXTIHAdi_t;
#define IDX_TIHADI_NONE			((IDXTIHAdi_t)0)

typedef uint8                       IDXIdxType_t;
#define IDX_IDXTYPE_ANY             ((IDXIdxType_t)255)
#define IDX_IDXTYPE_NONE            ((IDXIdxType_t)254)
#define IDX_IDXTYPE_MIN             ((IDXIdxType_t)0)
#define IDX_IDXTYPE_MAX             ((IDXIdxType_t)250)
#define IDX_IDXTYPE_MATCH(c,mask)   (   (c) == (mask)                         \
                                     || (mask) == IDX_IDXTYPE_ANY)

typedef char                        IDXKeyType_t;
#define IDX_KEYTYPE_OK(c)           (isalnum(c) || (c) == '@')
#define IDX_KEYTYPE_NONE            ((IDXKeyType_t) ' ')
#define IDX_KEYTYPE_ANY             ((IDXKeyType_t) '*')
#define IDX_KEYTYPE_MATCH(c,mask)   (   (c) == (mask)                         \
                                     || (mask) == IDX_KEYTYPE_ANY)
#define IDX_KEYTYPE_MASK_OK(mask)   (   IDX_KEYTYPE_OK(mask)                  \
                                     || (mask) == IDX_KEYTYPE_ANY)

typedef int16                       IDXIktNum_t;
#define IDX_IKTNUM_ANY              ((IDXIktNum_t)-1)
#define IDX_IKTNUM_NONE             ((IDXIktNum_t)0)
#define IDX_IKTNUM_MIN              ((IDXIktNum_t)1)
#define IDX_IKTNUM_MAX              ((IDXIktNum_t)SHRT_MAX)
#define IDX_IKTNUM_TITLE_WORDS	    ((IDXIktNum_t)4)
#define IDX_IKTNUM_ISBN		    ((IDXIktNum_t)7)
#define IDX_IKTNUM_ISSN		    ((IDXIktNum_t)8)
#define IDX_IKTNUM_PPN		    ((IDXIktNum_t)12)
#define IDX_IKTNUM_NUM		    ((IDXIktNum_t)1007)
#define IDX_IKTNUM_FULLTEXT         ((IDXIktNum_t)1010)
#define IDX_IKTNUM_NORMALISE	    ((IDXIktNum_t)1217)
#define IDX_IKTNUM_CLUSTERNUM       ((IDXIktNum_t)6128)
#define IDX_IKTNUM_OLC_SORTSTRING   ((IDXIktNum_t)6203)
#define IDX_IKTNUM_UTS   	    ((IDXIktNum_t)6207)
#define IDX_IKTNUM_SWETS	    ((IDXIktNum_t)8001)
#define IDX_IKTNUM_SIMILARITY	    ((IDXIktNum_t)8065)
#define IDX_IKTNUM_SOCIAL_DAILY_INSERTS	    ((IDXIktNum_t)8703)
#define IDX_IKTNUM_SOCIAL_QUERY_SRU	    ((IDXIktNum_t)8705)
#define IDX_IKTNUM_SELECTION        ((IDXIktNum_t)8880)
#define IDX_IKTNUM_PUBL_YEAR        ((IDXIktNum_t)9000)
#define IDX_IKTNUM_MATADI           ((IDXIktNum_t)9001)
#define IDX_IKTNUM_TIHADI           ((IDXIktNum_t)9041)
#define IDX_IKTNUM_TMEADI           ((IDXIktNum_t)9042)
#define IDX_IKTNUM_TDGADI           ((IDXIktNum_t)9043)
#define IDX_IKTNUM_TCT		    ((IDXIktNum_t)6212)
#define IDX_IKTNUM_TMM		    ((IDXIktNum_t)6213)
#define IDX_IKTNUM_TDR		    ((IDXIktNum_t)6214)
#define IDX_IKTNUM_BCLADI           ((IDXIktNum_t)9003)
#define IDX_IKTNUM_LANGUAGE         ((IDXIktNum_t)9004)
#define IDX_IKTNUM_COUNTRY          ((IDXIktNum_t)9005)
#define IDX_IKTNUM_DAY              ((IDXIktNum_t)9006)
#define IDX_IKTNUM_SUPPLIER         ((IDXIktNum_t)9008)
#define IDX_IKTNUM_SWD              ((IDXIktNum_t)9009)
#define IDX_IKTNUM_PUBLISHER_ADI    ((IDXIktNum_t)9016)
#define IDX_IKTNUM_RECTYPE          ((IDXIktNum_t)9017)
#define IDX_IKTNUM_ARTICLE_TYPE     ((IDXIktNum_t)9019)
#define IDX_IKTNUM_HOLDING_LIBRARY  ((IDXIktNum_t)9023)
#define IDX_IKTNUM_TITLESET         ((IDXIktNum_t)9024)
#define IDX_IKTNUM_INDEXSET         ((IDXIktNum_t)9025)
#define IDX_IKTNUM_SUBDATABASE      ((IDXIktNum_t)9026)
#define IDX_IKTNUM_ILN              ((IDXIktNum_t)9027)
#define IDX_IKTNUM_MEDIUM           ((IDXIktNum_t)9031)
#define IDX_IKTNUM_ILNCOUNT	    ((IDXIktNum_t)9039)

typedef char                        IDXRecType_t;
#define IDX_RECTYPE_NONE            ((IDXRecType_t) ' ')
#define IDX_RECTYPE_ANY             ((IDXRecType_t) '*')
#define IDX_RECTYPE_MATCH(c,mask) \
    ((c) == (mask) || (mask) == IDX_RECTYPE_ANY)
#define IDX_RECTYPE_OK(c) \
    (   (c) == IDX_RECTYPE_TITLE \
     || (c) == IDX_RECTYPE_AUTHORITY)
#define IDX_RECTYPE_MASK_OK(mask) \
    (   IDX_RECTYPE_OK(mask) \
     || (mask) == IDX_RECTYPE_ANY)
typedef enum IDXRecType_v {
    IDX_RECTYPE_TITLE     = '1',
    IDX_RECTYPE_AUTHORITY = '2',
    IDX_RECTYPE_SHORTCUT  = '3'     /* Only used within index routines */
} IDXRecType_v;

/* entry variant selection values */
#define IDX_VARIANT_ANY		(-1)
#define IDX_VARIANT_ORIGINAL	0
#define IDX_VARIANT_DERIVED	1

typedef char IDXFamType_t;
typedef enum IDXFamType_v {
    IDX_FAMTYPE_NONE = ' ',
    IDX_FAMTYPE_ANY = '*',
    IDX_FAMTYPE_NARROWER = '1',   /* NOTE: the order of these values  */
    IDX_FAMTYPE_RELATED,          /* must not be changed, since it is */
    IDX_FAMTYPE_REVERSE,          /* used in the search engine        */
    IDX_FAMTYPE_BROADER           /* and assumed to be fixed          */
} IDXFamType_v;


typedef struct IDXAdi_t {
    byte                       *buf;
    size_t                      siz;
    size_t                      len;
} IDXAdi_t, *IDXAdi_p;

typedef uint16                  IDXKwordNr_t;
#define IDX_KWORDNR_MIN         ((IDXKwordNr_t)0)
#define IDX_KWORDNR_MAX         ((IDXKwordNr_t)65535)
#define IDX_PHRASE_LEN_MAX      200
typedef struct IDXPhraseEntry_t {
    PICFileSet_t                fileset;
    PICIln_t                    iln;
    IDXIdxType_t                idxtype;
    const char			*PHrase;  /* may point in IDXIndexSet_t.str
    					   * Don't access directly, but through
					   * IDXPEPutPhrase() and
					   * IDXPEGetPhrase(). */
    uint8 			*usortkey;/* don't use anymore: will be eliminated */
    IDXKeyType_t                keytype;
    IDXRecType_t                rectype;
    int				variant;	/* from multi normalisation */
    UCcolStrength_t             colmode;
    const char                 *idxlocale;
} IDXPhraseEntry_t, *IDXPhraseEntry_p;
typedef struct IDXPhrasePointer_t {
    PICIdp_t                    idp;
    PICLevel_t                  lev;
    PICOccurrence_t             occ;
    IDXKwordNr_t                kwordnr;
    IDXKwordNr_t                kwordmax;
} IDXPhrasePointer_t, *IDXPhrasePointer_p;
typedef struct IDXPhraseIndex_t {
    IDXPhraseEntry_t            entry;
    IDXPhrasePointer_t          pointer;
} IDXPhraseIndex_t, *IDXPhraseIndex_p;

typedef struct IDXRelationEntry_t {
    PICFileSet_t                fileset;
    PICIdp_t                    idp;
    IDXIdxType_t                idxtype;
    IDXKeyType_t                keytype;
    PICIln_t                    iln;
} IDXRelationEntry_t, *IDXRelationEntry_p;
typedef struct IDXRelationPointer_t {
    PICIdp_t                    idp;
    PICLevel_t                  lev;
    PICOccurrence_t             occ;
} IDXRelationPointer_t, *IDXRelationPointer_p;
typedef struct IDXRelationIndex_t {
    IDXRelationEntry_t          entry;
    IDXRelationPointer_t        pointer;
} IDXRelationIndex_t, *IDXRelationIndex_p;

#define IDX_FAMSORT_LEN_MAX     40
typedef struct IDXFamilyEntry_t {
    PICFileSet_t                fileset;
    PICIdp_t                    idp;
    IDXRecType_t                rectype;
    IDXFamType_t                famtype;
    const char                 *SOrt; 	/* may point in IDXIndexSet_t.str
    					 * Don't access directly, but through
					 * IDXFEPutSort() and
					 * IDXFEGetSort(). */
    PICIln_t                    iln;
} IDXFamilyEntry_t, *IDXFamilyEntry_p;
typedef struct IDXFamilyPointer_t {
    PICIdp_t                    idp;
    PICLevel_t                  lev;
    PICOccurrence_t             occ;
} IDXFamilyPointer_t, *IDXFamilyPointer_p;
typedef struct IDXFamilyIndex_t {
    IDXFamilyEntry_t            entry;
    IDXFamilyPointer_t          pointer;
} IDXFamilyIndex_t, *IDXFamilyIndex_p;

typedef enum IDXIndexType_t {
    IDXTYPE_NONE,
    IDXTYPE_PHRASE,
    IDXTYPE_RELATION,
    IDXTYPE_FAMILY
} IDXIndexType_t;
typedef struct IDXIndex_t {
    IDXIndexType_t              typ;
    union {
        IDXPhraseIndex_t            phr;
        IDXRelationIndex_t          rel;
        IDXFamilyIndex_t            fam;
    }                           val;
    PICIdp_t origidp;
} IDXIndex_t, *IDXIndex_p;
typedef struct IDXIndexSet_t {
    IDXIndex_p	idx;	/* array of IDXIndex_t elements */
    size_t	siz;	/* number of allocated idx elements */
    size_t	len;	/* number of used idx elements */
    char *	str;	/* list of '\0'-terminated strings for phr.entry.phrase
			   | fam.entry.sort */
    size_t	strlen;	/* number of used bytes in str */
    size_t	strsiz;	/* number of allocated bytes in str */
} IDXIndexSet_t, *IDXIndexSet_p;

#define IDX_RECORD_BODY_SIZE    1529

typedef struct IDXBody_t {
    byte                        buf[IDX_RECORD_BODY_SIZE];
    size_t                      len;
} IDXBody_t      ;
/*
 * Index records consist of a key and a body. Note that the definition of the
 * keys are technically identical to the definition of single indexes. But
 * remember that the pointer part of the keys is a link to the next record;
 * it must NOT be confused with a pointer to a real data record.
 */
typedef struct IDXPhraseKey_t {
    IDXPhraseEntry_t            entry;
    IDXPhrasePointer_t          link;
} IDXPhraseKey_t, *IDXPhraseKey_p;
typedef struct IDXPhraseRecord_t {
    IDXPhraseKey_t              key;
    IDXBody_t                   body;
} IDXPhraseRecord_t, *IDXPhraseRecord_p;
typedef struct IDXRelationKey_t {
    IDXRelationEntry_t          entry;
    IDXRelationPointer_t        link;
} IDXRelationKey_t, *IDXRelationKey_p;
typedef struct IDXRelationRecord_t {
    IDXRelationKey_t            key;
    IDXBody_t                   body;
} IDXRelationRecord_t, *IDXRelationRecord_p;
typedef struct IDXFamilyKey_t {
    IDXFamilyEntry_t            entry;
    IDXFamilyPointer_t          link;
} IDXFamilyKey_t, *IDXFamilyKey_p;
typedef struct IDXFamilyRecord_t {
    IDXFamilyKey_t              key;
    IDXBody_t                   body;
} IDXFamilyRecord_t, *IDXFamilyRecord_p;

/* Types to handle word lists. */
#define IDXWORDLISTFILE_MAGIC 318839145UL
typedef struct IDXWordListHandle_t *IDXWordListHandle_p;
typedef struct IDXWordListMatch_t {
    const char *phrase;
    int errors;
} IDXWordListMatch_t, *IDXWordListMatch_p;

/* Handles to read indexes for searching */
typedef struct IDXPhraseHandle_t *IDXPhraseHandle_p;
typedef struct IDXRelationHandle_t *IDXRelationHandle_p;
typedef struct IDXFamilyHandle_t *IDXFamilyHandle_p;

/* Handles to read indexes sequentially */
typedef struct IDXIORead_t IDXIORead_t, *IDXIORead_p;
typedef struct IDXIOWrite_t IDXIOWrite_t, *IDXIOWrite_p; 

/* Handles for (de)compression of indexes */
typedef struct IDXHEncode_t IDXHEncode_t, *IDXHEncode_p;
typedef struct IDXHDecode_t IDXHDecode_t, *IDXHDecode_p;

/* 031A is used for the Online Contents sortstring phrase index*/
#define IDXOLC_SORTSTR_TAG_TYP	31
#define IDXOLC_SORTSTR_TAG_LEV	0
#define IDXOLC_SORTSTR_TAG_IND	'A'
#define IDXOLC_SORTSTR_TAG_OCC	PP_ANYOCC


#endif /* defined(IDXTYPES_H_INCLUDED) */
