/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: ln.h
 *  Description	: The interface of the LN-library
 *  Author(s)	: HSi
 *  Creation	: 05-12-1994
 *
 *  History:
 *  05-12-1994	: HSi   : creation
 *  16-12-1994	: eck      : Added #include's of some files
 *  07-03-1997	: eck      : LNILLSelect() and LNILLPresent().
 *  17-06-1999	: HV       : LNILLPassword(), LNILLDeposit(), LNTOOAddress and 
 *	     		     LNUPDAddress added.
 *  29-09-1999	: HV       : LNILLSelect() User param removed.
 *
 *  Notes	:
 *
 *========================================================================
 */

#if ! defined(LN_H_INCLUDED)
#define LN_H_INCLUDED

#pragma ident "$Id: ln.h 261102 2010-11-30 15:22:09Z sieburgw $"

#include <pica.h>
#include <cb.h>
#include <db.h>

ERROR_TYPE(LNErrno_t);

ERROR_DECLARE(LNE_NOERROR);		/* Ok				*/
ERROR_DECLARE(LNE_NOMEM);		/* Not enough memory		*/
ERROR_DECLARE(LNE_ERROR);		/* Not ok			*/
ERROR_DECLARE(LNE_BUFFER);		/* Buffer overflow		*/
ERROR_DECLARE(LNE_NOSUPPORT);		/* Function not supported	*/
ERROR_DECLARE(LNE_AUTHORITY);		/* Invalid authority		*/
ERROR_DECLARE(LNE_IDPASSWRONG);		/* Authorization failure	*/
ERROR_DECLARE(LNE_NOLIBS);		/* No supplying libraries	*/
ERROR_DECLARE(LNE_OWNLIB);		/* Title is in own library	*/
ERROR_DECLARE(LNE_TTLWRONG);		/* Invalid title		*/
ERROR_DECLARE(LNE_FLDWRONG);		/* Invalid (data in) field	*/
ERROR_DECLARE(LNE_MOREINFO);		/* more information needed	*/
ERROR_DECLARE(LNE_REJECT);		/* transaction rejected		*/
ERROR_DECLARE(LNE_NOINFO);		/* No info			*/
ERROR_DECLARE(LNE_LIBDEPOSIT);		/* library deposit exhausted	*/
ERROR_DECLARE(LNE_USRDEPOSIT);		/* user deposit exhausted	*/
ERROR_DECLARE(LNE_NOHITS);		/* No hits in LNILLSelect()	*/
ERROR_DECLARE(LNE_SETFULL);		/* Too many hits in LNILLSelect()*/
ERROR_DECLARE(LNE_NUMBER);		/* Error in number argument	*/

extern LNErrno_t LNerrno;

bool
LNIdentifyEnduser (			/* Identify enduser		*/
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to store attributes*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLForm (				/* Retrieve an ILL-form		*/
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLSend (				/* Send ILL-request		*/
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

int
LNILLSelect(
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*Options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLPresent(
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*Options,	/* like "format= ..."		*/
    int			FirstElt,	/* First element to show	*/
    int			Count,		/* Number of elements to show	*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);


bool
LNIdentifyBorrower(			/* Identify borrrower		*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*Ident,		/* Borrower identification	*/
    const char		*Password,	/* Borrower password		*/
    CBBuf_t		*CBbuf,		/* CB to store attributes	*/
    size_t		CBsize		/* size of CBbuf		*/
);

bool
LNListVolumes(				/* Retrieve list of volumes	*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*Epn,		/* Copy production number	*/
    long		Toffs,		/* The element offset		*/
    long		Eltcount,	/* Number of element found	*/
    long		*ELMfound,	/* Total elements found		*/
    CBBuf_t		*CBbuf,		/* The result buffer		*/
    size_t		CBsize		/* size of the result buffer	*/
);

bool
LNLoanInfo (				/* Get information about loans	*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*BorIdent,	/* Borrower identification	*/
    const char		*Password,	/* Borrower password		*/
    const char		*Options,	/* type=...;			*/
    long		Toffs,		/* First element		*/
    long		Eltcount,	/* Number of elemens wanted	*/
    long		*ELMfound,	/* Total elements found		*/
    CBBuf_t		*CBbuf,		/* The result buffer		*/
    size_t		CBsize		/* Size of result buffer	*/
);

bool
LNLoan(					/* Loan transaction		*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    CBBuf_t		*CBbuf,		/* CB with loan data		*/
    size_t		CBsize		/* size of CBbuf		*/
);

bool
LNRenewLoan(				/* Renew loan			*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*BorIdent,	/* Borrower identification	*/
    const char		*Password,	/* Borrower password		*/
    const char		*VolIdent,	/* Volume identification	*/
    CBBuf_t		*CBbuf,		/* CB to store attributes	*/
    size_t		CBsize		/* size of CBbuf		*/
);

bool
LNCancelReservation(			/* Cancel reservation		*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*BorIdent,	/* Borrower identification	*/
    const char		*Password,	/* Borrower password		*/
    const char		*ResIdent,	/* Reservation identification	*/
    CBBuf_t		*CBbuf,		/* CB to store attributes	*/
    size_t		CBsize		/* size of CBbuf		*/
);

bool
LNUpdPasswordBorrower(			/* Update password of borrower	*/
    DBHandle_p		DBhandle,	/* The DB-handle to use		*/
    const char		*BorIdent,	/* Borrower identification	*/
    const char		*OldPass,	/* Old password			*/
    const char		*NewPass	/* New password			*/
);

bool
LNILLPassword(				/* Update password of enduser   */
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLDeposit(				/* Display deposit of enduser   */
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLTOOAddress(			/* Display address of enduser   */
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

bool
LNILLUPDAddress(			/* Update address of enduser    */
    DBHandle_p		dbHandle,	/* The DB-handle to use		*/
    const char		*options,	/* Options to use		*/
    CBBuf_t		*cbBuf,		/* CB-buffer to attributes	*/
    size_t		cbSize		/* Size of cbBuf		*/
);

#endif /* ! defined(LN_H_INCLUDED) */
