/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: p3.h
 *  Description	: The Pica3-interface
 *  Author(s)	: Hans van Eck
 *  Creation	: 05-09-1994
 *
 *  History:
 *  05-09-1994	: eck      : creation
 *  06-12-1994	: HSi	   : Added P3IdentifyEnduser.
 *  08-12-1994	: HSi	   : Added P3ILLForm and P3ILLSend
 *  09-12-1994	: HSi	   : Added P3E_NOHITS and P3SDI-functions.
 *  08-06-1995	: eck      : Added P3Count().
 *  03-08-1995	: eck      : Added P3SetSort().
 *  05-10-1995	: eck      : Added P3Option().
 *  30-08-1996	: eck      : Added P3ScanDbsUpdates() and P3EditTitle().
 *  17-12-1996  : JV       : Added P3RecRelate.
 *  19-12-1996  : JV       : Added P3RecInfo.
 *  07-03-1997	: eck      : Added P3ILLSelect() and P3ILLPresent()
 *  31-10-1997	: HSi	   : Added P3FetchTitle()
 *  02-02-1999	: evd      : Added P3SelDbsUpdates().
 *  23-06-1999	: hv       : Added P3ILLTOOAddress() and P3ILLUPDAddress()
 *                           P3ILLDeposit() and P3ILLPassword()
 *  29-09-1999  : HV       : P3ILLSelect() User param removed.
 *  28-10-1999  : MT       : P3DBDelete(), P3DBStore(), P3DBUpdate() added.
 *  13-12-2001  : evd      : Added P3XLWMClose, P3XLWMOpen,
 *                           P3XLWMScanDbsUpdates
 *  18-12-2001  : evd      : P3XLWMSelect, P3XLWMLogon
 *  20-12-2001  : evd      : P3XLWMFetchTitle
 *  24-12-2001	: evd      : Added P3XLWMSelDbsUpdates().
 *  10-10-2013	: martinz  : Copied documentation from P3LIB.DOC.
 *  25-10-2013	: heesakkm : Removed obsolete P3Search() and P3Count().
 *  31-10-2013	: heesakkm : Removed obsolete functions.
 *
 *========================================================================
 */

#if ! defined(P3_H_INCLUDED)
#define P3_H_INCLUDED

#pragma ident "$Id: p3.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <stdlib.h>
#include <pica.h>
#include <qs.h>
#include <cb.h>

#define P3DATETIME_SIZ 36

/* declaration of server handle */
typedef const char	*P3Handle_p;

/* declaration of Pica3 set identification */
/* NULL terminated, all printables   */
typedef char		P3SetIdent_t[32];

/* declaration of P3-library error variable */
ERROR_TYPE(P3Errno_t);

ERROR_DECLARE(P3E_NOERROR);	/* ok */
ERROR_DECLARE(P3E_NOMEM);	/* not enough memory */
ERROR_DECLARE(P3E_SERVER);	/* server error */
ERROR_DECLARE(P3E_NOSUPPORT);	/* function not supported */
ERROR_DECLARE(P3E_AUTHORITY);	/* invalid authority */
ERROR_DECLARE(P3E_IDPASSWRONG);	/* identification failure */
ERROR_DECLARE(P3E_DISCONNECT);	/* server disconnected */
ERROR_DECLARE(P3E_PROTOCOL);	/* protocol error */
ERROR_DECLARE(P3E_TYPE);	/* illegal type */
ERROR_DECLARE(P3E_NOHITS);	/* no hits */
ERROR_DECLARE(P3E_NOINFO);	/* no information replied */
ERROR_DECLARE(P3E_SETFULL);	/* set is full */
ERROR_DECLARE(P3E_BUFFER);	/* buffer overflow */
ERROR_DECLARE(P3E_NOLIBS);	/* no supplying libraries */
ERROR_DECLARE(P3E_OWNLIB);	/* title is in own library */
ERROR_DECLARE(P3E_SETWRONG);	/* invalid set */
ERROR_DECLARE(P3E_TTLWRONG);	/* invalid title */
ERROR_DECLARE(P3E_FLDWRONG);	/* invalid (data in) field */
ERROR_DECLARE(P3E_EMAIL);	/* invalid email address */
ERROR_DECLARE(P3E_MAXCHANNEL);	/* all channels occupied */
ERROR_DECLARE(P3E_MOREINFO);	/* more information needed */
ERROR_DECLARE(P3E_REJECT);	/* transaction rejected */
ERROR_DECLARE(P3E_STOPWORD);	/* stopword detected */
ERROR_DECLARE(P3E_LIBDEPOSIT);	/* deposit of library exhausted */
ERROR_DECLARE(P3E_USRDEPOSIT);	/* deposit of user exhausted */

extern P3Errno_t P3errno;

P3Handle_p P3Open(const char *options);

bool P3Close(P3Handle_p P3handle);

bool P3Logon(P3Handle_p P3handle, const char *options);

int P3List(P3Handle_p P3handle, const char *DBname, const char *DBlast,
	    const char *options, int DBCount, CBBuf_t *CBbuf, size_t CBsize);

bool P3Info(P3Handle_p P3handle, const char *Dbattr,
	    CBBuf_t *CBbuf, size_t CBsize);

bool P3Select(P3Handle_p P3handle, const char *DBname, const char *Options);

bool P3Option(P3Handle_p P3handle, const char *Option);

bool P3ScanDbsUpdates(P3Handle_p P3Handle, const char *Offset,
			char Last[P3DATETIME_SIZ], int Count,
			const char *Options, CBBuf_t *CBbuf,
			size_t CBSize);

bool P3FetchTitle (P3Handle_p p3handle, const char *options,
		   const char *ppn, CBBuf_t *cbBuf, size_t cbSize);

bool P3SelDbsUpdates(P3Handle_p P3Handle, const char *Offset,
			char Last[P3DATETIME_SIZ], int Count,
			const char *Options, CBBuf_t *CBbuf,
			size_t CBSize);

P3Handle_p P3XLWMOpen(const char *options);

bool P3XLWMClose(P3Handle_p P3handle);

bool P3XLWMScanDbsUpdates(P3Handle_p P3Handle, const char *Offset,
			char Last[P3DATETIME_SIZ], int Count,
			const char *Options, CBBuf_t *CBbuf,
			size_t CBSize);

bool P3XLWMLogon(P3Handle_p P3handle, const char *options);

bool P3XLWMSelect(P3Handle_p P3handle, const char *DBname, const char *Options);

bool P3XLWMFetchTitle (P3Handle_p p3handle, const char *options,
		   const char *ppn, CBBuf_t *cbBuf, size_t cbSize);

bool P3XLWMSelDbsUpdates(P3Handle_p P3Handle, const char *Offset,
			char Last[P3DATETIME_SIZ], int Count,
			const char *Options, CBBuf_t *CBbuf,
			size_t CBSize);

#endif /* ! defined(P3_H_INCLUDED) */
