/*========================================================================
 *  Copyright (c) 2000 by Pica
 *
 *  File	: pxml.h
 *  Description	: The PXML-interface
 *  Author(s)	: Martin Talmon
 *  Creation	: 21-02-2000
 *
 *  History:
 *  21-02-2000	: mt       : creation
 *  01-02-2002	: mt       : PXMLE_IDX added
 *  01-02-2002	: listj    : added PXMLStore / PXMLDelete / PXMLUpdate
 *  06-03-2007  : listj    : added update URL parameter to 
 *                           PXMLStore / PXMLDelete / PXMLUpdate
 *                           
 *
 *  Notes	:
 *
 *========================================================================
 */

#if ! defined(PXML_H_INCLUDED)
#define PXML_H_INCLUDED

#pragma ident "$Id: pxml.h 261102 2010-11-30 15:22:09Z sieburgw $"  

#include <stdlib.h>
#include <pica.h>
#include <qs.h>
#include <cb.h>

typedef const char	*PXMLHandle_p;

typedef char		PXMLSetIdent_t[32];

ERROR_TYPE(PXMLErrno_t);

ERROR_DECLARE(PXMLE_NOERROR);
ERROR_DECLARE(PXMLE_NOMEM);
ERROR_DECLARE(PXMLE_SERVER);
ERROR_DECLARE(PXMLE_NOSUPPORT);
ERROR_DECLARE(PXMLE_AUTHORITY);
ERROR_DECLARE(PXMLE_IDPASSWRONG);
ERROR_DECLARE(PXMLE_DISCONNECT);
ERROR_DECLARE(PXMLE_PROTOCOL);
ERROR_DECLARE(PXMLE_TYPE);
ERROR_DECLARE(PXMLE_NOHITS);
ERROR_DECLARE(PXMLE_NOINFO);
ERROR_DECLARE(PXMLE_SETFULL);
ERROR_DECLARE(PXMLE_BUFFER);
ERROR_DECLARE(PXMLE_NOLIBS);
ERROR_DECLARE(PXMLE_OWNLIB);
ERROR_DECLARE(PXMLE_SETWRONG);
ERROR_DECLARE(PXMLE_TTLWRONG);
ERROR_DECLARE(PXMLE_FLDWRONG);
ERROR_DECLARE(PXMLE_EMAIL);
ERROR_DECLARE(PXMLE_MAXCHANNEL);
ERROR_DECLARE(PXMLE_MOREINFO);
ERROR_DECLARE(PXMLE_REJECT);
ERROR_DECLARE(PXMLE_STOPWORD);
ERROR_DECLARE(PXMLE_LIBDEPOSIT);
ERROR_DECLARE(PXMLE_USRDEPOSIT);
ERROR_DECLARE(PXMLE_NUMBER);
ERROR_DECLARE(PXMLE_TIMEOUT);
ERROR_DECLARE(PXMLE_IDX);

extern PXMLErrno_t PXMLerrno;

PXMLHandle_p PXMLOpen(const char *options);

bool PXMLClose(PXMLHandle_p PXMLhandle);

bool PXMLLogon(PXMLHandle_p PXMLhandle, const char *options);

bool PXMLSelect(PXMLHandle_p PXMLhandle, const char *DBname, const char *Options);

bool PXMLSearch(PXMLHandle_p PXMLhandle, QSQueryHandle_p Qhandle,
		PXMLSetIdent_t SetIdent, long *SetSize);

bool PXMLScan(PXMLHandle_p PXMLhandle, int KeyType, const char *Key, int count,
	    const char *options, CBBuf_t *CBbuf, size_t CBsize);

bool PXMLRelate(PXMLHandle_p PXMLhandle, const PXMLSetIdent_t SetIdent, 
		long Teltno, const char *options, PXMLSetIdent_t NewIdent, 
		long *NewSize);

bool PXMLRecRelate(PXMLHandle_p PXMLhandle, const char *recidt, 
                const char *options, PXMLSetIdent_t NewIdent, long *NewSize);

bool PXMLSetDelete(PXMLHandle_p PXMLhandle, const PXMLSetIdent_t SetIdent);

bool PXMLSetPresent(PXMLHandle_p PXMLhandle, const PXMLSetIdent_t SetIdent, 
		long Toffs, long Eltcount, const char *options,
		CBBuf_t *CBbuf, size_t CBsize);

bool PXMLExplain(PXMLHandle_p PXMLhandle, const char *DBattr,
		char *buf, size_t bufsize);

bool PXMLCost(PXMLHandle_p PXMLhandle, char *buf, size_t bufsize);

bool PXMLDelete(const char *host, const char *port, const char *url, CBBuf_t *Pkey, size_t Psize,
	      const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool PXMLStore(const char *host, const char *port, const char *url, CBBuf_t *Pkey, size_t Psize,
	      const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool PXMLReplace(const char *host, const char *port, const char *url, CBBuf_t *Pkey, size_t Psize,
	      const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

#endif /* ! defined(PXML_H_INCLUDED) */
