/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: qs.h
 *  Description	: Interface for the QS-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 09-12-1993
 *
 *  History:
 *  09-12-1993	: eck      : creation
 *  18-01-1994	: eck      : Added #include protection, changed errors
 *			   : Moved declarations from qslocal.h
 *  27-03-1995	: eck      : Added QS_IKT_BCL.
 *  03-08-1995	: eck      : Added QSSetSort()
 *  11-09-1995	: eck      : Added QS_IKT_SGN.
 *  16-12-1996  : jv       : Added QSRecRelate
 *  19-12-1996  : jv       : Added QSRecInfo
 *  09-05-2000  : mt       : Added QSE_ACCESS
 *
 *  Notes	:
 *  Includes <db.h>
 */

#if ! defined(QS_H_INCLUDED)
#define QS_H_INCLUDED

#pragma ident "$Id: qs.h 261102 2010-11-30 15:22:09Z sieburgw $"

#include <pica.h>
#include <db.h>

/* The types are meant to be neither perfect nor permanent */
typedef struct QSQuery_t *QSQueryHandle_p;
typedef const char *QSSetHandle_p;

/* The BCL and SGN are special */
#define QS_IKT_SGN		53
#define QS_IKT_BCL		5004

/* The ADI ikt's range from 9000 to 9999. */
#define QS_IKT_LOWADI		9000
#define QS_IKT_HIGHADI		9999

ERROR_TYPE(QSErrno_t);

ERROR_DECLARE(QSE_NOERROR);
ERROR_DECLARE(QSE_NOMEM);
ERROR_DECLARE(QSE_FATAL);
ERROR_DECLARE(QSE_NOSUPPORT);
ERROR_DECLARE(QSE_AUTHORITY);
ERROR_DECLARE(QSE_BUFFER);
ERROR_DECLARE(QSE_KEY);
ERROR_DECLARE(QSE_ADI);
ERROR_DECLARE(QSE_OPERATOR);
ERROR_DECLARE(QSE_SETWRONG);
ERROR_DECLARE(QSE_TTLWRONG);
ERROR_DECLARE(QSE_FLDWRONG);
ERROR_DECLARE(QSE_SETFULL);
ERROR_DECLARE(QSE_QUERY);
ERROR_DECLARE(QSE_NOHITS);
ERROR_DECLARE(QSE_NOINFO);
ERROR_DECLARE(QSE_SERVER);
ERROR_DECLARE(QSE_EMAIL);
ERROR_DECLARE(QSE_STOPWORD);
ERROR_DECLARE(QSE_LIBDEPOSIT);
ERROR_DECLARE(QSE_USRDEPOSIT);
ERROR_DECLARE(QSE_ACCESS);

extern QSErrno_t QSerrno;

typedef enum QSOperator_t {
    QSOP_AND, QSOP_OR, QSOP_ANDNOT, QSOP_NEAR
} QSOperator_t;

QSQueryHandle_p QSCreate(void);

bool QSDelete(QSQueryHandle_p Qhandle);

bool QSAddTerm(QSQueryHandle_p Qhandle, int KeyType, const char *Key);

bool QSAddSet(QSQueryHandle_p Qhandle, QSSetHandle_p Shandle);

bool QSAddOperator(QSQueryHandle_p Qhandle, QSOperator_t QSoper, ...);

QSSetHandle_p QSSearch(DBHandle_p DBhandle, QSQueryHandle_p Qhandle);

QSSetHandle_p QSSetCreate(DBHandle_p DBhandle, const char *Soptions);

bool QSSetDelete(QSSetHandle_p Shandle, const char *Options);

bool QSSetPresent(QSSetHandle_p Shandle, int FirstElt, int Count,
		    const char *Poptions, CBBuf_t *CBbuf, size_t CBsize);

bool QSSetCopy(QSSetHandle_p ShandleFrom, int Offs, int count,
		QSSetHandle_p ShandleTo);

QSSetHandle_p QSSetRelate(QSSetHandle_p Shandle, int Teltno,
			const char *Options);

QSSetHandle_p QSRecRelate(DBHandle_p DBhandle, const char *Recidt,
			  const char *Soptions); 

bool QSSetInfo(QSSetHandle_p Shandle, const char *Option,
		    CBBuf_t *CBbuf, size_t CBsize);

bool QSScan(DBHandle_p DBhandle, int KeyType, const char *Key,
		size_t count, const char *Options,
		CBBuf_t *CBbuf, size_t CBsize);

QSSetHandle_p QSSetSave(QSSetHandle_p ShandleFrom, const char *Options);

bool QSOrderForm(QSSetHandle_p Shandle, long Teltno,
		    const char *Options, CBBuf_t *CBbuf, size_t CBsize);

bool QSOrderSend(DBHandle_p DBhandle, CBBuf_t *CBbuf, size_t CBsize);

bool QSSDIPresent(DBHandle_p dbHandle, long tOffs, long eltCount,
		 const char *options, long *sdiFound,
		 CBBuf_t *cbBuf, size_t cbSize);

bool QSSDIForm (QSSetHandle_p qsSetHandle, long tEltNo, const char *options,
		CBBuf_t *cbBuf,size_t cbSize);

bool QSSDIInsert (DBHandle_p dbHandle, CBBuf_t *cbBuf, size_t cbSize);

bool QSSDIUpdate (DBHandle_p dbHandle, long tOffs, 
		   CBBuf_t *cbBuf, size_t cbSize);

bool QSTitleInfo(QSSetHandle_p Shandle, long Teltno, const char *Option,
		    CBBuf_t *CBbuf, size_t CBsize);

long QSCount(DBHandle_p DBhandle, QSQueryHandle_p Qhandle);

bool QSLocate(DBHandle_p DBhandle, const char *Options,
		CBBuf_t *CBbuf, size_t CBsize);

QSSetHandle_p QSSetSort(QSSetHandle_p Shandle, const char *Options);

bool QSCopyArticle(DBHandle_p DBhandle,
		    const char *recidt, const char *emailaddr);


bool QSRecInfo(DBHandle_p DBhandle, const char *Recidt,
		 const char *Options, CBBuf_t *CBbuf, size_t CBsize);


/*
 * The following part is not for use in applications.  Only the QS- and
 * SRV-library will use these.
 */
typedef enum { QNT_SET, QNT_OPERATOR, QNT_TERM } QSQNodeType_t;

typedef struct {
    int		qt_type;
    const char	*qt_key;
} QSTerm_t;

typedef struct QSQueryStack_t {
    QSQNodeType_t		qs_type;
    union {
	QSSetHandle_p		qu_sethandle;
	struct {
	    QSOperator_t	qo_operator;
	    int			qo_distance;
	} qu_oper;
	QSTerm_t		qu_term;
    } qs_union;
    struct QSQueryStack_t	*qs_next;
} *QSQueryStack_p;

#define QS_operator		qs_union.qu_oper.qo_operator
#define QS_distance		qs_union.qu_oper.qo_distance
#define QS_sethandle		qs_union.qu_sethandle
#define QS_term			qs_union.qu_term
#define QS_keytype		qs_union.qu_term.qt_type
#define QS_key			qs_union.qu_term.qt_key

struct QSQuery_t {
    QSQueryStack_p	qh_top;		/* top of the stack */
    QSQueryStack_p	qh_current;	/* for walking down the stack */
};

void QSQParseStart(QSQueryHandle_p Qhandle);

QSQueryStack_p QSQParseGet(QSQueryHandle_p Qhandle);

#endif /* ! defined(QS_H_INCLUDED) */
