/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: srv.h
 *  Description	: The interface to the SRV-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 24-11-1993
 *
 *  History:
 *  24-11-1993	: eck      : creation
 *  18-01-1994	: eck	   : Added #include protection
 *			   : Added #include of <cb.h>, <qs.h> and <adb.h>
 *  06-07-1994	: eck      : Added SRVSetRelate().
 *  06-12-1994	: HSi	   : Added SRVIndentifyEnduser, SRVILLForm and
 *			     SRVILLSend.
 *			     Added SRVE_AUTHORITY, SRVE_ERROR, SRVE_SETWRONG,
 *			     SRVE_TTLWRONG, SRVE_NOLIBS, SRVE_OWNLIB.
 *  12-12-1994	: HSi	   : added SRVE_EMAIL.
 *  28-12-1994  : HSi	   : added SRVE_NOHITS.
 *  08-06-1995	: eck      : Added SRVCount().
 *  03-08-1995	: eck      : Added SRVSetSort().
 *  05-10-1995	: eck      : Added SRVOption().
 *  17-12-1996	: JV       : Added SRVRecRelate().
 *  19-12-1996	: JV       : Added SRVRecInfo().
 *  07-03-1997	: eck      : Added SRVILLSelect() and SRVILLPresent().
 *  23-06-1999	: hv       : Added SRVILLTOOAddress(), SRVILLUPDAddress()
 *                           SRVILLPassword(), SRVILLDeposit()
 *  29-09-1999  : HV       : SRVILLSelect() User param removed.
 *  28-10-1999  : MT       : SRVDBDelete(), SRVDBStore(), SRVDBUpdate added().
 *
 *  Notes	:
 */

#if ! defined(SRV_H_INCLUDED)
#define SRV_H_INCLUDED

#pragma ident "$Id: srv.h 261102 2010-11-30 15:22:09Z sieburgw $"

#include <stdlib.h>
#include <pica.h>
#include <cb.h>
#include <qs.h>
#include <adb.h>

typedef const char *SRVHandle_p;
typedef const char *SRVSetHandle_p;

ERROR_TYPE(SRVErrno_t);

ERROR_DECLARE(SRVE_NOERROR);
ERROR_DECLARE(SRVE_NOMEM);
ERROR_DECLARE(SRVE_FATAL);
ERROR_DECLARE(SRVE_TYPE);
ERROR_DECLARE(SRVE_NOSUPPORT);
ERROR_DECLARE(SRVE_IDPASSWRONG);
ERROR_DECLARE(SRVE_CONNECT);
ERROR_DECLARE(SRVE_SEND);
ERROR_DECLARE(SRVE_RECEIVE);
ERROR_DECLARE(SRVE_PROTOCOL);
ERROR_DECLARE(SRVE_BUFFER);
ERROR_DECLARE(SRVE_NOKEYS);
ERROR_DECLARE(SRVE_SCAN);
ERROR_DECLARE(SRVE_SETFULL);
ERROR_DECLARE(SRVE_AUTHORITY);
ERROR_DECLARE(SRVE_ERROR);
ERROR_DECLARE(SRVE_SETWRONG);
ERROR_DECLARE(SRVE_TTLWRONG);
ERROR_DECLARE(SRVE_FLDWRONG);
ERROR_DECLARE(SRVE_NOLIBS);
ERROR_DECLARE(SRVE_OWNLIB);
ERROR_DECLARE(SRVE_EMAIL);
ERROR_DECLARE(SRVE_NOHITS);
ERROR_DECLARE(SRVE_NOINFO);
ERROR_DECLARE(SRVE_MAXCHANNEL);
ERROR_DECLARE(SRVE_STOPWORD);
ERROR_DECLARE(SRVE_MOREINFO);
ERROR_DECLARE(SRVE_REJECT);
ERROR_DECLARE(SRVE_LIBDEPOSIT);
ERROR_DECLARE(SRVE_USRDEPOSIT);
ERROR_DECLARE(SRVE_Z3950ACCESS);
ERROR_DECLARE(SRVE_NUMBER);

extern SRVErrno_t SRVerrno;

SRVHandle_p SRVOpen(ADBHandle_p ADBhandle, const char *options);

bool SRVClose(SRVHandle_p SRVhandle);

int SRVList(SRVHandle_p SRVhandle, const char *DBname, const char *DBlast,
	    const char *DBoptions, int DBcount,
	    CBBuf_t *CBbuf, size_t CBsize);

bool SRVSelect(SRVHandle_p SRVhandle, ADBHandle_p ADBhandle,
		const char *DBname, const char *options);

bool SRVInfo(SRVHandle_p SRVhandle, const char *DBattr,
		CBBuf_t *CBbuf, size_t CBsize);

SRVSetHandle_p SRVSearch(SRVHandle_p SRVhandle, QSQueryHandle_p Qhandle);

bool SRVScan(SRVHandle_p SRVhandle, int KeyType, const char *Key,
	    size_t Count, const char *options, CBBuf_t *CBbuf, size_t CBsize);

bool SRVSetInfo(SRVHandle_p SRVhandle,
		SRVSetHandle_p SRVSethandle, const char *options,
		CBBuf_t *CBbuf, size_t CBsize);

bool SRVSetDelete(SRVHandle_p SRVhandle, SRVSetHandle_p SVRSethandle);

bool SRVSetPresent(SRVHandle_p SRVhandle,
		    SRVSetHandle_p SVRSethandle, int FirstElt, int Count,
		    const char *Options, CBBuf_t *CBbuf, size_t CBsize);

SRVSetHandle_p SRVSetRelate(SRVHandle_p SRVhandle,
			    SRVSetHandle_p SRVSHandle,
			    int ELtno, const char *Options);

SRVSetHandle_p SRVRecRelate(SRVHandle_p SRVhandle,
			    const char *Recidt, const char *Options);

SRVSetHandle_p SRVSetSave(SRVHandle_p SRVhandle, SRVSetHandle_p Shandle,
			    const char *Options);

bool SRVIdentifyEnduser(SRVHandle_p srvHandle, const char *options,
			CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLForm(SRVHandle_p srvHandle, const char *options,
		CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLSend(SRVHandle_p srvHandle, const char *options,
		CBBuf_t *cbBuf, size_t cbSize);

int SRVILLSelect(SRVHandle_p srvHandle, const char *options, 
		CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLPresent(SRVHandle_p srvHandle, const char *Options,
		int FirstElt, int Count, CBBuf_t *cbBuf, size_t cbSize);

bool SRVOrderForm(SRVHandle_p SRVhandle, SRVSetHandle_p Shandle, long Teltno,
			const char *Options, CBBuf_t *CBbuf, size_t CBsize);

bool SRVOrderSend(SRVHandle_p SRVhandle, CBBuf_t *CBbuf, size_t CBsize);

bool SRVSDIPresent(SRVHandle_p srvHandle, long tOffs, long eltCount,
		   const char *options, long *sdiFound,
		   CBBuf_t *cbBuf, size_t cbSize);

bool SRVSDIForm (SRVHandle_p srvHandle, SRVSetHandle_p setHandle,
		 long tEltNo, const char *options,
		 CBBuf_t *cbBuf,size_t cbSize);

bool SRVSDIInsert (SRVHandle_p srvHandle, CBBuf_t *cbBuf, size_t cbSize);

bool SRVSDIUpdate (SRVHandle_p srvHandle, long tOffs,
		   CBBuf_t *cbBuf, size_t cbSize);

bool SRVTitleInfo(SRVHandle_p SRVhandle, SRVSetHandle_p Shandle, long Teltno,
		    const char *Options, CBBuf_t *CBbuf, size_t CBsize);

long SRVCount(SRVHandle_p SRVhandle, QSQueryHandle_p Qhandle);

bool SRVLocate(SRVHandle_p SRVhandle, const char *Options,
		    CBBuf_t *CBbuf, size_t CBsize);

bool SRVCost(SRVHandle_p SRVhandle, char *buf, size_t bufsize);

SRVSetHandle_p SRVSetSort(SRVHandle_p SRVhandle, SRVSetHandle_p Shandle,
			const char *Options);

bool SRVIdentifyBorrower(SRVHandle_p SRVhandle,
                        const char *Ident, const char *Password,
                        CBBuf_t *CBbuf, size_t CBsize);

bool SRVLoanInfo(SRVHandle_p SRVhandle,
		const char *BorIdent, const char *Password,
		const char *Options, long Toffs, long Eltcount,
		long *ELMfound, CBBuf_t *CBbuf, size_t CBsize);

bool SRVListVolumes(SRVHandle_p SRVhandle, const char *Epn,
                    long Toffs, long Eltcount, long *ELMfound,
                    CBBuf_t *CBbuf, size_t CBsize);

bool SRVLoan(SRVHandle_p SRVhandle, CBBuf_t *CBbuf, size_t CBsize);

bool SRVRenewLoan(SRVHandle_p SRVhandle,
                const char *BorIdent, const char *Password,
		const char *VolIdent, CBBuf_t *Cbbuf, size_t CBsize);

bool SRVCancelReservation(SRVHandle_p SRVhandle,
                        const char *BorIdent, const char *Password,
			const char *VolIdent, CBBuf_t *Cbbuf, size_t CBsize);

bool SRVUpdPasswordBorrower(SRVHandle_p SRVhandle,
                        const char *BorIdent,
                        const char *OldPass, const char *NewPass);

bool SRVCopyArticle(SRVHandle_p SRVHandle,
                    const char *recidt, const char *emailaddr);

bool SRVOption(SRVHandle_p SRVhandle, const char *Option);

bool SRVRecInfo(SRVHandle_p SRVhandle, const char *Recidt,
		    const char *Options, CBBuf_t *CBbuf, size_t CBsize);

bool SRVILLTOOAddress(SRVHandle_p srvHandle, const char *options,
		      CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLUPDAddress(SRVHandle_p srvHandle, const char *options,
		      CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLPassword(SRVHandle_p srvHandle, const char *options,
		      CBBuf_t *cbBuf, size_t cbSize);

bool SRVILLDeposit(SRVHandle_p srvHandle, const char *options,
		      CBBuf_t *cbBuf, size_t cbSize);

bool SRVDBDelete(SRVHandle_p SRVhandle, CBBuf_t *Pkey, size_t Psize,
		const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool SRVDBStore(SRVHandle_p SRVhandle, CBBuf_t *Pkey, size_t Psize,
		const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool SRVDBUpdate(SRVHandle_p SRVhandle, CBBuf_t *Pkey, size_t Psize,
		const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

#endif /* ! defined(SRV_H_INCLUDED) */
