/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: tb.h
 *  Description	: The interface to the TB-library
 *  Author(s)	: Clemens Buys
 *  Creation	: 20-12-1993
 *
 *  History:
 *  20-12-1993	: clemens  : creation
 *  20-02-1996	: eck      : Moved accounting support to AC-library.
 *  06-07-1998	: eck      : Made TBExit() visible.
 *  11-12-2009	: sutherlr : Added TBReadFile() and TBWriteRecordToFile()
 *  14-09-2015	: martinz  : TBExit is no longer visible, is called from
 *  		 	     atexit().
 *
 *  Notes	:
 *
 *========================================================================
 */

#if ! defined(TB_H_INCLUDED)
#define TB_H_INCLUDED

#pragma ident "$Id: tb.h 339567 2016-12-05 16:32:52Z heesakkm $"

#include <stdlib.h>
#include <stdio.h>

#include <stddef.h>
#include <pica.h>
#include <cb.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

ERROR_TYPE(TBErrno_t);

ERROR_DECLARE(TBE_NOERROR);
ERROR_DECLARE(TBE_IO);
ERROR_DECLARE(TBE_EOF);
ERROR_DECLARE(TBE_OVERFLOW);
ERROR_DECLARE(TBE_SECURITY);
ERROR_DECLARE(TBE_DUPLICATEKEY);
ERROR_DECLARE(TBE_ILLEGALKEY);
ERROR_DECLARE(TBE_KEYNOTFOUND);
ERROR_DECLARE(TBE_NOSERVER);
ERROR_DECLARE(TBE_OPEN);
ERROR_DECLARE(TBE_MAXOPENS);
ERROR_DECLARE(TBE_NOMEM);
ERROR_DECLARE(TBE_NOKEY);
ERROR_DECLARE(TBE_NODATA);

extern TBErrno_t TBerrno;

bool TBRead(const char *TBopts, const char *Pkey,
        char *Phit, size_t Phitsize, CBBuf_t *CBbuf, size_t CBsize);
bool TBReadN(const char *TBopts, const char *Pkey,
	char *Phit, size_t Phitsize,
	voidp Pbuf, size_t Bufsiz, size_t *Pretsize);

bool TBReadNX(const char *TBopts, const char * TBfilename, const char *Pkey,
	char *Phit, size_t Phitsize,
	voidp Pbuf, size_t Bufsiz, size_t *Pretsize);

bool TBNext(const char *TBopts,
        char *Phit, size_t Phitsize, CBBuf_t *CBbuf, size_t CBsize);
bool TBNextN(const char *TBopts,
	char *Phit, size_t Phitsize,
	voidp Pbuf, size_t Bufsiz, size_t *Pretsize);

bool TBStore(const char *TBopts, const char *Pkey, const CBBuf_t *CBbuf);
bool TBStoreN(const char *TBopts, const char *Pkey, cvoidp buf, size_t siz);

bool TBUpdate(const char *TBopts, const char *Pkey, const CBBuf_t *CBbuf);
bool TBUpdateN(const char *TBopts, const char *Pkey, cvoidp buf, size_t siz);

bool TBDelete(const char *TBopts, const char *Pkey, const CBBuf_t *CBbuf );
bool TBDeleteN(const char *TBopts, const char *Pkey, cvoidp buf, size_t siz);

bool TBReadFile(
	const char	*filename,
	const char	*options, 
	bool		(*store_func)(const char *key,
				      cvoidp data, 
				      size_t data_length, 
				      cvoidp userdata),
	bool		(*error_func)(const char *filename, 
				      long line_number,
				      bool is_error,
				      const char *message),
	cvoidp		userdata);

bool TBWriteRecordToFile(
	FILE            *stream,
	const char      *key,
	byte            *record,
	size_t          rec_size,
	size_t          rec_length,
	int             suppress,
	int		linelength);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(TB_H_INCLUDED) */
