/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wm.h
 *  Description	: Header file for the WM-library
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 13-12-1993
 *
 *  History:
 *  13-12-1993	: lolke    : creation
 *  12-01-1994  : rob      : making it work
 *  14-04-1994	: eck      : added WMMutatedVar()
 *  19-04-1994	: eck      : ANSI-fied some declarations
 *			   : added WME_NOTFOUND and WMGetForm()
 *  19-08-1994	: wimo	   : removed WMSend, WMReceive, WMSendRecv
 *			   : modified WMBuild and WMDissect
 *  21-07-1995	: eck      : Added WMClearCurrentContext()
 *  31-05-1996  : lolke    : WM_MAXWRMLEN too big for TANDEM
 *  12-06-1996  : lolke    : Added WMGet/Put/DelTemplate
 *  22-10-1996  : lolke    : Added WMWormToCB and defines
 *  22-11-1997  : lolke    : Added WMAppToField
 *  10-01-2002  : fb       : Added WMIsRegisteredField, WMIsMutatedField,
 *                           WMRegField and WMClearFieldReg for synchronisation
 *                           with WinIBW.
 *  31-03-2003  : HvdW     : tandem ifdefs removed
 *
 *  Notes	:
 *
 */

#if ! defined(WM_H_INCLUDED)
#define WM_H_INCLUDED

#pragma ident "$Id: wm.h 360943 2018-10-23 16:48:25Z heesakkm $"

#include <stddef.h>
#include <pica.h>
#include <cb.h>
#include <cs.h>

typedef int   WMContext_t;
typedef char  WMIdent_t[3];

#define WM_FIELDS 'F'
#define WM_VARIABLES 'V'
#define WM_MESSAGES 'M'
#define WM_NOCONTEXT	(-1)
#define WM_MAXWRMLEN	(32 * 1024)	/* To IBW without FEP */

typedef enum { 
    WM_CLIENT, WM_SERVER, WM_BOTH
} WMCSflag_t;

ERROR_TYPE(WMErrno_t);

ERROR_DECLARE(WME_NOERROR);
ERROR_DECLARE(WME_EOF);
ERROR_DECLARE(WME_OVERFLOW);
ERROR_DECLARE(WME_LENGTH);
ERROR_DECLARE(WME_ILLEGALCONTEXT);
ERROR_DECLARE(WME_MAXCONTEXT);
ERROR_DECLARE(WME_INTEGRITY);
ERROR_DECLARE(WME_NOIDENT);
ERROR_DECLARE(WME_MAXCOMMAND);
ERROR_DECLARE(WME_MAXMESSAGE);
ERROR_DECLARE(WME_HISIDENT);
ERROR_DECLARE(WME_UNKNOWNPART);
ERROR_DECLARE(WME_FEPERROR);
ERROR_DECLARE(WME_NOTFOUND);

extern WMErrno_t WMerrno;

/* context switch */
WMContext_t WMCreateContext(WMCSflag_t csflag);
WMContext_t WMSwitchContext(WMContext_t handle);
void WMDeleteContext(WMContext_t handle);
void WMClearCurrentContext(void);

/* io routines */
bool WMDissect(char *);
bool WMBuild(char *);
bool WMBuildCS(CSBuf_p worm);
bool WMWormToCB(char, CBBuf_t *, size_t);
bool WMWormComplete(const char *worm, size_t len);
bool WMObjectComplete(const char *object, size_t len);

/* interface functions using CS library */
bool WMGetFormCS	(CSBuf_p storage);
bool WMGetTemplateCS	(CSBuf_p storage);
bool WMGetCmdCS		(WMIdent_t id, CSBuf_p storage);
bool WMGetFieldCS	(const WMIdent_t id, CSBuf_p storage);
bool WMGetHisCS		(WMIdent_t id, CSBuf_p storage);
bool WMGetMsgByIDCS	(WMIdent_t id, CSBuf_p storage);
bool WMGetVarCS		(const WMIdent_t id, CSBuf_p storage);
bool WMGetFieldNCS	(int n, WMIdent_t id, CSBuf_p storage);
bool WMGetHisNCS	(int n, WMIdent_t id, CSBuf_p storage);
bool WMGetMsgNCS	(int n, WMIdent_t id, CSBuf_p storage);
bool WMGetVarNCS	(int n, WMIdent_t id, CSBuf_p storage);

/* buffer identifier */
bool WMGetBufid(WMIdent_t bident);
bool WMPutBufid(const WMIdent_t bident);
bool WMDelBuf(void);  

/* screen fields contents */
bool WMGetField (       const WMIdent_t fident, char *fbuf, size_t fsize);
bool WMGetFieldN(int n, WMIdent_t fident, char *fbuf, size_t fsize);
bool WMPutField(const WMIdent_t fident, const char *fbuf);
bool WMAppField(const WMIdent_t fident, const char *fbuf);
bool WMAppToField(WMIdent_t fident, const char *fbuf);
bool WMDelField(WMIdent_t fident);  
bool WMIsRegisteredField(char *id);
bool WMIsMutatedField(WMIdent_t fident);
bool WMRegField(WMIdent_t fident);
void WMClearFieldReg(void);

/* command line */
bool WMGetCmd(WMIdent_t cident,	char *cbuf, size_t csize);
bool WMPutCmd(const WMIdent_t cident, const char *cbuf);
bool WMDelCmd(void);  

/* global variables */
bool WMMutatedVar(WMIdent_t vident);
bool WMGetVar(const WMIdent_t vident, char *vbuf, size_t vsize);
bool WMGetVarN(int n, WMIdent_t mident, char *mbuf, size_t msize);
bool WMPutVar(const WMIdent_t vident, const char *vbuf);
bool WMRegVar(const WMIdent_t vident);
bool WMDelVar(const WMIdent_t vident);

/* screen identifier */
bool WMGetFrmid(WMIdent_t fident); 
bool WMPutFrmid(const WMIdent_t fident);
bool WMDelFrmid(void);  

/* screen form */
bool WMGetForm(char *form, size_t vsize);
void WMPutForm(const char *form);
void WMDelForm(void);

/* template */
bool WMGetTemplate(char *form, size_t vsize);
void WMPutTemplate(const char *form);
void WMDelTemplate(void);

/* messages */
bool WMGetMsg(int n, WMIdent_t mident, char *mbuf, size_t msize );  
bool WMPutMsg(       const WMIdent_t mident, const char *mbuf );
bool WMDelMsg(void);
  
/* history */
bool WMGetHis(const WMIdent_t hident, char *hbuf, size_t hsize );
bool WMPutHis(WMIdent_t hident, const char *hbuf);  
bool WMDelHis(void);  

/* Backward compatibility */

#define CLIENT WM_CLIENT
#define SERVER WM_SERVER

#endif /* ! defined(WM_H_INCLUDED) */
