/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: adbcreate.c
 *  Description	: The ADBCreat() routine
 *  Author(s)	: Hans van Eck
 *  Creation	: 08-11-1993
 *
 *  History:
 *  08-11-1993	: eck      : creation
 *  10-11-1993	: eck      : added comment && cleaned up the code a bit
 *
 *  Notes	:
 *
 *========================================================================
 */
#pragma ident "$Id: adbcreate.c 295831 2013-01-29 11:28:09Z helspere $"

#include <pica.h>
#include <adb.h>
#include "adblocal.h"
/*------------------------------------------------------------------------
 *  Function	: ADBCreate
 *  Purpose	: Create an empty ADB
 *  Method	: - create a new handle & stack-element & initialise them
 *
 *  History:
 *  08-11-1993	: eck      : creation
 *  22-04-1996	: eck      : Initialise ADBNameTable (only done first time).
 *
 *  Description :
 *  This function will create an empty ADB with the specified type.  A
 *  handle is returned, which can be used in succeeding calls to
 *  ADB-routines.
 *  In order to fill an ADB, ADBRead() must be called for ABD's with
 *  type is ADBT_FULLMEM.  How ADB's of other types are to be handled
 *  is not (yet) specified.
 *  If anything goes wrong, NULL is returned and ADBerrno will hold
 *  the error number.
 *
 *  Example     :
 *
 *	#include <adb.h>
 *
 *	ADBHandle_p	conn_adb;
 *	.
 *	.
 *	conn_adb = ADBCreate(ADBT_FULLMEM);
 *	if (conn_adb == NULL) {
 *		ERWrite(ADBerrno, jobname, "creating adb");
 *		return;
 *	}
 *	.
 *
 *  Returns	: pointer to the adb or NULL
 *
 *------------------------------------------------------------------------
 */
ADBHandle_p
ADBCreate(
	ADBType_t	ADBtype)	/* type of the ADB to create */
{
    ADBHandle_p		adb_p;

    ADBerrno = ADBE_NOERROR;

    if (ADBNameTable == NULL) {
	ADBNameTable = ADBNTCreate();
    }
    /* Allocate the handle */
    adb_p = ADBLNewHandle(ADBtype);

    if (adb_p != NULL) {
	adb_p->sh_first = ADBLNewStackelt("", &(adb_p->sh_adb));
	if (adb_p->sh_first != NULL) {
	    adb_p->sh_last = adb_p->sh_first;
	} else {
	    ADBLFreeHandle(adb_p);
	    adb_p = NULL;
	}
    }

    return adb_p;
} /* ADBCreate() */
