/*=========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: adbget.c
 *  Description	: The ADBGet-routine
 *  Author(s)	: Hans van Eck
 *  Creation	: 08-11-1993
 *
 *  History:
 *  08-11-1993	: eck      : creation
 *  13-02-1994	: eck      : added initialisation of ADBerrno
 *
 *=========================================================================
 */
#pragma ident "$Id: adbget.c 295831 2013-01-29 11:28:09Z helspere $"

#include <stddef.h>
#include <pica.h>
#include <adb.h>
#include "adblocal.h"

/*-------------------------------------------------------------------------
 *  Function	: ADBGet
 *  Purpose	: Retrieve the value of a variable
 *  Method	: - Get pointer to element
 *		: - return pointer to value if possible
 *
 *  History:
 *  09-11-1993	: eck      : creation
 *  13-02-1994	: eck      : added initialisation of ADBerrno
 *
 *  Notes	:
 *
 *  Description :
 *
 *  ADBHandle must be a valid handle (returned by ADBCreate()).
 *
 *  ADBVar must be a valid variable reference.
 *
 *  If anything goes wrong, NULL is returned, and ADBerrno should be
 *  checked.  Otherwise, a pointer to the value is returned.  This
 *  pointer may not be used for storing data, since it points directly
 *  into the ADB.  Since the string is '\0'-terminated, it can be used
 *  as a second argument to CSSetString() or strcpy(), if necessary.
 *
 *  Example     :
 *
 *	#include <adb.h>
 *	.
 *	ADBHandle_p	conn_adb;
 *	char			*id;
 *	.
 *	conn_adb = ADBCreate(ADBT_FULLMEM);
 *	ADBNewVar(conn_adb, "connection[:last].id", "DB1");
 *	.
 *	id = ADBGet(conn_adb, "connection[:last].id");
 *	if (id == NULL) {
 *		ERWrite(ADBerrno, jobname, "getting attribute");
 *		return;
 *	}
 *	.
 *
 *  Returns	: pointer to value or NULL on error
 *-------------------------------------------------------------------------
 */
const char *
ADBGet(
	ADBHandle_p	ADBhandle,	/* The ADB to get a value from */
	const char	*ADBvar)	/* Variable to get the value of */
{
    adb_elt_p		*elt_pp;

    ADBerrno = ADBE_NOERROR;

    elt_pp = ADBLGetElt(ADBhandle->sh_last->se_eltlist_pp, ADBvar);
    if ((elt_pp != NULL) && (*elt_pp != NULL)) {
	if ((*elt_pp)->ae_type == ADBELT_VALUE) {
	    return ((*elt_pp)->ae_value);
	}
	ADBerrno = ADBE_TYPE;
    }
    return NULL;
} /* ADBGet() */
