/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: adblocal.h
 *  Description	: declarations for maintaining ADB's
 *  Author(s)	: Hans van Eck
 *  Creation	: 05-11-1993
 *
 *  History:
 *  05-11-1993	: eck      : creation
 *  11-11-1993	: eck      : moved ADBNT* routines from adbnametable.h
 *			   : #define of STREQ
 *  18-01-1993	: eck	   : remove #define of STREQ, it's in <pica.h> now
 *  22-04-1996	: eck      : Adapted to new NMT-library.
 *
 *========================================================================
 */

#ifndef ADBLOCAL_H_INCLUDED
#define ADBLOCAL_H_INCLUDED

#pragma ident "$Id: adblocal.h 335412 2016-05-27 11:01:38Z heesakkm $"

#include <pica.h>
#include <nmt.h>

enum adb_elttyp_t {
    ADBELT_VALUE, ADBELT_RECORD
};

typedef struct adb_eltlist_t *adb_eltlist_p;

typedef struct adb_elt_t *adb_elt_p;

struct adb_eltlist_t {
    const char		*ael_name;
    adb_elt_p		ael_eltp;
    adb_eltlist_p	ael_next;
};

struct adb_elt_t {
    enum adb_elttyp_t	ae_type;
    union {
	adb_eltlist_p	ev_record; 
	const char	*ev_value; 
    } ae_eltval;
    adb_elt_p		ae_next;
};

#define ae_record	ae_eltval.ev_record
#define ae_value	ae_eltval.ev_value

struct ADB_stackhead_t {
    struct adb_stackelt_t	*sh_first;
    struct adb_stackelt_t	*sh_last;
    ADBType_t			sh_type;
    adb_eltlist_p		sh_adb;
};

struct adb_stackelt_t {
    struct adb_stackelt_t	*se_next;
    struct adb_stackelt_t	*se_prev;
    const char			*se_string;
    adb_eltlist_p		*se_eltlist_pp;
};

#define ADB_IDLEN_MAX	32

/* Declarations for local ADB functions.  They start with ADBL. */
adb_elt_p *ADBLGetElt(adb_eltlist_p *eltlist_pp, const char *varp);
adb_eltlist_p *
ADBLGetEltList(adb_eltlist_p *eltlist_pp, const char *var, const char **fldnam);

ADBHandle_p ADBLNewHandle(ADBType_t ADBtype);
void ADBLFreeHandle(ADBHandle_p ADBhandle);
struct adb_stackelt_t *ADBLNewStackelt(const char *path, adb_eltlist_p *adb_pp);
void ADBLFreeStackelt(struct adb_stackelt_t *selt_p);

adb_elt_p ADBLNewElt(const char *ADBvalue);
adb_eltlist_p ADBLNewEltList(const char *ADBvar);
void ADBLFreeEltList(adb_eltlist_p eltlist_p);
void ADBLFreeElt(adb_elt_p elt_p);

extern NMTHandle_p ADBNameTable;
extern cvoidp ADBNameNewOld(const char *, cvoidp, cvoidp);
#define ADBNTCreate()		(NMTCreate(NULL, ADBNameNewOld))
#define ADBNTLookup(name)	(NMTGet(ADBNameTable, name))
#define ADBNTEnter(name)	(NMTPut(ADBNameTable, name, ""))

#endif /* ADBLOCAL_H_INCLUDED */
