/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: adbsetvar.c
 *  Description	: The ADBSetVar-routine
 *  Author(s)	: Hans van Eck
 *  Creation	: 08-11-1993
 *
 *  History:
 *  08-11-1993	: eck      : creation
 *
 *========================================================================
 */
#pragma ident "$Id: adbsetvar.c 295831 2013-01-29 11:28:09Z helspere $"

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <pica.h>
#include <adb.h>
#include "adblocal.h"

/*------------------------------------------------------------------------
 *  Function	: ADBSetVar
 *  Purpose	: Change the value of a variable
 *  Method	: - Get pointer to (pointer to) the variable
 *		: - Change the value if the type id ok.
 *
 *  History:
 *  09-11-1993	: eck      : creation
 *
 *  Notes	:
 *  This function cannot change the type of the element: passing NULL as
 *  value will result in a crash.
 *
 *  Description :
 *
 *  ADBHandle must be a valid handle (returned by ADBCreate(ADBT_FULLMEM)).
 *
 *  ADBVar must be a valid variable reference (the variable must already exist).
 *
 *  ADBValue must be a '\0'-terminated string.
 *
 *  The function will set the variable specified by ADBVar to (a copy
 *  of) ADBValue.  If something goes wrong, FALSE is returned and
 *  ADBerrno will contain the error.  If all went well, the variable
 *  is set, and TRUE is returned.
 *
 *  Example     :
 *
 *  to be supplied.
 *
 *  Returns	: indication of success
 *------------------------------------------------------------------------
 */
bool
ADBSetVar(
	ADBHandle_p	ADBhandle,	/* The ADB to set a variable in */
	const char	*ADBvar,	/* The variable to change */
	const char	*ADBvalue)	/* The value to set it to */
{
    adb_elt_p		*elt_pp;

    ADBerrno = ADBE_NOERROR;

    elt_pp = ADBLGetElt(ADBhandle->sh_last->se_eltlist_pp, ADBvar);
    if ((elt_pp != NULL) && (*elt_pp != NULL)) {
	if ((*elt_pp)->ae_type == ADBELT_VALUE) {
	    if ((*elt_pp)->ae_value != NULL) {
		free((void *)(*elt_pp)->ae_value);
		(*elt_pp)->ae_value = NULL;
	    }
	    (*elt_pp)->ae_value = malloc(strlen(ADBvalue) + 1);

	    if ((*elt_pp)->ae_value != NULL) {
		(void) strcpy((char *)(*elt_pp)->ae_value, ADBvalue);
	    } else {
		ADBerrno = ADBE_NOMEM;
	    }
	} else {
	    ADBerrno = ADBE_TYPE;
	}
    }
    return ADBerrno == ADBE_NOERROR;
} /* ADBSetVar() */
