<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File		: lib_cb.md
#  Description	: General description of the CB library
#  Author(s)	: Maarten Heesakkers
#  Creation	: 30-09-2013
#
#  History:
#  30-09-2013	: heesakkm : creation
#
#  SVN ident	: $Id: lib_cb.md 307506 2014-01-22 10:52:31Z sieburgw $
-->

The CB library
==============

Counted Buffer Structure
------------------------
Counted buffers provide a flexible way to store and manipulate all kinds of
information. All tables f.i. are based on counted buffer structures (see: TB
Library).

The logical structure of a counted buffer is:

    COUNTED_BUFFER	= <BUFFER_LENGTH>{<ELEMENT>}

    <BUFFER_LENGTH>	= <CB_LENGTH> (BUFFER_LENGTH itself included)

    <ELEMENT>		= <ELEMENT_LENGTH>[<ELEMENT_VALUE>]

    <ELEMENT_LENGTH>	= <CB_LENGTH> (ELEMENT_LENGTH itself included)

    <ELEMENT_VALUE>	= unsigned char[] (all characters allowed)

    <CB_LENGTH>		= unsigned char[CB_LENGTH_SIZE]

The maximum length of a counted buffer is determined by the system parameter
CB_LENGTH_SIZE. The length is calculated as: length =
CB_LENGTH[0]+256*CB_LENGTH[1].
Assigning this parameter  the value 2 allows a maximum length of 64K.
CB_LENGTH_SIZE is only visible in the CB-library itself, not in the application
or other libraries.

In other words:

- a counted buffer consists of a length parameter, containing the length of the
  counted buffer (itself included), optionally followed by a variable number of
  elements
- an element consists of a length parameter, containing the length of the
  element (itself included), optionally followed by an element value,
  containing the contents of the element
- both buffer and element length parameters have the same structure : a fixed
  length string from which the real value can be derived in a standard,
  portable way

Table structure
---------------
As mentioned before, the structure of tables is based on counted buffers.
A table is stored into a counted buffer and consists of a set of, so called,
attributes. Each attribute is stored into an element of the counted buffer.

The structure of an attribute is:

    ATTRIBUTE		= <ATTRIBUTE_LABEL><NULLBYTE><ATTRIBUTE_VALUE><NULLBYTE>

    ATTRIBUTE_LABEL	= char[] ('A'..'Z','a'..'z','0'..'9')

    NULLBYTE		= '\0'

    ATTRIBUTE_VALUE	= unsigned char[] (all characters allowed but \0)

The CB library has routines to manipulate counted buffers sec, elements in a
counted buffer and attributes in elements.

Notes
-----
- The maximum length of a counted buffer is determined by the system parameter
  CB_LENGTH_SIZE. Assigning this parameter  the value 2 allows a maximum length
  of 64K.
- CB_LENGTH_SIZE is only visible in the CB-library itself, not in the
  application or other libraries.
