<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File		: lib_conn.md
#  Description	: General description of the CONN library
#  Author(s)	: Maarten Heesakkers
#  Creation	: 02-10-2013
#
#  History:
#  02-10-2013	: heesakkm : creation
#
#  SVN ident	: $Id: lib_conn.md 307506 2014-01-22 10:52:31Z sieburgw $
-->

The CONN library
================

Connection interface
--------------------

The connection interface is a wrapper around the OS socket functions. Instead
of a sequence of name/address-resolution and socket calls, a single call will
suffice to create a connection or a server-socket or to accept connections.

The functions return file descriptors, so they can be used as arguments in the
EVT-library. All file descriptors returned are set to close-on-exec.

The functions can also operate on local domain sockets (aka UNIX domain
sockets).

Connection functions
--------------------

| Function		    | Return Value			| Options String |
| :------------------------ | :-------------------------------- | :------------: |
| CONN_valid_ipv4()	    | boolean				|		 |
| CONN_valid_ipv6()	    | boolean				|		 |
| CONN_address_equal()	    | boolean				|	X	 |
| CONN_parse_addrspec()	    | boolean				|	X	 |
| CONN_ip_order()	    | string				|		 |
| CONN_my_name()	    | string				|		 |
| CONN_my_ipaddr()	    | string				|		 |
| CONN_my_ipaddr_opts()	    | string				|	X	 |
| CONN_passive_tcp()	    | valid file-descriptor or -1	|		 |
| CONN_passive_tcp_opts()   | valid file-descriptor or -1	|	X	 |
| CONN_passive_tcp4()	    | valid file-descriptor or -1	|	X	 |
| CONN_passive_tcp6()	    | valid file-descriptor or -1	|	X	 |
| CONN_connect_tcp()	    | valid file-descriptor or -1	|		 |
| CONN_connect_tcp_opts()   | valid file-descriptor or -1	|	X	 |
| CONN_connect_tcp4()	    | valid file-descriptor or -1	|	X	 |
| CONN_connect_tcp6()	    | valid file-descriptor or -1	|	X	 |
| CONN_passive_local()	    | valid file-descriptor or -1	|		 |
| CONN_connect_local()	    | valid file-descriptor or -1	|		 |
| CONN_connect()	    | valid file-descriptor or -1	|		 |
| CONN_connect_opts()	    | valid file-descriptor or -1	|	X	 |
| CONN_connect_oneof()	    | valid file-descriptor or -1	|		 |
| CONN_connect_oneof_opts() | valid file-descriptor or -1	|	X	 |
| CONN_accept()		    | valid file-descriptor or -1	|		 |
| CONN_close()		    | 0 on successful completion or -1	|		 |
| CONN_bufsize()	    | boolean				|		 |
| CONN_close_on_exec()	    | boolean				|		 |
| CONN_keepalive()	    | boolean				|		 |
| CONN_nodelay()	    | boolean				|		 |
| CONN_nonblock()	    | boolean				|		 |

The CONN_connect*() functions all have a string as input address argument.
The CONN_passive*() functions have a Counted-String as address argument (used
for input and output).

#### Address syntax ####
The syntax for the address strings accepted by the CONN_connect*() functions is:

    CONN_connect_oneof	:	<CONN_connect> [ '|' <CONN_connect> ]*
    CONN_connect	:	"T=" <CONN_connect_tcp4> | "S=" <CONN_connect_tcp6> | "L=" <CONN_connect_local>
    CONN_connect_tcp*	:	[ <ipv4-addr> | <ipv6-addr> ] ':' <tcp-port>
    CONN_connect_tcp4	:	[ <ipv4-addr> ] ':' <tcp-port>
    CONN_connect_tcp6	:	[ <ipv6-addr> ] ':' <tcp-port>
    CONN_connect_local	:	<path-to-file>
    <ipv4-addr>		:	<hostname> | <dotted-decimal>
    <ipv6-addr>		:	'[' <hostname> | <colon-hexadecimal> ']'

The syntax for the address strings accepted by the CONN_passive*() functions is:

    CONN_passive_tcp*	:	[ <ipv4-addr> | <ipv6-addr> ] [ ':' <tcp-port> ]
    CONN_passive_tcp4	:	[ <ipv4-addr> ] [ ':' <tcp-port> ]
    CONN_passive_tcp6	:	[ <ipv6-addr> ] [ ':' <tcp-port> ]
    CONN_passive_local	:	<path-to-file>
    <ipv4-addr>		:	<hostname> | <dotted-decimal>
    <ipv6-addr>		:	'[' <hostname> | <colon-hexadecimal> ']'

##### Remarks: #####
- <ipv4-addr> can be a name or a dotted-decimal string.
- <ipv6-addr> can be a name or a colon-hexadecimal string (note that an IPv6
  address must be enclosed in square brackets).
- <tcp-port> should be a port-number
- <path-to-file> should be the absolute path of a file

#### Options string ####
A number of functions in the table above have an options string in their
interface that allows manipulation of the default behavior. These functions are
marked with an X in the Options String column.

The syntax of the options string is:

    <optionname>=<value>[;<optionname>=<value>]*

#### File descriptor attributes ####
For the CONN_passive_*() and CONN_connect_*() functions that end with 'tcp4',
'tcp6' and 'opts' and that have an X in the Options String column, a number of
file descriptor attributes can be configured using the options string.
The option name for these cases is defined as "<protocol level>-<optionname>".

The following table lists the supported file descriptor attributes that can be
configured via the options string. The second column contains the name of the
function, if any, that have the similar effect.

| Option Name              | Function                     |
| :----------------------- | :--------------------------- |
| SOL_SOCKET-SO_KEEPALIVE  | CONN_keepalive()             |
| SOL_SOCKET-SO_REUSEADDR  |                              |
| SOL_SOCKET-SO_RCVBUF     | CONN_bufsize(SO_RCVBUF, ...) |
| SOL_SOCKET-SO_SNDBUF     | CONN_bufsize(SO_SNDBUF, ...) |
| SOL_SOCKET-SO_DEBUG      |                              |
| IPPROTO_TCP-TCP_NODELAY  | CONN_nodelay()               |
| IPPROTO_TCP-TCP_MAXSEG   |                              |
| IPPROTO_IPV6-IPV6_V6ONLY |                              |

For CONN_connect_tcp*() functions, the option "SOL_SOCKET-SO_KEEPALIVE=1" is
set by default if it is missing from the options string.

For CONN_passive_tcp*() functions, the options "SOL_SOCKET-SO_REUSEADDR=1" and
"IPPROTO_IPV6-IPV6_V6ONLY=1" are set by default if they are missing from the
options string.

#### Miscellaneous ####
This library does not have its own CONerrno. Errors are logged, and in case of
system errors, errno will be set.

The following defines are available in the conn.h header file:

    #define CONN_ADDRESS_UNKNOWN    '?'
    #define CONN_ADDRESS_IPV4       'T'
    #define CONN_ADDRESS_IPV6       'S'
    #define CONN_ADDRESS_LOCAL      'L'

    #define CONN_IP_VERSION4        '4'
    #define CONN_IP_VERSION6        '6'
