<!--
!
!  Copyright (c) 2013 by OCLC
!
!  File		: lib_cs.md
!  Description	: General description of the CS library
!  Author(s)	: Eric Helsper
!  Creation	: 23-01-2013
!
!  History:
!  23-01-2013	: eric     : creation of this header
!
!  SVN ident	: $Id: lib_cs.md 305171 2013-11-01 13:41:52Z sieburgw $
-->

The CS library
==============

Counted String Structure
------------------------

The logical structure of a counted string is:

    COUNTED_STRING		=
    	<STRING_SIZE><STRING_LENGTH>
    	<STRING_TYPE><STRING_CONTENTS_PTR>

    STRING_SIZE		= size_t

    STRING_LENGTH	= size_t

    STRING_TYPE		= global, local, dynamic or fully-dynamic

    STRING_CONTENTS	= <ARRAY OF CHAR> length depends on decl.

    STRING_CONTENTS_PTR	= pointer to <STRING_CONTENTS>

In other words:

- A Counted String consists of a structure with a size, a length,
  a type and a buffer field.
- The minimum maximum size of a counted string is 32K bytes: the
  minimum maximum size of C objects.
- Only a fully-dynamic CS can be a member of a struct or union.
- There is a dynamic variant of the CS, for which memory for the
  buffer is automatically allocated and de-allocated when necessary.
- For a fully-dynamic variant, the structure is allocated using a
  function.

Notes
-----

- The structure of a counted string is invisible to other libraries.
- The contents of a counted string will be zero-terminated.
  This is enforced by the library.
- A counted string may contain nul-characters.
- A dynamic or fully-dynamic CS requires the programmer to deallocate
  when the variable goes out of scope.
- The CSBuf_t type may be used only by the CS-library.
- Variables must be defined with CS_GLOBAL(name, size),
  CS_LOCAL(name, size), CS_DYNAMIC(name).
  CSBuf_p (initialized with CSCreate()) can be used to create a fully-dynamic
  counted string.

