/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: envgetopt.c
 *  Description	: The sure way for getopt().
 *  Author(s)	: Hans van Eck
 *  Creation	: 22-08-1994
 *
 *  History:
 *  22-08-1994	: eck      : creation
 *
 *  Notes	:
 *  This should work the same as the POSIX getopt()-routine, but it is
 *  necessary for non-POSIX systems.
 *
 *========================================================================
 */

#pragma ident "$Id: envgetopt.c 305171 2013-11-01 13:41:52Z sieburgw $"

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pica.h>
#include <env.h>

/** Will be set to the argument of an option if it requires one. */
char	*ENVoptarg;
/** Index in argv to the current arg, which is possibly an option. */
int	ENVoptind = 1;
/** The current option letter. */
int	ENVoptopt;
/** If not 0, errors (like unknown option) will be written to stderr. */
int	ENVopterr = 0;

/*------------------------------------------------------------------------
 *  Function	: ENVoptionerror
 *  Purpose	: Write an error about an option to stderr.
 *
 *  History:
 *  22-08-1994	: eck      : creation
 *  07-02-1995	: eck      : use ENVBasename()
 *
 *------------------------------------------------------------------------
 */
static void
ENVoptionerror(
	const char	*prog,		/* Name of application (argv[0]) */
	const char	*string)	/* Error to display */
{
    if (ENVopterr == 0) return;

    (void) fprintf(stderr,"%s: %s -- %c\n",
			    ENVBasename(prog), string, ENVoptopt);
} /* ENVoptionerror() */

/*------------------------------------------------------------------------
 *  Function	: ENVGetOpt
 *  Purpose	: Make sure we have getopt() functionality.
 *
 *  History:
 *  22-08-1994	: eck      : creation
 *
 *  Notes	:
 *  Has POSIX functionality. The variables have ENV prepended. ENVopterr
 *  is initialized differently.
 *
 *  Returns	: The option recognised, '?' or ':'.
 *------------------------------------------------------------------------
 */
int
ENVGetOpt(
	int		argc,		/* number of input args */
	char		*argv[],	/* input args array */
	const char	*optstring)	/* string describing recognized options */
{
    static int	subidx = 1;
    const char	*tmpp;

    ENVoptarg = NULL;
    if (ENVoptind > argc) {
	return -1;
    }
    if ((argv[ENVoptind] == NULL)
	|| (*argv[ENVoptind] != '-')
	|| (*(argv[ENVoptind] + 1) == '\0')) {
	return -1;
    }
    if (STREQ(argv[ENVoptind], "--")) {
	ENVoptind++;
	return -1;
    }
    ENVoptopt = argv[ENVoptind][subidx++];
    if ((tmpp = strchr(optstring, ENVoptopt)) != NULL) {
	if (argv[ENVoptind][subidx] == '\0') {
	    ENVoptind++;
	    subidx = 0;
	}

	if (*(tmpp + 1) == ':') {
	    /* The option has an argument */
	    if (argv[ENVoptind] == NULL) {
		if (optstring[0] == ':') return ':';
		else {
		    ENVoptionerror(argv[0], "option has no argument");
		    return '?';
		}
	    } else {
		ENVoptarg = argv[ENVoptind] + subidx;
		ENVoptind++;
		subidx = 1;
	    }
	} else if (subidx == 0) {
	    subidx = 1;
	}
	return ENVoptopt;
    } else {
	if (argv[ENVoptind][subidx] == '\0') {
	    ENVoptind++;
	    subidx = 1;
	}
	if (optstring[0] != ':') {
	    ENVoptionerror(argv[0], "unknown option");
	}
	return '?';
    }
} /* ENVGetOpt() */
