/*========================================================================
 *  Copyright (c) 2004 by OCLC PICA
 *
 *  File	: hsht.h
 *  Description	: The hash-table library
 *  Author(s)	: Maarten Heesakkers
 *  Creation	: 06-09-2004
 *
 *  History:
 *  06-09-2004	: heesakkm : creation
 *  12-03-2014	: heesakkm : Added HSHTGetNrEntries().
 *
 *  Notes	:
 *
 *========================================================================
 */

#ifndef HSHT_H_INCLUDED
#define HSHT_H_INCLUDED

#pragma ident "$Id: hsht.h 363677 2019-01-10 15:10:10Z heesakkm $"

#include <pica.h>


ERROR_TYPE(HSHTErrno_t);

ERROR_DECLARE(HSHTE_NOERROR);	/* ok */
ERROR_DECLARE(HSHTE_NOMEM);	/* not enough memory */

extern HSHTErrno_t HSHTerrno;

/**
 * The library expects HSHTKey_t to be an unsigned integral type.
 * At the moment it is 32 bits, but in the future this could change to 64 bits.
 * Thus a programmer should not rely on the type being uint32.
 */
typedef uint32			HSHTKey_t;
typedef struct HSHTable_t	*HSHTHandle_p;

/**
 * HSHT_NOFUNC is a function-argument for HSHTCreate(). It can be used when
 * no allocation/deallocation should take place.
 */
#define HSHT_NOFUNC	((cvoidp (*)(HSHTKey_t, cvoidp, cvoidp)) 0)

/**
 * HSHTNewOldString() is also a function argument for HSHTCreate().
 * This routine will allocate and deallocate '\0'-terminated strings.
 */
cvoidp HSHTNewOldString(HSHTKey_t name, cvoidp newp, cvoidp oldp);

HSHTHandle_p HSHTCreate(const char *options,
		cvoidp (*NewOld)(HSHTKey_t name, cvoidp newp, cvoidp oldp));

cvoidp HSHT(HSHTHandle_p table, HSHTKey_t key, cvoidp value, bool mustput);

cvoidp HSHTGet(HSHTHandle_p table, HSHTKey_t key);

cvoidp HSHTPut(HSHTHandle_p table, HSHTKey_t key, cvoidp value);

cvoidp HSHTDel(HSHTHandle_p table, HSHTKey_t key);

#define HSHTGet(table, key)		HSHT(table, key, NULL, FALSE)
#define HSHTPut(table, key, value)	HSHT(table, key, value, TRUE)
#define HSHTDel(table, key)		HSHT(table, key, NULL, TRUE)

void HSHTDestroy(HSHTHandle_p table);

void HSHTClean(HSHTHandle_p table);

typedef bool (*HSHTHandleElt_f) (HSHTKey_t key, cvoidp eltp, voidp xtraparam);

bool HSHTWalk(HSHTHandle_p table,
	      HSHTHandleElt_f HandleElt,
	      voidp xtraparam);

uint32 HSHTGetNrEntries(HSHTHandle_p table);
#endif /* HSHT_H_INCLUDED */
