/*========================================================================
 *  Copyright (c) 2011 by OCLC
 *
 *  File	: ihl.h
 *  Description	: IP address handling functions
 *  Author(s)	: Hans van Eck
 *  Creation	: 16-03-2011
 *
 *  History:
 *  16-03-2011	: eck      : creation
 *
 *========================================================================
 */

#ifndef IHL_H_INCLUDED
#define IHL_H_INCLUDED

#pragma ident "$Id: ihl.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <pica.h>

ERROR_TYPE(IHLErrno_t);

ERROR_DECLARE(IHLE_NOERROR);
ERROR_DECLARE(IHLE_NOMEM);
ERROR_DECLARE(IHLE_INTERNAL);

/**
 * Need room for 2 addresses (at least for IPv4),
 * INET6_ADDRSTRLEN == 46, so make the buffer 100 (2*46 + slack)
 */
#define IHL_ADDRBUF_SIZE	(100)

typedef struct IHL_Entry_t	*IHL_Entry_p;
typedef struct IHL_Handle_t	*IHL_Handle_p;

IHL_Handle_p
IHL_Add(IHL_Handle_p hostlist, const char *hostrange, cvoidp userdata);

void
IHL_Destroy(IHL_Handle_p hostlist);

IHL_Entry_p
IHL_Check(IHL_Handle_p hostlist, const char *host);

cvoidp
IHL_Userdata_Get(IHL_Entry_p ihe);

bool
IHL_IsValidAddress(const char *host);

const char *
IHL_EntryToString(char *buf, IHL_Entry_p address);

IHL_Entry_p
IHL_Entry_Parse(const char **host, cvoidp userdata);

void
IHL_Entry_Destroy(IHL_Entry_p ihe);

bool
IHL_Walk(IHL_Handle_p hostlist,
	    bool (*HandleElt)(IHL_Entry_p entry, voidp xtraparam),
	    voidp userdata);

bool
IHL_CheckAddressInRange(
	IHL_Entry_p range,
	IHL_Entry_p address);

IHL_Handle_p
IHL_Create(void);

/*
 * For debugging only
 */
void
IHL_Print_Entry(
	IHL_Entry_p	ihep);

#endif /* IHL_H_INCLUDED */
