<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File		: lib_ihl.md
#  Description	: Markdown file for lib/ihl
#  Author(s)	: Maarten Heesakkers
#  Creation	: 01-10-2013
#
#  History:
#  01-10-2013	: heesakkm : creation
#
#  SVN ident	: $Id: lib_ihl.md 307506 2014-01-22 10:52:31Z sieburgw $
-->

The IHL library
==============

IP-hostlist
-----------

The IP-hostlist interface is a simple library for maintaining a list if
IP-hosts. Such a list can be used for checking allow/deny of a list of hosts.

The library contains support for IP-ranges and CIDR blocks.

At the moment of this writing, IPv6 support has not been implemented, but
should require no interface changes.

IP-hostrangelist functions
--------------------------

1. IHL_Add()
2. IHL_Destroy()
3. IHL_Check()
4. IHL_IsValidAddress()
5. IHL_EntryToString()

The list of hostranges is implemented as a IHL_Handle_p with each element of
type IHL_Entry_p. This is an opaque datatype. There is a function to convert
such an entry to a string.

The syntax for addresses accepted by the IHL_add functions is:

    <hostrange>		::= <dotted_address>
    			    | <dotted_address> '-' <dotted_address>
    			    | <dotted_address> '/' <nr_mask_bits> ;

    <dotted_address>	::= <octet_value> '.' <octet_value> '.' <octet_value> '.' <octet_value> ;

    <octet_value>	::= 0 .. 255 ;

    <nr_mask_bits>	::= 0 .. 32 ;

Examples of valid hostranges:

    74.125.77.99
    74.125.77.99-74.125.77.110
    74.125.77.99/24


Example
-------
\code
    #include <ihl.h>
    .
    static bool
    handle_ihl_entry(
	    IHL_Entry_p	entryp,
	    voidp		userdata)
    {
	printf("entry: %s\n", IHL_EntryToString(NULL, entryp));	
    }

    IHL_Handle_p	hostlist;

    hostlist = IHL_Add(NULL, "1.2.3.4");
    hostlist = IHL_Add(hostlist, "10.20.30.40-10.20.30.50");
    hostlist = IHL_Add(hostlist, "100.100.100.174/30");

    IHL_Check(hostlist, "1.2.3.4");		// returns TRUE
    IHL_Check(hostlist, "1.2.3.5");		// returns FALSE
    IHL_Check(hostlist, "10.20.30.43");		// returns TRUE
    IHL_Check(hostlist, "100.100.100.171");	// returns FALSE
    IHL_Check(hostlist, "100.100.100.172");	// returns TRUE
    IHL_Check(hostlist, "100.100.100.175");	// returns TRUE
    IHL_Check(hostlist, "100.100.100.176");	// returns FALSE

    IHL_Walk(hostlist, handle_ihl_entry, NULL);
	    // Prints entry: 1.2.3.4
	    //        entry: 10.20.30.40-10.20.30.50
	    //        entry: 100.100.100.172/30
\endcode
