/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: adb.h
 *  Description	: The interface to the ADB-routines
 *  Author(s)	: Hans van Eck
 *  Creation	: 08-11-1993
 *
 *  History:
 *  08-11-1993	: eck      : creation
 *  11-11-1993	: eck      : added #include <stdio.h>, changed ABDWithEnd()
 *  12-11-1993	: eck      : added ADBE_WITHEND, deleted ADB_IDXNOTPRESENT
 *  18-01-1994	: eck      : added #include protection and ADB_KEYLENGTH_MAX
 *
 *  Notes	:
 *  Some fix should be found for the #include <stdio.h>.
 *  It is not conform pica-standards but necessary for DBPrint().  Not
 *  using DBPrint() would otherwise counter-intuitively require the #include.
 */

#if ! defined(ADB_H_INCLUDED)
#define ADB_H_INCLUDED

#pragma ident "$Id: adb.h 261102 2010-11-30 15:22:09Z sieburgw $"

#include <stdio.h>
#include <pica.h>

#define ADB_KEYLENGTH_MAX	200

typedef struct ADB_stackhead_t *ADBHandle_p;

typedef enum { ADBT_FULLMEM } ADBType_t;

ERROR_TYPE(ADBErrno_t);

ERROR_DECLARE(ADBE_NOERROR);
ERROR_DECLARE(ADBE_NOMEM);
ERROR_DECLARE(ADBE_TYPE);
ERROR_DECLARE(ADBE_FILE);
ERROR_DECLARE(ADBE_ILLEGALKEY);
ERROR_DECLARE(ADBE_NOTPRESENT);
ERROR_DECLARE(ADBE_INTEGRITY);
ERROR_DECLARE(ADBE_IO);
ERROR_DECLARE(ADBE_WITHEND);
ERROR_DECLARE(ADBE_SELFCOPY);

extern ADBErrno_t ADBerrno;

ADBHandle_p ADBCreate(ADBType_t ADBtype);

void ADBDestroy(ADBHandle_p ADBhandle);

bool ADBReadFile(ADBHandle_p ADBhandle, const char *ADBfile);

const char *ADBGet(ADBHandle_p ADBhandle, const char *ADBVar);

bool ADBSetVar(ADBHandle_p ADBhandle, const char *ADBvar, const char *ADBvalue);

bool ADBNewVar(ADBHandle_p ADBhandle, const char *ADBvar, const char *ADBvalue);

bool ADBDelVar(ADBHandle_p ADBhandle, const char *ADBvar);

bool ADBWithBegin(ADBHandle_p ADBhandle, const char *ADBvar);

bool ADBWithEnd(ADBHandle_p ADBhandle);

bool ADBPrint(FILE *file, ADBHandle_p ADBhandle, bool compiled);

bool ADBCopy(ADBHandle_p ADBto, ADBHandle_p ADBfrom);

#endif /* ! defined(ADB_H_INCLUDED) */
