/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: db.h
 *  Description	: The interface to the DB-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 03-11-1993
 *
 *  History:
 *  03-11-1993	: eck      : creation
 *  18-01-1994	: eck	   : Added #include protection
 *			   : added #include of <cb.h>
 *			   : Changed some error numbers
 *  15-09-1994	: eck      : Added DBopts to DBNameInfo().
 *  08-06-1995	: eck      : Added DBCount().
 *  05-10-1995	: eck      : Added DBOption().
 *  28-10-1999	: martin   : Added DBDelete(), DBStore(), DBUpdate().
 *  10-08-2000	: martin   : DBE_NOSUPPORT added.
 *
 *  Notes	:
 *
 */

#if ! defined(DB_H_INCLUDED)
#define DB_H_INCLUDED

#pragma ident "$Id: db.h 261102 2010-11-30 15:22:09Z sieburgw $"

#include <pica.h>
#include <cb.h>

typedef const char *DBHandle_p;

ERROR_TYPE(DBErrno_t);

ERROR_DECLARE(DBE_NOERROR);
ERROR_DECLARE(DBE_FATAL);
ERROR_DECLARE(DBE_BUFFER);
ERROR_DECLARE(DBE_ISDATABASE);
ERROR_DECLARE(DBE_COULDNOTMAP);
ERROR_DECLARE(DBE_IDPASS);
ERROR_DECLARE(DBE_SERVER);
ERROR_DECLARE(DBE_MAXCHANNEL);
ERROR_DECLARE(DBE_LIBDEPOSIT);
ERROR_DECLARE(DBE_USRDEPOSIT);
ERROR_DECLARE(DBE_AUTHORITY);
ERROR_DECLARE(DBE_Z3950ACCESS);
ERROR_DECLARE(DBE_NOSUPPORT);

extern DBErrno_t DBerrno;

int DBList(const char *DBname, const char *DBlast, const char *Dbopts,
	    int DBcount, CBBuf_t *CBbuf, size_t CBsize);

DBHandle_p DBOpen(const char *DBname, const char *DBopts);

bool DBClose(DBHandle_p DBhandle);

bool DBRead(DBHandle_p DBhandle, CBBuf_t *Pkey, size_t Psize,
	    const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool DBStore(DBHandle_p DBhandle, CBBuf_t *Pkey, size_t Psize,
	    const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool DBUpdate(DBHandle_p DBhandle, CBBuf_t *Pkey, size_t Psize,
	    const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool DBDelete(DBHandle_p DBhandle, CBBuf_t *Pkey, size_t Psize,
	    const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

bool DBInfo(DBHandle_p DBhandle, const char *DBattr,
	    CBBuf_t *CBbuf, size_t CBsize);

bool DBNameInfo(const char *DBname, const char *DBattr,
		const char *DBopts, CBBuf_t *CBbuf, size_t CBsize);

void DBClean(const char *options);

bool DBCost(DBHandle_p DBhandle, char *buf, size_t bufsize);

bool DBOption(DBHandle_p DBhandle, const char *Option);

#endif /* ! defined(DB_H_INCLUDED) */
