/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: env.h
 *  Description	: The interface to the ENV-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 23-11-1993
 *
 *  History:
 *  23-11-1993	: eck      : creation
 *  14-12-1993  : clemens  : extra parameter ENVInit(..,appl_version)
 *                           ENVReadfile() no longer static
 *                           ENVRefresh() added
 *  09-05-1994	: HSi	   : Added ENVE_VARMISSING: environment variable (used
 *			     internally ) is missing
 *			     Added ENVEN_NOACTREC: no application-configuration
 *			     record found for application.
 *  07-02-1995	: eck      : Changed interface of ENVInit().
 *			     Added ENVBasename().
 *  10-01-1996	: eck      : Removed ENVRefresh() and ENVE_NOACTREC.
 *  11-09-2003	: paul     : Added ENVutf8 (Unicode Master Switch)
 *  24-02-2005	: sutherlr : Made ENVExpandVariables() public.
 *  23-03-2005	: paul     : Added ENV_max_copies
 *  09-10-2013	: heesakkm : Made ENVversion publicly available.
 *
 *========================================================================
 */

#if ! defined(ENV_H_INCLUDED)
#define ENV_H_INCLUDED

#pragma ident "$Id: env.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <pica.h>
#include <picatypes.h>

#ifdef  __cplusplus
extern "C" {
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
#endif

ERROR_TYPE(ENVErrno_t);

ERROR_DECLARE(ENVE_NOERROR);
ERROR_DECLARE(ENVE_NOMEM);
ERROR_DECLARE(ENVE_IO);
ERROR_DECLARE(ENVE_FILEMAP);
ERROR_DECLARE(ENVE_OPTION);
ERROR_DECLARE(ENVE_VARMISSING);

extern ENVErrno_t ENVerrno;

extern char	*ENVoptarg;
extern int	ENVoptind;
extern int	ENVopterr; 
extern int	ENVoptopt;

extern bool	ENVutf8;

extern const char	*ENVversion;
extern PICCopyNum_t	ENV_max_copies;

#define ENV_NOFUNC	((void (*)(int)) 0)

bool ENVReadFile(const char *filename);

bool ENVInit(int argc, char *argv[], const char *appl_version,
	    const char *opts, void (*func)(int option));

int ENVGetOpt(int argc, char *argv[], const char *optstring);

bool ENVSetVar(const char *name, const char *Value);

const char *ENVGetVar(const char *name);

const char *ENVBasename(const char *path);

bool ENVWalk(bool (*HandleElt)( const char *name, 
				const char *value, 
				voidp xtra), 
	    voidp xtra);

const char *ENVExpandVariables(const char *value_in);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(ENV_H_INCLUDED) */
