/*========================================================================
 *  Copyright (c) 1995 by Pica
 *
 *  File	: evt.h
 *  Description	: The interface for the EVT-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 07-05-1997
 *
 *  History:
 *  07-05-1997	: eck      : creation
 *  10-02-1998	: BdB	   : Added EVTT_FDWRITE events
 *  02-03-2000	: eck      : Added EVTE_FILDES (EVTPost wrong file-descriptor)
 *
 *========================================================================
 */

#if ! defined(EVT_H_INCLUDED)
#define EVT_H_INCLUDED

#pragma ident "$Id: evt.h 305171 2013-11-01 13:41:52Z sieburgw $"

#include <pica.h>


ERROR_TYPE(EVTErrno_t);

ERROR_DECLARE(EVTE_NOERROR);	/* (ok) */
ERROR_DECLARE(EVTE_NOMEM);	/* (not enough memory) */
ERROR_DECLARE(EVTE_FILDES);	/* (file-descriptor error) */

extern EVTErrno_t EVTerrno;

/** Event id. */
typedef unsigned long	EVTId_t;

/**
 * EVT_NOEVENT is an event id. It is returned when an event could not be created.
 */
#define EVT_NOEVENT	((EVTId_t) 0)

typedef enum {
    EVTT_FILDES,	/* Trigger on file-descriptor */
    EVTT_FDWRITE,	/* Trigger on writable file descriptor */
    EVTT_MILLIS,	/* Trigger after a number of milliseconds */
    EVTT_SECONDS,	/* Trigger after a number of seconds */
    EVTT_ABS_MILLIS,	/* Trigger on specific time (in seconds) */
    EVTT_ABS_SECONDS	/* Trigger on specific time (milliseconds) */
} EVTType_t;

void EVTMain(void);

EVTId_t EVTPost(
	EVTType_t	evt_typ,
	unsigned long	fildes_or_time,
	void		(*callback_handler)(cvoidp argument),
	cvoidp		callback_argument);

bool EVTDelete(EVTId_t event_id);

#endif /* ! defined(EVT_H_INCLUDED) */
