/*
 *  Copyright (c) 1998 by Pica
 *
 *  File	: fioous.h
 *  Description	: Header for OUS tables FIO types and routines
 *  Author(s)	: Els van Doorn
 *  Creation	: 13-01-1999
 *
 *  History:
 *  13-01-1999	: EvD 	: creation
 *  06-05-1999  : EvD   : fno in textline routines changed to int8
 *
 *  Notes	:
 *
 */

#ifndef FIOOUS_H_INCLUDED
#define FIOOUS_H_INCLUDED

#pragma ident "$Id: fioous.h 261102 2010-11-30 15:22:09Z sieburgw $"

#define SIZE_OUS_FREE_TEXT 54
#define SIZE_OUS_SHELF_MARK 64
#define SIZE_OUS_VOLUME_INFORMATION 15
#define SIZE_OUS_DBDATETIME 30
#define SIZE_OUS_VOLUME_BAR 20
#define SIZE_OUS_VOLUME_BARCODE 50

/* Record containing all fields (with the same names) as the volume
   table in OUS. */
typedef struct fioous_datetime_t {
    int		days;
    int		time;
} fioous_datetime_t;

typedef struct OUS_volume_t {
    int32 volume_number;
    char volume_bar[SIZE_OUS_VOLUME_BAR + 1];
    PICIln_t iln;
    PICIpn_t epn;
    int32 container_number;
    short cum_loans;
    short cum_requests;
    short cum_reservations;
    char date_of_availability[SIZE_OUS_DBDATETIME + 1];
    char edit_date[SIZE_OUS_DBDATETIME + 1];
    char extra_barcode[SIZE_OUS_VOLUME_BARCODE + 1];
    char free_text[SIZE_OUS_FREE_TEXT + 1];
    char input_date[SIZE_OUS_DBDATETIME + 1];
    int8 loan_indication;
    int8 loan_status;
    char shelfmark_defiant[SIZE_OUS_SHELF_MARK + 1];
    int8 source_code;
    int8 type_of_material;
    char volume_information[SIZE_OUS_VOLUME_INFORMATION + 1];
    int8 volume_status;
    fioous_datetime_t volume_ts;
} OUS_volume_t;

/* Record containing all fields (with the same names) as the iln 
   table in OUS. */
typedef struct OUS_iln_t {
    PICIln_t iln;
} OUS_iln_t;

#define MAX_LNIC_CONV_L 50

typedef struct OUS_lnic_conv_rec_t
{
    char ggc_lnic;
    int8 ous_lnic;
} OUS_lnic_conv_rec_t;

typedef struct OUS_lnic_conv_arr_t
{
    int16 nr_of_lines;
    OUS_lnic_conv_rec_t inh[MAX_LNIC_CONV_L];
} OUS_lnic_conv_arr_t;

#define MAX_CONV_TABLE_L 256
typedef struct OUS_conv_table_arr_t
{
    char tbl[MAX_CONV_TABLE_L][2];
} OUS_conv_table_arr_t;

#define MAXCONTENTSIZE( a, b ) ((a)>(b) ? (a) : (b)) 
#define SIZE_OUS_MAX_CONTENT MAXCONTENTSIZE( MAXCONTENTSIZE( sizeof(OUS_lnic_conv_arr_t),sizeof(OUS_conv_table_arr_t) ) , 0 )

typedef struct OUS_data_t {
    byte content[SIZE_OUS_MAX_CONTENT];
} OUS_data_t;

#define SIZE_OUS_PARAMCODE 6
#define SIZE_OUS_PARAM_CONTENT 255

typedef struct OUS_param_t {
    char paramcode[SIZE_OUS_PARAMCODE + 1];
    int  soort;
    int  vlgnr;
    char content1[SIZE_OUS_PARAM_CONTENT + 1];
    char content2[SIZE_OUS_PARAM_CONTENT + 1];
} OUS_param_t;

#define SIZE_OUS_LANGUAGE 2
#define SIZE_OUS_TEXTLINE 80

typedef struct OUS_textlines_t {
    PICFileSet_t	fno;
    int16		typ;
    int			number;
    char		language[SIZE_OUS_LANGUAGE + 1];
    int16		line_count;
    char		textline[SIZE_OUS_TEXTLINE + 1];
} OUS_textlines_t;

/* Find all ilns in ous */
bool
FIOOUS_iln_MSEL1(FIOHandle_p handle, const char *options,
		OUS_iln_t *ousiln);

/* Find another iln for the most recent MSEL routine. */
bool
FIOOUS_iln_NEXT(FIOHandle_p handle, const char *options,
		OUS_iln_t *ousiln);

/* Find all records till maxrows in the volume table with given iln and epn  */
bool
FIOOUS_volume_MSEL3(FIOHandle_p handle, const char *options,
			    PICIln_t iln, PICIpn_t epn, 
			    OUS_volume_t *record);

/* Find another record for the most recent MSEL routine. */
bool
FIOOUS_volume_NEXT(FIOHandle_p handle, const char *options,
			   OUS_volume_t *record);

bool
FIOOUS_data_MSEL1(FIOHandle_p handle, const char *options,
		  int16 data_id_nr, int16 number,
		  byte *prm, size_t prm_l);

/* find param record on paramcode, soort and vlgnr */
bool
FIOOUS_param_MSEL1(FIOHandle_p handle, const char *options,
		   const char *paramcode, int soort, int vlgnr, 
		   OUS_param_t *record);

/* find textline on fno, typ, number and language */
bool
FIOOUS_textlines_MSEL0(FIOHandle_p handle, const char *options,
		       int8 fno, int16 typ, int number, const char *language,
		       char *textline);

bool
FIOOUS_textlines_NEXT0(FIOHandle_p handle, const char *options,
		       char *textline);

/* find textlines on fno, typ and language */
bool
FIOOUS_textlines_MSEL1(FIOHandle_p handle, const char *options,
		       int8 fno, int16 typ, const char *language,
		       OUS_textlines_t *record);

bool
FIOOUS_textlines_NEXT1(FIOHandle_p handle, const char *options,
		       OUS_textlines_t *record);

/* Find the record in the volume table with given iln and volume_number  */
bool
FIOous_volume_SEL0(FIOHandle_p handle, const char *options,
		   PICIln_t iln, int32 volume_number, 
	           OUS_volume_t *record);

#endif /* FIOOUS_H_INCLUDED */
