/*=====================================================================
 *  Copyright (c) 2003 by OCLC PICA
 *
 *  File        : idg.h
 *  Description : header file for idg library
 *  Author(s)   : Guido van den Heuvel
 *  Creation    : 30-10-2003
 *
 *  History:
 *  30-10-2003  : Guy     : creation
 *  14-06-2004  : paul    : adapted IDGRPPHandle for using PP-struct's
 *                        : added IDGRPPLevelCleared()+IDGRPPLevelClear()+
 *                          IDGRPPLevelScan()+IDGRPPExpandBcl()
 *                        : removed IDGRPPClear() and IDGRPPFromIntern()
 *                        : use picatypes in IDGRPPHandle_t
 *  28-06-2004  : paul    : added IDG_PTR_* defines
 *                        : added IDGRPPLevelScanRestore()
 *                        : added IDGGenModify()
 *                        : changed IDGGenerateADI() so it actuall has output
 *                        : added IDGGenerateMatAdi()
 *                        : added IDG_NOFUNC()
 *                        : added IDGAdiFieldToExtern()
 *  30-06-2004  : paul    : added IDGGenGetComp()+IDGGenGetPpntab()
 *                          +IDGRPPExpandBclRestore()
 *                        : new end_of_level added in IDGRPPHandle_t
 *  05-07-2004	: paul    : added IDGAdiFieldToIkt()
 *  07-07-2004  : paul    : added IDGSetFill()+IDGSetFillData_t
 *                        : added IDGE_IDX
 *                        : added IDGOnlineUpdate()
 *  03-08-2004	: paul	  : added IDGSetFillUnique()
 *  12-10-2005	: jaap     : Add IDGNKeywordResetAll()
 *  12-10-2005	: jaap     : Add IDGNNormalise2() and IDGNormPerson2()
 *  19-01-2006	: jaap     : Add IDGOnlineUpdateFromSet()
 *  21-04-2006	: paul     : Improved interface of IDGCompileGen()
 *  07-06-2006  : paul     : Added sortstr_from_pptag callback to IDGGenInit()
 *  01-09-2006  : paul     : Added options parameter to IDGPChconvParse()
 *  18-09-2006  : paul     : Undone previous change
 *  22-09-2006  : paul     : Added IDGImplicitPhraseSearch()
 *  11-10-2006	: paul     : Added IDGCompPrefixKeywordAddedLength()
 *  18-10-2006	: paul     : Removed graphemecount argument from
 *                           IDGCompPrefixKeywordAddedLength()
 *                           Added IDGCompPrefixKeywordIkt()
 *                         : Added IDGCompImplicitPhraseSearch()
 *  24-10-2006  : paul     : Added IDGScriptSeparator()
 *                         : Added IDGScriptSensitiveIkt()
 *                         : Changed HMSIDGCompImplicitPhraseSearch()
 *  24-10-2006  : paul     : Added IDGDenormalisePrefixKeyword()
 *                         : Added IDGNormaliseCodePoint()
 *                         : Added IDGScriptsHandle_p
 *                         : Added IDGScriptsHandleGet()
 *                         : Changed IDGScriptSeparator()
 *  07-03-2007  : paul     : Added options parameter to IDGCompShow()
 *  11-04-2007	: paul     : Added IDGOptimiseInit2()
 *  03-03-2008	: paul     : Added IDGDeNormaliseTerm()
 *  13-02-2008	: daalmantz : Define IDGKeyTranslate_f callback for pointer
 *                            translation. 
 *  22-02-2008	: daalmanf : the rel and fam unions in IDGEntry_t now contain
 *                         : a PICIpn_t ipn instead of "int ipn" in preparation
 *                         : to a variant with PICIdp_t to be written later.
 *  29-02-2008	: daalmanf : added IDP-indexed rel and fam records to IDGEntry_t
 *  19-12-2008	: paul     : Added IDGFindIndexPhraseVariants() for
 *                           highlighting multi-indexed titles
 *  29-01-2009	: paul     : moved IDGNDictFinalize(), IDGNDictFree()
 *                           and IDGNDictAdd() from idgn_dict.h
 *                         : added IDGNDictCompile()
 *  03-03-2009	: paul     : Fixed bug in online index update, due to
 *                           duplication of sources
 *  05-03-2009	: paul     : Allow tags of level 3 (social)
 *  12-08-2009	: paul     : Added IDGDeNormaliseTermIdxType()
 *  04-02-2010	: paul     : Added IDGUtf8Cleanup()
 *  01-03-2010	: paul     : Added IDGCompAddLev0Tags()
 *  18-06-2010	: paul     : CBS-1229 preparation; added IDGCompCodeReplace()
 *  10-10-2011	: martinz  : Added IDGCompileGenOpt().
 *  18-02-2013	: martinz  : Added adidir to IDGOnlineIndexsetStore().
 *  09-04-2014	: martinz  : Added parse types/functions.
 *  09-05-2014	: martinz  : Added bytecode types/functions.
 *  05-06-2014	: martinz  : Replaced IDGAstByteCode() by
 *  			     IDGAstByteCodeCreate().
 *  05-06-2014	: martinz  : Added IDGAstByteCodeDestroy().
 *  14-07-2014	: martinz  : Added IDGParseCleanup() and IDGParseInit().
 *  14-07-2014	: martinz  : Added several functions and types.
 *  16-09-2014	: martinz  : Added idg_functions_p definition.
 *  21-10-2014	: martinz  : Added IDGParseDiagnostics().
 *  19-11-2014	: martinz  : Changed IDGAstByteCodeCreate() interface.
 *  15-12-2014	: martinz  : Added diagnostic messages interface.
 *  21-05-2015	: martinz  : Changed IDGGenerate() interface.
 *  03-06-2015	: martinz  : Got rid of opt_print in IDGOptimiseInit2().
 *
 *  Notes       :
 *=====================================================================
 */
#pragma ident "$Id: idg.h 367425 2019-05-15 06:58:27Z smitp $"

#if ! defined(IDG_H_INCLUDED)
#define IDG_H_INCLUDED

#include <stddef.h>

#include <pica.h>
#include <picatypes.h>
#include <picamisc.h>
#include <idxtypes.h>
#include <idx.h>
#include <pp.h>
#include <cs.h>
#include <nmt.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

#define IDGVM_TEMPLATE '\007' /* no-one in his right frame of mind
			       * would use ASCII BELL for anything */
#define IDGVM_TPL_UP 'c'
#define IDGVM_TPL_NOUP 'C'
#define IDGVM_TPL_LOW 'm'
#define IDGVM_TPL_NOLOW 'M'
#define IDGVM_TPL_ANUM 'x'
#define IDGVM_TPL_NOANUM 'X'
#define IDGVM_TPL_NUM 'd'
#define IDGVM_TPL_NONUM 'D'
#define IDGVM_TPL_ALPHA 'l'
#define IDGVM_TPL_NOALPHA 'L'
#define IDGVM_TPL_ANY '*'

ERROR_TYPE(IDGErrno_t);

ERROR_DECLARE(IDGE_NOERROR);
ERROR_DECLARE(IDGE_NOMEM);
ERROR_DECLARE(IDGE_FORMAT);
ERROR_DECLARE(IDGE_PARAM);
ERROR_DECLARE(IDGE_WRONGTYPE);
ERROR_DECLARE(IDGE_PARSE);
ERROR_DECLARE(IDGE_OVERFLOW);
ERROR_DECLARE(IDGE_INTERNAL);
ERROR_DECLARE(IDGE_IDX);	/* look at IDXerrno */

extern IDGErrno_t IDGerrno;

typedef struct IDGMsg_t *IDGMsg_p;
typedef struct idg_ast_t *idg_ast_p;	/* Type of Abstract Syntax Tree */
typedef struct idg_ast_bytecode_t *idg_ast_bytecode_p;	/* Bytecode handle */
typedef struct idg_parsed_code_t *idg_parsed_code_p;	/* vmcode handle */

typedef enum {
    PARSE_NOERROR,
    PARSE_WARNING,
    PARSE_ERROR
} parse_error_e;

typedef parse_error_e* parse_error_p;

#define IDGRPP_TAG_HASH_COUNT	((PIC_LEVEL_MAX + 1) * 100)
typedef struct IDGRPPHandle_t {
    PICLevel_t		level;
    PICIpn_t		ipn;
    PICIdp_t		idp;
    PICIln_t		iln;
    PICCopyNum_t	occ;
    PPTag_p		tag[IDGRPP_TAG_HASH_COUNT];
    PPTag_p		end_of_level;
    bool		cleared[PIC_LEVEL_MAX + 1];
} IDGRPPHandle_t, *IDGRPPHandle_p;

typedef enum IDGEntryType_t {
    IDGTYPE_NONE = 0,
    IDGTYPE_PHRASE,
    IDGTYPE_RELATION,
    IDGTYPE_FAMILY
} IDGEntryType_t;

typedef struct IDGEntry_t {
    IDGEntryType_t type;
    int fileset;
    union {
	struct {
	    int iln;
	    int idxtype;
	    byte *entry;
	    size_t entry_len;
	    char keytype;
	    char rectype;
	    byte variant;	/* from multiple normalisation? */
	} phr;
	struct {
	    PICIpn_t idp;
	    int idxtype;
	    char keytype;
	    int iln;
	} rel;
	struct {
	    PICIpn_t idp;
	    char rectype;
	    char famtype;
	    byte *sortkey;
	    int iln;
	} fam;
    } contents;
}  IDGEntry_t, *IDGEntry_p;

typedef enum IDGRectype_t {
    IDG_RECTYPE_TITLE = '1',
    IDG_RECTYPE_AUTHORITY = '2'
} IDGRectype_t;

typedef struct idgcomp_t * IDGComp_p;
typedef struct IDGGen_t IDGGen_t, *IDGGen_p;
typedef struct idgnchconv_handle_t *IDGChconv_p;
typedef struct IDGSTHandle_t *IDGSTHandle_p;
typedef NMTHandle_p idg_functions_p;

/* Diagnostic functions. */
IDGMsg_p IDGMsgCreate(void);
void IDGMsgDestroy(IDGMsg_p msg);
const char *IDGMsgGet(IDGMsg_p msg);

/* Parse functions */
idg_ast_p IDGFcv2AstCreate(IDGMsg_p, IDGSTHandle_p, const char *, int, int,
			   const char *);
void IDGAstDestroy(idg_ast_p);
const char *IDGAstDump(idg_ast_p);
idg_ast_bytecode_p IDGAstByteCodeCreate(IDGMsg_p, idg_ast_p, bool, bool, bool,
					idg_functions_p);
idg_parsed_code_p IDGAstByteCodeTearOff(idg_ast_bytecode_p);
idg_parsed_code_p IDGAstByteCodeGet(idg_ast_bytecode_p);
void IDGAstByteCodeDestroy(idg_ast_bytecode_p);
bool IDGAstFlatten(idg_ast_p);
IDGSTHandle_p IDGAstCtxCreate(void);
void IDGAstCtxDestroy(IDGSTHandle_p);

int IDGCompOptimised(IDGComp_p hnd);
bool IDGCompCodeReplace(IDGComp_p dest, IDGComp_p src, const char *options);

bool IDGCompActivateProfiler(IDGComp_p hnd);

void
IDGRPPDestroy(
	      IDGRPPHandle_p hnd
	      );
PPTag_p
IDGRPPLevelScan(
	IDGRPPHandle_p	hnd,
	PICLevel_t	level,
	PICIpn_t	ipn,
	PICIdp_t	idp,
	PICIln_t	iln,
	PICCopyNum_t	copynum,
	PPTtl_p		ppttl
    );
    
bool
IDGRPPLevelScanRestore(
	PPTtl_p		ppttl);
    
bool
IDGRPPExpandBcl(
	PPTtl_p		ppttl,
	PPTtl_p		titleread(const char *ppn, void *userdata),
	void		*titlereaduserdata
    );
bool
IDGRPPExpandBCL(
	IDGRPPHandle_p	hnd,
	PPTtl_p		ppttl,
	PPTtl_p		titleread(const char *ppn, void *userdata),
	void		*titlereaduserdata);

bool
IDGRPPExpandBclRestore(
	PPTtl_p		ppttl
    );
    
PPTag_p
IDGRPPTagFind(
	      IDGRPPHandle_p hnd, 
	      PPTag_p tag,
	      int8 typ, 
	      int8 lev, 
	      char ind, 
	      int8 occ
	      );

PPTag_p 
IDGRPPTagNext(
	      IDGRPPHandle_p hnd,
	      PPTag_p tag,
	      int level
	      );

PPSbf_p
IDGRPPSbfFirst(
	       PPTag_p tag
	       );

PPSbf_p
IDGRPPSbfNext(
	      PPSbf_p sbf
	      );

PPSbf_p
IDGRPPSbfFind(
	      PPTag_p tag,
	      PPSbf_p sbf,
	      char type
	      );

void
IDGRPPLevelClear(
	    IDGRPPHandle_p	hnd,
	    int			level
    );
    
bool
IDGRPPLevelCleared(
	    IDGRPPHandle_p	hnd,
	    int			level
            );
    
IDGRPPHandle_p
IDGRPPInit(
	   void
	   );

void
IDGRPPShow(
	   IDGRPPHandle_p hnd
	   );

/* Compile the passed table into an idgcomp structure. */
IDGComp_p IDGCompileGen( const char* table, const char** error_msg, parse_error_p parse_result );

IDGComp_p IDGCompileGenOpt(const char* table, const char** error_msg, parse_error_p parse_result, const char *options);

/* Frees an compiled table. */
void IDGCompDestroy( IDGComp_p table );

void
IDGGenDestroy(
	      IDGGen_p hnd
	      );

typedef bool		(*IDGStoreIdxCB_f)(IDGEntry_p, byte *, size_t, void *);
#define IDG_NOFUNC	((IDGStoreIdxCB_f)0)

typedef bool		(*IDGStoreAdiCB_f)(const byte *, size_t, void *);
#define IDG_ADI_NOFUNC	((IDGStoreAdiCB_f)0)

typedef PICIdp_t (*EDPTranslate_f)(PICFileSet_t indexset, const char *edp, size_t edplen);

IDGGen_p
IDGGenInit(
	   IDGComp_p compiled_table,
	   bool use_iln,
	   IDGStoreIdxCB_f store_index,
	   void *userdata,
	   bool debug,
	   CSBuf_p (*sortstr_from_pptag)(CSBuf_p dest, const char *, PPTag_p,
					 bool),
	   PICIdp_t (*idp_from_edp)(PICFileSet_t, const char *, long)
	   );
IDGGen_p
IDGGenCreate(
	   IDGComp_p compiled_table,
	   bool use_iln,
	   IDGStoreIdxCB_f store_index,
	   IDGStoreAdiCB_f store_adi,
	   void *userdata,
	   bool debug,
	   CSBuf_p (*sortstr_from_pptag)(CSBuf_p dest, const char *, PPTag_p,
					 bool),
	   PICIdp_t (*idp_from_edp)(PICFileSet_t, const char *, long)
	   );

void
IDGGenModify(
	IDGGen_p	hnd,
	bool		use_iln,
	IDGStoreIdxCB_f	store_index,
	void		*userdata
    );
void
IDGGenModifyOpt(
	IDGGen_p	hnd,
	bool		use_iln,
	IDGStoreIdxCB_f	store_index,
	void		*userdata,
	const char	*options
    );
void
IDGGenModifyHandle(
	IDGGen_p	hnd,
	bool    	use_iln,
	IDGStoreIdxCB_f	store_index,
	IDGStoreAdiCB_f	store_adi,
	void		*userdata,
	const char	*options
    );
    
IDGComp_p
IDGGenGetComp(
	IDGGen_p	hnd
    );
    
void
IDGOptimiseInit2(
	bool	opt_flag,
	bool	opt_understandable);
    
void
IDGOptimiseInit(
		bool opt_flag
		);

bool
IDGGenerate(
	    IDGGen_p hnd,
	    IDGRPPHandle_p title,
	    PICFileSet_t index_fileset,
	    PICFileSet_t title_fileset
	    );

bool
IDGGenerateIndex(
		 IDGGen_p hnd,
		 IDGRPPHandle_p title
		 );

bool
IDGGenerateADI(
	IDGGen_p	hnd,
	IDGRPPHandle_p	title,
	IDXAdi_p	adi
    );

IDXMatAdi_t
IDGGenerateMatAdi(
	IDGGen_p	hnd,
	IDGRPPHandle_p	title);

const char *
IDGUtf8Cleanup(
	const char	*utf8str,
	size_t		*utf8len,
	size_t		utf8maxlen,
	IDGGen_p	gen,
	IDGEntryType_t	type);

size_t
IDGNormaliseTerm(
		 int                 iktnum,
		 IDGComp_p           hnd,
		 const char         *term,
		 char               *phrase,
		 size_t              phrasesiz,
		 const char         *special_in,
		 const char         *special_out
		 );
size_t
IDGNPhraseVariants(
	int		iktnum,
	IDGComp_p	hnd,
	const char	*term,
	CSBuf_p		phrases,
	const char	*options);

bool
IDGDeNormaliseFamSort(
	CSBuf_p			dest,
	const char		*famsort);

typedef enum IDGDeNormFlags_t {
    IDGND_NOFLAGS		= 0x00,	/* no flags */
    IDGND_PREFIX_KEYWORD_UNDO	= 0x01,	/* remove IDXGEN_PREFIX_KEYWORD_SEP? */
    IDGND_APPEND_SEPARATOR	= 0x02 	/* append separator behind keyword? */
} IDGDeNormFlags_t;
bool
IDGDeNormaliseTermIdxType(
	CSBuf_p			dest,
	IDXIdxType_t		idxtype,
	IDGComp_p		idgcomp,
	const char		*phrase,
	IDGDeNormFlags_t	flags);
bool
IDGDeNormaliseTermCS(
	CSBuf_p			dest,
        IDXIktNum_t		iktnum,
	IDGComp_p		idgcomp,
	const char		*phrase,
	IDGDeNormFlags_t	flags);
bool
IDGDeNormaliseTerm(
        IDXIktNum_t	iktnum,
	IDGComp_p	idgcomp,
	const char	*phrase,
	char		*term,
	size_t		termlen);

bool
IDGDenormaliseTerm(
		   const char *phrase,
		   char *term,
		   size_t termlen
		   );

bool
IDGDenormalisePrefixKeyword(
	IDGComp_p	idgcomp,
	IDXIktNum_t	iktnum,
	const char	*phrase,
	char		*term,
	size_t		termlen);
    
bool
IDGNormalise2(
	     IDGComp_p idgcomp,
	     int idxtype,
	     char *term,
	     int term_len,
	     IDGEntry_t *entry,
	     byte *pointer,
	     int pointer_len,
	     IDGStoreIdxCB_f store_index,
	     void *store_data
	     );

bool
IDGNormPerson2(
	      IDGComp_p idgcomp,
	      int idxtype,
	      char *forename,
	      int flen,
	      char *surname,
	      int slen,
	      bool rotate,
	      IDGEntry_t *entry,
	      byte *pointer,
	      int pointer_len,
	      IDGStoreIdxCB_f store_index,
	      void *store_data
	      );

void
IDGNKeywordResetAll(
		 IDGComp_p idgcomp
		 );

const char *
IDGFindIndexPhrase(
	const char	*term,
	const char	**termphrase,
	size_t		*termphrlen,
	bool		*is_stopword,
	IDXIdxType_t	idxtype,
	IDGComp_p	idgcomp);
const char *
IDGFindIndexPhraseVariants(
	CSBuf_p		phrases,
	const char	*term,
	const char	**termphrase,
	size_t		*termphrlen,
	bool		*is_stopword,
	IDXIdxType_t	idxtype,
	IDGComp_p	idgcomp);

const char *
IDGNormaliseCodePoint(
	const char	*term,
	IDXIdxType_t	idxtype,
	IDGComp_p	idgcomp);
    
void
IDGCompShow(
	    const char *options,
	    IDGComp_p hnd
	    );

typedef struct idgnscripts_handle_t	*IDGScriptsHandle_p;
IDGScriptsHandle_p
IDGScriptsHandleGet(
	IDGComp_p		idgcomp,
	IDXIktNum_t		ikt,
	const IDXIktInfo_t	**iktinfop);

char
IDGScriptSeparator(
	IDGComp_p		idgcomp,
	const IDXIktInfo_t	*iktinfo,
	IDGScriptsHandle_p	scripts,
	const char		*term);
    
bool
IDGCompImplicitPhraseSearch(
	IDGComp_p	idgcomp,
	IDXIktNum_t	ikt,
	const char	*left,
	const char      *right);
    
int
IDGCompPrefixKeywordAddedLength(
	IDGComp_p		idgcomp,
	IDXIktNum_t		ikt,
	const char		*index_phrase);
    
bool
IDGCompPrefixKeywordIkt(
	IDGComp_p		idgcomp,
	IDXIktNum_t		ikt);

bool
IDGCompIsLBS( IDGComp_p idgcomp );
    
bool
IDGScriptSensitiveIkt(
	IDGComp_p		idgcomp,
	IDXIktNum_t		ikt);
    
IDXIktNum_t *
IDGIktListCreate(
	const char	*options,
	IDGComp_p	idgcomp);

void
IDGIktListDestroy(
	IDXIktNum_t	*iktlist);

bool
IDGGetIktInfo(
	IDXIktNum_t	iktnum,
	IDGComp_p	idgcomp,
	IDXIktInfo_t	*iktinfo);

char*
IDGGetTriggerTags(
        IDGComp_p idgcomp,
	IDXIdxType_t idx_type,
	IDXKeyType_t key_type );
    
const char *
IDGGetCharset(
	IDGComp_p	idgcomp);

const char *
IDGImplicitPhraseSearch(
	const char	**script_name,
	bool		*prefix_keyword,
	char		*index_phrase,
	IDXIdxType_t	idxtype,
	IDGComp_p	idgcomp);

size_t
IDGSizeofAdi(
	IDGComp_p	idgcomp);

bool
IDGIgnoreRelationIndexType(
	IDGComp_p	idgcomp);

const char *
IDGGetLocale(
	IDGComp_p	idgcomp);

bool
IDGStopword(
	IDXIdxType_t	idxtype,
	IDGComp_p	idgcomp,
	const char	*phrase,
	bool		*is_stopword);

IDXIktNum_t
IDGNextIktInfo(
	IDXIktNum_t	iktnum,
	IDGComp_p	idgcomp,
	IDXIktInfo_t	*iktinfo);

const char *
IDGGetXpnTab(
	IDGComp_p	idgcomp);

bool
IDGAdiFilterReset(
	IDXAdiFilter_p	adifilter,
	IDGComp_p	idgcomp);

void
IDGAdiFilterDestroy(
	IDXAdiFilter_p	adifilter);

IDXAdiFilter_p
IDGAdiFilterCreate(
	IDGComp_p	idgcomp);

bool
IDGGetAdiField(
	int		seqnr,
	const IDXAdi_t	*adi,
	IDGComp_p	idgcomp,
	IDXAdiField_t	*adifield);

IDXIktNum_t
IDGAdiFieldToExtern(
	const char			*options,
	int				seqnr,
	const IDXAdi_t			*adi,
	IDXRecType_t			rectype,
        IDGComp_p			idgcomp,
	char				*ext_intern,
	size_t				ext_internsize,
	char				*ext_extern,
	size_t				ext_externsize,
	const struct IDXIktMnemos_t	*iktmnemos);
    
IDXIktNum_t 
IDGAdiFieldToIkt(
        int			seqnr,
	IDXRecType_t		rectype,
	IDGComp_p		idgcomp);
    
bool
IDGIktMnemosAddAdiCode(
        const char          *intrn,
        const char          *extrn,
        const char          *desc,
	IDXIktNum_t          iktnum,
        IDXIktMnemos_p       iktmnemos,
	IDGComp_p            idgcomp);

const char *IDGAdiToString(IDGComp_p idgtab, const IDXAdi_t *adi);
    
bool IDGSetFillAdi(const byte *adiptr, size_t adilen, void *userdata);
bool IDGSetFillIdx(IDGEntry_p entry, byte *pointer, size_t pointer_len, void *userdata);
bool IDGSetFill(IDGEntry_p entry, byte *pointer, size_t pointer_len,
		void *userdata);
bool IDGSetFillUnique(IDGEntry_p entry, byte *pointer, size_t pointer_len,
		      void *userdata);
void IDGSetFillStringMove(IDXIndexSet_t *indexSet, const char *realloced_str);

/*
 * userdata has type IDGSetFillData_t *
 */
typedef struct IDGSetFillData_t {
    IDXIndexSet_t	*indexSet;
    IDXAdi_t		*adi;
} IDGSetFillData_t;

typedef struct IDGOIndexset_t *IDGOIndexset_p;
IDGOIndexset_p IDGOnlineIndexsetCreate(void);
void IDGOnlineIndexsetDestroy(IDGOIndexset_p set);
void IDGOnlineIndexsetClear(IDGOIndexset_p set);
bool IDGOnlineIndexsetStore(PICFileSet_t fileset,
			    PICIpn_t ipn,
			    IDXRecType_t rectype,
			    const IDGOIndexset_p set,
			    const char *charset,
    			    const char *adidir);

bool IDGOnlineIndexsetPut(PICIpn_t ipn,
			  PICIdp_t idp,
			  PICFileSet_t indexSet,
			  IDGComp_p idgComp,
			  const IDXIndexSet_t *idx,
			  const IDXAdi_t *adi,
			  voidp idxset_data);
void
IDGOnlineIndexsetStoreAdiPut(
	IDGOIndexset_p	set,
	bool		store_adi);
    
bool
IDGOnlineUpdateFromSet(
    PICFileSet_t	index_fileset,
    PICIpn_t		ipn,
    PICIdp_t		idp,
    IDXRecType_t	rectype,
    IDGComp_p		idgcomp,
    const IDXIndexSet_t	*idx,
    const IDXAdi_t	*adi);

void
IDGOSetDirectory(const char *directory);
    
bool
IDGNChconv2(
	IDGChconv_p hnd,
	char *in,
	char *out);

IDGChconv_p
IDGNChconvInit2(
	void);

bool
IDGNChconvCompile2(
	IDGChconv_p hnd);

typedef struct IDGNDictHandle_tp *IDGNDictHandle_p;

IDGNDictHandle_p
IDGNDictCompile(
	const char	*basename,
	const char	*src_text,
	const char	*src_filename);

bool
IDGNDictFinalize(
		      IDGNDictHandle_p handle);


void
IDGNDictFree(
		  IDGNDictHandle_p handle);

bool IDGCompAddLev0Tags(PPTtl_p ppttl, IDGComp_p handle, const char *options);

bool
IDGNormalisePhrase(
	int                 iktnum,
	IDGComp_p           hnd,
	const char         *term,
	IDGStoreIdxCB_f     store_index,
	voidp		    callback_data
	);

/* General cleanup. Should be called when no more idgparse_ calls are made. */ 
void IDGParseCleanup(void);

bool IDGParseInit(void);

typedef struct IDGPhraseList_t		*IDGPhraseList_p;

IDGPhraseList_p
IDGPhraseListGet(
	const char		*envvar,
	const char		*options,
	CSBuf_p			cserr);

bool
IDGPhraseListContains(
	IDGPhraseList_p		phraselist,
	const char		*phrase,
	int			phrase_len,
	CSBuf_p			cserr);

bool IDGCompIndexedLevelsDefined(IDGComp_p handle);

PPTtl_p
IDGCompIndexedLevelsClone(
	IDGComp_p	handle,
	PPTtl_p		ppttl,
	const char	*options);

bool
IDGCompOrsCheck001X(
	IDGComp_p	handle);

PICFileSet_t
IDGCompOrsAdiFileset(
	IDGComp_p	handle);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif
