/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File        : idx.h
 *  Description : The interface for the IDX-library
 *  Author(s)   : Paul Smit
 *  Creation    : 22-10-1996
 *
 *  History:
 *  22-10-1996  : paul     : creation
 *  07-07-1997  : HSi      : Added some errorcodes and functions for reading
 *                           indexes.
 *  09-07-1997  : paul     : Added IDXIktToIndexType(), IDXIktListCreate(),
 *                           IDXIktListDestroy() and IDXIndexTypeToSRStruct()
 *  21-07-1997  : HSi      : Added IDXCompareIndex(), IDXStore
 *  01-08-1997  : paul     : Added IDXIndexIktDef_t and IDXDenormalizeTerm()
 *  20- 8-1997  : HSi      : Added IDXE_NOSUPPORT.
 *  25- 8-2997  : HSi      : Added IDXRelationRead(), IDXRelationNext().
 *   8- 9-2997  : HSi      : Added IDXFamilyRead(), IDXFamilyNext().
 *  08-09-1997  : paul     : srstruct new member in IDXIndexIktDef_t;
 *                           IDXGetIndexIktDef() changed into IDXGetIktInfo()
 *                           IDXSizeofAdi() added
 *                           IDXGetAdiField() and IDXAdiField_t added
 *                           IDXIndexIktDef_t renamed to IDXIndexIktInfo_t
 *                           IDXStopword() added;  special_in,special_out
 *                           extra parameters in IDXDenormalizeTerm()
 *   9- 9-1997  : HSi      : Added parameter entryChanged to IDX*Next routines.
 *  10-10-1997  : paul     : IDXFindTermPhrase() added
 *  14-10-1997  : paul     : pointers in IDXAdiField_t modified to const *
 *  29-10-1997  : paul     : IDXAdiFilter_p, IDXAdiFilterToIntern(),
 *                           IDXAdiFilterToExtern() and IDXFilterAdi() added
 *                           IDXAdiCode_t, IDXIktMnemo_t, IDXIktMnemos_p,
 *                           IDXIktToMnemo(), IDXMnemoToIkt() added
 *  31-10-1997  : paul     : IDXE_IKT changed to IDXE_PARAMETER;
 *                           IDXAdiFilterAllowed() and *Old() added
 *  25-11-1997  : paul     : IDXIktMnemosReset(), IDXAdiFilterFldOffset()
 *                           and IDXScanIktNum() added
 *  05-12-1997  : paul     : IDXAdiFilterCopy() added
 *  09-12-1997  : paul     : extra parameter maxfilter in IDXAdiFilterToIntern
 *  23-12-1997  : paul     : IDXAdiFilterToExtern(): options parameter added
 *  09-01-1998  : paul     : IDXADIFILTERFLDOFFSET(): extra parameter
 *                           deffilter, and adifilter as output parameter
 *  18-02-1998  : paul     : IDXDEF_SRSTRUCT_*NUMBERED_KEYWORD introduced;
 *                           IDXShortcut() removed;
 *                           IDX_DEVIATION_ABSTRACT_OLC_LEVEL introduced;
 *                           IDXStore()+IDXUpdate(): idxdef parameter removed
 *                           entrySearched introduced in IDX*Read()
 *  09-03-1998  : paul     : IDXFindTermPhrase() renamed: IDXFindIndexPhrase()
 *  09-04-1998  : paul     : IDXUnpackAdiField() added
 *  22-06-1998  : BdB      : Get rid of useless options in IDX*Next()
 *                           Add len field to IDXAdiIktInfo_t
 *                           IDXFilterAscii() added
 *  29-06-1998  : fb       : pathway.h instead of csrc.h
 *                           interface of IDX*Old functions changed.
 *  25-03-1999  : paul     : IDXAdiField_t: signed_interval added
 *                           IDXDEF_ADIACCESS_SIGNED_INTERVAL +
 *                           IDXDEF_ADITYPE_SIGNED_INTERVAL added
 *                           IDX_ADI_INTERVAL_SIGN added
 *  04-05-1999  : BdB      : Add level argument to IDXGenerateIndexes
 *  02-06-1999  : BdB   : Add ADI access type IDXDEF_ADIACCESS_BCLCODE
 *  16-06-1999  : BdB   : Add IDXReadChecktime() routine
 *  17-08-1999  : BdB   : Add wordlist functions and (UN)PACK macros
 *  23-09-1999  : BdB   : Add IDXGenerateADIJVU
 *  17-02-2000  : BdB   : Add new IDX read interface with handles
 *  23-05-2000  : BdB   : Change IDXGenerate* interface
 *  07-06-2000  : BdB   : Add IDXHideIfLinkNotIndexed
 *  20-12-2000  : paul  : Add IDXDEF_SRSTRUCT_DATE
 *  31-07-2001  : paul  : Added IDXGenAdiPeriodPublication
 *  09-11-2001  : paul  : Added IDXIgnoreRelationIndexType()
 *  18-12-2001  : paul  : Added IDXGenerateMatAdiList()
 *  21-03-2002  : paul  : Added IDXLinkableSubfield()
 *  24-05-2002  : Guy   : Added IDXGetXpnTab
 *  29-05-2002  : paul  : Add rectype field to IDXAdiIktInfo_t
 *  05-06-2002  : paul  : added IDXDenormalizedTerm()
 *  06-06-2002  : paul  : added IDXAdiFieldToExtern()
 *  06-09-2002  : paul  : renamed IDAXAdiFieldToExtern() to IDXAdiFieldToExtern
 *  23-10-2002  : Guy   : added headers for idxio.c and idxhuffman.c
 *  09-12-2002  : paul  : added IDXNextIktInfo()
 *  23-12-2002  : paul  : Added IDXAdiFilterToInternNew()
 *  31-03-2003  : HvdW  : Tandem ifdefs removed
 *  09-04-2003	: jaap  : Remove obsolete function IDXStore
 *  22-10-2003  : Guy   : moved IDX_PACK* and IDX_UNPACK* to PM library
 *  23-02-2004  : paul  : Added IDXAdiFilterIntersect()
 *                      : IDXGenerateMatAdiList() returns ADI field seqnr,
 *                        instead success indication
 *                      : Added IDXAdiFieldToIkt()
 *  10-07-2006  : paul  : Added IDXGEN_PERSON_UTF8_SEP
 *  20-09-2006  : paul  : Added IDXGEN_PREFIX_KEYWORD_SEP
 *  04-05-2007  : paul  : Added IDXAdiFilterFldDisagreement()
 *  18-01-2008	: daalmantz : Added IDXKey* functions and IDXE_DUPLICATEKEY
 *                        and IDXE_UNICODE
 *  13-02-2008	: daalmantz : Added HMSKEY_NOFUNC = NULL value for 
 *                            IDGKeyTranslate_f pointer translation callback
 *                            (see lib/include/idg.h)
 *  21-02-2008	: daalmanf : Removed edp (former idxkey) stuff to better place
 *  04-12-2009  : daalmanf : implement IDXDEF_ADITYPE_LONG_NUMBER = 'L'
 *                         : (Jaap Lutz, GBV, for CollectionWS/PerLis)
 *  08-02-2012	: martinz  : Added IDXLinkableSubfield().
 *  13-02-2012	: martinz  : Added IDXGenerateJVUADI().
 *  02-04-2013	: martinz  : Added IDXIndexSetCreate(), IDXIndexSetDestroy()
 *  			     and IDXIndexSetReset().
 *  10-05-2013	: martinz  : Use access functions for phrase & sort entries.
 *  12-11-2018	: zwaalm   : CBS-8478 Defined ADI-fields with their sizes.
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(IDX_H_INCLUDED)
#define IDX_H_INCLUDED

#pragma ident "$Id: idx.h 368339 2019-07-04 14:42:35Z heesakkm $"

#include <stddef.h>

#include <pica.h>
#include <picamisc.h>
#include <picatypes.h>
#include <pp.h>
#include <cb.h>
#include <idxtypes.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif
  
ERROR_TYPE(IDXErrno_t);

ERROR_DECLARE(IDXE_NOERROR);
ERROR_DECLARE(IDXE_INTERNAL);
ERROR_DECLARE(IDXE_SYNTAX);
ERROR_DECLARE(IDXE_NOMEM);
ERROR_DECLARE(IDXE_TITLE);
ERROR_DECLARE(IDXE_OVERFLOW);
ERROR_DECLARE(IDXE_PARAMETER);
ERROR_DECLARE(IDXE_IO);
ERROR_DECLARE(IDXE_EOF);
ERROR_DECLARE(IDXE_KEYNOTFOUND);
ERROR_DECLARE(IDXE_NOSUPPORT);
ERROR_DECLARE(IDXE_DUPLICATEKEY);
ERROR_DECLARE(IDXE_UNICODE);
ERROR_DECLARE(IDXE_CALLBACK);

extern IDXErrno_t        IDXerrno;
/*
 *  Interval ADI can be signed. The internal representation of the minus
 *  sign is fixed; the external value may be defined in the ADITXT table.
 */
#define IDX_ADI_INTERVAL_SIGN   'c'

/*
 *  Generating indexes on CBS is slightly different from generating them on
 *  LBS: the pointer does not contain level information
 */
typedef enum IDXPointerType_t {
    IDX_POINTERTYPE_COMPLETE,
    IDX_POINTERTYPE_NO_LEVEL
} IDXPointerType_t;
/*
 *  In case of the ABSTRACT_OLC deviation, the pointer generated on an abstract
 *  data record contains the IPN of the higher level OLC record. This is
 *  indicated by the level part of the pointer:
 */
#define IDX_DEVIATION_ABSTRACT_OLC_LEVEL    ((PICLevel_t) 3)
/*
 *  IDXDiagnostic is a buffer that will contain a diagnostic error message,
 *  in case IDXGenCompile() fails to compile the source table.
 */
#define IDX_DIAG_SIZE    3000
extern char              IDXDiagnostic[IDX_DIAG_SIZE];
/*
 * SRStruct: defining the structure of a search term
 */
typedef enum IDXDefSRStruct_t {
    IDXDEF_SRSTRUCT_NONE                = ' ',
    IDXDEF_SRSTRUCT_AUTHOR_TITLE        = 'a',
    IDXDEF_SRSTRUCT_COMPOSER_TITLE      = 'c',
    IDXDEF_SRSTRUCT_PHRASE              = 'p',
    IDXDEF_SRSTRUCT_PERSON              = 'P',
    IDXDEF_SRSTRUCT_TITLE_ACRONYM       = 't',
    IDXDEF_SRSTRUCT_UNNUMBERED_KEYWORD  = 'U',
    IDXDEF_SRSTRUCT_TAGNUMBERED_KEYWORD = 'T',
    IDXDEF_SRSTRUCT_RECNUMBERED_KEYWORD = 'R',
    IDXDEF_SRSTRUCT_SWD                 = 's',
    IDXDEF_SRSTRUCT_DATE                = 'd'
} IDXDefSRStruct_t;

/*
 * Sizes and max. values of ADI fields of different types.
 */
#define IDXDEF_ADISIZE_ASCII           (sizeof(char))
#define IDXDEF_ADISIZE_NUMBER          (sizeof(uint16))
#define IDXDEF_ADISIZE_SHORT_NUMBER    (sizeof(byte))
#define IDXDEF_ADISIZE_LONG_NUMBER     (sizeof(uint32))
#define IDXDEF_ADISIZE_INTERVAL 	(2 * IDXDEF_ADISIZE_NUMBER)
#define IDXDEF_ADISIZE_SHORT_INTERVAL 	(2 * IDXDEF_ADISIZE_SHORT_NUMBER)
#define IDXDEF_ADISIZE_BCLCODE         (sizeof(uint16))
#define IDXDEF_ADI_ASCII_MAX           IDX_UCHAR_MAX
#define IDXDEF_ADI_SHORT_NUMBER_MAX    IDX_UCHAR_MAX

/*
 * Table containing ADI-types and possible sizes.
 * Size 0 means: any size.
 */
#define ADITYPE_TABLE 							\
X(ADITYPE_NONE,			' ',	0)				\
X(ADITYPE_ASCII,          	'A',	0)				\
X(ADITYPE_BITMAP,         	'B',	0)				\
X(ADITYPE_INTERVAL,       	'I',	IDXDEF_ADISIZE_INTERVAL)	\
X(ADITYPE_SIGNED_INTERVAL,	'P',	IDXDEF_ADISIZE_INTERVAL)	\
X(ADITYPE_NUMBER,         	'N',	IDXDEF_ADISIZE_NUMBER)		\
X(ADITYPE_SHORT_NUMBER,   	'n',	IDXDEF_ADISIZE_SHORT_NUMBER)	\
X(ADITYPE_LONG_NUMBER,    	'L',	IDXDEF_ADISIZE_LONG_NUMBER)	\
X(ADITYPE_SHORT_INTERVAL,	'i',	IDXDEF_ADISIZE_SHORT_INTERVAL)	\

/*
 * The adi-type is the type of the information contained in an adi field.
 * The adi-access identifies the way in which an adi command is linked to
 * the adi-type of the corresponding adi-field.
 */
#define X(type, value, size) IDXDEF_ ## type = value,
typedef enum IDXDefAdiType_t {
    ADITYPE_TABLE
    ADITYPE_COUNT = 256
} IDXDefAdiType_t;
#undef X

typedef enum IDXDefAdiAccess_t {
    IDXDEF_ADIACCESS_NONE           = ' ',
    IDXDEF_ADIACCESS_ASCII          = 'A',
    IDXDEF_ADIACCESS_BITMAP         = 'B',
    IDXDEF_ADIACCESS_INTERVAL       = 'I',
    IDXDEF_ADIACCESS_SIGNED_INTERVAL= 'P',
    IDXDEF_ADIACCESS_STOPWORDS      = 'S',
    IDXDEF_ADIACCESS_NUMBER         = 'N',
    IDXDEF_ADIACCESS_UPSHIFT        = 'U',
    IDXDEF_ADIACCESS_BCLCODE        = 'C',
    IDXDEF_ADIACCESS_FLAG           = 'F',
    IDXDEF_ADIACCESS_DAY            = 'D',
    IDXDEF_ADIACCESS_BITLIST        = 'L'    
} IDXDefAdiAccess_t;
/*
 *  The (external) definition of an index ikt...
 */
#define IDX_IMPLICIT_FAM_SIZE    9
typedef struct IDXIndexIktInfo_t {
    IDXDefSRStruct_t     srstruct;
    IDXIdxType_t         idxtype;
    IDXKeyType_t         ttltype;
    PICIln_t             ttlowner;
    IDXKeyType_t         auttype;
    PICIln_t             autowner;
    IDXKeyType_t         reltype;
    struct IDXFamRelay_t {
        IDXRecType_t         rectype;
        IDXFamType_t         famtype;
    }                    implicit_fam[IDX_IMPLICIT_FAM_SIZE];
} IDXIndexIktInfo_t;
typedef struct IDXAdiIktInfo_t {
    int                  seqnr;
    IDXDefAdiType_t      type;
    IDXDefAdiAccess_t    access;
    int                  len;
    IDXRecType_t         rectype;
    int                  offset;    /* IDG library only */
} IDXAdiIktInfo_t;
typedef enum IDXDefIktType_t {
    IDXDEF_IKTTYPE_ANY,
    IDXDEF_IKTTYPE_INDEX,
    IDXDEF_IKTTYPE_ADI
} IDXDefIktType_t;
typedef struct IDXIktInfo_t {
    IDXIktNum_t          num;       /* IDG library only */
    IDXDefIktType_t      type;      /* must NOT contain IDXDEF_IKTTYPE_ANY */
    union {
        IDXIndexIktInfo_t     idx;  /* IDXDEF_IKTTYPE_INDEX */
        IDXAdiIktInfo_t       adi;  /* IDXDEF_IKTTYPE_ADI */
    }                    val;
} IDXIktInfo_t;
/*
 * Typedef of the structure that holds the information of ADI fields.
 * Parts of this struct can point to information contained in the actual
 * ADI part of the index, which is in struct IDXAdi_t.
 */
typedef struct IDXAdiField_t {
    IDXDefAdiType_t      type;
    union {
        struct {
            const char    *buf;
            size_t         len;
        }              ascii;
        struct {
            const byte    *buf;
            size_t         len;
        }              bitmap;
        struct {
            int16          lo, hi;
        }              signed_interval;
        struct {
            uint16         lo, hi;
        }              interval;
        uint16         number;
        byte           short_number;
	uint32         long_number;
        struct {
            byte           lo, hi;
        }              short_interval;
    }                    val;
} IDXAdiField_t;
#define IDX_ADIFLD_MAX       20
typedef struct IDXAdiFields_t {
    size_t               len;
    IDXAdiField_t        fld[IDX_ADIFLD_MAX];
} IDXAdiFields_t;

typedef struct IDXAdiFilter_t *IDXAdiFilter_p;
/*
 * Typedef of the list of ikt/mnemonic combinations and of a single
 * element in this list and of a code of an ADI ikt.
 */
typedef struct IDXIktMnemos_t *IDXIktMnemos_p;
typedef struct IDXAdiCode_t {
    const char                 *extrn;
    const char                 *desc;
    const struct IDXAdiCode_t  *next;
    byte                        intrn;
} IDXAdiCode_t;
typedef struct IDXIktMnemo_t {
    const char                 *mnemo;
    const char                 *desc;
    const IDXAdiCode_t         *adicode;
    IDXIktNum_t                 iktnum;
} IDXIktMnemo_t;

/*
 * A person index is separated into two parts: the surname- and the forename
 * part. The separating character is contained both in the search term and in
 * the index phrase. For correct Unicode sortkeys we need a different
 * character: the form feed seems to be working correctly.
 */
#define IDX_PERSON_TERM_SEP              ','
#define IDXGEN_PERSON_INDEX_SEP             '\037'
#define IDXGEN_PERSON_UTF8_SEP              '\f'
/*
 * A prefix keyword would have been a stopword if there hadn't been a trick to
 * generate an index on it. The trick is that the first grapheme of the next
 * keyword is appended to the index phrase. If a separator has been defined, a
 * separator should be added between them.
 */
#define IDXGEN_PREFIX_KEYWORD_SEP           '\037'
/*
 * A composer-title index is separated into two parts: the composer- and the
 * title part. The separating string is contained both in the search term and
 * in the index phrase.
 */
#define IDX_COMPOSER_TITLE_SEP           " : "
#define IDX_COMPOSER_TITLE_SEP_LEN       3
/*
 * An SWD index is separated into several parts. The separating string is
 * contained both in the search term and in the index phrase.
 */
#define IDX_SWD_SEP                      " / "
#define IDX_SWD_SEP_LEN                  3

/*---------------------------------------------------------------------------
 *   Declaration for IDXIOStat_t
 *---------------------------------------------------------------------------
 */
typedef struct IDXIOStat_t {
    char   *filename;
    int     num;
    uint32 *table;
} IDXIOStat_t, *IDXIOStat_p;

    /* 
     * the IDXIO functions can return this pointer to signify that 
     * the last hit of a given entry has been read from the index file
     */
#define IDXIO_END_OF_ENTRY ((byte *) -1)

void
IDXOSetDirectory(const char *directory);

enum {
    IDXOF_COMMON_ILN = 1
};

bool IDXGenerateJVUADI(PPTtl_p ttl, uint16 *jvubeg, uint16 *jvuend);

void IDXGenAdiPeriodPublication(
    PPTag_p            tag,
    const char        *sbfset,
    uint16            *jvubeg,
    uint16            *jvuend);

/* Generate the day ADI value (days from 1-1-1920). */
uint16 IDXGenerateDayADI(int year, int month, int day, bool end);

#define IDX_DEFAULT_LOCALE "en__US"
/*
 * Ikt-mnemonic functions...
 */
bool IDXIktMnemosReset(const CBBuf_t            *cbuf,
                       size_t                    cbsize,
                       IDXIktMnemos_p            iktmnemos);
IDXIktMnemos_p IDXIktMnemosCreate(const CBBuf_t *cbuf,
                                  size_t         cbsize);
void IDXIktMnemosDestroy(IDXIktMnemos_p          iktmnemos);
bool IDXAddIktMnemo(IDXIktNum_t	iktnum,
        	const char	*mnemo,
        	const char	*desc,
        	IDXIktMnemos_p	iktmnemos);
const IDXIktMnemo_t *IDXIktToMnemo(IDXIktNum_t                  iktnum,
                                   const struct IDXIktMnemos_t *iktmnemos);
const IDXIktMnemo_t *IDXMnemoToIkt(const char                  *mnemo,
                                   const struct IDXIktMnemos_t *iktmnemos);
IDXIktNum_t IDXScanIktNum(const char                  *iktstr,
                          const struct IDXIktMnemos_t *iktmnemos,
                          const IDXIktMnemo_t        **iktmnemo,
                          size_t                      *scanlen);
const IDXAdiCode_t *IDXAdiCodeToExtern(byte                   intrn,
                                       const IDXAdiCode_t    *adicode);
const IDXAdiCode_t *IDXAdiCodeToIntern(const char            *extrn,
                                       const IDXAdiCode_t    *adicode);
bool IDXSortAdiCodes(struct IDXIktMnemos_t	*iktmnemos,
		     IDXIktNum_t		iktnum);

/*
 * ADI-filter functions...
 */
void IDXAdiFilterDestroy(IDXAdiFilter_p adifilter);
bool IDXAdiFilterCopy(const struct IDXAdiFilter_t *srcfilter,
                      IDXAdiFilter_p               dstfilter);
bool
IDXAdiFilterToInternNew(
    const char                  *options,
    const char                  *extadifilter,
    IDXAdiFilter_p               adifilter,
    const struct IDXAdiFilter_t *maxfilter,
    const struct IDXIktMnemos_t *iktmnemos);
    
bool IDXAdiFilterToIntern(const char                  *extadifilter,
                          IDXAdiFilter_p               adifilter,
                          const struct IDXAdiFilter_t *maxfilter,
                          const struct IDXIktMnemos_t *iktmnemos);
bool IDXAdiFilterToExtern(const char                   *options,
                          const struct IDXAdiFilter_t  *adifilter,
                          const struct IDXAdiFilter_t  *defaultadi,
                          char                         *extadifilter,
                          size_t                        extadifiltersize,
                          const struct IDXIktMnemos_t  *iktmnemos);
IDXIktNum_t IDXAdiFilterFldDisagreement(
				const struct IDXAdiFilter_t*adifilter1,
				const struct IDXAdiFilter_t *adifilter2);
int IDXAdiFilterFldOffset(      struct IDXAdiFilter_t *adifilter,
                          const struct IDXAdiFilter_t *deffilter);
bool IDXFilterAdi(const byte                  *adibuf,
                  IDXRecType_t                 rectype,
                  const struct IDXAdiFilter_t *adifilter);
bool IDXFilterAscii(const struct IDXAdiFilter_t *adifilter,
                    IDXIktNum_t ikt,
                    const char *ascii);
IDXRecType_t IDXRecTypeFilter(const struct IDXAdiFilter_t *adifilter);
bool IDXAdiFilterAllowed(const struct IDXAdiFilter_t *actual,
                         const struct IDXAdiFilter_t *allowed);
IDXAdiFilter_p
IDXAdiFilterIntersect(
	const struct IDXAdiFilter_t	*filt0,
	const struct IDXAdiFilter_t	*filt1);

IDXRecType_t
IDXAdiFilterRectypeMaskGet(
	const struct IDXAdiFilter_t	*adifilter);
void
IDXAdiFilterRectypeMaskSet(
	IDXAdiFilter_p	adifilter,
	IDXRecType_t	rectypemask);

/* Functions for reading indexes
 */
bool
IDXPhraseRead (
    const char                  *options,
    const IDXPhraseEntry_t      *entrySearched,
    IDXPhraseEntry_t            *entryFound,
    IDXPhrasePointer_t          *pointerFound,
    IDXAdi_p			*adiFound
    );

bool
IDXPhraseNext (
    IDXPhraseEntry_t            *entryFound,
    IDXPhrasePointer_t          *pointerFound,
    IDXAdi_p			*adiFound,
    bool                        *entryChanged
    );

void
IDXPhraseReset(void);

bool
IDXRelationRead (
    const char                  *options,
    const IDXRelationEntry_t    *entrySearched,
    IDXRelationEntry_t          *entryFound,
    IDXRelationPointer_t        *pointerFound,
    IDXAdi_p			*adiFound
    );

bool
IDXRelationNext (
    IDXRelationEntry_t          *entryFound,
    IDXRelationPointer_t        *pointerFound,
    IDXAdi_p			*adiFound,
    bool                        *entryChanged
    );

void
IDXRelationReset(void);

bool
IDXFamilyRead (
    const char                  *options,
    const IDXFamilyEntry_t      *entrySearched,
    IDXFamilyEntry_t            *entryFound,
    IDXFamilyPointer_t          *pointerFound,
    IDXAdi_p			*adiFound
    );

bool
IDXFamilyNext (
    IDXFamilyEntry_t            *entryFound,
    IDXFamilyPointer_t          *pointerFound,
    IDXAdi_p			*adiFound,
    bool                        *entryChanged
    );

void
IDXFamilyReset(void);

/* Make a following IDX*Read() routine check the modification time of the index
   file and reopen it if necessary.  Obsolete, will be removed as soon as I
   (BdB) check in the new idx read code. */
bool
IDXReadChecktime(void);

int
IDXCompareIndex (
    const IDXIndex_t            *a,
    const IDXIndex_t            *b
    );

/* On Solaris 2.6, we can use 64 bit files by defining _LARGEFILE_SOURCE and
   setting _FILE_OFFSET_BITS to 64.  A problem is that fseek/ftell use longs,
   which are 32 bits, so the UNIX 98 spec defines new functions fseeko/ftello
   that use 64-bit off_t. */
#ifdef _LARGEFILE_SOURCE
/* Make sure we know of off_t. */
#include <unistd.h>
typedef off_t IDX_offset_tp;    /* To be used in IDX_FSEEK/FTELL */
#define IDX_OFFSET_PRINTF_FORMAT "%lld" /* To printf IDX_offset_tp */
#define IDX_FSEEK fseeko
#define IDX_FTELL ftello
#define IDX_LARGEFILES 1
#else /* _LARGEFILE_SOURCE */
/* Attempt to differentiate between 32 and 64 (actually >= 48) bit machines.
   If this is compiled on a machine with 40 bit longs, it could fail.  But we
   cannot increase the constant in the comparison, because it may not be
   understood on 32 bit machines.  (In the new C0X standard, these preprocessor
   calculations must be done in at least 64 bits.) */
#if ULONG_MAX > 4294967295UL
#define IDX_LARGEFILES 1
#else /* ULONG_MAX > 4294967295UL */
#define IDX_LARGEFILES 0
#endif /* ULONG_MAX > 4294967295UL */
typedef long int IDX_offset_tp; /* To be used in IDX_FSEEK/FTELL */
#define IDX_OFFSET_PRINTF_FORMAT "%ld" /* To printf IDX_offset_tp */
#define IDX_FSEEK fseek
#define IDX_FTELL ftell
#endif /* _LARGEFILE_SOURCE */

/* Various utility macros to (un)pack data stored in index files in bigendian
   format. Moved to PM library (Guy 22-10-2003) */

#define IDX_PACK2(p,x) PM_PACK2(p,x)
#define IDX_UNPACK2(p) PM_UNPACK2(p)

#define IDX_PACK4(p,x) PM_PACK4(p,x)
#define IDX_UNPACK4(p) PM_UNPACK4(p)

#define IDX_PACK6(p,x) PM_PACK6(p,x)
#define IDX_UNPACK6(p) PM_UNPACK6(p)

/* New IDX read interface allowing multiple simultaneous reads using
   handles. */
void
IDXPhraseDestroy(IDXPhraseHandle_p handle);
IDXPhraseHandle_p
IDXPhraseCreate(void);
void
IDXPhraseFinalize(IDXPhraseHandle_p handle);
bool
IDXPhrasePosition(
    IDXPhraseHandle_p handle,
    const char *options,
    const IDXPhraseEntry_t *entry);
bool
IDXPhraseGet(
    IDXPhraseHandle_p handle,
    IDXPhraseEntry_t *entry,
    IDXPhrasePointer_t *pointer,
    bool *entryChanged);

void
IDXRelationDestroy(IDXRelationHandle_p handle);
IDXRelationHandle_p
IDXRelationCreate(void);
void
IDXRelationFinalize(IDXRelationHandle_p handle);
bool
IDXRelationPosition(
    IDXRelationHandle_p handle,
    const char *options,
    const IDXRelationEntry_t *entry);
bool
IDXRelationGet(
    IDXRelationHandle_p handle,
    IDXRelationEntry_t *entry,
    IDXRelationPointer_t *pointer,
    bool *entryChanged);

void
IDXFamilyDestroy(IDXFamilyHandle_p handle);
IDXFamilyHandle_p
IDXFamilyCreate(void);
void
IDXFamilyFinalize(IDXFamilyHandle_p handle);
bool
IDXFamilyPosition(
    IDXFamilyHandle_p handle,
    const char *options,
    const IDXFamilyEntry_t *entry);
bool
IDXFamilyGet(
    IDXFamilyHandle_p handle,
    IDXFamilyEntry_t *entry,
    IDXFamilyPointer_t *pointer,
    bool *entryChanged);

void
IDXFilesetClose(
	PICFileSet_t	fileset);

const char *
IDXReadDirectory(void);


bool
IDXIOReadClose(
	       IDXIORead_p hnd   
	       );

IDXIORead_p
IDXIOReadOpen(const char *path,          
	      IDXIndexType_t typ,  
	      int indexset,        
	      int length,          
	      int overflow,         
	      bool use_aio
	      );

byte *
IDXIOReadEntry(
	       IDXIORead_p  hnd,   /* IN : handle to idx read control block */
	       int         *len    /* OUT: length of next entry             */
	       );

byte *
IDXIOReadHit(
	     IDXIORead_p  hnd,   /* IN : handle to idx read control block */
	     int         *len    /* OUT: length of next entry             */
	     );

bool
IDXIOWriteClose(
		IDXIOWrite_p hnd
		);

IDXIOWrite_p 
IDXIOWriteOpen(
	       const char *path,    /* IN : directory in which file resides */
	       IDXIndexType_t typ,  /* IN : type of index                   */
	       int indexset,        /* IN : indexset of index               */
	       int length,          /* IN : length of read buffer           */
	       int overflow,        /* IN : amount of buffer overflow       */
	       bool use_aio         /* IN : TRUE when AIO is to be used     */
	       );

bool
IDXIOWriteEntry(
		IDXIOWrite_p hnd,
		byte *buf,
		int len
		);

bool
IDXIOWriteHit(
	      IDXIOWrite_p hnd,
	      byte *buf,
	      int len
	      );

typedef struct IDXAdiHandle_tp *IDXAdiHandle_p;

bool
IDXAdiClose(
	IDXAdiHandle_p	handle);

IDXAdiHandle_p
IDXAdiCreate(
	const char	*options,
	PICFileSet_t	indexset,
	const char	*path,
	int		adilen);

IDXAdiHandle_p
IDXAdiOpen(
	const char	*options,
	PICFileSet_t	indexset,
	const char	*path);

bool
IDXAdiGet(
	IDXAdiHandle_p	handle,
	PICIdp_t	idp,
	IDXRecType_t	*rectype,
	byte		*buf);

bool
IDXAdiSet(
	IDXAdiHandle_p	handle,
	PICIdp_t	idp,
	IDXRecType_t	rectype,
	byte		*buf);

bool
IDXAdiTraverse(
	IDXAdiHandle_p	handle,
	bool		(*callback)(PICIdp_t, IDXRecType_t, IDXAdi_t *, void *),
	void		*userdata);

int
IDXAdiLength(
	IDXAdiHandle_p	handle);

PICIdp_t
IDXAdiMaxIDP(
        IDXAdiHandle_p  handle);

IDXAdi_p
IDXPhraseAdiGet(
	IDXPhraseHandle_p	handle,
	PICIdp_t		idp,
	IDXRecType_t		*rectype);

IDXAdi_p
IDXRelationAdiGet(
	IDXRelationHandle_p	handle,
	PICIdp_t		idp,
	IDXRecType_t		*rectype);

IDXAdi_p
IDXFamilyAdiGet(
	IDXFamilyHandle_p	handle,
	PICIdp_t		idp,
	IDXRecType_t		*rectype);

bool
IDXLinkableSubfield(
    IDXRecType_t            rectype,
    PICLevel_t              lev,
    PICTagType_t            typ,
    PICIndicator_t          ind,
    PICOccurrence_t         occ,
    PICSubfType_t           sbftyp);

IDXIndexSet_p
IDXIndexSetCreate(
	size_t idxsiz,
	size_t strsiz);

void
IDXIndexSetDestroy(
	IDXIndexSet_p indexset);

void
IDXIndexSetReset(IDXIndexSet_p indexset);

const char *
IDXPEGetPhrase(IDXPhraseEntry_p entry);

const char *
IDXPEPutPhrase(IDXPhraseEntry_p	entry, const char *phrase);

const char *
IDXFEGetSort(IDXFamilyEntry_p entry);

const char *
IDXFEPutSort(IDXFamilyEntry_p	entry, const char *sort);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(IDX_H_INCLUDED) */
