/*
 *  Copyright (c) 2000 by Pica
 *
 *  File	: idxutree.h
 *  Description	: 
 *  Author(s)	: Bas de Bakker
 *  Creation	: 04-02-2000
 *
 *  History:
 *  04-02-2000	: BdB	: creation
 *  24-01-2002  : Guy   : added utf8 stuff in IDXTreeInsert
 *  10-07-2006	: paul  : Allow family sortstrings to contain Unicode
 *
 *  Notes	:
 *
 */

#ifndef IDXUTREE_H_INCLUDED
#define IDXUTREE_H_INCLUDED

#pragma ident "$Id: idxutree.h 301473 2013-07-18 12:09:20Z sieburgw $"

typedef struct IDXUFile_t *IDXUFile_p;

void
IDXUFileClose(IDXUFile_p file);

IDXUFile_p
IDXUFileOpen(const char *filename, int flags);

bool
IDXUIsOutOfDate(IDXUFile_p file, bool writer);

void
IDXUMakeOutOfDate(IDXUFile_p file, bool writer);

void
IDXUClearOutOfDate(IDXUFile_p hash, bool writer);

bool
IDXUMapToSize(IDXUFile_p file);

size_t
IDXUFileSize(IDXUFile_p file);

bool
IDXTreeCreate(const char *filename);

size_t
IDXTreeFind(IDXUFile_p tree, byte *key, int keylen, bool backward);

size_t
IDXTreeFindN(IDXUFile_p tree, byte *key, int keylen, bool backward,
	     int(*compar)(const byte *entry,
			  const byte *cmstr,
			  size_t len,
			  voidp userdata),
	     voidp userdata);

size_t
IDXTreeNext(IDXUFile_p tree, size_t n, bool backward);

const byte *
IDXTreeData(IDXUFile_p tree, size_t n);

bool
IDXTreeInsert(IDXUFile_p tree, byte *data, int keylen, int datalen);

bool
IDXTreeInsertN(IDXUFile_p tree, 
	       byte *data, 
	       int keylen, 
	       int datalen,
	       int        (*cmp_f) (const void *,
				    const void *,
				    size_t));

bool
IDXHashCreate(const char *filename, unsigned int nbuckets);

unsigned int *
IDXHashInsert(IDXUFile_p hash, PICIpn_t ipn);

unsigned int
IDXHashFind(IDXUFile_p hash, PICIpn_t ipn);

bool
IDXUFileLock(IDXUFile_p hash, bool set);

bool
IDXUFileProtect(IDXUFile_p file, bool set);

#endif /* IDXUTREE_H_INCLUDED */
