/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: nmt.h
 *  Description	: The nametable-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 15-04-1996
 *
 *  History:
 *  15-04-1996	: eck      : creation
 *  18-04-1996	: eck      : Added NMT() as general interface, added NMTDel(),
 *			   : added macro's
 *  19-04-1996	: eck      : Added name parameter to NewOld() interface.
 *  08-05-1996	: eck      : Added NMTClean().
 *  18-07-1997	: eck      : Added NMTWalk().
 *  14-08-1997	: eck      : Added extra parameter to NMTWalk().
 *  24-08-1998	: BdB	   : Allow NMTWalk() to quit before end of table
 *  05-03-2008	: sieburgw : Added NMTWalkOrdered
 *  12-09-2013	: martinz  : Added documentation from NMTLIB.DOC.
 *
 *========================================================================
 */

#if ! defined(NMT_H_INCLUDED)
#define NMT_H_INCLUDED

#pragma ident "$Id: nmt.h 305354 2013-11-08 11:55:59Z sieburgw $"

#include <pica.h>

ERROR_TYPE(NMTErrno_t);

ERROR_DECLARE(NMTE_NOERROR);	/* ok			*/
ERROR_DECLARE(NMTE_NOMEM);	/* not enough memory	*/

extern NMTErrno_t NMTerrno;

typedef struct NMTable_t *NMTHandle_p;
typedef struct NMT_Iterator_t *NMT_Iterator_p;

/**
 * NMT_NOFUNC is a function-argument for NMTCreate(). It can be used when no
 * allocation/deallocation should take place.
 */
#define NMT_NOFUNC	((cvoidp (*)(const char *, cvoidp, cvoidp)) 0)

cvoidp NMTNewOldString(const char *name, cvoidp newp, cvoidp oldp);

const char *NMTToString(NMTHandle_p table);
void NMTToStringSet(NMTHandle_p table,
		    bool (*EltToString)(const char *, cvoidp eltp, voidp xtra));
bool NMTPointerToString(const char *name, cvoidp eltp, voidp xtra);
bool NMTStringToString(const char *name, cvoidp eltp, voidp xtra);
const char *NMTToStringElt(NMTHandle_p table, const char *name);

NMTHandle_p NMTCreate(const char *options,
		cvoidp (*NewOld)(const char *name, cvoidp newp, cvoidp oldp));
NMTHandle_p NMT_Create(
	const char *options,
	cvoidp (*NewOld)(const char *, cvoidp newp, cvoidp oldp),
	bool (*EltToString)(const char *, cvoidp eltp, voidp xtra));

cvoidp NMT(NMTHandle_p table, const char *key, cvoidp value, bool mustput);

cvoidp NMTGet(NMTHandle_p table, const char *key);

cvoidp NMTPut(NMTHandle_p table, const char *key, cvoidp value);

cvoidp NMTDel(NMTHandle_p table, const char *key);

#define NMTGet(table, key)		NMT(table, key, NULL, FALSE)
#define NMTPut(table, key, value)	NMT(table, key, value, TRUE)
#define NMTDel(table, key)		NMT(table, key, NULL, TRUE)

void NMTDestroy(NMTHandle_p table);

void NMTClean(NMTHandle_p table);

bool NMTWalk(NMTHandle_p table,
	    bool (*HandleElt)(const char *, cvoidp eltp, voidp xtraparam),
	    voidp xtraparam);

bool NMTWalkOrdered(NMTHandle_p table,
	    bool (*HandleElt)(const char *, cvoidp eltp, voidp xtraparam),
	    void  *xtraparam);

NMT_Iterator_p
NMT_IteratorCreate(
	NMTHandle_p	nmt);

bool
NMT_IteratorFirst(
	NMT_Iterator_p	iter,
	const char**	key,
	cvoidp*		value);

bool
NMT_IteratorLast(
	NMT_Iterator_p	iter,
	const char**	key,
	cvoidp*		value);

bool
NMT_IteratorNext(
	NMT_Iterator_p	iter,
	const char**	key,
	cvoidp*		value);

bool
NMT_IteratorPrev(
	NMT_Iterator_p	iter,
	const char**	key,
	cvoidp*		value);

void
NMT_IteratorDestroy(
	NMT_Iterator_p iter);

#endif /* ! defined(NMT_H_INCLUDED) */
