/*
 *  Copyright (c) 1993 by Pica
 *
 *  File        : pica.h
 *  Description : The general pica header.  Defines some standard types.
 *  Author(s)   : Hans van Eck
 *  Created     : 13-09-1993
 *
 *  History:
 *  13-09-1993  : eck      : creation
 *  13-09-1993  : eck      : Fixed stupid inclusion test (was #if defined ...)
 *  01-10-1993  : eck      : New header format, added byte, changed int8
 *  03-11-1993  : eck      : Put #pragma inside #if
 *  15-11-1993  : eck      : Changed order inside bool type
 *  23-11-1993  : eck      : Added *ERR_FIRST for error handling
 *  14-12-1993  : lolke    : Changed WOERR_FIRST to WMERR_FIRST
 *  22-12-1993  : lolke    : Changed OPERR_FIRST to IDBERR_FIRST
 *  18-01-1994  : eck      : Added definition of STREQ()
 *  10-02-1994  : eck      : added FMTERR_FIRST
 *  18-02-1994  : eck      : added CCVERR_FIRST
 *  09-03-1994  : eck      : added APPLERR_FIRST
 *  31-05-1996  : lolke    : merged TANDEM and UNIX pica.h
 *  21-11-1996  : PvdM     : adjusted Windows bool type collision
 *  27-02-1997  : lolke    : bool enum is problem on TANDEM
 *  19-02-1998  : paul     : #pragma NOWARN added for use under GUARDIAN
 *  08-06-1998  : lolke    : int64 and uint64 added.
 *                           Only on TANDEM and ALPHA arithmetic can be done
 *
 *  25-03-1999  : paul     : INT8MIN, ..., UINT32MAX added
 *  31-03-2003  : HvdW     : tandem if's removed
 *  04-09-2003	: jaap     : Add 64-bits integers for linux and update defaults
 *  14-06-2004	: paul     : Undid delta 1.35 (addition of include <string.h>)
 *  23-01-2008	: fb       : Changed the definition of bool to typedef, added
 *                           typedef for PATH_MAX (Windows).
 *  20-05-2016	: heesakkm : Removed Windows support.
 *
 *  Notes       :
 *
 */

#if ! defined(PICA_H_INCLUDED)
#define PICA_H_INCLUDED


#if defined(_MSC_VER)
#pragma warning ( disable : 4068 )
#endif

#pragma ident "$Id: pica.h 374577 2020-02-20 13:50:32Z smitp $"

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

#if ! defined(__GNUC__)
#define __attribute__(x)
#endif

/* The following is necessary for Digital UNIX.  Let's hope that the bool
 * is not a problem.
 */
#if defined(TRUE)
#undef TRUE
#undef FALSE
#endif

typedef enum { FALSE=0, TRUE=1 } bool;

typedef signed char     int8;
typedef short           int16;
#if defined (__alpha)
typedef int             int32;
typedef long            int64;
#elif defined (__sun)
typedef int             int32;
typedef long long       int64;
#elif defined (__linux)
typedef int             int32;
typedef long long       int64;
#else
typedef int             int32;
typedef long long       int64;
#endif

typedef unsigned char   uint8;
typedef unsigned short  uint16;
#if defined(__alpha)
typedef unsigned int   uint32;
typedef unsigned long  uint64;
#elif defined(__sun)
typedef unsigned int		uint32;
typedef unsigned long long	uint64;
#elif defined(__linux)
typedef unsigned int		uint32;
typedef unsigned long long	uint64;
#else
typedef unsigned int		uint32;
typedef unsigned long long	uint64;
#endif

typedef unsigned char   byte;
#if !defined(_ZCONF_H) && !defined(ZCONF_H)
/* Unfortunately, zconf.h from the zlib compression library also contains the
   next line.  So we avoid a double typedef with this #ifdef.  This means that
   <zlib.h> must be included before <pica.h>. */
typedef void            *voidp;
#endif /* !defined(_ZCONF_H) && !defined(ZCONF_H) */
typedef const void      *cvoidp;

#define INT8MIN   ((int8)         -128)
#define INT8MAX   ((int8)          127)
#define INT16MIN  ((int16)      -32768)
#define INT16MAX  ((int16)       32767)
#define INT32MIN  ((int32) -2147483647-1)
#define INT32MAX  ((int32)  2147483647)
#define INT64MIN  ((int64) -9223372036854775807-1)
#define INT64MAX  ((int64)  9223372036854775807)
#define UINT8MAX  ((uint8)         255)
#define UINT16MAX ((uint16)      65535)
#define UINT32MAX ((uint32) 4294967295U)
#define UINT64MAX ((uint64) 18446744073709551615)

/* The STREQ()- definition may only be used when <string.h> is included. */
#define STREQ(a, b)     (strcmp((a), (b)) == 0)

#define ERROR_TYPE(type)        typedef const char *type
#define ERROR_DECLARE(error)    extern const char error[]
#define ERROR_DEFINE(error)     const char error[] = # error

ERROR_TYPE(APPLErrno_t);

ERROR_DECLARE(APPLE_NOERROR);
ERROR_DECLARE(APPLE_NOMEM);
ERROR_DECLARE(APPLE_INTERNAL);
ERROR_DECLARE(APPLE_FATAL);
ERROR_DECLARE(APPLE_INCIDENTAL);
ERROR_DECLARE(APPLE_PARAM);
ERROR_DECLARE(APPLE_FILEOPEN);
ERROR_DECLARE(APPLE_FILEREAD);
ERROR_DECLARE(APPLE_FILEWRITE);
ERROR_DECLARE(APPLE_FILECLOSE);
ERROR_DECLARE(APPLE_BUFFEROVERFLOW);
ERROR_DECLARE(APPLE_CURSESINIT);
ERROR_DECLARE(APPLE_CURSES);

extern APPLErrno_t APPLerrno;

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* defined(PICA_H_INCLUDED) */
