/*========================================================================
 *  Copyright (c) 1997 by Pica
 *
 *  File        : picatypes.h
 *  Description : General Pica-specific types
 *  Author(s)   : Paul Smit
 *  Creation    : 10-03-1997
 *
 *  History:
 *  10-03-1997  : paul     : creation
 *  03-07-1997  : HSi      : Added PICTimeStamp, PIC_TITLEREC_SIZE and
 *                           PIC_IPN_ANY.
 *  04-07-1997  : HSi      : Added PICTTLBuf.
 *  09-07-1997  : paul     : Added PIC_IKTNUM_NONE and PIC_IKTNUM_ANY
 *                           sys/types.h replaced by stddef.h
 *  01-08-1997  : paul     : Added PIC_ILN_COMMON and PIC_ILN_MIN;
 *                           changed PIC_ILN_NONE from 0 to -2
 *   2-10-1997  : HSi      : Added PIC_IDXTYPE_ANY.
 *  09-10-1997  : paul     : #include <limits.h> added
 *  05-11-1997  : paul     : PIC_IKTNUM_RECTYPE and PIC_RECTYPE_MASK_OK() added
 *  05-11-1997  : HSi      : PIC_FILESET_ANY added
 *  20-11-1997  : paul     : PIC_IKTNUM_SWD added
 *  14-01-1998  : BdB      : Change non-tandem PIC_TITLEREC_SIZE to 32K
 *  23-02-1998  : paul     : PICMatAdi_t,PICIdxType_t,PICKeyType_t,PICIktNum_t,
 *                           PICRecType_t,PICFamType_t moved to idxtypes.h
 *                           PIC_ILN_MAX introduced
 *  27-07-1999  : BdB      : Add PIC_PPN_MAXLEN and increase PIC_IPN_MAX
 *  19-10-1999  : fb       : PIC_ILN_MAX reduced to 400 (current limit)
 *  18-10-2000  : paul     : PIC_CHARBIT, PIC_CHARMASK and
 *                           PIC_USERBITS_NONE introduced
 *  30-10-2000  : paul     : PIC_ILN_DECIMAL_MAX introduced
 *  28-11-2000  : paul     : #define PIC_COPYNUM_MIN changed from 0 to 1
 *                           #define PIC_COPYNUM_NONE added
 *  12-01-2001	: eck      : Added PIC_RECTYPE_TITLE and PIC_RECTYPE_AUTHORITY
 *  18-09-2001  : paul     : Added PIC_ILN_TITLE_OWNER
 *  06-01-2003  : paul     : Changed PIC_ILN_MAX from 400 to 999
 *  31-03-2003  : HvdW     : Tandem if's removed
 *  11-07-2003  : paul     : Added PIC_FILESET_EXTERNAL;
 *                           changed PIC_FILESET_NONE from 0 to -2
 *  24-09-2003  : paul     : PIC_SUBFTYPE_OK() extended with '{' and '}'
 *  01-12-2004  : paul     : Added PIC_XOCC_*
 *                         : Changed PIC_COPYNUM_MAX from 99 to 999
 *  20-12-2004  : paul     : Added PIC_RECTYPE_ILLREQUEST and
 *                           PIC_RECTYPE_FROM_MATTYPE()
 *  24-12-2005  : paul     : Added PIC_OCCURRENCE_LEN (the length of the
 *                           decimal representation of PIC_OCCURRENCE_MAX)
 *  28-06-2006  : paul     : Added PIC_ILN_ONLINE_CONTENTS
 *  21-02-2008	: daalmanf : Added PICIdp_t (Internal Document  Pointer)
 *  11-02-2009  : evd      : Added PIC_LEVEL_SOCIAL = 3 (sns project)
 *  05-03-2009	: paul     : Allow tags of level 3 (social)
 *  12-03-2009	: paul     : Added PIC_LEVEL_ORDER()
 *  17-06-2009	: paul     : Added PIC_SOCIAL_KARMA_*
 *  07-09-2009	: paul     : CBS-1429 Added PIC_LINKAGE_OCC_NONE and
 *                           PIC_SUBFTYPE_LINKAGE_OCC
 *  08-12-2009	: paul     : CBS-1621: bugfix in holding-analysis presentation
 *  11-12-2009	: paul     : CBS-607: PPTtlSort should sort level 2 tags on $x
 *                           value
 *  21-04-2010	: paul     : Undone change of 24-09-2003
 *  06-12-2010	: paul     : CBS-2360 Added PIC_RECTYPE_NONE, PIC_RECTYPE_ANY
 *  07-03-2011	: paul     : CBS-2756 Added PIC_OCCURRENCE_MAX_LEV01
 *  05-07-2013	: heesakkm : CBS-4809: Added PIC_OCCURRENCE_SELECTED
 *  28-09-2017	: smitp    : CBS-7722 Added @ as valid Pica+ subfield-code
 *  31-10-2017  : petersh  : CBS-7742 Added note to update setSubfieldType
 *			      in csf_elvis.pm whenever PIC_SUBFTYPE_OK is modified
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(PICATYPES_H_INCLUDED)
#define PICATYPES_H_INCLUDED

#pragma ident "$Id: picatypes.h 373344 2020-01-16 07:13:36Z smitp $"

#include <limits.h>
#include <pica.h>
#include <ctype.h>
#include <stddef.h>

/*
 * PIC_CHARBIT is the number of bits contained in a byte in 001@$0 (userbits
 * subfield). PIC_CHARMASK is the unsigned number that represents exactly
 * PIC_CHARBIT 1 bits.
 * PIC_USERBITS_NONE is a string representing userbits that are all set to 0.
 */
#define PIC_CHARBIT       8
#define PIC_CHARMASK      ((unsigned) UINT8MAX)
#define PIC_USERBITS_NONE "-"

#define PIC_TITLEREC_SIZE           (32768)

#define PIC_RECTYPE_NONE                ' '
#define PIC_RECTYPE_ANY                 '*'
#define PIC_RECTYPE_TITLE		'1'
#define PIC_RECTYPE_AUTHORITY		'2'
#define PIC_RECTYPE_ILLREQUEST		'3'
#define PIC_RECTYPE_FROM_MATTYPE(mattype) \
    ((mattype) == NULL ? PIC_RECTYPE_TITLE \
     : (*(mattype) == 'T' ? PIC_RECTYPE_AUTHORITY \
	: (*(mattype) == PIC_RECTYPE_ILLREQUEST ? PIC_RECTYPE_ILLREQUEST \
	   : PIC_RECTYPE_TITLE)))

/*
 * The order of levels. A level not between 0 and 9 is illegal and returns -1.
 * Beware of levels from artificial tags like 999@, which should behind level 3
 * but in front of levels 1 and 2.
 * In a multi-ILN title, level 1 and 2 blocks of each ILN will be at the end of
 * the title, just like in a single-ILN title.
 */
#define PIC_LEVEL_ORDER(level) \
    ((level) == PIC_LEVEL_MAIN         ? 0 : \
     ((level) == PIC_LEVEL_LOCAL       ? 8 : \
      ((level) == PIC_LEVEL_COPY       ? 9 : \
       (((level) >= 0 && (level) <= 9) ? ((int)level - 2) : -1))))

#define PIC_LEVEL_MAIN              ((PICLevel_t) 0)
#define PIC_LEVEL_LOCAL             ((PICLevel_t) 1)
#define PIC_LEVEL_COPY              ((PICLevel_t) 2)
#define PIC_LEVEL_SOCIAL            ((PICLevel_t) 3)
#define PIC_LEVEL_MIN               PIC_LEVEL_MAIN
#define PIC_LEVEL_MAX               PIC_LEVEL_SOCIAL
#define PIC_LEVEL_ANY               ((PICLevel_t) -1)
#define PIC_LEVEL_NONE              ((PICLevel_t) -2)
#define PIC_LEVEL_ERROR             ((PICLevel_t) -3)
#define PIC_TAGTYPE_MIN             ((PICTagType_t) 0)
#define PIC_TAGTYPE_MAX             ((PICTagType_t) 99)
#define PIC_INDICATOR_MIN           ((PICIndicator_t) '@')
#define PIC_INDICATOR_MAX           ((PICIndicator_t) 'Z')
#define PIC_INDICATOR_ANY           ((PICIndicator_t) '*')
#define PIC_INDICATOR_OK(c)         (   PIC_INDICATOR_MIN <= (c)              \
                                     && (c) <= PIC_INDICATOR_MAX)
#define PIC_INDICATOR_MASK_OK(c)    (   PIC_INDICATOR_OK(c)                   \
                                     || (c) == PIC_INDICATOR_ANY)
#define PIC_INDICATOR_MATCH(c,mask) (   (c) == (mask)                         \
                                     || (mask) == PIC_INDICATOR_ANY)
#define PIC_OCCURRENCE_MIN          ((PICOccurrence_t) 0)
#define PIC_OCCURRENCE_MAX          ((PICOccurrence_t) 999)
#define PIC_OCCURRENCE_MAX_LEV01    ((PICOccurrence_t) 99)
#define PIC_OCCURRENCE_ANY          ((PICOccurrence_t) -1)
#define PIC_OCCURRENCE_SELECTED     ((PICOccurrence_t) -2)
#define PIC_OCCURRENCE_NONE         ((PICOccurrence_t) -3)
/* The length of the decimal representation of PIC_OCCURRENCE_MAX: */
#define PIC_OCCURRENCE_LEN          3

/* value of a PIC_SUBFTYPE_LINKAGE_OCC subfield indicating: not linked */
#define PIC_LINKAGE_OCC_NONE	"00"
#define PIC_LINKAGE_OCC_MIN	"01"
#define PIC_LINKAGE_OCC_MAX	"99"
#define PIC_LINKAGE_OCC_LEN	2

/* The length of the contents of $x subfields in level 2 tags, used for the
 * Pica3 tag occurrence
 */
#define PIC_XOCC_LEN		    2
#define PIC_XOCC_MIN		    0
#define PIC_XOCC_MAX		    99
/* type of the subfield xoccurrence of a level 2 tag */
#define PIC_XOCC_SBFTYP             'x'
/* implicit xoccurrence of a level 2 tag if the subfield is missing */
#define PIC_XOCC_DEFAULT            "00"

// The implicit value of $S, if it is omitted
#define PIC_DEFAULT_INDICATORS      "##"

#define PIC_SUBFTYPE_NONE           ((PICSubfType_t) ' ')
#define PIC_SUBFTYPE_ANY            ((PICSubfType_t) '*')
#define PIC_SUBFTYPE_INDICATORS     ((PICSubfType_t) 'S')
#define PIC_SUBFTYPE_LINKAGE_OCC    ((PICSubfType_t) 'T')
#define PIC_SUBFTYPE_SCRIPTCODE     ((PICSubfType_t) 'U')
#define PIC_SUBFTYPE_SCRIPTLANG     ((PICSubfType_t) 'L')
#define PIC_SUBFTYPE_PPN_LINK       ((PICSubfType_t) '9')
#define PIC_SUBFTYPE_PPN_EXPANSION  ((PICSubfType_t) '8')

// PIC_SUBFTYPE_OK defines the types that a Pica database can contain
// This define cannot match the following chars:
// $ since '$$' is used in FCV tables
// ~ since '$~' is used in both FCV and IDG tables
// The characters matched by FCV_ISSUBFSYM(): '<' '>' '%' '-'
// The continuation subfield in internal Pica+: SBF_CONTINUE: ((char) 25)
//
// Beware that if you add a new char, you may also have to add it to:
// 'PPSBF' in xpnlex.lex
// 'SUBFIELD' in idgp_lexer.lex
// 'SUBFIELD' in tfltlex.lex
// sub setSubfieldType in csf_elvis.pm
#define PIC_SUBFTYPE_OK(c)          (isalnum((unsigned char)(c)) \
                                     || (c) == '@')

// An OCN, excluding the leading "ocn" prefix, may have limited length (some say 12)
// Since we don't know the exact maximum, this define is a broad estimate
#define PIC_OCN_MAXLEN              22

/* A PPN can have length from PIC_PPN_LEN to PIC_PPN_MAXLEN.  PPNs that would
   be smaller than PIC_PPN_LEN have leading zeros added up to PIC_PPN_LEN. */
#define PIC_PPN_LEN                 9
#define PIC_PPN_MAXLEN              11
#define PIC_IPN_ANY                 ((PICIpn_t) -1)
#define PIC_IPN_NONE                ((PICIpn_t) 0)
#define PIC_IPN_MIN                 ((PICIpn_t) 1)
/* We use 2^32-2, to allow software to use PIC_IPN_MAX+1 internally as a
   sentinel value larger than all real IPNs. */
#define PIC_IPN_MAX                 ((PICIpn_t) 2147483646)

#define PIC_ILN_MIN                 ((PICIln_t) 1)
#define PIC_ILN_COMMON              ((PICIln_t) 0)
#define PIC_ILN_ANY                 ((PICIln_t) -1)
#define PIC_ILN_NONE                ((PICIln_t) -2)
/* PIC_ILN_TITLE_OWNER is used in index definition tables to indicate the
   title_owner field of DBS-parameters; it must differ from both PIC_ILN_ANY 
   and PIC_ILN_NONE */
#define PIC_ILN_TITLE_OWNER         ((PICIln_t) -3)
#define PIC_ILN_ONLINE_CONTENTS     ((PICIln_t) 999)
#define PIC_ILN_MAX                 ((PICIln_t) INT16MAX - 1)
#define PIC_ILN_SENTINEL            ((PICIln_t) INT16MAX)
/* PIC_ILN_DECIMAL_MAX represents the length of decimal representation of
   PIC_ILN_MAX; to be increased when PIC_ILN_MAX is increased */
#define PIC_ILN_DECIMAL_MAX         5
/* PIC_OWNER_TO_ILN is an old macro that used to contain a % 1000 trick for
 * converting owners to iln. These are now the same. */
#define PIC_OWNER_TO_ILN(owner)     (owner)
/* PIC_ILN_RANGEBUF_SIZE is the size of the ILN range buffers 
   NOTE: worst case is consecutive pairs, eg: "1,2,4,5,7,8,..."
   This means that 2/3 of the number in range are represented. */
#define PIC_ILN_RANGEBUF_SIZE ((PIC_ILN_DECIMAL_MAX+1) * (PIC_ILN_MAX + 1) * 2 / 3)
/* PIC_USERBITS_BUF_SIZE is the required size of the userbit buffer */
#define PIC_USERBITS_BUF_SIZE	    ((PIC_ILN_MAX/PIC_CHARBIT) + 1)

#define PIC_COPYNUM_MIN             ((PICCopyNum_t) 1)
#define PIC_COPYNUM_MAX             ((PICCopyNum_t) 999)
#define PIC_COPYNUM_MAX_EXTERN_PP   ((PICCopyNum_t) INT16MAX - 1)
#define PIC_COPYNUM_MAX_SENTINEL    ((PICCopyNum_t) INT16MAX)
#define PIC_COPYNUM_MIN_SENTINEL    ((PICCopyNum_t) INT16MIN)
#define PIC_COPYNUM_NONE            ((PICCopyNum_t) 0)
#define PIC_COPYNUM_ANY             ((PICCopyNum_t) PIC_OCCURRENCE_ANY)
#define PIC_COPYNUM_LEN             3
/* PIC_COPYNUM_DECIMAL_MAX represents the length of decimal representation of
   PIC_COPYNUM_MAX; to be increased when PIC_COPYNUM_MAX is increased */
#define PIC_COPYNUM_DECIMAL_MAX     PIC_COPYNUM_LEN
/* PIC_COPYNUM_RANGEBUF_SIZE is the size of the ILN range buffers 
   NOTE: worst case is consecutive pairs, eg: "1,2,4,5,7,8,..."
   This means that 2/3 of the number in range are represented. */
#define PIC_COPYNUM_RANGEBUF_SIZE ((PIC_COPYNUM_DECIMAL_MAX+1) * (PIC_COPYNUM_MAX + 1) * 2 / 3)

#define PIC_FILESET_EXTERNAL        ((PICFileSet_t) 0)
#define PIC_FILESET_MIN             ((PICFileSet_t) 1)
#define PIC_FILESET_MAX             ((PICFileSet_t) INT16MAX - 1)
#define PIC_FILESET_ANY             ((PICFileSet_t) -1)
#define PIC_FILESET_NONE            ((PICFileSet_t) -2)

typedef int16                       PICFileSet_t;
typedef char                        PICPpn_t[PIC_PPN_MAXLEN + 1];
typedef int32                       PICIpn_t;
typedef int16                       PICIln_t;
typedef int16                       PICCopyNum_t;
typedef int8                        PICLevel_t;
typedef int16                       PICOccurrence_t;
typedef char                        PICIndicator_t;
typedef int8                        PICTagType_t;
typedef char                        PICSubfType_t;

#define PIC_TIMESTAMP_SIZE              (8)
typedef byte PICTimeStamp_t [PIC_TIMESTAMP_SIZE];
typedef PICTimeStamp_t *PICTimeStamp_p;

typedef struct {
    PICTimeStamp_t      timeStamp;      /* Title timestamp              */
    PICFileSet_t        fileSet;        /* Title fileset                */
    PICLevel_t          lev;            /* Title level                  */
    PICIpn_t            ipn;            /* Title number                 */
    PICIln_t            iln;            /* Title owner                  */
    PICOccurrence_t     occ;            /* Copy occurrence              */
    PICIpn_t            epn;            /* Copy number                  */
    bool                isActive;       /* Active flag (logical deleted)*/
    char                status;         /* Title status                 */
    size_t              buflen;         /* Used length of buffer        */
    byte                buf[PIC_TITLEREC_SIZE]; /* Title buffer         */
} PICTTLBuf_t, *PICTTLBuf_p;


/* An IDP (Internal Document - Pointer) is a number used for PSI indexing
   which is equivalent to a PPN. It doesn't have any meaning in Sybase
   (i.e. it's not an IPN), however each IDP also maps to one IPN  */
typedef int32                       PICIdp_t;
#define PIC_IDP_ANY                 ((PICIdp_t) -1)
#define PIC_IDP_NONE                ((PICIdp_t) 0)
#define PIC_IDP_MIN                 ((PICIdp_t) 1)
/* We use 2^31-2, to allow software to use PIC_IDP_MAX+1 internally as a
   sentinel value larger than all real IDPs. */
#define PIC_IDP_MAX                 ((PICIdp_t) 2147483646)

/* Valid range of values for usr_social.karma */
#define PIC_SOCIAL_KARMA_MIN	0
#define PIC_SOCIAL_KARMA_MAX	9999
/* The highest value that represents bad karma */
#define PIC_SOCIAL_KARMA_BAD \
	((PIC_SOCIAL_KARMA_MAX - PIC_SOCIAL_KARMA_MIN + 1) / 10)
/* The karma value that a newly inserted usr_social gets */
#define PIC_SOCIAL_KARMA_START \
	(PIC_SOCIAL_KARMA_BAD + PIC_SOCIAL_KARMA_BAD / 2)


#endif /* defined(PICATYPES_H_INCLUDED) */
