/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File        : pp.h
 *  Description : The pica-plus library
 *  Author(s)   : Hans van Eck
 *  Creation    : 22-01-1996
 *
 *  History:
 *  22-01-1996  : eck      : creation
 *  02-02-1996  : eck      : Lots of changes in the interface
 *  04-09-1998  : BdB      : PPFromIntern takes const byte * instead of byte *
 *  11-06-1999  : BdB   : Add more const's to pointer arguments
 *  11-04-2000  : BdB   : Add PPTtlConcatenate
 *  20-07-2000  : paul  : Added PP_ANYSBF
 *  23-08-2000  : paul  : Moved PP*Create() and PP*Destroy() from pplocal.h
 *  12-10-2000  : paul  : PPToNormal() introduced
 *  18-10-2000  : paul  : PPFromNormal() introduced
 *  03-05-2001  : evd   : ppsbf_length from short to int for titles > 64 k
 *  20-08-2001	: eck      : Changed interfaces from short to int
 *  17-10-2001  : jelle : Added PPEltClone()
 *  16-11-2001  : paul  : Added PPEltListAppend()
 *  21-01-2002	: paul  : Added PPTagHideUndo()
 *  11-03-2002  : paul  : Added PPBitsToExtern()
 *  08-05-2002  : paul  : Added PPEltInsert()
 *  10-05-2002  : paul  : moved PPEltUnlink() from pplocal.h
 *                      : PPEltAppend() changed into PPEltListAppend()
 *                      : PPEltInsert() changed into PPEltListInsert()
 *  01-10-2002	: paul  : Added PPTo*Create() and PPBuffer*()
 *  03-04-2003	: paul  : Added PPToInternCreateCS() + PPToInternAppendCS()
 *  07-04-2003	: paul  : Added PPToExternCreateCS() + PPToExternAppendCS()
 *                      : +     PPToNormalCreateCS() + PPToNormalAppendCS()
 *  09-04-2003	: paul  : Added PPBitsToExternAppend()
 *  08-11-2004  : michiel  : multi-byte occurrence
 *  20-10-2004  : paul  : moved PPTagCmp from pplocal.h
 *  26-11-2004  : guy   : added PPCompare()
 *  08-04-2008  : paul  : added PPTtlSortCustom()
 *  11-02-2009	: martinz  : Added PPFromExtern2(), PPFromNormal2().
 *  05-03-2009	: paul     : Allow tags of level 3 (social)
 *  01-09-2009	: sieburgw : Added PPEltDbg.
 *  01-04-2010	: paul     : Added PPEltListClone()
 *  20-05-2011	: paul     : CBS-2906 added PPTagFromSbf() + PPTtlFromTag()
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(PP_H_INCLUDED)
#define PP_H_INCLUDED

#pragma ident "$Id: pp.h 350873 2017-12-05 11:44:17Z smitp $"

#include <stddef.h>
#include <pica.h>
#include <picatypes.h>
#include <cs.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

ERROR_TYPE(PPErrno_t);

ERROR_DECLARE(PPE_NOERROR);
ERROR_DECLARE(PPE_NOMEM);
ERROR_DECLARE(PPE_TYPE);
ERROR_DECLARE(PPE_FORMAT);
ERROR_DECLARE(PPE_OVERFLOW);

extern PPErrno_t PPerrno;

/*
 * All elements of a title (whole-title, tag and subfield) reside in the
 * same basic structure.  This makes it possible to doubly-link lists of
 * the same elements while having a "previous" pointer in the first
 * sub-element point to the higher-up element.  This makes it easy to
 * insert and delete element at any place.
 */

typedef enum {
    PPT_NONE, PPT_TITLE, PPT_TAG, PPT_SUBFIELD
} PPElementTyp_t;

typedef int8                        PP_Level_t;
typedef int16                       PP_Occurrence_t;
typedef char                        PP_Indicator_t;
typedef int8                        PP_TagType_t;
typedef char                        PP_SubfType_t;

#define PPLEV_MAIN              ((PP_Level_t) 0)
#define PPLEV_LOCAL             ((PP_Level_t) 1)
#define PPLEV_COPY              ((PP_Level_t) 2)
#define PPLEV_SOCIAL            ((PP_Level_t) 3)
#define PP_LEVEL_MIN               PPLEV_MAIN
#define PP_LEVEL_MAX               PPLEV_SOCIAL
#define PP_LEVEL_ANY               ((PP_Level_t) -1)
#define PP_TAGTYPE_MIN             ((PP_TagType_t) 0)
#define PP_TAGTYPE_MAX             ((PP_TagType_t) 99)
#define PP_TAGTYPE_ANY             ((PP_TagType_t) -1)
#define PP_INDICATOR_MIN           ((PP_Indicator_t) '@')
#define PP_INDICATOR_MAX           ((PP_Indicator_t) 'Z')
#define PP_INDICATOR_ANY           ((PP_Indicator_t) '*')
#define PP_INDICATOR_OK(c)         (   PP_INDICATOR_MIN <= (c)              \
                                     && (c) <= PP_INDICATOR_MAX)
#define PP_OCCURRENCE_MIN          ((PP_Occurrence_t) PIC_OCCURRENCE_MIN)
#define PP_OCCURRENCE_MAX          ((PP_Occurrence_t) PIC_OCCURRENCE_MAX)
#define PP_OCCURRENCE_MAX_LEV01    ((PP_Occurrence_t) PIC_OCCURRENCE_MAX_LEV01)
#define PP_OCCURRENCE_ANY          ((PP_Occurrence_t) PIC_OCCURRENCE_ANY)
#define PP_OCCURRENCE_SELECTED     ((PP_Occurrence_t) PIC_OCCURRENCE_SELECTED)
#define PP_OCCURRENCE_NONE         ((PP_Occurrence_t) PIC_OCCURRENCE_NONE)

#define PP_ANYTYP       ((PP_TagType_t) -1)
#define PP_ANYLEV       ((PP_Level_t) -1)
#define PP_ANYIND       ('*')
#define PP_ANYOCC       PP_OCCURRENCE_ANY
#define PP_ANYSBF       ('*')

// The character to be used for PP_ANYOCC in Normalized/External Pica+
#define PP_OCCURRENCE_ANY_NORMALIZED	'*'

typedef struct PPElement_t *PPElement_p;

typedef struct {
    PPElement_p ppttl_firsttag;         /* First tag of the title */
} PPTypeTitle_t;

typedef struct {
    PPElement_p         pptag_firstsbf; /* First subfield of the tag */
    PICTagType_t        pptag_typ;      /* Type of the tag */
    PICLevel_t                pptag_lev;      /* Level of the tag */
    char                pptag_ind;      /* Indicator */
    PICOccurrence_t     pptag_occ;      /* Occurrence */
} PPTypeTag_t;

typedef struct {
    char                ppsbf_typ;      /* subfield type (from [a-zA-Z0-9]) */
    int                 ppsbf_length;   /* contents-length (without '\0') */
    voidp               ppsbf_contents; /* contents */
} PPTypeSubfield_t;

typedef struct PPElement_t {
    PPElement_p         pp_eltnext;     /* next with same pp_elttype */
    PPElement_p         pp_eltprev;     /* previous ... */
    PPElementTyp_t      pp_elttyp;
    union {
        PPTypeTitle_t           pp_title;
        PPTypeTag_t             pp_tag;
        PPTypeSubfield_t        pp_subfield;
    }                   pp_elt;
} PPElement_t;

/* Some #defines to make the programmers' lifo more easy. */
#define PP_title        pp_elt.pp_title
#define PP_tag          pp_elt.pp_tag
#define PP_subfield     pp_elt.pp_subfield

#define PPTTL_firsttag  PP_title.ppttl_firsttag

#define PPTAG_firstsbf  PP_tag.pptag_firstsbf
#define PPTAG_typ       PP_tag.pptag_typ
#define PPTAG_lev       PP_tag.pptag_lev
#define PPTAG_ind       PP_tag.pptag_ind
#define PPTAG_occ       PP_tag.pptag_occ

#define PPSBF_typ       PP_subfield.ppsbf_typ
#define PPSBF_length    PP_subfield.ppsbf_length
#define PPSBF_contents  PP_subfield.ppsbf_contents

typedef PPElement_p PPTtl_p;            /* with pp_elttyp is PPT_TITLE */
typedef PPElement_p PPTag_p;            /* with pp_elttyp is PPT_TAG */
typedef PPElement_p PPSbf_p;            /* with pp_elttyp is PPT_SUBFIELD */

/*
 * The following variables modify the behaviour of PP[To|From][Intern|Extern]
 */
typedef enum {
    PPEND_LITTLE, PPEND_BIG
} PPEndianness_t;

extern PPEndianness_t   PP_int_ttlendian;

extern char     PP_ext_tagend;
extern char     PP_ext_sbfmark;

PPTtl_p PPFromIntern(PPTtl_p PPtitle, const byte *ttlbuf, size_t ttlsiz);

bool PPToIntern(PPTtl_p PPtitle, PICLevel_t lev, PICOccurrence_t occ,
                byte *ttlbuf, size_t bufsiz, size_t *ttlsiz);

CSBuf_p PPToInternCreateCS(PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);
    
CSBuf_p PPToExternCreateCS(PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);
    
CSBuf_p PPToNormalCreateCS(PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);

bool PPIntOutTagCS(CSBuf_p ttl, PPTag_p tag);
bool PPToInternAppendCS(CSBuf_p ttl, PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);
    
CSBuf_p PPTagToNormal(CSBuf_p csbuf, PPTag_p tag);
bool PPExtOutTagCS(CSBuf_p ttl, PPTag_p tag);
bool PPToExternAppendCS(CSBuf_p ttl, PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);
    
bool PPToNormalAppendCS(CSBuf_p ttl, PPTtl_p title, PICLevel_t lev, PICOccurrence_t occ);
    
char * PPToString(PPTtl_p PPtitle);

PPTag_p PPTagFromExtern(const char *tag_start);

PPTtl_p PPFromExtern(PPTtl_p PPtitle, char *ttlbuf);

PPTtl_p PPFromNormal(PPTtl_p PPtitle, char *ttlbuf);

bool PPBitsToExtern(char *ttlbuf, size_t bufsiz, size_t *ttlsiz,
		    const byte *bitbuf, size_t bitlen);

bool PPBitsToExternAppend(CSBuf_p ttl, const byte *bitbuf, size_t bitlen);
    
bool PPToExtern(PPTtl_p PPtitle, PICLevel_t lev, PICOccurrence_t occ,
                char *ttlbuf, size_t bufsiz, size_t *ttlsiz);

bool PPToNormal(PPTtl_p PPtitle, PICLevel_t lev, PICOccurrence_t occ,
                char *ttlbuf, size_t bufsiz, size_t *ttlsiz);

int PPTagCmp(PPElement_p tag, PPElement_p cmptag);

typedef int	(*PPTagCmp_f)(PPTag_p, PPTag_p);
void PPTtlSortCustom(PPTtl_p title, PPTagCmp_f tagcmp, const char *options);
void PPTtlSort(PPTtl_p PPtitle);

void PPEltDestroy(PPElement_p PPelt);

PPElement_p PPEltClone(PPElement_p PPelt);
PPElement_p PPEltListClone(PPElement_p PPelt);

PPElement_p PPEltListAppend(PPElement_p oldelt, PPElement_p newelt);
    
PPElement_p PPEltListInsert(PPElement_p oldelt, PPElement_p newelt);
    
void PPEltUnlink(PPElement_p elt);

PPTag_p PPTagAppend(PPElement_p PPelt, PICTagType_t typ, PICLevel_t lev, char ind, PICOccurrence_t occ);

PPTag_p PPTTLTagAddCopy(PPElement_p PPelt, PPTag_p PPTag);

PPTag_p PPTagFind(PPElement_p elt, PICTagType_t typ, PICLevel_t lev, char ind, PICOccurrence_t occ);

void PPTagHide(PPTag_p PPtag);
void PPTagHideUndo(PPTag_p tag);
    
PPSbf_p PPSbfInsert(PPElement_p PPelt, char typ, cvoidp contents, int len);

PPSbf_p PPSbfAppend(PPElement_p PPelt, char typ, cvoidp contents, int len);

bool PPSbfAppendContents(PPSbf_p PPsbf, cvoidp contents, int len);

PPSbf_p PPSbfFind(PPElement_p PPelt, char typ);

bool PPSbfReplaceContents(PPSbf_p PPsubfield, cvoidp contents, int len);

voidp PPSbfGet(PPTtl_p PPtitle, PICTagType_t tagtyp, PICLevel_t lev, char ind, PICOccurrence_t occ,
                char sbftyp);

PPTtl_p
PPTtlConcatenate(PPTtl_p dest, PPTtl_p src);

PPTtl_p PPTtlCreate(void);
void PPTtlDestroy(PPTtl_p PPtitle);

PPTag_p PPTagCreate(PICTagType_t typ, PICLevel_t lev, char ind, PICOccurrence_t occ);
void PPTagDestroy(PPTag_p PPtag);

PPElement_p PPSbfCreate(char typ, cvoidp contents, int len);
void PPSbfDestroy(PPSbf_p PPsubfield);

PPTtl_p  PPReadCopy(PPTtl_p *src, int16 *iln);

bool 
PPCompare(PPTag_p first1, PPTag_p last1, PPTag_p first2, PPTag_p last2);

PPTtl_p PPFromExtern2(PPTtl_p PPtitle, const char *ttlbuf);

PPTtl_p PPFromNormal2(PPTtl_p PPtitle, const char *ttlbuf);

void
PPEltDbg(CSBuf_p buf, PPElement_p elt, int level, PPElement_p prev);

PPTag_p PPTagFromSbf(PPSbf_p sbf);
PPTtl_p PPTtlFromTag(PPTag_p tag);

#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* ! defined(PP_H_INCLUDED) */
