/*========================================================================
 *  Copyright (c) 1997 by Pica
 *
 *  File        : prs.h
 *  Description : The interface for the PRS-library
 *  Author(s)   : Paul Smit
 *  Creation    : 29-09-1997
 *
 *  History:
 *  29-09-1997  : paul  : creation
 *  02-10-1997  : paul  : PRSFull(): idxdef instead of iktlist
 *  10-10-1997  : paul  : PRSExpandClickEffects() added
 *  19-01-1998  : HSi   : Added some errorstatuses.
 *  28-01-1998  : paul  : Added PRS_SC_INDENT_CHAR
 *  09-03-1998  : paul  : PRSHighlight() added
 *  20-03-1998  : BdB   : Added userdata argument to highlight callback.
 *  30-03-1998  : paul  : Added layout parameter (PRSLayout_t) in PRSFull()
 *  25-06-1998  : paul  : Added member owners in PRSLayout_t
 *                        Added options parameter in PRSLayout()
 *                        PRSE_INVALIDTITLE changed to PRSE_PARAMETER
 *                        PRSTableHtml_t introduced
 *                        Removed PRS_SC_INDENT_CHAR
 *                        Added interpreter() parameter: PRSFull()
 *                        Removed member noindent from PRSLayout_t
 *  20-11-1998  : paul  : PRSBrief(): html parameter removed
 *  21-06-1999  : OvV   : Add PRSHolding();
 *  06-06-2000  : paul  : Added #include <html.h>
 *  27-11-2000  : paul  : PRSPresFull() introduced
 *  06-02-2001  : paul  : PRSPresFull() replaced by PRSFullCreate() and
 *                        PRSFullDestroy()
 *  20-02-2001  : MH    : PRSBriefCompile() and PRSBriefMatch() added
 *  23-02-2001  : MH    : PRSBriefCreate(), PRSBriefDestroyList(),
 *                        PRSBriefColumnise() and PRSBriefPres_t added
 *  08-03-2001  : MH    : Lots of changes for brief presentations
 *  07-06-2001  : paul  : PRSLayout(): removed
 *  17-10-2001  : paul  : maxcols parameter removed from PRSBriefMatch()
 *                      : PRSBriefMatchSet() added
 *  14-02-2002  : paul  : PRSBriefColumnise(): new parameter truncation_allowed
 *  19-02-2002  : paul  : PRSLayout_t: maxindent, maxpage, repeatedlines and
 *                        continuationtext added
 *  17-04-2002  : paul  : Added #define PRS_BRIEF_COLUMNS_UNDEFINED
 *  11-12-2003  : paul  : Added utf8 parameter in PRSExpandClickEffects() and
 *                        PRSHighlight()
 *  23-03-2004  : paul  : Introduced the global variable PRSutf8
 *  05-05-2004	: jaap	: Add PRSFullIDG() and PRSFullCreateIDG()
 *			  Remove PRSExpandClickEffects() and PRSHighlight()
 *  24-05-2005  : paul  : Added xml element definitions in PRSLayout_t
 *                      : Added PRSLayoutStruct_t
 *  08-09-2005  : paul  : Renamed PRS_LAYOUT_STRUCT_XML
 *                        to PRS_LAYOUT_STRUCT_XMLPRS
 *                      : Added PRS_LAYOUT_STRUCT_XML
 *  08-03-2007  : paul  : Added PRS_XML_IMG_NAME
 *  24-12-2008	: paul     : A term that found only original index entries
 *                           may not highlight a derived presentation word
 *  07-09-2009	: paul     : CBS-1429 Added PRSPPLinkageRenumber()
 *  19-03-2010	: paul     : CBS-1835 Added PRSTtlScriptSort()
 *  12-07-2016	: martinz  : Changed highlighting userdata.
 *  25-09-2018	: zwaalm   : CBS-7440 Generate PRSLayoutStruct_t with X-macro.
 *  31-10-2018	: zwaalm   : CBS-7440 Code review changes.
 *  27-02-2019	: zwaalm   : CBS-8671 Refactored PRSFullDestroy(),
 *  			     PRSFullPres_t and PRSFullPres_p.
 *  27-02-2019	: zwaalm   : CBS-8671 Added PRSBriefFilterCBCreate().
 *
 *  Notes       :
 *
 *========================================================================
 */

#if ! defined(PRS_H_INCLUDED)
#define PRS_H_INCLUDED

#pragma ident "$Id: prs.h 365431 2019-02-28 11:23:08Z zwaalm $"

#include <stddef.h>
#include <pica.h>
#include <picatypes.h>
#include <html.h>
#include <pp.h>
#include <fcv.h>
#include <idx.h>
#include <idg.h>

#define PRS_BRIEF_MAX_COLUMNS           10      /* display columns */
#define PRS_BRIEF_COLUMNS_UNDEFINED     (-1)    /* Don't know maxcols yet */

typedef struct  PRSHighlighting_t {
      IDXIdxType_t            idxtype;
      bool                  (*highlight_term_phrase)(
                                  const char         *options,
                                  const char         *indexphr,
                                  bool                is_stopword,
				  int                 variant,
                                  void               *userdata);
      void                   *userdata;
} PRSHighlighting_t;

/* Table with all possible layout_struct elements.  */
#define PRS_LAYOUT_STRUCT_TABLE			\
X(PRS_LAYOUT_STRUCT_NONE,   	"none")      	\
X(PRS_LAYOUT_STRUCT_FLAT,	"flat")		\
X(PRS_LAYOUT_STRUCT_HTML,	"html")		\
X(PRS_LAYOUT_STRUCT_XMLPRS,	"xmlprs")	\
X(PRS_LAYOUT_STRUCT_XML,	"xml")

/* X-macro that defines layout_struct enums; PRS_LAYOUT_STRUCT_NONE etc. */
#define X(struct_enum, struct_name) struct_enum,
typedef enum PRSLayoutStruct_t {
    PRS_LAYOUT_STRUCT_TABLE
    PRS_LAYOUT_STRUCT_COUNT
} PRSLayoutStruct_t;
#undef X

/* Presented text is structured into lines by xml elements with this name */
#define PRS_XML_LINE_NAME	"line"
/* Children of the xml line element have one of the following names: */
#define PRS_XML_TEXT_NAME	"text"
#define PRS_XML_IMG_NAME	"img"

typedef struct  PRSTableHtml_t {
      const char             *table,     /* html TABLE open tags */
                             *row,       /* html TR open tags */
                             *data;      /* html TD open tags */
} PRSTableHtml_t;
typedef struct  PRSLayout_t {
    size_t              maxline;           /* flat maximum line length */
    size_t              minline;           /* flat minimum line length */
    size_t              maxpage;           /* flat max. lines per page */
    size_t              repeatedlines;     /* flat for continuation pages */
    const char          *continuationtext; /* text separating repeatedlines */
    struct {                               /* and continuation page */
	size_t              minindent;     /* flat minimum indent size */
	size_t              maxindent;     /* flat maximum indent size */
	const char         *label;         /* html TH open tags */
	PRSTableHtml_t   tablehtml;
    }                  labelled;
    PRSTableHtml_t     owners;
    struct {
	const char	*label;         /* xml label element */
	PRSTableHtml_t	labelled;	/* xml labelled table elements */
	PRSTableHtml_t	table;		/* xml elements for other tables */
    }                  xml;
} PRSLayout_t;

typedef struct PRSPres_t {
      char	*ptr;
      size_t	len;
      size_t	size;
} PRSPres_t, *PRSPres_p;

/* For backward compatibility. */
typedef PRSPres_t PRSFullPres_t;
typedef PRSPres_p PRSFullPres_p;
#define PRSFullDestroy(pres) PRSDestroy((pres))


typedef struct PRSBriefPres_t {
    const char          *pbp_buf;
    size_t              pbp_totlen;
    const char          *pbp_app_data;
    size_t              pbp_app_data_len;
    struct {
        const char      *pbpc_ptr;
        size_t          pbpc_len;
    } pbp_column[PRS_BRIEF_MAX_COLUMNS];
    int                 *pbp_columnwidths;
    struct PRSBriefPres_t *pbp_next;
} PRSBriefPres_t, *PRSBriefPres_p;

ERROR_TYPE(PRSErrno_t);

ERROR_DECLARE(PRSE_NOERROR);
ERROR_DECLARE(PRSE_ERROR);
ERROR_DECLARE(PRSE_NOMEM);
ERROR_DECLARE(PRSE_BUFFEROVERFLOW);
ERROR_DECLARE(PRSE_PARAMETER);
ERROR_DECLARE(PRSE_INTERNAL);

extern PRSErrno_t PRSerrno;

extern bool PRSutf8;

typedef const char *(*PRSInterpreter_f)(HTMLArgList_p);

// Structure containing all callbacks+userdata needed by PRS-library
typedef struct {
    PRSInterpreter_f	interpreter;
    FCVConvertCB_t	fcvcb;
} PRSCB_t, *PRSCB_p;

void
PRSCBClear(
	PRSCB_p	cb);

bool PRSRemoveDuplicateTagsCS(
        const char	*options,	/* option string */
        CSBuf_p 	src,		/* normalized input buffer */
        CSBuf_p 	dest);		/* output buffer */

bool PRSFull(
      const char                  *options,
      PRSInterpreter_f		   interpreter,
      PPTtl_p                      ttl,
      FCVSynBuf_p                  synbuf,
      FCVSynBuf_p                  htmlsyn,
      IDGComp_p                    idgcomp,
      const PRSHighlighting_t     *highlighting,
      const PRSLayout_t           *layout,
      char                        *destbuf,
      size_t                       destbufsize);
bool PRSFullCB(
	const char		*options,
	PPTtl_p			ttl,
	FCVSynBuf_p		synbuf,
	FCVSynBuf_p		htmlsyn,
	IDGComp_p		idgcomp,
	const PRSHighlighting_t	*highlighting,
	const PRSLayout_t	*layout,
	char			*destbuf,
	size_t			destbufsize,
	PRSCB_p			cb);

void PRSDestroy(PRSPres_p pres);

PRSPres_p
PRSFullCreate(
      const char		*options,
      PRSInterpreter_f		interpreter,
      PPTtl_p			ttl,
      FCVSynBuf_p		synbuf,
      FCVSynBuf_p		htmlsyn,
      FCVSynBuf_p		filter,
      IDGComp_p			idgcomp,
      const PRSHighlighting_t	*highlighting,
      const PRSLayout_t		*layout);
PRSPres_p
PRSFullCreateCB(
      const char		*options,
      PPTtl_p			ttl,
      FCVSynBuf_p		synbuf,
      FCVSynBuf_p		htmlsyn,
      FCVSynBuf_p		filter,
      IDGComp_p			idgcomp,
      const PRSHighlighting_t	*highlighting,
      const PRSLayout_t		*layout,
      PRSCB_p			cb);

bool PRSBrief(const char *options,
        PPTtl_p     ttl,
        FCVSynBuf_p synbuf,
        size_t      ttlnr,
        char       *destbuf,
        size_t      destbufsize);
bool PRSBriefCB(
	const char	*options,
	PPTtl_p		ttl,
	FCVSynBuf_p	synbuf,
	size_t		ttlnr,
	char		*destbuf,
	size_t		destbufsize,
	PRSCB_p		cb);

bool
PRSBriefFilter(
      const char	*options,
      PPTtl_p		ttl,
      FCVSynBuf_p	synbuf,
      FCVSynBuf_p	filter,
      size_t		ttlnr,
      char		*destbuf,
      size_t		destsize);
bool
PRSBriefFilterCB(
	const char	*options,
	PPTtl_p		ttl,
	FCVSynBuf_p	synbuf,
	FCVSynBuf_p	filter,
	size_t		ttlnr,
	char		*destbuf,
	size_t		destsize,
	PRSCB_p     	cb);

PRSPres_p
PRSBriefFilterCBCreate(
	const char	*options,
	PPTtl_p		ttl,
	FCVSynBuf_p	synbuf,
	FCVSynBuf_p	filter,
	size_t		ttlnr,
	PRSCB_p     	cb);

bool PRSBriefMultiple(const char **options,
        PPTtl_p      ttl,
        FCVSynBuf_p* synbufs,
        size_t       ttlnr,
        char**	     destbuf,
        size_t       destbufsize,
	int	     n_bufs );
bool
PRSBriefMultipleCB(
	const char	**options,
	PPTtl_p		ttl,
	FCVSynBuf_p	*synbufs,
	size_t		ttlnr,
	char		**destbufs,
	size_t		destsize,
	int		n_bufs,
	PRSCB_p		cb);

void            PRSBriefDestroyList(PRSBriefPres_p  preslist);
PRSBriefPres_p  PRSBriefCreate(
        const char      *options,
        PPTtl_p         ttl,
        FCVSynBuf_p     synbuf,
        int             *columnwidths,
        const char      *appldata);
PRSBriefPres_p  PRSBriefCreateCB(
        const char	*options,
        PPTtl_p		ttl,
        FCVSynBuf_p	synbuf,
        int		*columnwidths,
        const char	*appldata,
	PRSCB_p		cb);

bool PRSHolding(const char *options,
              PRSInterpreter_f	interpreter,
              PPTtl_p     ttl,
              FCVSynBuf_p synbuf,
              size_t      ttlnr,
              char       *destbuf,
              size_t      destbufsize);
bool PRSHoldingCB(
	const char	*options,
	PPTtl_p		ttl,
	FCVSynBuf_p	synbuf,
	size_t		ttlnr,
	char		*destbuf,
	size_t		destbufsize,
	PRSCB_p		cb);

bool
PRSBriefColumnise(
        PRSBriefPres_p  preslist,
        const char      *options,
	bool            truncation_allowed,
        int             maxcols,
        size_t          maxtotalsize);

PRSBriefPres_p
PRSBriefTaggedCreate(
        const char      *options,
        PPTtl_p         ttl,
        FCVSynBuf_p     MatcodeSyn,
        FCVSynBuf_p     FullSyn,
        FCVSynBuf_p     FilterSyn,
        int             *columnwidths,
        const char      *appldata);
PRSBriefPres_p
PRSBriefTaggedCreateCB(
        const char	*options,
        PPTtl_p		ttl,
        FCVSynBuf_p	MatcodeSyn,
        FCVSynBuf_p	FullSyn,
        FCVSynBuf_p	FilterSyn,
        int		*columnwidths,
        const char	*appldata,
	PRSCB_p		cb);

typedef struct PRSBriefSpecEntry_t {
    int         bse_prestype;
    int         bse_subsystem;
    int         bse_filenumber;
    char        bse_matcode[3];         /* materialcode + '\0' */
    char        bse_fcvkey[100];        /* FCV-key + '\0' */
    int         bse_width[PRS_BRIEF_MAX_COLUMNS];
} PRSBriefSpecEntry_t, *PRSBriefSpecEntry_p;

typedef struct PRSBriefSpec_t *PRSBriefSpec_p;

PRSBriefSpec_p PRSBriefCompile(
        const char *text);

PRSBriefSpecEntry_p PRSBriefMatch(
        PRSBriefSpec_p table, int prestype,
        int subsystem, int filenumber, const char *matcode);

int
PRSBriefMatchSet(
    PRSBriefSpec_p  table,
    int             prestype,
    int             subsystem,
    int             filenumber);

bool
PRSPPLinkageRenumber(
	const char	*options,
	PPTtl_p		ttl);

bool
PRSTtlScriptSort(
	PPTtl_p		ttl,
	const char	*FCVoptions);

// Name and values of the FORMATSPEC OPTION "prs-subf_spacing"
#define PRS_SUBF_SPACING_OPT_NAME	"subf_spacing"
typedef enum {
    PRS_SS_NONE,
    PRS_SS_AROUND,	// "subf_spacing=around"
    PRS_SS_SENTINEL	// Keep this one last
} PRSSubfSpacingOpt_t;
PRSSubfSpacingOpt_t PRSSubfSpacingOptFetch(const char *optval);
    
#endif /* ! defined(PRS_H_INCLUDED) */
