/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File        : ttl.h
 *  Description : The interface for the TTL-library
 *  Author(s)   : Hans van Eck
 *  Creation    : 07-03-1996
 *
 *  History:
 *  07-03-1996  : eck      : creation
 *  30-07-1996  : michel   : [1] timestamp_t removed (declared in sqlddl.h)
 *			     [2] #define TTL_H_INCLUDED: added
 *  07-08-1996  : michel   : TTLNumberIn2Ex() and TTLNumberEx2In(): added
 *  03-07-1997	: HSi	   : Changed types according to picatypes.h
 *  04-07-1997	: HSi	   : TTLBuf -> PICTTLBuf.
 *  17-07-1997	: HSi	   : Added TTLDBSet.
 *  20-11-1997	: HSi	   : Changed interfaces of TTLStore,Update,Delete
 *			     ( PICTTLBuf_t -> const PICTTLBuf_t *).
 *  19-09-1997  : paul     : TTLReadLogical() added
 *  01-12-1997	: HSi	   : The Tandem and the Unix variant use different
 *			     typedefinitions. The Unix variant used the
 *			     the typedefinitions from picatypes.h
 *			     The Unix variant uses local definitions and
 *			     definitions from sqlddl.h
 *  04-12-1997	: HSi	   : Fixed type in name TTLStore.
 *  20-01-1998	: lolke    : Merged Tandem and Pica variant.
 *  05-03-1998  : eric     : Added TTLBuf_t field ttl_matcode
 *  10-03-1998  : lolke    : ttl_timestamp_t equal for all platforms (TANDEM variant chosen)
 *  15-04-1998  : eric     : function prototypes from ttlidx.c added; 
 *                           TTLE_INDEXING added
 *  28-04-1998  : eric     : TTLDupKey_t, _p added/modified
 *  26-05-1998  : eric     : TTLBuf_t is now defined as FIOTitle_t (fio.h) Has
 *                           a lot more fields, so servers should access these
 *                           fields rather than title tags. no previously
 *                           existing fields were removed or changed.
 *  26-05-1998  : eric     : Interfaces of TTLStore, Update, Delete:
 *			     TTLBuf_t * can no longer be const because it
 *                           has to be modified
 *  12-06-1998  : eric     : TTLGetPpnTab added
 *  06-07-1998  : eric     : ttl_timestamp_ -> TTLTimestamp; same for ttl_date_
 *                           TTLGetPpnTab (removed) -> PMGetPpnTab
 *  08-07-1998  : eric     : TTLGetPpnTab back, but accesses ppntab from 
 *                           ttlhandle
 *  15-07-1998  : fb       : Added definition of TTLParameters_t. Added 
 *                           TTLParameters to interface of TTLReadLogical.
 *                           xpdhandle_t -> TTLXpdHandle_t.
 *  21-08-1998  : eric     : Everything having to do with indexing and
 *                           timestamping removed
 *  13-01-1999  : fb       : Updated TTLParameters_t to use sourcecode_t
 *  10-03-1999  : eh       : Interface of TTLDBClose changed
 *  20-09-1999	: lolke    : TTLGetFIOHandle also available under UNIX.
 *  11-04-2000	: BdB	: Add TTLReadForIndex
 *  25-04-2000	: BdB	: Add TTLDBGetFileSet
 *  23-05-2000	: BdB	: Add TTLReadForBulkindex
 *  22-11-2000	: eck      : #ifdef-fed TTLNewNumber() for Tandem only
 *  31-03-2003  : HvdW     : Tandem ifdefs removed
 *  18-09-2003  : paul     : Changed TTL_LVL_MAIN  from 0 to PIC_LEVEL_MAIN
 *                         : Changed TTL_LVL_LOCAL from 1 to PIC_LEVEL_LOCAL
 *                         : Changed TTL_LVL_COPY  from 2 to PIC_LEVEL_COPY
 *  11-02-2008  : evd      : Added TTL_LVL_SOCIAL = PIC_LEVEL_SOCIAL = 3
 *  19-03-2009	: paul     : Removed TTLReadForIndex() + TTLReadForBulkIndex()
 *                         : Moved titleread_callback_tp to pccttl.h
 *
 *  Notes:
 *
 *========================================================================
 */

#pragma ident "$Id: ttl.h 335828 2016-06-10 11:33:48Z heesakkm $"

#ifndef TTL_H_INCLUDED
#define TTL_H_INCLUDED

#include <stddef.h>
#include <pica.h>
#include <picatypes.h>
#include <fio.h>
#include <fiottlpublic.h>
#include <picamisc.h>
#include <pp.h>

#ifdef  __cplusplus
#ifndef bool
#define __bool_defined_here
#define bool Cbool
#elif bool != Cbool
#error "When #define'd 'bool' must be 'Cbool'"
#endif
extern "C" {
#endif

#define TTL_FLIPPER     (-1)

#define TTL_MAXBUF               FIOTTL_MAXBUFFER
#define TTL_MAXMATERIALCODE      FIOTTL_MAXMATERIALCODE
#define TTL_MAXSELECTIONKEY      FIOTTL_MAXSELECTIONKEY
#define TTL_MAXUSEDBYILNS        FIOTTL_MAXUSEDBYILNS
#define TTL_MAXBUFFER            FIOTTL_MAXBUFFER

#define TTL_LVL_MAIN    PIC_LEVEL_MAIN
#define TTL_LVL_LOCAL   PIC_LEVEL_LOCAL
#define TTL_LVL_COPY    PIC_LEVEL_COPY
#define TTL_LVL_SOCIAL  PIC_LEVEL_SOCIAL

ERROR_TYPE(TTLErrno_t);

ERROR_DECLARE(TTLE_NOERROR);
ERROR_DECLARE(TTLE_NOMEM);              /* ??? HvE added */
ERROR_DECLARE(TTLE_DATABASE);           /* database error */
ERROR_DECLARE(TTLE_DUPLICATEKEY);       /* duplicate key (on write?) */
ERROR_DECLARE(TTLE_EOF);                /* eof */
ERROR_DECLARE(TTLE_ERROR);              /* unspecified internal error */
ERROR_DECLARE(TTLE_OPTION);             /* invalid option (value?) */
ERROR_DECLARE(TTLE_SECURITY);           /* security violation */
ERROR_DECLARE(TTLE_TIMESTAMP);          /* invalid timestamp, record changed */
ERROR_DECLARE(TTLE_BUFFER);             /* buffer overflow */
ERROR_DECLARE(TTLE_NOTSUP);             /* function not supported */
ERROR_DECLARE(TTLE_KEYNOTFOUND);	/* no record with specified key */
ERROR_DECLARE(TTLE_LOGGING);	        /* logging error */
ERROR_DECLARE(TTLE_FORMAT);	/* Error in internal PP format */
ERROR_DECLARE(TTLE_DEADLOCK);

extern TTLErrno_t TTLerrno;

typedef struct TTLHandle_t *TTLHandle_p;

typedef FIOTimestamp_t TTLTimestamp_t;
typedef FIODate_t      TTLDate_t;

typedef FIOTitle_t TTLBuf_t;
typedef FIOTitle_p TTLBuf_p;

typedef short  TTLXpdHandle_t;

typedef struct TTLParameters_t {
    TTLTimestamp_t                  timestamp;
    PICIln_t                        updated_by_iln;
    sourcecode_t                    updated_by_source;
    TTLDate_t                       date_creation;
    sourcecode_t                    created_by_source;
    TTLDate_t                       date_completion;
    sourcecode_t                    completed_by_source;
    TTLDate_t                       date_deletion;
    sourcecode_t                    deleted_by_source;
    PICIln_t                        main_extension;
    PICIln_t                        owner;
    PICFileSet_t                    fileset;
    PICIpn_t                        ipn;
    bool                            active;
    char                            status;
    char                            material_code[FIOTTL_MAXMATERIALCODE + 1];
    STRDEFINE(FIOTTL_MAXUSEDBYILNS) used_by_ilns;
    PICCopyNum_t                    nr_of_copies;
    struct {
        TTLTimestamp_t              timestamp;
        PICIln_t                    updated_by_iln;
        sourcecode_t                updated_by_source;
    } local;
    struct {
        TTLTimestamp_t              timestamp;
        PICIln_t                    updated_by_iln;
        sourcecode_t                updated_by_source;
        PICIpn_t                    epn;
        PICCopyNum_t                num;
        TTLDate_t                   date_selection;
        char                        selection_key[FIOTTL_MAXSELECTIONKEY + 1];
    } copy[PIC_COPYNUM_MAX];
} TTLParameters_t, *TTLParameters_p;

TTLHandle_p TTLDBOpen(int16 set, const char *options);

FIOHandle_p TTLGetFIOHandle(TTLHandle_p ttlhandle);

int16 TTLDBGetFileSet(TTLHandle_p handle);

bool TTLDBSetFileSet(TTLHandle_p ttlhandle, const char *options, int16);

bool TTLDBClose(TTLHandle_p ttlhandle, const char *options);

bool TTLRead (
    TTLHandle_p	 	ttlhandle,
    int16		lev,
    int32		ipn,
    int16		iln,
    int16		occ,
    int32		epn,
    const char *	options,
    TTLBuf_t *		title);
FIOTitleCS_p TTLReadCS (
    TTLHandle_p	 	ttlhandle,
    int16		lev,
    int32		ipn,
    int16		iln,
    int16		occ,
    int32		epn,
    const char *	options);

bool TTLNext (
    TTLHandle_p	 	ttlhandle,	/* handle to file	*/
    int16		level,
    const char *	options,	/* options		*/
    TTLBuf_t *		title);		/* Title part		*/
FIOTitleCS_p TTLNextCS (
    TTLHandle_p	 	ttlhandle,
    int16		level,
    const char *	options);

bool TTLReadLogical(
    TTLHandle_p         ttlhandle, 
    PICIpn_t            ipn,
    PICIln_t            iln,
    PICCopyNum_t        copynum, 
    const char         *options,
    TTLXpdHandle_t      xpdnr,
    char               *outbuf, 
    size_t             *outlen, 
    size_t              outsize, 
    TTLParameters_p     ttlpar);

bool TTLStore(TTLHandle_p ttlhandle, const char *options, TTLBuf_t *ttl_buf);
bool TTLStoreCS(TTLHandle_p ttlhandle, const char *options, FIOTitleCS_p ttlbuf);

bool TTLUpdate(TTLHandle_p ttlhandle, const char *options, TTLBuf_t *ttl_buf);
bool TTLUpdateCS(TTLHandle_p ttlhandle, const char *options, FIOTitleCS_p ttlbuf);

bool TTLDelete(TTLHandle_p ttlhandle, const char *options, const TTLBuf_t *ttl_buf);
bool TTLDeleteCS(TTLHandle_p ttlhandle, const char *options, FIOTitleCS_cp ttlbuf);

bool TTLExpandCodes ( TTLBuf_t *ttl, int iln, char *language );
bool TTLExpandCodesCS ( FIOTitleCS_p ttl, int iln, char *language );

/* This typedef is here only for backward compatibility; do not use anymore */
typedef bool (*titleread_callback_tp)(PICFileSet_t fileset,
				      PICIpn_t ipn,
				      PPTtl_p total,
				      void *userdata);


#ifdef  __cplusplus
} /* extern "C" */
#ifdef __bool_defined_here
#undef bool
#endif
#endif

#endif /* #ifndef TTL_H_INCLUDED */

