<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File		: pm.md
#  Description	: General description of the PM library
#  Author(s)	: Martin Zwaal
#  Creation	: 13-09-2013
#
#  History:
#  13-09-2013	: martinz  : creation
#
#  SVN ident	: $Id: pm.md 375146 2020-03-10 16:26:44Z heesakkm $
-->

The PM library
==============

The Pica-Miscellaneous library contains functions that can be used generally,
but that are not large enough to warrant their own library. These include
functions for handling options-strings passed to the library routines,
conversion-routines to ASCII (as lightweight sprintf()-routines), a routine for
generating a unique symbol, a hash function and routines to handle
range-strings.

Functions may use an options-string or a range-string in their interface:

An options-string consists of concatenated "<name>=<value>"-type strings, separated
by semicolons. This means that the value may not contain a semicolon.

A range-string consists of a number of "<nr>"- or "<nr>-<nr>"-type strings,
separated by comma's or spaces.

Function			| Description
--------------------------------|------------------------------------------
PM_atosize()			| Convert string with size suffix to number
PM_Base64Decode()		| Base64-decoding of a string
PM_Base64Encode()		| Base64-encoding of a string
PM_bit_count()			| Count the number of set bits in a bit string
PM_cluster_num_from_phrase()	| Extract the cluster number from its index phrase
PM_cluster_num_phrase()		| Print a cluster number index phrase
PM_cluster_num_phrase_pos()	| Print a cluster number index phrase that can be used in IDXPhrasePosition()
PM_cluster_type_valid()		| Validate a cluster type
PM_CRC32()			| Calculate Cyclic Redundancy Check
PM_DESDecode()			| Decode a string
PM_DESEncode()			| Encode a string
PM_DESKeyCreate()		| Create a key object from a key string for DES encryption
PM_DESKeyDestroy()		| Destroy a key object
PM_EAN_gen()			| Generate an EAN check digit from a digit-string
PM_EAN_test()			| Test an EAN check-digit in a string
PM_epn_from_ipn()		| Convert an IPN to a EPN
PM_epn_to_ipn()			| Convert an EPN to an IPN
PM_file_tmpnam()		| Generate a unique temporary filename
PM_file_tempnam()		| Generate and allocate a unique temporary filename
PM_gensym()			| Generate a unique symbol string
PM_hash()			| Generate hash-value from string
PM_hex_pack()			| Convert ascii to binary
PM_hex_unpack()			| Convert binary to ascii
PM_hyphenate_isbn()		| Put hyphens in ISBN
PM_hyphenate_issn()		| Put hyphens in ISSN
PM_is_ascii_space()		| Portable version of isspace() for ascii bytes
PM_isbn_10_to_13()		| Convert a 10-digit ISBN to a 13-digit ISBN (a.k.a. Bookland EAN number)
PM_isbn_13_to_10()		| Convert a 13-digit ISBN (Bookland EAN number) to a 10-digit ISBN
PM_itoa()			| Convert integer to an ASCII-string
PM_LBSStamp()			| Get a timestamp
PM_ltoa()			| Convert long to an ASCII-string
PM_MakeSortString()		| Formats a sort string with year, month, day, volume, number, page.
PM_Mod11Gen()			| Generate a check digit from a digit-string
PM_Mod11GenBNF()		| Generate a BNF check digit from a digit-string
PM_Mod11GenISNI()		| Generate an ISNI check digit from a digit-string
PM_Mod11Test()			| Test a check-digit in a string
PM_month2l()			| Convert Month / Season name to integer
PM_old_scramble()		| Scramble NO encryption of code
PM_opt_copyval()		| Copy the value of an options-string
PM_opt_equal()			| Test option-name in options-string
PM_opt_fetch()			| Find and copy specific option
PM_opt_findend()		| Find the end of an option
PM_opt_findname()		| Find the name-part of an option
PM_opt_findnameend()		| Find the end of the name of an option
PM_opt_findnext()		| Find the next option
PM_opt_findstart()		| Find option-name in options-string
PM_opt_findval()		| Find the value part of an option
PM_opt_findvalend()		| Find the end of the value of an option
PM_ppn_from_ipn()		| Convert from IPN to PPN
PM_ppn_tab_create()		| Create a PPN conversion table
PM_ppn_tab_destroy()		| Destroy a PPN conversion table
PM_ppn_to_ipn()			| Convert from PPN to IPN
PM_range_add()			| Add a range to a range-string
PM_range_clean()		| Clean a range-string
PM_range_count()		| Count the number of ranges in a range-string
PM_range_del_number()		| Delete a number from a range-string
PM_range_error()		| Find the first error in a range-buffer
PM_range_from_bits()		| Convert a bit-array to a range-buffer
PM_range_get_bounds()		| Get a <low>,<high> pair from a range-string
PM_range_intersect()		| Intersect two range-buffers
PM_range_test_number()		| Test if a number is in a range-string
PM_range_to_bits()		| Convert a range-buffer to a bit-array
PM_range_to_bits_check()	| Convert a range-buffer to a bit-array and check that all chars in the range-buffer are used
PM_range_unite()		| Unite two range-buffers
PM_range_x_add()		| Add a range to a hexadecimal range-string
PM_range_x_clean()		| Clean a hexadecimal range-string
PM_range_x_count()		| Count the number of ranges in a hexadecimal range-string
PM_range_x_del_number()		| Delete a number from a hexadecimal range-string
PM_range_x_error()		| Find the first error in a hexadecimal range-buffer
PM_range_x_get_bounds()		| Get a <low>,<high> pair from a hexadecimal range-string
PM_range_x_intersect()		| Intersect two hexadecimal range-buffers
PM_range_x_test_number()	| Test if a number is in a hexadecimal range-string
PM_range_x_unite()		| Unite two hexadecimal range-buffers
PM_roman2l()			| Convert Roman numerals to integer
PM_scramble()			| Procedure used for login security
PM_sizetoa()			| Convert a value to a size string
PM_strcmpi()			| Case-insensitive strcmp()
PM_strdup()			| Duplicate a string
PM_strncmpi()			| Case-insensitive strncmp()
PM_strncpy()			| Copy a string over a length, enforcing '\0'-termination
PM_time_CalculateDateTime()	| Get new date and time in specified format
PM_time_DifferDateTime()	| Calculate time difference
PM_time_GetDateTime()		| Get current date and time in specified format
PM_time_local_to_UTC()		| Convert local time to UTC (ISO 8601)
PM_time_MakeCTime()		| Convert time string to time_t
PM_tolower()			| Lowercase single PICA character
PM_toupper()			| Uppercase single PICA character
PM_ultoa()			| Convert unsigned long to an ASCII-string
PM_UsedByIlnsAscToBin()		| Convert used-by-ilns (ranges) to binary (userbits) form
PM_UsedByIlnsBinToAsc()		| Convert userbits to used-by-ilns (ranges)
PM_userbit_next()		| Get index of first set userbit after reference iln
PM_userbit_previous()		| Get index of last set userbit before reference iln
PM_userbit_test()		| Test if user bit (from 001@ $0) is set
PM_Utf32CharRefGet()		| Get an XML character reference
PM_Utf32CharRefPut()		| Put an hexadecimal XML character reference
PM_Utf32CharToUtf8()		| Put a utf32 char into a utf8 string
PM_utoa()			| Convert unsigned to an ASCII-string
PM_version_strcmp()		| Compares two version strings
