/*=====================================================================
 *  Copyright (c) 2004 by OCLC PICA
 *
 *  File        : pm_charset.c
 *  Description : utilities dealing with the PICA character set
 *  Author(s)   : Guido van den Heuvel
 *  Creation    : 21-04-2004
 *
 *  History:
 *  21-04-2004  : Guy     : creation
 *
 *=====================================================================
 */
#pragma ident "$Id: pm_charset.c 305171 2013-11-01 13:41:52Z sieburgw $"

#include <picamisc.h>

static const unsigned char pica_case_tab[256][2] = {
    /* uppercase equivalent, lowercase equivalent */
    {0x00, 0x00},
    {0x01, 0x01},
    {0x02, 0x02},
    {0x03, 0x03},
    {0x04, 0x04},
    {0x05, 0x05},
    {0x06, 0x06},
    {0x07, 0x07},
    {0x08, 0x08},
    {0x09, 0x09},
    {0x0a, 0x0a},
    {0x0b, 0x0b},
    {0x0c, 0x0c},
    {0x0d, 0x0d},
    {0x0e, 0x0e},
    {0x0f, 0x0f},

    {0x10, 0x10},
    {0x11, 0x11},
    {0x12, 0x12},
    {0x13, 0x13},
    {0x14, 0x14},
    {0x15, 0x15},
    {0x16, 0x16},
    {0x17, 0x17},
    {0x18, 0x18},
    {0x19, 0x19},
    {0x1a, 0x1a},
    {0x1b, 0x1b},
    {0x1c, 0x1c},
    {0x1d, 0x1d},
    {0x1e, 0x1e},
    {0x1f, 0x1f},

    {0x20, 0x20}, /* space */
    {0x21, 0x21},
    {0x22, 0x22},
    {0x23, 0x23},
    {0x24, 0x24},
    {0x25, 0x25},
    {0x26, 0x26},
    {0x27, 0x27},
    {0x28, 0x28},
    {0x29, 0x29},
    {0x2a, 0x2a},
    {0x2b, 0x2b},
    {0x2c, 0x2c},
    {0x2d, 0x2d},
    {0x2e, 0x2e},
    {0x2f, 0x2f},

    {0x30, 0x30},
    {0x31, 0x31},
    {0x32, 0x32},
    {0x33, 0x33},
    {0x34, 0x34},
    {0x35, 0x35},
    {0x36, 0x36},
    {0x37, 0x37},
    {0x38, 0x38},
    {0x39, 0x39},
    {0x3a, 0x3a},
    {0x3b, 0x3b},
    {0x3c, 0x3c},
    {0x3d, 0x3d},
    {0x3e, 0x3e},
    {0x3f, 0x3f},

    {0x40, 0x40},
    {0x41, 0x61}, /* A */
    {0x42, 0x62},
    {0x43, 0x63},
    {0x44, 0x64},
    {0x45, 0x65},
    {0x46, 0x66},
    {0x47, 0x67},
    {0x48, 0x68},
    {0x49, 0x69},
    {0x4a, 0x6a},
    {0x4b, 0x6b},
    {0x4c, 0x6c},
    {0x4d, 0x6d},
    {0x4e, 0x6e},
    {0x4f, 0x6f},

    {0x50, 0x70},
    {0x51, 0x71},
    {0x52, 0x72},
    {0x53, 0x73},
    {0x54, 0x74},
    {0x55, 0x75},
    {0x56, 0x76},
    {0x57, 0x77},
    {0x58, 0x78},
    {0x59, 0x79},
    {0x5a, 0x7a}, /* Z */
    {0x5b, 0x5b},
    {0x5c, 0x5c},
    {0x5d, 0x5d},
    {0x5e, 0x5e},
    {0x5f, 0x5f},

    {0x60, 0x60},
    {0x41, 0x61}, /* a */
    {0x42, 0x62},
    {0x43, 0x63},
    {0x44, 0x64},
    {0x45, 0x65},
    {0x46, 0x66},
    {0x47, 0x67},
    {0x48, 0x68},
    {0x49, 0x69},
    {0x4a, 0x6a},
    {0x4b, 0x6b},
    {0x4c, 0x6c},
    {0x4d, 0x6d},
    {0x4e, 0x6e},
    {0x4f, 0x6f},

    {0x50, 0x70},
    {0x51, 0x71},
    {0x52, 0x72},
    {0x53, 0x73},
    {0x54, 0x74},
    {0x55, 0x75},
    {0x56, 0x76},
    {0x57, 0x77},
    {0x58, 0x78},
    {0x59, 0x79},
    {0x5a, 0x7a}, /* z */
    {0x7b, 0x7b},
    {0x7c, 0x7c},
    {0x7d, 0x7d},
    {0x7e, 0x7e},
    {0x7f, 0x7f},

    {0x80, 0x80},
    {0x81, 0x81},
    {0x82, 0x82},
    {0x83, 0x83},
    {0x84, 0x84},
    {0x85, 0x85},
    {0x86, 0x86},
    {0x87, 0x87},
    {0x88, 0x88},
    {0x89, 0x89},
    {0x8a, 0x8a},
    {0x8b, 0x8b},
    {0x8c, 0x8c},
    {0x8d, 0x8d},
    {0x8e, 0x8e},
    {0x8f, 0x8f},

    {0x90, 0x90},
    {0x91, 0x91},
    {0x92, 0x92},
    {0x93, 0x93},
    {0x94, 0x94},
    {0x95, 0x95},
    {0x96, 0x96},
    {0x97, 0x97},
    {0x98, 0x98},
    {0x99, 0x99},
    {0x9a, 0x9a},
    {0x9b, 0x9b},
    {0x9c, 0x9c},
    {0x9d, 0x9d},
    {0x9e, 0x9e},
    {0x9f, 0x9f},

    {0xa0, 0xa0},
    {0xa1, 0xb1}, /* L dash */
    {0xa2, 0xb2}, /* O slash */
    {0xa3, 0xb3}, /* D dash */
    {0xa4, 0xb4}, /* Thorn */
    {0xa5, 0xb5}, /* AE */
    {0xa6, 0xb6}, /* OE */
    {0xa7, 0xa7},
    {0xa8, 0xa8},
    {0xa9, 0xa9},
    {0xaa, 0xaa},
    {0xab, 0xab},
    {0xac, 0xbc}, /* O hook */
    {0xad, 0xbd}, /* U hook */
    {0xae, 0xae},
    {0xaf, 0xbf}, /* Angstrom */

    {0xb0, 0xb0},
    {0xa1, 0xb1}, /* l dash */
    {0xa2, 0xb2}, /* o slash */
    {0xa3, 0xb3}, /* d dash */
    {0xa4, 0xb4}, /* thorn */
    {0xa5, 0xb5}, /* ae */
    {0xa6, 0xb6}, /* oe */
    {0xb7, 0xb7},
    {0x49, 0xb8}, /* lowercase i without dot */
    {0xb9, 0xb9},
    {0xba, 0xba},
    {0xbb, 0xbb},
    {0xac, 0xbc}, /* o hook */
    {0xad, 0xbd}, /* u hook */
    {0xbe, 0xbe}, 
    {0xaf, 0xbf}, /* angstrom */

    {0xc0, 0xd0}, /* IJ */
    {0xc1, 0xd1}, /* A umlaut */
    {0xc2, 0xd2}, /* O umlaut */
    {0xc3, 0xd3}, /* U umlaut */
    {0xc4, 0xd4}, /* inverted C */
    {0xc5, 0xd5}, /* inverted E */
    {0xc6, 0xc6},
    {0xc7, 0xc7},
    {0xc8, 0xc8},
    {0xc9, 0xc9},
    {0xca, 0xca},
    {0xcb, 0xcb},
    {0xcc, 0xcc},
    {0xcd, 0xcd},
    {0xce, 0xce},
    {0xcf, 0xcf},

    {0xc0, 0xd0}, /* ij */
    {0xc1, 0xd1}, /* a umlaut */
    {0xc2, 0xd2}, /* o umlaut */
    {0xc3, 0xd3}, /* u umlaut */
    {0xc4, 0xd4}, /* inverted c */
    {0xc5, 0xd5}, /* inverted e */
    {0xd6, 0xd6},
    {0xd7, 0xd7},
    {0xd8, 0xd8},
    {0xd9, 0xd9},
    {0xda, 0xda},
    {0xdb, 0xdb},
    {0xdc, 0xdc},
    {0xdd, 0xdd},
    {0xde, 0xde},
    {0xdf, 0xdf},

    {0xe0, 0xe0},
    {0xe1, 0xe1},
    {0xe2, 0xe2},
    {0xe3, 0xe3},
    {0xe4, 0xe4},
    {0xe5, 0xe5},
    {0xe6, 0xe6},
    {0xe7, 0xe7},
    {0xe8, 0xe8},
    {0xe9, 0xe9},
    {0xea, 0xea},
    {0xeb, 0xeb},
    {0xec, 0xec},
    {0xed, 0xed},
    {0xee, 0xee},
    {0xef, 0xef},

    {0xf0, 0xf0},
    {0xf1, 0xf1},
    {0xf2, 0xf2},
    {0xf3, 0xf3},
    {0xf4, 0xf4},
    {0xf5, 0xf5},
    {0xf6, 0xf6},
    {0xf7, 0xf7},
    {0xf8, 0xf8},
    {0xf9, 0xf9},
    {0xfa, 0xfa},
    {0xfb, 0xfb},
    {0xfc, 0xfc},
    {0xfd, 0xfd},
    {0xfe, 0xfe},
    {0xff, 0xff}
};

/*---------------------------------------------------------------------
 *  Function    : PM_tolower
 *  Purpose     : downcase single PICA character
 *  Method      :
 *
 *  History:
 *  21-04-2004  : Guy     : creation
 *
 *  Returns     : the lowercase equivalent of the input
 *---------------------------------------------------------------------
 */
unsigned int
PM_tolower(
	   unsigned int char_in
	   )
{
    return pica_case_tab[char_in][1];
}  /* PM_tolower */

/*---------------------------------------------------------------------
 *  Function    : PM_toupper
 *  Purpose     : upcase single PICA character
 *  Method      :
 *
 *  History:
 *  21-04-2004  : Guy     : creation
 *
 *  Returns     : the uppercase equivalent of the input
 *---------------------------------------------------------------------
 */
unsigned int
PM_toupper(
	   unsigned int char_in
	   )
{
    return pica_case_tab[char_in][0];
}  /* PM_toupper */

/*------------------------------------------------------------------------
 *  Function	: PM_is_ascii_space
 *  Purpose	: Portable version of isspace() for ascii bytes
 *  Method	:
 *
 *  History:
 *  21-03-2013	: sieburgw : creation
 *
 *  Notes	: isspace() may return TRUE for some none-ascii bytes
 *
 *  Returns	: 1 in case of a space char, 0 otherwise
 *------------------------------------------------------------------------
 */
int
PM_is_ascii_space(
	int c)
{
    if (c & 0x80){
	return 0; /* non-ascii */
    }
    switch (c){
    case '\t':
    case '\r':
    case '\n':
    case '\v':
    case '\f':
    case ' ':
	return 1; /* one of the space chars for the C locale */
    default:
	return 0;
    }
} /* PM_is_ascii_space() */
