/*========================================================================
 *  Copyright (c) 2018 by OCLC
 *
 *  File	: pm_lang.c
 *  Description	: Miscelaneous language functions
 *
 *  History:
 *  04-04-2018	: smitp    : CBS-7486 creation
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: pm_lang.c 354663 2018-04-04 09:34:20Z smitp $"

#include <string.h>

#include <pica.h>
#include <picamisc.h>

/*------------------------------------------------------------------------
 *  Function	: PM_LangToISO639_1
 *  Purpose	: Convert a CBS language to ISO 639-1
 *
 *  History:
 *  04-04-2018	: smitp    : CBS-7486 creation
 *
 *  Returns	:
 *  - a string constant representing the equivalent language in ISO 639-1
 *  - or NULL in case of unknown language
 *
 *------------------------------------------------------------------------
 */
const char *
PM_LangToISO639_1(
	const char	*langCBS)
{
    const char *langISO = NULL;

    if (langCBS == NULL) {
	langISO = NULL;
    } else if (STREQ(langCBS, "NE")) {
	langISO = "nl";
    } else if (STREQ(langCBS, "FR")) {
	langISO = "fr";
    } else if (STREQ(langCBS, "DU")) {
	langISO = "de";
    } else if (STREQ(langCBS, "EN")) {
	langISO = "en";
    } else if (STREQ(langCBS, "TR")) {
	langISO = "tr";
    }

    return langISO;
} // PM_LangToISO639_1()

/*------------------------------------------------------------------------
 *  Function	: PM_LangFromISO639_1
 *  Purpose	: Convert an ISO 639-1 language to CBS
 *
 *  History:
 *  04-04-2018	: smitp    : CBS-7486 creation
 *
 *  Returns	:
 *  - a string constant representing the equivalent language in ISO 639-1
 *  - or NULL in case of unknown language
 *
 *------------------------------------------------------------------------
 */
const char *
PM_LangFromISO639_1(
	const char	*langISO)
{
    const char *langCBS = NULL;

    if (langISO == NULL) {
	langCBS = NULL;
    } else if (STREQ(langISO, "nl")) {
	langCBS = "NE";
    } else if (STREQ(langISO, "fr")) {
	langCBS = "FR";
    } else if (STREQ(langISO, "de")) {
	langCBS = "DU";
    } else if (STREQ(langISO, "en")) {
	langCBS = "EN";
    } else if (STREQ(langISO, "tr")) {
	langCBS = "TR";
    }

    return langCBS;
} // PM_LangFromISO639_1()

