/*========================================================================
 *  Copyright (c) 2015 by OCLC
 *
 *  File	: pm_math.c
 *  Description	: Miscellaneous mathematical functions
 *
 *  History:
 *  17-03-2015	: paul     : creation
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: pm_math.c 325092 2015-04-02 15:15:14Z heesakkm $"

#include <pica.h>

#include <picamisc.h>

/*------------------------------------------------------------------------
 *  Function	: PM_math_pow2roundup
 *  Purpose	: Round up to power of 2 that is equal or higher
 *
 *  History:
 *  16-03-2015	: paul     : CBS-6100 creation
 *
 *  Returns	: power of 2 that is equal to or higher than the given number
 *                or zero if it doesn't fit into 32 bits
 *
 *------------------------------------------------------------------------
 */
uint32
PM_math_pow2roundup(
	uint32	x)
{
    if (x == 0) return 1;
    if (--x & 0x80000000) return 0;

    x |= x >> 1;
    x |= x >> 2;
    x |= x >> 4;
    x |= x >> 8;
    x |= x >> 16;

    return x + 1;
} // PM_math_pow2roundup()

