/*========================================================================
 *  Copyright (c) 2008 by OCLC
 *
 *  File	: pm_version.c
 *  Description	: Compares two version strings.
 *  Author(s)	: Clemens Buys
 *  Creation	: 10-11-2008
 *
 *  History:
 *  10-11-2008	: clemens  : creation
 *
 *========================================================================
 */

#pragma ident "$Id: pm_version.c 305171 2013-11-01 13:41:52Z sieburgw $"

#include <stdlib.h>
#include <string.h>

#include <pica.h>
#include <picamisc.h>

/*------------------------------------------------------------------------
 *  Function	: PM_version_strcmp
 *  Purpose	: Compares two version strings.
 *  Method	:
 *
 *  History:
 *  10-11-2008	: clemens  : creation
 *
 *  Notes	:
 *  This function expects the input to be version strings containing only
 *  digits and dots.
 *  Missing elements in a version string are taken to be 0.
 *
 *  Returns	:
 *  - <0 : desired version smaller than client version
 *  - =0 : desired version and client version are identical
 *  - >0 : desired version greater than client version
 *------------------------------------------------------------------------
 */
int
PM_version_strcmp(
        const char      *desired_version,       /* e.g. "3.3" */
        const char      *client_version)        /* e.g. "3.3.0.1" */
{
    int dv;
    int cv;

    while ((desired_version != NULL) && (*desired_version != '\0')) {
	dv = atoi(desired_version);
	cv = client_version ? atoi(client_version) : 0;

	if (dv != cv) {
	    return dv - cv;
	}

	desired_version = strchr(desired_version, '.');
	if (desired_version != NULL) {
	    desired_version++;
	}
	if (client_version) {
	    client_version = strchr(client_version, '.');
	    if (client_version != NULL) {
		client_version++;
	    }
	}
    }
    while ((client_version != NULL) && (*client_version != '\0')){
	cv = atoi(client_version);
	if (cv != 0) return -cv;

	client_version = strchr(client_version, '.');
	if (client_version != NULL) {
	    client_version++;
	}
    }
    return 0;
} /* PM_version_strcmp() */
