/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: pmhexpack.c
 *  Description	: Convert from ascii to binary and vice versa and pack every
 *  		  ascii byte to a half binary byte and the other way around a
 *  		  binary byte expands to two ascii bytes.
 *  Author(s)	: Rudy Tan
 *  Creation	: 27-08-1996
 *
 *  History:
 *  27-08-1996	: rudy     : creation
 *  12-09-2013	: martinz  : Added documentation from PMLIB.DOC.
 *
 *========================================================================
 */

#pragma ident "$Id: pmhexpack.c 335412 2016-05-27 11:01:38Z heesakkm $"

#include <string.h>

#include <pica.h>
#include <picamisc.h>

/*------------------------------------------------------------------------
 *  Function    : PM_hex_unpack
 *  Purpose     : Converts data into a hexrepresentation. ie 0x3F => "3F".
 *  Method      :
 *  Author(s)   : Rudy Tan
 *  Creation    : 01-07-1996
 *
 *  History:
 *  01-07-1996  : rudy          : creation
 *  19-08-1996  : rudy/lolke: Interface changed
 *
 *  Notes       : Actlen is the number of bytes to be unpacked
 *
 *  Returns     :
 *  The number of converted half-bytes. So this may be uneven, same as outsiz
 *------------------------------------------------------------------------
 */
size_t
PM_hex_unpack(
       char *out,
       size_t outsiz,
       const byte *in,
       int packedlen)
{
    size_t i;
    unsigned char tmp = 0x00;
    unsigned char *ptr1 = (unsigned char *)out;
    unsigned char *ptr2 = (unsigned char *)in;

    if (packedlen < 0) return 0;
    i = 0;
    while (( i <= outsiz) && (packedlen != 0)){
        tmp = ptr2[0];
        tmp >>= 4;
        tmp &= 0x0F;
        if (tmp <= 9) {
            tmp += '0';
        } else {
            tmp += ('A' - 0x0A);
        }
        ptr1[i] = tmp;
        i++;
        if (i <= outsiz) {
            tmp = *ptr2++;
            tmp &= 0x0F;
            if (tmp <= 9) {
                tmp +=  '0';
            } else {
                tmp += ('A' - 0x0A);
            }
            ptr1[i] = tmp;
            i++;
            packedlen--;
        }
    }
    return i;
} /* PM_hex_unpack() */

/*------------------------------------------------------------------------
 *  Function    : PM_hex_pack
 *  Purpose     : Converts a HEXARRAY to data. ie "3F" => 0x3F
 *  Method      : Yes.
 *  Author(s)   : Rudy Tan
 *  Creation    : 01-07-1996
 *
 *  History:
 *  01-07-1996  : rudy     : creation
 *
 *------------------------------------------------------------------------
 */
size_t
PM_hex_pack(
	byte *out,
	size_t	out_size,
	const char *in,
	int unpackedlen)
{
    unsigned char tmp = 0x00;
    unsigned char *ptr1 = (unsigned char *) in;
    unsigned char *ptr2 = (unsigned char *) out;

    if (unpackedlen < 0) unpackedlen = (int) strlen(in);
    while ((out_size != 0) && (unpackedlen != 0)) {
        tmp = *ptr1++;
        unpackedlen--;
        if (tmp <= '9') {
            tmp -= '0';
        } else {
            tmp -= ('A' - 0x0A);
        }
        *ptr2 = (tmp<<4 & 0xF0);
        if (unpackedlen != 0) {
            tmp = *ptr1++;
            unpackedlen--;
            if (tmp <= '9') {
                tmp -= '0';
            } else {
                tmp -= ('A' - 0x0A);
            }
            *ptr2++ |= tmp;
        }
        out_size--;
    }
    return (size_t) (ptr2 - (unsigned char *) out);
} /* PM_hex_pack() */
