<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File		: p3.md
#  Description	: General description of the P3 library
#  Author(s)	: Martin Zwaal
#  Creation	: 16-10-2013
#
#  History:
#  16-10-2013	: martinz  : creation
#
#  SVN ident	: $Id: p3.md 339156 2016-11-18 18:26:14Z heesakkm $
-->

The P3 library
==============

Pica3 Database Access
---------------------
The P3 library describes the interaction between a client and a Pica3 server.
That interaction is based on the Pica3 protocol, developed by Pica.

The Pica3 protocol is an open protocol, quite different from international
protocols like Z39.50 or SR (Search&Retrieve).

When we simplify some things you may say that a Pica3 message is a variable
length Ascii string, composed of one or more message-parts. Each message-part
contains a specific piece of information and consists of a header, the real
information and an end-delimiter.

The following message-parts are defined:

 Message   | Description
 ----------|:------------------------------------------------------------
 command   | one or more (concatenated) command(s) to instruct the server
 buffer    | information entered at screens at the client
 message   | errors, warnings and confirmations
 screenid  | screen identification
 screendef | screen definition
 window    | information to be displayed at screens at the client
 history   | (search) history commands
 variable  | variables (context and global), mostly used to synchronize between client and server

All message-parts but variables are sent one-way; either to the server or to
the client:

    +))))))))))),               +))))))))))),
    * Client    /))))))))))))))>1 Server    *
    *           *  command      *           *
    *           *  buffer       *           *
    *           *               *           *
    *           /)<)))))))))))))1           *
    *           *  screenid     *           *
    *           *  screendef    *           *
    *           *  window       *           *
    *           *  message      *           *
    *           *  history      *           *
    *           *               *           *
    *           /)<))))))))))))>1           *
    *           *  variable     *           *
    .)))))))))))-               .)))))))))))-

The variety of Pica3 functions depends on the Pica3 server. Not only search and
retrieve facilities, but also ILL and database functions may be supported.

In this document, a basic set of functions is specified. Each function call
will normally result in sending a command to the Pica3 server (with or without
buffer information) and analysing the reply from the server.

The Pica3 library will (internally) use the worm library (WM) to manipulate
data to be sent to and retrieve from the server. The
connect/disconnect/send-/receive operations are performed by calling the
appropriate routines from the FE library.

The external interface to the Pica3 routines is kept simple and, as far as
possible, identical to equivalent functions in the SRV library.

Attention: not all P3 functions may be used at any time.
The following rules must be obeyed:

- the first routine to be called must be P3Open(), which establishes a physical
  connection to the Pica3 server. As long as there is no valid connection, no
  other routines may be called.
- a connection may be closed by calling P3Close().
- after P3Open() has been called successfully, a call to P3Logon must be
  performed. P3Logon() connects to the Pica3 server application itself unless
  the authorization failed.
- after P3Logon() has been called successfully, only the routines P3List() and
  P3Select() may be called. Another call to P3Logon() is allowed too. P3List()
  returns a list of available databases, P3Select() selects a database.
- only when P3Select() has been called successfully, the other functions may be
  called.

Beware that the information replied by the Pica3 server may depend on the user
that logged on and/or the database that was selected.

Pica3 Database Access Functions
-------------------------------

The following functions are defined:

Function			| Description
:-------------------------------|:-------------------------
P3Open()			| connect to a Pica3 server
P3Close()			| disconnect a Pica3 server
P3Logon()			| log-on to a Pica3 server
P3List()			| retrieve list of databases
P3Select()			| select a database
P3Info()			| retrieve info about opened database
P3Option()			| set option
P3ScanDbsUpdates()		| scan database updates
P3FetchTitle()			| fetch (part of) a title
P3SelDbsUpdates()		| select database updates
P3XLWMOpen()			| open a connection
P3XLWMClose()			| close a connection
P3XLWMLogon()			| send logon command
P3XLWMSelect()			| select a database as the current one
P3XLWMScanDbsUpdates()		| scan database updates
P3XLWMFetchTitle()		| fetch (part of) a title
P3XLWMSelDbsUpdates()		| select database updates
