/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: p3close.c
 *  Description	: The close routine for a Pica3-server
 *  Author(s)	: Hans van Eck
 *  Creation	: 06-09-1994
 *
 *  History:
 *  06-09-1994	: eck      : creation
 *  18-12-2001  : evd      : Added P3XLWMClose
 *  10-10-2013	: martinz  : Copied documentation from P3LIB.DOC.
 *
 *========================================================================
 */

#pragma ident "$Id: p3close.c 305171 2013-11-01 13:41:52Z sieburgw $"

#include <stdlib.h>

#include <pica.h>
#include <p3.h>
#include <fep.h>

#include "p3local.h"

/*------------------------------------------------------------------------
 *  Function	: P3Close
 *  Purpose	: Close a connection to a Pica3-server
 *  Method	: Call FEClose().
 *
 *  History:
 *  05-10-1994	: eck      : creation
 *
 *  Description	:
 *  - P3handle must be a valid P3Handle_p previously returned by P3Open() (and
 *    not already closed).
 *
 *  This function will disconnect the Pica3 server. When the disconnect fails
 *  FALSE is returned and P3errno should be examined.
 *
 *  Error values:
 *
 *	P3E_DISCONNECT	(server disconnected)
 *	P3E_NOERROR	(ok)
 *	P3E_PROTOCOL	(protocol error)
 *	P3E_SERVER	(server error)
 *
 *  Returns	: Indication of success
 *------------------------------------------------------------------------
 */
bool
P3Close(
	P3Handle_p	P3handle)		/* Handle to close */
{
    p3_connection_p	p3_conn = P3C_GetConnection(P3handle);

    if (p3_conn != NULL) {
	FEStop(p3_conn->p3cn_fe_handle);
	(void) P3C_DelConnection(p3_conn);
    } else {
	P3errno = P3E_SERVER;
	(void) ERWrite(P3errno, "P3Close" , "invalid handle");
    }

    return (P3errno == P3E_NOERROR);
} /* P3Close() */

/*------------------------------------------------------------------------
 *  Function	: P3XLWMClose
 *  Purpose	: Close a connection to a Pica3-server
 *  Method	: Call NIOClose().
 *
 *  History:
 *  18-12-2001  : evd     : Copied from P3Close
 *
 *  Returns	: Indication of success
 *------------------------------------------------------------------------
 */
bool
P3XLWMClose(
	P3Handle_p	P3handle)		/* Handle to close */
{
    p3_XLWMconnection_p	p3_conn = P3C_XLWMGetConnection(P3handle);

    if (p3_conn != NULL) {
	NIOClose(p3_conn->p3XLWMcn_nio_handle);
	(void) P3C_XLWMDelConnection(p3_conn);
    } else {
	P3errno = P3E_SERVER;
	(void) ERWrite(P3errno, "P3XLWMClose" , "invalid handle");
    }

    return (P3errno == P3E_NOERROR);
} /* P3XLWMClose() */
