/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: p3local.h
 *  Description	: Local declaration for P3-library
 *  Author(s)	: Hans van Eck
 *  Creation	: 06-09-1994
 *
 *  History:
 *  06-09-1994	: eck      : creation
 *  30-10-1997	: HSi	   : Added P3L_SyntaxAdbToP3
 *  13-05-1998	: eck      : Added P3ScanCacheClear()
 *  08-09-1998	: eck      : Added FLG_UPDATE
 *  19-06-2000	: martin   : Added P3GetCatDiagnostic()
 *  13-12-2001  : evd      : Added P3L_XLWMelt2CB
 *  13-12-2001  : evd      : Added p3_XLWMconnection_[tp]
 *  13-12-2001  : evd      : Added P3C_XLWMAddConnection
 *  13-12-2001  : evd      : Added P3C_XLWMGetConnection
 *  13-12-2001  : evd      : Added P3C_XLWMDelConnection
 *  18-12-2001  : evd      : Added P3L_XLWMSendRecv
 *  06-08-2002	: martin   : Added P3L_cnvZeroStrChar
 *  27-08-2004	: huug	   : Added define P3L_CHARSET_MAX
 *  21-04-2007	: clemens  : Increased MAXLEN 4000 --> 8000
 *  14-12-2009	: paul     : Copied CBS_VAR_* defines from pcctypes.h
 *
 *========================================================================
 */

#ifndef P3LOCAL_H_INCLUDED
#define P3LOCAL_H_INCLUDED

#pragma ident "$Id: p3local.h 360943 2018-10-23 16:48:25Z heesakkm $"

#include <pica.h>
#include <picamisc.h>
#include <adb.h>
#include <p3.h>
#include <fep.h>
#include <wm.h>
#include <cs.h>
#include <nio.h>

#define MAXLEN		(WM_MAXWRMLEN > 8000 ? 8000 : WM_MAXWRMLEN)
#define STR_LANGUAGE	"language"

#define P3L_CHARSET_MAX	100

typedef struct p3_XLWMconnection_t	*p3_XLWMconnection_p;

typedef struct p3_XLWMconnection_t {
    P3Handle_p		p3XLWMcn_p3_handle;
    NIOHandle_t		p3XLWMcn_nio_handle;
    WMContext_t		p3XLWMcn_wm_context;
    p3_XLWMconnection_p	p3XLWMcn_next;
} p3_XLWMconnection_t;

typedef struct p3_connection_t	*p3_connection_p;

typedef struct p3_connection_t {
    P3Handle_p		p3cn_p3_handle;
    FEHandle_t		p3cn_fe_handle;
    WMContext_t		p3cn_wm_context;
    p3_connection_p	p3cn_next;
} p3_connection_t;

#define ETX "\003"
#define LF  "\012"
#define FF  "\014"
#define CR  PM_CR
#define ESC PM_ESC
#define EOO "\035"
#define EOV "\036"
#define SEP "\037"

#define P3_VAR_RESPONSE_CHARSET "#C"
#define P3_VAR_CLIENT_CHARSET "!C"
#define P3_VAR_USERID         "!U"
#define P3_VAR_USERNAME       "!P"
#define P3_VAR_LIBRARYID      "!L"
#define P3_VAR_LIBRARYNAME    "!B"
#define P3_VAR_CURSORADDR     "!A"
#define P3_VAR_CURSORFIELD    "!V"
#define P3_VAR_CURSORPOS      "!P"
#define P3_VAR_CURSORLINE     "!R"
#define P3_VAR_COPYSIGNAL     "!K"
#define P3_VAR_VERSION        "!S"
#define P3_VAR_TERMTYPE       "!T"
#define P3_VAR_PURGECACHE     "!X"
#define P3_VAR_HOSTID         "#S"
#define P3_VAR_ERROR          "/V"
#define P3_VAR_CATSTATUS      "/C"
#define P3_VAR_ILLREQNUM      "A*"
#define P3_VAR_ADDRORG        "A1"
#define P3_VAR_ADDRDEP        "A2"
#define P3_VAR_ADI            "AD"
#define P3_VAR_HISTLINE       "CO"
#define P3_VAR_ILLSHORTFORMAT "AK"
#define P3_VAR_ILLFULLFORMAT  "AL"
#define P3_VAR_PURGILLCACHE   "AX"
#define P3_VAR_MAILSYSTEM     "B1"
#define P3_VAR_MAILDBASENAME  "B2"
#define P3_VAR_MAILORG        "B3"
#define P3_VAR_MAILDEP        "B4"
#define P3_VAR_MAILUSER       "B5"
#define P3_VAR_MAILSTARTDATE  "B6"
#define P3_VAR_MAILSTOPDATE   "B7"
#define P3_VAR_MAILSURFNAME   "B8"
#define P3_VAR_MAILFLAGS      "B9"
#define P3_VAR_FILE           "BE"
#define P3_VAR_REMOTEDBSID    "RI"
#define P3_VAR_REMOTEDBSNAME  "RN"
#define P3_VAR_REMOTETITLE    "RT"
#define P3_VAR_DBASEID        "BI"
#define P3_VAR_DBASENUM       "BN"
#define P3_VAR_MAILNUM        "BR"
#define P3_VAR_MAILSETSIZE    "BS"
#define P3_VAR_ILLPERMIT      "BA"
#define P3_VAR_EDITFORMAT     "DB"
#define P3_VAR_DOWNLFORMAT    "DD"
#define P3_VAR_SHORTFORMAT    "DK"
#define P3_VAR_FULLFORMAT     "DL"
#define P3_VAR_EUSLASTACCESS  "EA"
#define P3_VAR_EUSBALANCE     "EB"
#define P3_VAR_EUSLIB         "EL"
#define P3_VAR_EPN            "EP"
#define P3_VAR_HELPITEM       "HI"
#define P3_VAR_INDEXPOS       "IP"
#define P3_VAR_ILLSETSIZE     "IS"
#define P3_VAR_INDEXTYPE      "IT"
#define P3_VAR_LEVEL          "LV"
#define P3_VAR_LIBRARY        "OI"
#define P3_VAR_ILN            "OJ"
#define P3_VAR_LIBSHORTNAME   "LS"
#define P3_VAR_LIBMNEMO       "LM"
#define P3_VAR_MORE           "MO"
#define P3_VAR_MAILNUMBER     "MR"
#define P3_VAR_MULTIRECTYPE   "OT"
#define P3_VAR_PAGENUM        "PA"
#define P3_VAR_PPN            "PP"
#define P3_VAR_PRESFORMAT     "PR"
#define P3_VAR_RECTYPE        "RC"
#define P3_VAR_SCRAMBLE_SEED  "RR"
#define P3_VAR_SAVESETID      "S0"
#define P3_VAR_SDIREQNUM      "SD"
#define P3_VAR_SET            "SE"
#define P3_VAR_SETID          "SI"
#define P3_VAR_SDISETSIZE     "SS"
#define P3_VAR_SETSIZE        "SZ"
#define P3_VAR_SYSTEM         "SY"
#define P3_VAR_LANGUAGE       "TA"
#define P3_VAR_DATE           "TD"
#define P3_VAR_PREVSCRID      "TG"
#define P3_VAR_TITLE          "TI"
#define P3_VAR_TTLNUMMAX      "TM"
#define P3_VAR_TIMESTAMP      "TS"
#define P3_VAR_USERNUM        "UN"
#define P3_VAR_USRSETSIZE     "UZ"
#define P3_VAR_USRPAGESIZE    "UP"
#define P3_VAR_USRSEQ         "US"
#define P3_VAR_SCRVERSION     "VE"
#define P3_VAR_INDEXWORD      "WO"
#define P3_VAR_SAVESETSIZE    "X0"
#define P3_VAR_ILLSLIPNUM     "YN"
#define P3_VAR_ILLSLIPMAX     "YM"

typedef enum {
    P3_OK,		/* request successfully handled */
    P3_NOTSUP,		/* not supported */
    P3_NOAUTH,		/* no authority */
    P3_NOTNOW,		/* request not allowed yet */
    P3_REJECT,		/* transaction rejected */
    P3_NOINFO,		/* no information available */
    P3_NONODE,		/* no node specified */
    P3_NOTDBS,		/* no real database specified */
    P3_MAXSET,		/* set overflow */
    P3_NOHITS,		/* nothing found */
    P3_NOLIBS,		/* no supplying libraries */
    P3_OWNLIB,		/* title part is in own library */
    P3_ERRSET,		/* invalid set */
    P3_ERRTTL,		/* invalid title(s) */
    P3_ERRCMD,		/* invalid command */
    P3_ERRFLD,		/* invalid field */
    P3_EMAIL,		/* invalid email address */
    P3_ERROR,		/* server error */
    P3_MAXCHN,		/* maximum nr. of channels exceeded */
    P3_MRINFO,		/* more information needed */
    P3_STPWRD,		/* search with stopword(s) only */
    P3_LIBDEP,		/* library deposit exhausted */
    P3_USRDEP,		/* user deposit exhausted */
    P3_ERRNUM,		/* invalid number */

    P3_UNKNOWN		/* unknown status returned by server */
} P3Status_t;

extern P3Status_t P3status;

bool P3L_SendRecv(FEHandle_t fe_handle);

void P3L_InternToPica3Name(const char *iname, CSBuf_p oname);

void P3L_ToCache(P3Handle_p P3handle, const P3SetIdent_t SetIdent, long Teltno,
		    const char *option, const char *value);

bool P3L_FromCache(P3Handle_p P3handle, const P3SetIdent_t SetIdent,
		    long Teltno, const char *option,
		    CBBuf_t *CBbuf, size_t CBsize);

p3_connection_p P3C_AddConnection(FEHandle_t fe_handle);

p3_connection_p P3C_GetConnection(P3Handle_p p3_handle);

bool P3C_DelConnection(p3_connection_p p3_conn);

bool P3C_AddToConnection(P3Handle_p p3_handle, const char *attrib,
			const char *value);

const char *P3C_GetFromConnection(P3Handle_p p3_handle, const char *attrib);

void P3L_InsertLanguageCommand(CSBuf_p command, const char *options);

WMContext_t P3L_SwitchAndClearContext(WMContext_t handle);

#define FLG_FIELD	0x01
#define FLG_VAR		0x02
#define FLG_OPTIONAL	0x04
#define FLG_SPLITLINE	0x08
#define FLG_ADDID	0x10
#define FLG_UPDATE	0x20
int P3L_WMelt2CB(const WMIdent_t id, CBBuf_t *CBbuf, size_t CBsize,
		    uint16 options, const char *location);

const char * P3L_SyntaxAdbToP3 (const char *adbSyntax);

void P3ScanCacheClear(void);

int P3GetCatDiagnostic(void);

int P3L_XLWMelt2CB(const WMIdent_t id, CBBuf_t *CBbuf, size_t CBsize,
		    uint16 options, const char *location);

p3_XLWMconnection_p P3C_XLWMAddConnection(NIOHandle_t nio_handle);

p3_XLWMconnection_p P3C_XLWMGetConnection(P3Handle_p p3_handle);

bool P3C_XLWMDelConnection(p3_XLWMconnection_p p3_conn);

bool P3L_XLWMSendRecv(NIOHandle_t nio_handle);

void P3L_cnvZeroStrChar( char *string, size_t length, char searchChar,
		char replaceChar);

#endif /* P3LOCAL_H_INCLUDED */
