/*========================================================================
 *  Copyright (c) 1999 by Pica
 *
 *  File	: p3seldbsupdates.c
 *  Description	:
 *  Copythor(s)	: Els van Doorn
 *  Creation	: 03-02-1999
 *
 *  History:
 *  03-02-1999	: els      : creation
 *  24-12-2001  : evd      : Added P3XLWMSelDbsUpdates
 *  04-07-2003  : huug     : \SRC option in \SEL \UPD command
 *		:	   : reduced duplicate code
 *
 *========================================================================
 */

#pragma ident "$Id: p3seldbsupdates.c 305171 2013-11-01 13:41:52Z sieburgw $"

#include <string.h>

#include <pica.h>
#include <picamisc.h>
#include <p3.h>
#include <cs.h>

#include "p3local.h"

#define IDLEN_MAX	 40

#define STR_SOURCE	"source"
#define STR_LIBRARY	"library"
#define STR_BYLIB	"bylib"
#define STR_RECTYPE	"rectype"

/*------------------------------------------------------------------------
 *  Function	: DetermineSourceArgs
 *  Purpose	: Determine source descriptor from options
 *  Method	: obvious
 *
 *  History:
 *  02-07-2003	: huug      : creation
 *
 *  Returns	: TRUE if a source was specified, FALSE otherwise.
 *------------------------------------------------------------------------
 */
static bool
DetermineSourceArgs(
	const char	*options,
	char		source[IDLEN_MAX + 1])
{
    source[0] = '\0';

    options = PM_opt_findstart(options);
    while ((options != NULL) && (*options != '\0')) {
	if (PM_opt_equal(options, STR_SOURCE)) {
	    options = PM_opt_findval(options);
	    PM_opt_copyval(options, source, IDLEN_MAX + 1);
	}
	options = PM_opt_findnext(options);
    }
    return (source[0] != '\0');
} /* DetermineSourceArgs() */

/*------------------------------------------------------------------------
 *  Function	: DetermineLibraryArgs
 *  Purpose	: Determine values for BIB, DOO and REC from options
 *  Method	: obvious
 *
 *  History:
 *  25-07-1996	: PvdM      : creation
 *
 *  Returns	: TRUE if a library was specified, FALSE otherwise.
 *------------------------------------------------------------------------
 */
static bool
DetermineLibraryArgs(
	const char	*options,
	char		library[IDLEN_MAX + 1],
	char		bylib[IDLEN_MAX + 1],
	char		rectype[IDLEN_MAX + 1])
{
    library[0] = '\0';
    bylib[0] = '\0';
    rectype[0] = '\0';

    options = PM_opt_findstart(options);
    while ((options != NULL) && (*options != '\0')) {
	if (PM_opt_equal(options, STR_LIBRARY)) {
	    options = PM_opt_findval(options);
	    PM_opt_copyval(options, library, IDLEN_MAX + 1);
	}
	if (PM_opt_equal(options, STR_BYLIB)) {
	    options = PM_opt_findval(options);
	    PM_opt_copyval(options, bylib, IDLEN_MAX + 1);
	}
	if (PM_opt_equal(options, STR_RECTYPE)) {
	    options = PM_opt_findval(options);
	    PM_opt_copyval(options, rectype, IDLEN_MAX + 1);
	}
	options = PM_opt_findnext(options);
    }
    return ((library[0] != '\0') || (bylib[0] != '\0') || (rectype[0] != '\0'));
} /* DetermineLibraryArgs() */

/*------------------------------------------------------------------------
 *  Function	: BuildSelUpdCommand
 *  Purpose	: Build \sel \upd command
 *  Method	: CSPrintf()
 *
 *  History:
 *  03-07-2003	: huug     : creation
 *
 *  Notes	: The \sel \upd command has the form:
 *		  \sel Count upd [ \src source ] \sed Offselt [ \doo library ] ;
 *		  source and library are found in the Options string
 *
 *------------------------------------------------------------------------
 */
static void
BuildSelUpdCommand(
	CSBuf_p 	command,
	const char	*Offset,
	int		Count,
	const char	*Options)
{
    char		source[IDLEN_MAX+1];
    char		library[IDLEN_MAX + 1];
    char		bylib[IDLEN_MAX + 1];
    char		rectype[IDLEN_MAX + 1];
    CS_LOCAL(		src_opt, 100);
    CS_LOCAL(		bib_opt, 100);
    CS_LOCAL(		by_opt, 100);
    CS_LOCAL(		rec_opt, 100);

    CSClear(src_opt);
    CSClear(bib_opt);
    CSClear(by_opt);
    CSClear(rec_opt);
    if ( DetermineSourceArgs(Options, source)) {
	CSPrintf(src_opt, "\\src %s ", source);
    }
    if ( DetermineLibraryArgs(Options, library, bylib, rectype)) {
	if (strlen(library) > 0) {
	    CSPrintf(bib_opt, "\\bib %s ", library);
	}
	if (strlen(bylib) > 0) {
	    CSPrintf(by_opt, "\\doo %s ", bylib);
	}
	if (strlen(rectype) > 0) {
	    CSPrintf(rec_opt,"\\rec %s", rectype);
	}
    }
    CSPrintf(command, "\\sel %d %s\\upd \\sed %s %s%s%s;",
	     Count,
	     CSContents(src_opt), /* may be empty */
	     Offset,
	     CSContents(bib_opt), /* may be empty */
	     CSContents(by_opt),  /* may be empty */
	     CSContents(rec_opt));/* may be empty */
} /* BuildSelUpdCommand */

static void
FillReturnedInfo(
	char*	Last,
	CBBuf_t	*CBbuf,
	size_t	CBsize)
{
    char	varbuf[MAXLEN];
    const char	*location = "P3SelDbsUpdates.FillReturnedInfo";

    if (!WMGetField("ET", varbuf, sizeof(varbuf))) {
        P3errno = P3E_BUFFER;
        (void) ERWrite(P3errno, location, "getting field ET");
        return;
    }
    else {
	(void) strcpy(Last, varbuf);
    }

    (void) P3L_WMelt2CB("OW", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_WMelt2CB("BY", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_WMelt2CB("RC", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_WMelt2CB("TX", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID | FLG_SPLITLINE | FLG_OPTIONAL, location);
} /* FillReturnedInfo */

/*------------------------------------------------------------------------
 *  Function	: P3SelDbsUpdates
 *  Purpose	: Select database updates with command SEL UPD SRC SED BIB DOO REC
 *  Method	:
 *
 *  History:
 *  03-02-1999	: EvD      : creation
 *
 *  Notes	: Source, Library bylib and rectype need not be specified.
 *
 *  Returns	: Indication of success
 *------------------------------------------------------------------------
 */
bool
P3SelDbsUpdates(
	P3Handle_p	P3handle,	/* Handle to Pica3-server */
	const char	*Offset,	/* valid date/time */
	char		Last[P3DATETIME_SIZ],
					/* date/time of last element */
	int		Count,		/* maximum number to retrieve */
	const char	*Options,	/* options for SEL cmd */
	CBBuf_t		*CBbuf,		/* Buffer to contain updates */
	size_t		CBsize)		/* Size of buffer */
{
    p3_connection_p	p3_conn = P3C_GetConnection(P3handle);
    CS_LOCAL(		command, 160);
    WMContext_t		prev_context;

    P3errno = P3E_NOERROR;

    if (p3_conn != NULL) {
	prev_context = P3L_SwitchAndClearContext(p3_conn->p3cn_wm_context);
	BuildSelUpdCommand(command, Offset, Count, Options);
	WMPutCmd("US", CSContents(command));
	if (P3L_SendRecv(p3_conn->p3cn_fe_handle)) {
	    switch (P3status) {
	    case P3_NOTNOW:
		    P3errno = P3E_PROTOCOL;
		    break;
	    case P3_NOTSUP:
		    P3errno = P3E_NOSUPPORT;
		    break;
	    case P3_NOAUTH:
		    P3errno = P3E_AUTHORITY;
		    break;
	    case P3_ERROR:
		    P3errno = P3E_SERVER;
		    break;
	    case P3_OK:
		    break;
	    default:
		    if (P3errno == P3E_NOERROR) P3errno = P3E_SERVER;
	    }

	    if (P3errno == P3E_NOERROR) {
		FillReturnedInfo(Last, CBbuf, CBsize);
	    }
	}
	(void) WMSwitchContext(prev_context);
    } else {
	/* The handle is not valid */
	P3errno = P3E_SERVER;
	(void) ERWrite(P3errno, "P3SelDbsUpdates", "invalid handle");
    }

    return (P3errno == P3E_NOERROR);
} /* P3SelDbsUpdates() */

static void
XLWMFillReturnedInfo(
	char*	Last,
	CBBuf_t	*CBbuf,
	size_t	CBsize)
{
    CS_DYNAMIC(varbuf);
    const char	*location = "P3XLWMSelDbsUpdates.XLWMFillReturnedInfo";

    if (!WMGetFieldCS("ET", varbuf)) {
        P3errno = P3E_BUFFER;
        (void) ERWrite(P3errno, location, "getting field ET");
	CSDestroy(varbuf);
        return;
    }
    else {
	(void) strcpy(Last, CSContents(varbuf));
    }

    (void) P3L_XLWMelt2CB("OW", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_XLWMelt2CB("BY", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_XLWMelt2CB("RC", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID, location);
    (void) P3L_XLWMelt2CB("TX", CBbuf, CBsize,
		FLG_FIELD | FLG_ADDID | FLG_SPLITLINE | FLG_OPTIONAL, location);
    CSDestroy(varbuf);
} /* XLWMFillReturnedInfo */

/*------------------------------------------------------------------------
 *  Function	: P3XLWMSelDbsUpdates
 *  Purpose	: Select database updates with command SEL UPD SED BIB DOO REC
 *  Method	:
 *
 *  History:
 *  24-12-2001	: EvD      : copied from P3SelDbsUpdates
 *
 *  Notes	: Library need not be specified.
 *
 *  Returns	: Indication of success
 *------------------------------------------------------------------------
 */
bool
P3XLWMSelDbsUpdates(
	P3Handle_p	P3handle,	/* Handle to Pica3-server */
	const char	*Offset,	/* valid date/time */
	char		Last[P3DATETIME_SIZ],
					/* date/time of last element */
	int		Count,		/* maximum number to retrieve */
	const char	*Options,	/* options for SEL cmd */
	CBBuf_t		*CBbuf,		/* Buffer to contain updates */
	size_t		CBsize)		/* Size of buffer */
{
    p3_XLWMconnection_p	p3_conn = P3C_XLWMGetConnection(P3handle);
    CS_LOCAL(		command, 160);
    WMContext_t		prev_context;

    P3errno = P3E_NOERROR;

    if (p3_conn != NULL) {
	prev_context = P3L_SwitchAndClearContext(p3_conn->p3XLWMcn_wm_context);
	BuildSelUpdCommand(command, Offset, Count, Options);
	WMPutCmd("US", CSContents(command));
	if (P3L_XLWMSendRecv(p3_conn->p3XLWMcn_nio_handle)) {
	    switch (P3status) {
	    case P3_NOTNOW:
		    P3errno = P3E_PROTOCOL;
		    break;
	    case P3_NOTSUP:
		    P3errno = P3E_NOSUPPORT;
		    break;
	    case P3_NOAUTH:
		    P3errno = P3E_AUTHORITY;
		    break;
	    case P3_ERROR:
		    P3errno = P3E_SERVER;
		    break;
	    case P3_OK:
		    break;
	    default:
		    if (P3errno == P3E_NOERROR) P3errno = P3E_SERVER;
	    }

	    if (P3errno == P3E_NOERROR) {
		XLWMFillReturnedInfo(Last, CBbuf, CBsize);
	    }
	}
	(void) WMSwitchContext(prev_context);
    } else {
	/* The handle is not valid */
	P3errno = P3E_SERVER;
	(void) ERWrite(P3errno, "P3XLWMSelDbsUpdates", "invalid handle");
    }

    return (P3errno == P3E_NOERROR);
} /* P3XLWMSelDbsUpdates() */
