<!--
#
#  Copyright (c) 2014 by OCLC
#
#  File         : lib_pa.md
#  Description  : General description of the PA library
#  Author(s)    : Huug Peters
#  Creation     : 18-09-2013
#
#  History:
#  25-02-2014   : huug  : creation
#
#  SVN ident    : $Id: lib_pa.md 313247 2014-06-13 12:57:34Z heesakkm $
-->
The PA library
==============

PA library data types
---------------------
1.  `PAHandle_p`
    The PAHandle_t is the basic data structure of the PA library. It contains
    all the data necessary to properly maintain the pointer array.
    This data structure should be regarded as an opaque type; setting and
    retrieving values should be done through the appropriate set and get
    functions.
2.  `PA_NewOldCallback_f`
    The PANewOldCallback_f callback function type is provided to allow the
    developer specification of:
    -   the allocation strategy for new pointer array elements;
    -   the deallocation strategy for cleanup of existing elements.
    It works similarly to the NewOld function used in nametables (NMTNewOld). 
3. `PA_SortCompare_f`
    The PASortCompare_f callback function type is provided to allow the
    developer specification of the sorting strategy used in sorting the pointer
    array. The type `PA_SortCompare_f` is defined as:

	typedef int (*PA_SortCompare_f)(cvoidp arg1, cvoidp arg2);

    The comparison function should return an integer value less than, equal to
    or greater than 0, if the first argument is considered respectively less
    than, equal to or greater than the second argument.
4.  `PA_DumpElement_f`
    The PA_DumpElement_f callback function type is provided to allow the
    developer specification of the dump function. The type `PA_DumpElement_f`
    is defined as:

	typedef void	(*PA_DumpElement_f)(size_t idx, cvoidp ptr);

    The function must itself cast `ptr` to the proper type.

The PANewOldCallback_f function
-------------------------------
This function is defined as:

    typedef cvoidp (*PA_NewOldCallback_f)(cvoidp newp, cvoidp oldp);

The function must handle the consequences of `oldp` removed from the pointer
array and return the pointer that should replace `oldp`. In general terms it
should free the structure that `oldp` refers to and allocate a structure to
hold a copy of the structure refered to by `newp`.

The following situations can exist in any combination:
|Situation      | Action
|---------------|-----------------------------------------------------------------
|`oldp != NULL` | Deallocate the used memory associated with `oldp`
|`oldp == NULL` | No action for `oldp`
|`newp != NULL` | Allocate the memory to hold the data associated with `newp`
|               | Return the pointer to the newly allocated memory
|`newp == NULL` | Return `newp`

`PA_NewOldCallback_f` needs to return the value to be put into the pointer array.
The function PA_NOFUNC can be seen
as a function that always returns newp.
