/*========================================================================
 *  Copyright (c) 2006 by OCLC PICA
 *
 *  File	: pa.h
 *  Description	: Header file for pointer array library
 *  Author(s)	: Johan List
 *  Creation	: 14-06-2006
 *
 *  History:
 *  14-06-2006	: listj    : creation
 *  23-06-2006  : listj    : added PA_NOFUNC
 *                           GetLastIndex() renamed to GetNrEntries()
 *  10-02-2010	: listj    : added callback possibility for pointer 
 *			     array element dump
 *  17-05-2010	: martinz  : Added PA_SortU() and PA_Search().
 *  28-08-2018	: zwaalm   : CBS-8356 Added PA_SearchDirect().
 *
 *  Notes	:
 *
 *========================================================================
 */

/** \addtogroup pa_internal
 * @{
 */
#ifndef PA_H_INCLUDED
#define PA_H_INCLUDED

#pragma ident "$Id: pa.h 359038 2018-08-29 16:05:28Z zwaalm $"

#include <stdio.h>
#include <pica.h>

ERROR_DECLARE(PAE_NOERROR);
ERROR_DECLARE(PAE_NOMEM);

ERROR_TYPE(PAErrno_t);
extern	PAErrno_t	PAerrno;

typedef struct 	PAHandle_t*	PAHandle_p;
typedef cvoidp			(*PA_NewOldCallback_f)(cvoidp newptr, cvoidp oldptr);
typedef void			(*PA_DumpElement_f)(size_t idx, cvoidp ptr);
typedef int			(*PA_SortCompare_f)(cvoidp arg1, cvoidp arg2);
typedef int			(*PA_Compare_f)(cvoidp arg1, cvoidp arg2, cvoidp userdata);

#define PA_NOFUNC       	((PA_NewOldCallback_f)0)
#define PA_DUMPELEMENT_NOFUNC	((PA_DumpElement_f)0)

cvoidp
PA_NewOldString(
	cvoidp	newptr,
	cvoidp	oldptr);

PAHandle_p	
PA_Create(
	const char		*options,
	PA_NewOldCallback_f	NewOld);

void
PA_Destroy(
	PAHandle_p	paHandle);

void 
PA_Clear(
	PAHandle_p	paHandle);

cvoidp
PA_Add(
	PAHandle_p	paHandle,
	cvoidp		ptr);

cvoidp
PA_AddAtN(
	PAHandle_p	paHandle,
	int		idx,
	cvoidp		ptr);

void
PA_DelN(
	PAHandle_p	paHandle,
	int		idx);

cvoidp
PA_GetN(
	PAHandle_p	paHandle,
	int		idx);

cvoidp
PA_SetN(
	PAHandle_p	paHandle,
	int		idx,
	cvoidp		p);

size_t
PA_GetCapacity(
	PAHandle_p	paHandle);

size_t
PA_GetNrEntries(
	PAHandle_p	paHandle);

void 
PA_Sort(
	PAHandle_p		paHandle,
	PA_SortCompare_f	compare);

void
PA_SortU(
	PAHandle_p	paHandle,
	PA_Compare_f	compare,
	cvoidp		userdata);

int
PA_Search(
	PAHandle_p	paHandle,
	cvoidp		keyptr,
	PA_Compare_f	compare,
	cvoidp		userdata);

int
PA_SearchDirect(
	PAHandle_p	paHandle,
	cvoidp		key,
	PA_Compare_f	compare,
	cvoidp		userdata);

void
PA_Dump(
	PAHandle_p		paHandle,
	PA_DumpElement_f 	DumpElt);

bool
PA_Walk(
	PAHandle_p	paHandle,
	bool		(*HandleElt)(size_t idx, cvoidp eltp, voidp xtraparam),
	voidp		xtraparam);

#endif /* PA_H_INCLUDED */
/** @} */
