<!--
#
#  Copyright (c) 2013 by OCLC
#
#  File         : lib_wm.md
#  Description  : General description of the WM library
#  Author(s)    : Els van Doorn
#  Creation     : 12-05-2014
#
#  History:
#  12-05-2014   : els     : creation
#
-->

The WM library
===============


Worm Structure
--------------

This library deals with the phenomenon called worm.
The name worm is introduced by Pica to indicate the 'string' containing a Pica3 
(application) message. 
A worm is sent to and from between client and server (fi. IBW and CBS/LBS).
The message is structured conform the Pica3 application protocol, which is only 
superficially described in this document.
A worm consists of one or more parts. 
A part can be one of:

  - command
  - message (error-/warning-)
  - window
  - buffer
  - variable (context or global)
  - screenform label
  - screen form
  - screen template
  - history modification

Syntactically:

WORM             = {<PART><EOO>}

<PART>           = <PART_TYPE><PART_ID>[<PART_CONTENTS>]

<PART_TYPE>      = char[1] 
                     (one of 'M','K','V','G','W','S','B','C','D','T')

<PART_ID>        = char[2] (identification of part)

<PART_CONTENTS>  = {<FIELD><EOV>} | char[]

<FIELD>          = <FIELD_TYPE><FIELD_ID>[<MODIFY_TAG>][<FIELD_CONTENTS>]

<FIELD_TYPE>     = char (at this moment always 'V')

<FIELD_ID>       = char[2]
                     (identificat= char ('M' for modified)

<FIELD_CONTENTS> = char[] (contents of field)

<EOO>            = char (value: 29)
<EOV>            = char (value: 30)

Only buffer and window parts have a field segmentation.
Although the contents of a field may also be structured, this is not further elaborated 
in this document because it is not (yet) covered by the worm library functions.
The function of the worm library is to provide a rather simple interface on worm component 
structures ; the library hides the internal structure of these components from the 
application. In this way the library can be seen as a black box.
Actually, the library maintains hidden datastructures for:

  - buffers&windows
  - commands
  - variables (global&context)
  - screenforms
  - screentemplates
  - history
  - messages


Worm Functions
--------------

The following functions allow manipulating of components of worms:

Function name           | description
:-----------------------|:----------------------------------
WMCreateContext         | create worm context
WMSwitchContext         | switch worm context
WMDeleteContext         | delete context
WMClearCurrentContext   | clear current context
WMDissect               | dissect a worm ; store internally
WMBuild                 | construct worm from internals
WMWormToCB              | copy worm parts to CB
WMGetBufid              | retrieve buffer identification
WMPutBufid              | store buffer identification
WMDelBuf                | delete buffer
WMGetField              | retrieve field
WMGetFieldN             | retrieve field N
WMAppField              | append field to internal structure
WMAppToField            | append string to contents of field
WMPutField              | store field
WMDelField              | delete field
WMGetCmd                | retrieve command
WMPutCmd                | store command
WMDelCmd                | delete command
WMGetVar                | retrieve variable
WMGetVarN               | retrieve next variable
WMPutVar                | store variable
WMDelVar                | delete variable
WMRegVar                | register variable
WMMutatedVar            | register variable being updated
WMGetFrmid              | retrieve screenform identification
WMPutFrmid              | store screenform identification
WMDelFrmid              | delete screenform identification
WMGetForm               | retrieve screenform
WMPutForm               | store screenform
WMDelForm               | delete screenform
WMGetMsg                | retrieve message
WMPutMsg                | store message
WMDelMsg                | delete message
WMGetHis                | retrieve history
WMPutHis                | store history
WMDelHis                | delete history
WMGetTemplate           | retrieve screentemplate
WMPutTemplate           | store screentemplate
WMDelTemplate           | delete screentemplate


General remark and warning:

The WM-library is built upon the IDB-library. In many cases the value of WMerrno is not 
relevant or not defined, although the boolean functionvalue may have been propagated from
the associated IDB-functioncall, indicating failure. It should, however, never be necessary
to inspect the value of IDBerrno.

Declarations

declaration of context handle
typedef int WMContext_t;

declaration of client-server switch 
typedef enum {
	WM_CLIENT, WM_SERVER, WM_BOTH
	} WMCSflag_t;

declaration of part or field identification
typedef char WMIdent_t[3] 

declaration of WM-library error variable
ERROR_TYPE(WMErrno_t); 

variable name		| description
:-----------------------|:------------------------
WME_NOERROR		| ok
WME_EOF			| end of file
WME_OVERFLOW		| overflow
WME_LENGTH		| length 
WME_ILLEGALCONTEXT	| illegal context handle
WME_MAXCONTEXT		| too many contexts
WME_INTEGRITY		| integrity
WME_NOIDENT		| no identifier
WME_MAXCOMMAND		| max command
WME_MAXMESSAGE		| max message
WME_HISIDENT		| hisident
WME_UNKNOWNPART		| unknown part
WME_FEPERROR		| fep error
WME_NOERROR		| ok
WME_NOTFOUND		| identifier not found

