/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wmcommand.c
 *  Description	: Message routines from the WM-library
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 17-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  11-01-1994  : rob      : recreation
 *
 *  Notes	: this is a crazy bombastic implementation
 *
 *========================================================================
 */

#pragma ident "$Id: wmcommand.c 360943 2018-10-23 16:48:25Z heesakkm $"

#include <string.h>

#include <pica.h>
#include <er.h>
#include <wm.h>
#include "wmlocal.h"

/*------------------------------------------------------------------------
 *  Function	: WMGetCmd, WMPutCmd, WMDelCmd
 *  Purpose	: Get, Put or Del contents of command
 *  Notes       : Only one command exists
 *
 *  WMGetCmd
 *  --------
 *
 *  Description	:
 *
 *  cident is a (returned) pointer to a command identification.
 *  Wild cards are allowed (see: WMGetField()).
 *
 *  cbuf is a pointer to a command contents.
 *
 *  csize is the size of the command contents.
 *
 *  The function will copy the contents of the first matching command to cbuf 
 *  and the identification to cident. In erroneous situations FALSE will be
 *  Error values:
 *
 *  none
 *
 *  Example	:
 *
 *      #include <wm.h>
 *      char		cid[3];    Will contain "US" or "PC" 
 *      char		cbuf[80];
 *
 *      if (!WMGetCmd(cid, cbuf, sizeof(cbuf))) {
 *          ERWrite(WMerrno, jobname, "getting command");
 *          return;
 *      }
 *
 *  WMPutCmd
 *  --------
 *
 *  Description	:
 *
 *  cident is a pointer to a command identification.
 *  Wild cards are not allowed.
 *
 *  cbuf is a pointer to a (zero-terminated) command contents.
 *
 *  The function will store the command contents with the proper identification.
 *  In case there was already a command part defined the contents of the command
 *  to store is concatenated to the contents in the library, preceded by the
 *  semicolon character (';'). In erroneous situations FALSE will be returned and
 *  WMerrno should be examined.
 *
 *  Error values:
 *
 *  none
 *
 *  Example	:
 *
 *      #include <wm.h>
 *
 *      if (!WMPutCmd("US", "KORT")) {
 *          ERWrite(WMerrno, jobname, "putting command");
 *      }   * As if the user had entered the "KORT" command *
 *
 *  WMDelCmd
 *  --------
 *
 *  Description	:
 *
 *  The function will delete all command parts in the library. In case no command
 *  parts were found, success is returned. In erroneous situations FALSE will be
 *  returned. WMerrno is not affected.
 *
 *  Error values:
 *
 *  none
 *
 *  Example	:
 *
 *      #include <wm.h>
 *
 *      (void) WMPutCmd("US", "KORT");   Simulate "KORT" command
 *
 *      (void) WMDelCmd();               Ah well, maybe not...
 *
 *
 *------------------------------------------------------------------------
 */
bool
WMGetCmd(WMIdent_t mident, char *mbuf, size_t msize)
{
    WMerrno = WME_NOERROR;
    return IDBGetN(WMContext[WMCurrentC]->Commands, 0, mident, mbuf, msize);
} /* WMGetCmd */

bool
WMDelCmd(void)
{
    WMerrno = WME_NOERROR;
    IDBEmpty(WMContext[WMCurrentC]->Commands);
    return TRUE;
} /* WMDelCmd */

bool
WMPutCmd(const WMIdent_t mident, const char *mbuf)
{    
    WMerrno = WME_NOERROR;
    IDBEmpty(WMContext[WMCurrentC]->Commands);
    return IDBPut(WMContext[WMCurrentC]->Commands, mident, mbuf);
} /* WMPutCmd */
