/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wmhistory.c
 *  Description	: History routines from the WM-library
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  26-10-1994  : remy     : made WMGetHis.
 *                           added a few things to WMPutHistory.
 *  23-10-1995	: HSi	   : Implemented server side of history in a crude way.
 *			     The history lines are put in an IDB-buffer with
 *			     the ident as id. Because the id in an IDB-
 *			     buffer has a length of 2 (excluding '\0') nothing
 *			     more can be used. So only one line per ident
 *			     is possible. The ident is AD, DE, RE etc.
 *  19-01-1998	: lolke    : Use of WM_BOTH. Deleted obsolete code.
 *  24-01-2002  : fb       : Bugfix in WMPutHistory for WinIBW.
 *			      
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: wmhistory.c 360943 2018-10-23 16:48:25Z heesakkm $"

#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include <pica.h>
#include <er.h>
#include <wm.h>
#include "wmlocal.h"

/*------------------------------------------------------------------------
 *  Function	: FindHistory
 *  Purpose	: Try to find historyline with setnr
 *  Method	: Linear search
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *
 *  Notes	:
 *
 *  Returns	: -1 if not found else index in history array
 *
 *------------------------------------------------------------------------
 */
static int
FindHistory(int setnr)
{
    int i;

    for (i = 0; i < WMContext[WMCurrentC]->HistoryP &&
		WMContext[WMCurrentC]->History[i].n != setnr; i++) {
	;
    }
    if (i < WMContext[WMCurrentC]->HistoryP) {
	return i;
    }
    return -1;
} /* FindHistory */

/*------------------------------------------------------------------------
 *  Function	: WMInitHistory
 *  Purpose	: 
 *  Method	: Set HistoryP to zero
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  25-10-1994  : remy     : added boolean set to true.
 *
 *  Notes	:
 *
 *  Returns	: nothing
 *
 *------------------------------------------------------------------------
 */
void
WMInitHistory(void)
{
    WMerrno = WME_NOERROR;

    WMContext[WMCurrentC]->HistoryP = 0;
    WMContext[WMCurrentC]->HistoryInit = TRUE;
} /* WMInitHistory */

/*------------------------------------------------------------------------
 *  Function	: WMDelHistory
 *  Purpose	: Delete history element with setnummer equals setnr
 *  Method	: Find and overwrite tail
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *
 *  Notes	:
 *
 *  Returns	: nothing
 *
 *------------------------------------------------------------------------
 */
void
WMDelHistory(int setnr)
{
    int i;
    int j;

    WMerrno = WME_NOERROR;
    if ((i = FindHistory(setnr)) >= 0) {
	for (j = i + 1; j <= WMContext[WMCurrentC]->HistoryP; j++) {
	    WMContext[WMCurrentC]->History[j - 1] = WMContext[WMCurrentC]->History[j];
	}
	WMContext[WMCurrentC]->HistoryP--;
    }
} /* WMDelHistory */

/*------------------------------------------------------------------------
 *  Function	: WMMarkHistory
 *  Purpose	: Set boolean for highlighting
 *  Method	: Find and set boolean
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *
 *  Notes	:
 *
 *  Returns	: nothing
 *
 *------------------------------------------------------------------------
 */
void
WMMarkHistory(int setnr, bool mark)
{
    int i;

    WMerrno = WME_NOERROR;
    if ((i = FindHistory(setnr)) >= 0) {
	WMContext[WMCurrentC]->History[i].m = mark;
    }
} /* WMMarkHistory */

/*------------------------------------------------------------------------
 *  Function	: WMPutHistory
 *  Purpose	: Put history element in the WMContext
 *  Method	: Make room and put
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  25-10-1994  : remy     : added calculation of hlen and the else part.
 *  02-12-1997	: lolke    : hlp no longer out of bounds.
 *  24-01-2002  : fb       : Added terminating zero for WinIBW.
 *
 *  Notes	:
 *
 *  Returns	: nothing
 *
 *------------------------------------------------------------------------
 */
void
WMPutHistory(int setnr, const char *hbuf)
{
    size_t hlen;
    int hlp;

    WMerrno = WME_NOERROR;
    WMDelHistory(setnr);
    hlp = WMContext[WMCurrentC]->HistoryP;
    if (hlp >= MAXHISTORY) {
	WMDelHistory(WMContext[WMCurrentC]->History[0].n);
	hlp = MAXHISTORY - 1;
    }
    WMContext[WMCurrentC]->History[hlp].p = FALSE;
    WMContext[WMCurrentC]->History[hlp].m = FALSE;
    WMContext[WMCurrentC]->History[hlp].n = setnr;
    hlen = strlen(hbuf);
    if (hlen >= MAXHISTORYTEXT) {
	hlen = MAXHISTORYTEXT - 1;
    }
    (void) strncpy(WMContext[WMCurrentC]->History[hlp].c, hbuf, hlen);
    WMContext[WMCurrentC]->History[hlp].c[hlen] = '\0';
    WMContext[WMCurrentC]->HistoryP++;
} /* WMPutHistory */

/*------------------------------------------------------------------------
 *  Function	: WMGetHis
 *  Purpose	: 
 *  Method	: 
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  25-10-1994  : remy     : implementation
 *
 *  Notes	: hident is not used as it is not needed.
 *
 *  Description	:
 *
 *  hident is a (returned) pointer to a history identification.
 *  Wild cards are allowed (see: WMGetField()).
 *
 *  hbuf is a pointer to a history contents.
 *
 *  hsize is the size of the history contents.
 *
 *  The function will copy the first matching history contents to hbuf and the history
 *  identification to hident. In erroneous situations FALSE will be returned and WMerrno
 *  should be examined.
 *
 *  Notes	:
 *
 *  This function is not yet implemented.
 *
 *  Error values:
 *
 *  | Value			| Description			|
 *  |-------------------------: | :---------------------------- |
 *  | WME_NOERROR               | ok                            |
 *  | WME_OVERFLOW              | overflow                      |
 *  | WME_ILLEGALCONTEXT        | illegal context handle        |
 *  | WME_MAXCONTEXT            | too many contexts             |
 *  | WME_IDENT                 | invalid identifier            |
 *  | WME_NOTFOUND              | identifier not found          |
 *
 *  Returns	:
 *
 *  | Value	| Meaning			|
 *  |---------: | :---------------------------- |
 *  | TRUE      | ok                            |
 *  | FALSE     | something fails               |
 *
 *------------------------------------------------------------------------
 */
bool
WMGetHis(const WMIdent_t hident, char *hbuf, size_t hsize)
{ 
    int i,j,k;
    char *temp = NULL;

    (void) hident;	// Keep lint happy

    WMerrno = WME_NOERROR;
    hbuf[0] = '\0';
    j = 0;
    k = 0;
    for (i = 0; i < WMContext[WMCurrentC]->HistoryP; i++){
	temp = WMContext[WMCurrentC]->History[i].c;
	for ( ; *temp != *SEP; temp++) ;
	temp++;
	if (WMContext[WMCurrentC]->History[i].m)
	   k = sprintf(hbuf,"%c%c%s\r",*ESC,*PM_ESC_PROTECT_OPEN,temp);
        else
	   k = sprintf(hbuf,"%c%c%s\r",*ESC,'U',temp);
	j += k;
	if (j < hsize) {
	    hbuf += k;
        }
	else  {
	    WMerrno = WME_OVERFLOW;  /* is it the right error ? */
	    if (hsize > 0) hbuf[hsize-1] = '\0';
	    else hbuf[0] = '\0';
	    return FALSE;
	}
    }
    return TRUE;
} /* WMGetHis */

/*------------------------------------------------------------------------
 *  Function	: WMPutHis
 *  Purpose	: Put history part hident in context
 *  Method	: 
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  23-10-1995	: HSi	   : added code to put history line in HistoryServer
 *			     IDB-buffer. Ident is used as key. As a result
 *			     only one set may be added etc.
 *
 *  Notes	: 
 *
 *  Description	:
 *
 *  hident is a pointer to an history identification.
 *  Wild cards are not allowed.
 *
 *  hbuf is a pointer to an (zero-terminated) history contents.
 *
 *  The function will store the history contents with the proper identification. In case
 *  there was already an history part with the same identification defined, that one is
 *  replaced. In erroneous situations FALSE will be returned and WMerrno should be 
 *  examined.
 *
 *  Notes	:
 *
 *  This function is not yet implemented
 *
 *  Error values:
 *
 *  | Value			| Description			|
 *  |-------------------------: | :---------------------------- |
 *  | WME_NOERROR               | ok                            |
 *  | WME_OVERFLOW              | overflow                      |
 *  | WME_ILLEGALCONTEXT        | illegal context handle        |
 *  | WME_MAXCONTEXT            | too many contexts             |
 *  | WME_IDENT                 | invalid identifier            |
 *  | WME_NOTFOUND              | identifier not found          |
 *
 *  Returns	:
 *
 *  | Value	| Meaning			|
 *  |---------: | :---------------------------- |
 *  | TRUE      | ok                            |
 *  | FALSE     | something fails               |
 *
 *------------------------------------------------------------------------
 */
bool
WMPutHis(WMIdent_t hident, const char *hbuf)
{
    WMerrno = WME_NOERROR;
    return IDBPutA ( WMContext[WMCurrentC]->HistoryServer, hident, hbuf );
} /* WMPutHis */

/*------------------------------------------------------------------------
 *  Function	: WMDelHis
 *  Purpose	: Delete the whole history context
 *  Method	: initialise
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *
 *  Notes	: Works only in WM_SERVER or WM_BOTH mode
 *
 *  Description	:
 *
 *  The function will delete all history parts in the library. In case no such history
 *  parts were found, success is returned. In erroneous situations FALSE will be returned
 *  and WMerrno should be examined.
 *
 *  Notes	:
 *
 *  This function is not yet implemented.
 *
 *  Error values:
 *
 *  | Value			| Description			|
 *  |-------------------------: | :---------------------------- |
 *  | WME_NOERROR               | ok                            |
 *  | WME_OVERFLOW              | overflow                      |
 *  | WME_ILLEGALCONTEXT        | illegal context handle        |
 *  | WME_MAXCONTEXT            | too many contexts             |
 *  | WME_IDENT                 | invalid identifier            |
 *  | WME_NOTFOUND              | identifier not found          |
 *
 *  Returns	:
 *
 *  | Value	| Meaning			|
 *  |---------: | :---------------------------- |
 *  | TRUE      | ok                            |
 *  | FALSE     | something fails               |
 *
 *------------------------------------------------------------------------
 */
bool
WMDelHis(void)
{
    WMerrno = WME_NOERROR;
    if (WMContext[WMCurrentC]->csflag == WM_SERVER || WMContext[WMCurrentC]->csflag == WM_BOTH) {
        IDBEmpty ( WMContext[WMCurrentC]->HistoryServer );
    }

    return TRUE;
} /* WMDelHis */
