/*
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wmlocal.h
 *  Description	: Local WM types and defines and variabels
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 13-12-1993
 *
 *  History:
 *  13-12-1993	: lolke    : creation
 *  23-10-1995	: HSi	   : Added HistoryServer to context. This field is
 *			     only used for the server side of this library.
 *			     Refer to wmhistory.c for details if usage.
 *  14-10-1997	: HSi	   : Modified decl of WMInitHistory to (void). The
 *			     OSF C-compiler needs this to see it as a
 *			     prototype.
 *  10-01-2002  : fb       : Added fields FieldRegister and FieldRegisterSize
 *                           to ContextStruct for cynchronisation with WinIBW.
 *
 *  Notes	: This file may be included only after wm.h is included. It
 *                is included in all the wm*.c files. As idb.h is included
 *                here, do NOT include idb.h in the wm*.c files.
 *
 */

#ifndef WMLOCAL_H_INCLUDED
#define WMLOCAL_H_INCLUDED

#pragma ident "$Id: wmlocal.h 360943 2018-10-23 16:48:25Z heesakkm $"

#include <pica.h>
#include <picamisc.h>
#include <idb.h>

#define MAXCONTEXT 10
#define MAXFIELDS 60
#define MAXHISTORY 100
#define MAXHISTORYTEXT 200

#define ETX "\003"
#define LF  "\012"
#define FF  "\014"
#define CR  PM_CR
#define ESC PM_ESC
#define EOO "\035"
#define EOV "\036"
#define SEP "\037"

typedef char WMPart_t;

typedef struct {
    int  n;
    bool m;
    bool p;
    char c[MAXHISTORYTEXT];
} history_t;

typedef struct {
    WMIdent_t  id;
    int        x;
    int        y;
    int        l;
    int        o;
    bool       uitv;
    bool       prot;
    bool       hide;
    int        ip;
    int        il;
} field_t;

typedef struct {
    WMCSflag_t     csflag;
    WMIdent_t      BufferID;
    WMIdent_t      ScreenID;
    WMIdent_t      ScreenDA;
    WMIdent_t      ScreenDest;
    IDBHandle_p    Fields;
    IDBHandle_p    Messages;
    IDBHandle_p    Commands;
    IDBHandle_p    Variables;
    IDBHandle_p	   HistoryServer;
    history_t      History[MAXHISTORY];
    int            HistoryP;
    bool           HistoryInit;
/* the following should later change type */
/* for screen */
    char	   *Screen;
/* for template */
    char	   *Template;
/* registerd vars */
    char           *VarRegister;
    int            VarRegisterSize;    
/* registered fields */
    char           *FieldRegister;
    int            FieldRegisterSize;
/*
 *   field_t        Field[MAXFIELDS];
 *   int            FieldP;
*/
} ContextStruct_t, *ContextStruct_p;

extern ContextStruct_p WMContext[MAXCONTEXT];
extern int WMCurrentC;

extern bool WMIsRegistered(const WMIdent_t);
extern bool WMIsMutatedVar(char *);

extern void WMInitHistory(void);
extern void WMMarkHistory(int setnr, bool mark);
extern void WMDelHistory(int setnr);
extern void WMPutHistory(int setnr, const char *hbuf);

#endif /* WMLOCAL_H_INCLUDED */
