/*========================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wmtemplate.c
 *  Description	: Template routines from the WM-library
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 22-02-1996
 *
 *  History:
 *  17-12-1993	: lolke    : creation
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: wmtemplate.c 313247 2014-06-13 12:57:34Z heesakkm $"

#include <string.h>
#include <stdlib.h>
#include <pica.h>
#include <er.h>
#include <wm.h>
#include "wmlocal.h"

/*------------------------------------------------------------------------
 *  Function	: WMGet/Put/DelTemplate
 *
 *  History:
 *  22-02-1996	: lolke    : creation
 *
 *  Notes	:
 *
 *  WMGetTemplate
 *  -------------
 *
 *  Description	:
 *
 *  sform is pointer to a buffer, used to store the screenform.
 *
 *  vsize is the size of sform.
 *
 *  Beware that there is never more than one screentemplate.
 *  The function will copy the screenform to sform. In erroneous situations FALSE will
 *  be returned and WMerrno should be examined.
 *
 *  Error values:
 *
 *  | Value		| Description		|
 *  | ----------------: | :-------------------- |
 *  | WME_NOERROR       | ok                    |
 *  | WME_OVERFLOW      | overflow              |
 *  | WME_NOTFOUND      | identifier not found  |
 *
 *  Example	:
 *
 *      #include <wm.h>
 *      char     buffer[1500];
 *
 *      if (!WMGetTemplate(buffer, sizeof(buffer))) {
 *          ERWrite(WMerrno, jobname, "getting template");
 *      }
 *
 *  WMPutTemplate
 *  -------------
 *
 *  Description	:
 *
 *  sform is a pointer to a screenform.
 *  Wild cards are not allowed.
 *
 *  The function will store the screentemplate fident. In case there was already a
 *  screenform defined, that one is replaced. The function return no errors.
 *
 *  Error values:
 *
 *  (none)
 *
 *  Example	:
 *
 *      #include <wm.h>
 *      char     buffer[1500];
 * 
 *      (void) WMPutTemplate(buffer);
 *
 *  WMDelTemplate
 *  -------------
 *
 *  Description	:
 *
 *  The function will delete the screentemplate. No errors are returned.
 *
 *  Error values:
 *
 *  (none)
 *
 *  Example	:
 *
 *      #include <wm.h>
 *
 *      (void) WMDelTemplate();
 *
 *------------------------------------------------------------------------
 */
bool
WMGetTemplate(char *form, size_t vsize)
{
    size_t	len;
    
    WMerrno = WME_NOERROR;

    if (WMContext[WMCurrentC]->Template == NULL) {
	WMerrno = WME_NOTFOUND;
    } else {
	len = strlen(WMContext[WMCurrentC]->Template);

	if (vsize > len) {
	    (void) strcpy(form, WMContext[WMCurrentC]->Template);
	} else {
	    *form = '\0';
	    WMerrno = WME_OVERFLOW;
	}
    }
    return (WMerrno == WME_NOERROR);
} /* WMGetTemplate() */

void
WMPutTemplate(const char *form)
{
    int l;
    
    WMerrno = WME_NOERROR;
    /* copieer het formulier */
    l = strlen(form);
    free(WMContext[WMCurrentC]->Template);
    WMContext[WMCurrentC]->Template = (char *) malloc ( l+1 );
    (void) strcpy(WMContext[WMCurrentC]->Template, form);

    /* remove a possible etx */
    if (l>0){
	if (form[l-1]==3){
	    WMContext[WMCurrentC]->Template[l-1] = 0;
      	}
    }
} /* WMPutForm */

void
WMDelTemplate (void)
{
    WMerrno = WME_NOERROR;
    free (WMContext[WMCurrentC]->Template);
    WMContext[WMCurrentC]->Template = NULL;    
} /* WMDelForm */
