/*========================================================================================
 *  Copyright (c) 1993 by Pica
 *
 *  File	: wmworm.c
 *  Description	: The part of the WM-library which communicates with the
 *                SC-library and puts it in the internal structure
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 13-12-1993
 *
 *  History:
 *  13-12-1993	: lolke    : creation
 *  12-01-9194  : rob      : made it work
 *  19-04-1994	: eck      : added WMSendRecv()
 *  29-07-1994  : wimo	   : changed call to FEStart using STDIN & STDOUT
 *  19-08-1994  : wimo	   : removed WMSend, WMReceive, WMSendRecv
 *			   : modified WMBuild and WMDissect
 *  26-10-1994  : remy     : modified HisDissect and WMDissect (slightly)
 *  18-11-1994  : remy     : modified BufDissect
 *  08-12-1994	: HSi	   : modified BufBuild (added modified flag if usage
 *			     is client)
 *  29-12-1994  : rob      : added (bad) code to delete cursor position
 *			     added code to handle multiple field sends of
 *	                     same id.
 *  19-01-1998  : lolke    : Added use of WM_BOTH. WM_BOTH equals WM_SERVER 
 *                           except for history parts. 
 *			     For WM_BOTH WMDissect and WMBuild are each other 
 *                           inverse routine.
 *  01-09-1998  : fb       : In BufDissect creation of editscript header only
 *                           for WM_CLIENT.
 *  27-03-2003  : fb       : BufDissect addapted for large dynamic buffers.
 *
 *  Notes	:
 *
 *======================================================================================
 */

#pragma ident "$Id: wmworm.c 360943 2018-10-23 16:48:25Z heesakkm $"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include <pica.h>
#include <er.h>
#include <cb.h>
#include <wm.h>
#include "wmlocal.h"

static int  WormP;
static char *wm_p;

#define ETX "\003"
#define LF  "\012"
#define FF  "\014"
#define EOO "\035"
#define EOV "\036"
#define SEP "\037"

#define WM_VAR_CURSORADDR     "!A"
#define WM_VAR_CURSORFIELD    "!V"
#define WM_VAR_CURSORPOS      "!P"
#define WM_VAR_CURSORLINE     "!R"

/*------------------------------------------------------------------------
 *  Function	: BufDissect
 *  Purpose	: Dissect buffer and window parts
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  18-11-1994  : remy     : added WMGetField for "TX" one.
 *                           added the edit field part.
 *  01-09-1998  : fb       : creation of editscript header only for WM_CLIENT.
 *  27-03-2003  : fb       : Allow for large dynamic buffers.
 *
 *  Notes	: Parses Window data and Buffer data, as defined in the
 *                Technical Manual for the IBW v1.2.
 *
 *  Returns	: FALSE if something fails
 *------------------------------------------------------------------------
 */
static bool
BufDissect( char *ident, char *str )
{
    char      *hlp;
    char      *script;
    char       temp[1000];
    CS_DYNAMIC(csdtmp);
    int        i;
    int        j;
    int        s;
    WMIdent_t  id;
    bool       rc = TRUE;


    /* copy the buffer identifier */
    WMContext[WMCurrentC]->BufferID[0] = ident[0];
    WMContext[WMCurrentC]->BufferID[1] = ident[1];

    /* copy the fields */
    hlp = str+3;
    while (hlp[0]==*EOV){ hlp++; }

    while ( hlp[0] == 'V' ){
        /* first the id */
	id[0] = hlp[1];
	id[1] = hlp[2];
	id[2] = '\0';

	/* look for the end of our quest */
	i = 0;
	while (hlp[i]!=*EOV && hlp[i] != 0) {i++;}

	/* should be *EOV now, skip mutation flags */
	s = 3;
	if (str[0]=='B') { s=4; };
	if (hlp[i]!=*EOV || i<s) { 
	    rc = FALSE;
	    break;
	}

	/* copy the field */
	hlp[i] = 0;

	/* create editscript header in case of an edit field */
	temp[0] = '\0';
	if ((hlp[s]=='T') && (WMContext[WMCurrentC]->csflag == WM_CLIENT)) {
	    script = strchr (hlp,*SEP);
	    if (script!=NULL) {
		/* one sprintf should do ... rk */
		j = 1 + (int)(script - hlp);
		strcat  (temp,"\033H\033P\033%\033V");
		strncat (temp,hlp+s+1,3);
		strcat  (temp,"\033H");
		strncat (temp,hlp+s+4,j-7);
		sprintf (temp+strlen(temp),"%lu",(unsigned long) strlen(hlp+j));
		strcat  (temp,"\033V\033U\r");
		s = j;
	    }
	}

	/* Retrieve the value of this field already stored
	   (will not be found the first time): */
	CSClear(csdtmp);
	if (!WMGetFieldCS(id, csdtmp) && (WMerrno != WME_NOTFOUND)) {
	    rc = FALSE;
	    break;
	}

        /* In case of an edit field, append the header just created,
	   and the script: */
	if (*temp) {
	    (void) CSAppend(csdtmp, temp);
	    (void) CSAppend(csdtmp, hlp+s);
	    (void) WMPutField(id, CSContents(csdtmp));
	} else if (CSLength(csdtmp) > 0) {
	    /* Append CR as separator: */
	    if (CSContents(csdtmp)[CSLength(csdtmp)-1] != '\r') {
		(void) CSAppendC(csdtmp, '\r');
	    }
	    /* Add the entire contents of the field: */
	    (void) CSAppend(csdtmp, hlp+s);
	    (void) WMPutField(id, CSContents(csdtmp));
	} else {
	    (void) WMPutField(id, hlp+s);
	}

	hlp[i] = *EOV;
	hlp += i+1;

        /* allow for programming errors! */
	while (hlp[0]==*EOV){ hlp++; }
    }

    /* Clean up: */
    CSDestroy(csdtmp);

    return rc;
} /* BufDissect */


/*------------------------------------------------------------------------
 *  Function	: HisDissect
 *  Purpose	: Dissect history part
 *  Method	: obvious
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *  26-10-1994  : remy     : boolean for WMMarkHistory were wrong.
 *                           in case of SE,DE,PU it was acting only
 *                           on the first set number, changed it.
 *
 *  Notes	:
 *
 *  Returns	: FALSE if something fails
 *
 *------------------------------------------------------------------------
 */
static bool
HisDissect(
	const WMIdent_t	hident, 
	const char	*hbuf)
{
    int setnr;

    WMerrno = WME_NOERROR;
    if (strcmp(hident, "IN") == 0) {  /* because hbuf is "" when IN */
	WMInitHistory();
    }
    while (*hbuf != '\0'){
       setnr = -1;
       if (isdigit(*hbuf)) {
          setnr = atoi(hbuf);
       }
       if (strcmp(hident, "AD") == 0 || strcmp(hident, "RE") == 0) {
	  WMPutHistory(setnr, hbuf);
	  while (*hbuf != '\0') hbuf++;
       } else if (strcmp(hident, "DE") == 0) {
	  WMMarkHistory(setnr, TRUE);
       } else if (strcmp(hident, "PU") == 0) {
	  WMDelHistory(setnr);
       } else if (strcmp(hident, "SE") == 0) {
	  WMMarkHistory(setnr, FALSE);
       } else {
	  WMerrno = WME_HISIDENT;
	  (void) ERWrite(WMerrno, "HISDissect", "Ident not known");
	  return FALSE;
       }
       while ((*hbuf != *SEP) && (*hbuf != '\0')) hbuf++;
       if (*hbuf != '\0') hbuf++;
    }
    return TRUE;
} /* HisDissect */

/*------------------------------------------------------------------------
 *  Function	: DefDissect
 *  Purpose	: Dissect the screen definition part
 *  Method	: 
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 16-12-1993
 *
 *  History:
 *  16-12-1993	: lolke    : creation
 *		  wimo	   : made it work
 *
 *  Notes	: Does a WMPutForm
 *
 *  Returns	: TRUE
 *
 *------------------------------------------------------------------------
 */
static bool
DefDissect(
	const char	*dbuf)
{
    WMPutForm((char *)dbuf);
    WMerrno = WME_NOERROR;
    return TRUE;
} /* DefDissect */

/*------------------------------------------------------------------------
 *  Function	: TemplateDissect
 *  Purpose	: Dissect the template part
 *  Method	: WMPutTemplate
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 02-07-1996
 *
 *  History:
 *  02-07-1996	: lolke    : creation
 *
 *  Notes	: Copied from DefDissect
 *
 *  Returns	: TRUE
 *
 *------------------------------------------------------------------------
 */
static bool
TemplateDissect(
	const char	*dbuf)
{
    WMPutTemplate((char *)dbuf);
    WMerrno = WME_NOERROR;
    return TRUE;
} /* TemplateDissect */

/*------------------------------------------------------------------------
 *  Function	: WMDissect
 *  Purpose	: Dissect the worm to the local datastructure of WMContext
 *  Method	: Uses strtok
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 13-12-1993
 *
 *  History:
 *  13-12-1993	: lolke    : creation
 *  19-04-1994	: eck      : moved WMMutatedVar()-loop to WMSendRecv().
 *  25-10-1994  : remy     : added +3 to str in call of HisDissect.
 *
 *  Description		:
 *
 *  The function will scan the worm. Each new part, except buffers and windows 
 *  (which are stored directly into the library) will be stored in dedicated 
 *  data-structures. In erroneous situations FALSE will be returned. 
 *  It indicates that the worm is corrupted.
 *
 *  Notes		: the worm should be EOT and zero terminated
 *
 *  WMerrno is set to WME_NOERROR in this routine.
 *
 *  Error values	:
 *
 *  | Value		| Description		|
 *  | ----------------: | :-------------------- |
 *  | WME_NOERROR	| ok			|
 *
 *  Returns		:
 *
 *  | Value	| Meaning		|
 *  | --------: | :-------------------- |  
 *  | FALSE	| something fails	|
 *  | TRUE	| Succeeded		|
 *
 *---------------------------------------------------------------------------
 */
bool
WMDissect(char *Worm)
{
    bool      VSeen;
    bool      WSeen;
    bool      PSeen;
    bool      RSeen;

    char      *str;
    WMIdent_t ident;
    int	      i;

    WMerrno = WME_NOERROR;
    WSeen = FALSE;
    VSeen = FALSE;
    PSeen = FALSE;
    RSeen = FALSE;

    str = Worm;
    while ( str[0]>*ETX ){
	/* string points to the next wormpart */

	/* they all need the ident */
   	   ident[0] = str[1];
	   ident[1] = str[2];
	   ident[2] = '\0';
	   if (ident[0]<' ' || ident[1]<' '){
	      /* thats no good! */
	      return FALSE;
	   }

	/* patch the end to zero */
	   i = 0;
	   while (str[i] && str[i]!=*ETX && str[i]!=*EOO){ i++;}
	   if (str[i]!=*EOO){
	       /* thats no good! */
	       return FALSE;
	   }
	   str[i] = 0;

	switch (str[0]) {
	  case 'M':
	    (void) WMPutMsg(ident, str+3);
	    break;
          case 'V':
	  case 'K':
    	    (void) WMPutVar(ident, str+3);
	    if (str[0] == 'K'){ (void) WMRegVar(ident);}
	    if (ident[0]=='!'){
		VSeen = VSeen || ident[1] == 'V';
		WSeen = WSeen || ident[1] == 'W';
		PSeen = PSeen || ident[1] == 'P';
		RSeen = RSeen || ident[1] == 'R';
	    }
	    break;
	  case 'S':
	    (void) WMPutFrmid(ident);
	    break;
	  case 'C':
	    (void) WMPutCmd(ident, str+3);
	    break;
	  case 'W' :
	  case 'B' :
	    (void) BufDissect(ident, str);
	    if (!WSeen) WMDelVar("!W");
	    if (!VSeen) WMDelVar(WM_VAR_CURSORFIELD);
	    if (!PSeen) WMDelVar(WM_VAR_CURSORPOS);
	    if (!RSeen) WMDelVar(WM_VAR_CURSORLINE);
	    break;
	  case 'D' :
	    (void) DefDissect(str);
	    break;
	  case 'T' :
	    (void) TemplateDissect(str);
	    break;
	  case 'G' :
	    if ( WMContext[WMCurrentC]->csflag == WM_BOTH) {
		WMPutHis(ident, str+3);
	    } else {
	        (void) HisDissect(ident, str+3);
	    }
	    break;
	  default :
	    /* we don't pretend to know all! */
	    /* no need to make a fuzz */;
        }

	/* patch back the end */
	   str[i] = *EOO;

	/* go to the next part */
	   str += i+1;

    }
    return TRUE;
} /* WMDissect */

/*-----------------------------------------------------------------------
 *  Function    : AddStrToWorm
 *  Purpose     : Add string to worm
 *  Note        : protects against overflow
 *                assumes WormP is correct
 *		  Worm stays zero terminated
 *  History     : rk 04/02/94 created
 *-----------------------------------------------------------------------
 */
static void
AddStrToWorm (const char *str)
{
    while (*str && (WormP < WM_MAXWRMLEN)) {
	*wm_p++ = *str++;
	WormP++;
    }
    *wm_p = 0;
} /* AddStrToWorm */


/*------------------------------------------------------------------------
 *  Function	: BufBuild
 *  Purpose	: Put buffer id and fields in worm
 *  History	:
 *  08-12-1994	: HSi	: Added modified flag if type is B (i.e. client).
 *  10-01-2002  : fb    : Synchronised with WinIBW version.
 *------------------------------------------------------------------------
 */
static void
BufBuild(const char *type)
{
    WMIdent_t id;
    char      c[WM_MAXWRMLEN];
    int       i;

    if (WMGetBufid(id)) {
	AddStrToWorm(type);
	AddStrToWorm(id);

	for (i = 0; WMGetFieldN(i, id, c, sizeof(c)); i++) {
	    AddStrToWorm("V");
	    AddStrToWorm(id);
	    if (type[0] == 'B') {
		if (WMIsMutatedField(id)) {
		    AddStrToWorm("M");
		} else {
		    AddStrToWorm("0");
		}
	    }
	    AddStrToWorm(c);
	    AddStrToWorm(EOV);
	}
	AddStrToWorm(EOO);
    }
} /* BufBuild() */


/*------------------------------------------------------------------------
 *  Function	: ComBuild
 *  Purpose	: Put command in worm
 *------------------------------------------------------------------------
 */
static void
ComBuild(void)
{
    WMIdent_t	id;
    char	hlp[301];

    id[0] = id[1] = '?';
    hlp[0] = 0;
    if (WMGetCmd(id, hlp, sizeof(hlp))) {
	AddStrToWorm("C");
	AddStrToWorm(id);
	AddStrToWorm(hlp);
	AddStrToWorm(EOO);
    }
} /* ComBuild */

/*------------------------------------------------------------------------
 *  Function	: DefBuild
 *  Purpose	: Put screen definition in worm
 *  Method	: Read definition and copy to worm
 *  Notes       : Since definitions might already be terminated by an EOO
 *                we check this and only append an EOO if necessary.
 *                Nearly identical to TemplateBuild.
 *------------------------------------------------------------------------
 */
static void
DefBuild(void)
{
    if (WMContext[WMCurrentC]->Screen != NULL) {
	char *  val = WMContext[WMCurrentC]->Screen;
	size_t  len = strlen(val);

	AddStrToWorm(val);
	WMDelForm();
	if (WMObjectComplete(val, len) == FALSE) {
	    AddStrToWorm(EOO);
	}
    }
} /* DefBuild */

/*------------------------------------------------------------------------
 *  Function	: TemplateBuild
 *  Purpose	: Put template in worm
 *  Method	: Read definition and copy to worm
 *------------------------------------------------------------------------
 */
static void
TemplateBuild(void)
{
    if (WMContext[WMCurrentC]->Template != NULL) {
	char *  val = WMContext[WMCurrentC]->Template;
	size_t  len = strlen(val);

	AddStrToWorm(val);
	WMDelTemplate();
	if (WMObjectComplete(val, len) == FALSE) {
	    AddStrToWorm(EOO);
	}
    }
} /* TemplateBuild */

/*------------------------------------------------------------------------
 *  Function	: FrmBuild
 *  Purpose	: Put "Sxx" in worm
 *------------------------------------------------------------------------
 */
static bool
FrmBuild(void)
{
    char id[3];
    static char hlp[] = "SXX" EOO;

    if (WMGetFrmid(id) != TRUE) { return FALSE; }
    hlp[1] = id[0];
    hlp[2] = id[1];
    AddStrToWorm (hlp);
    return TRUE;

} /* FrmBuild */

/*------------------------------------------------------------------------
 *  Function	: HisBuild
 *  Purpose	: Put history records in worm
 *  Notes	: Copied from MesBuild ( HSi, 23-10-1995 ), use direct call
 *		  to IDB, because WMGetHis seems to be used in client-mode.
 *------------------------------------------------------------------------
 */
static void
HisBuild(void)
{
    int i;
    char id[3];
    char message[800];

    i = 0;
    while ( IDBGetN ( WMContext[WMCurrentC]->HistoryServer, i,
		     id, message, sizeof ( message ) ) ) {
	AddStrToWorm ("G");
	AddStrToWorm (id);
	AddStrToWorm (message);
	AddStrToWorm (EOO);
	i++;
    }
    (void) WMDelHis();


} /* HisBuild */

/*------------------------------------------------------------------------
 *  Function	: MesBuild
 *  Purpose	: Put messages in the worm
 *------------------------------------------------------------------------
 */
static void
MesBuild(void)
{
    int i;
    char id[3];
    char message[800];

    i = 0;
    while (WMGetMsg(i, id, message, sizeof(message))==TRUE){
	AddStrToWorm ("M");
	AddStrToWorm (id);
	AddStrToWorm (message);
	AddStrToWorm (EOO);
	i++;
    }
    (void) WMDelMsg();

} /* MesBuild */

/*------------------------------------------------------------------------
 *  Function	: VarBuild
 *  Purpose	: Put variables in worm
 *  Notes	: All variables are put in the worm
 *                Should be all changed!
 *------------------------------------------------------------------------
 */
static void
VarBuild(void)
{
    WMIdent_t id;
    char inh[WM_MAXWRMLEN];
    int n;

    inh[0] = '\0';
    n = 0;
    while (WMGetVarN(n, id, inh, sizeof(inh))) {
	if (WMIsMutatedVar(id)) {
	    if (WMContext[WMCurrentC]->csflag == WM_CLIENT) {
		AddStrToWorm("V");
	    } else {
		AddStrToWorm("K");
	    }
	    AddStrToWorm (id);
	    AddStrToWorm (inh);
	    AddStrToWorm (EOO);
	} else if (WMContext[WMCurrentC]->csflag == WM_SERVER || WMContext[WMCurrentC]->csflag == WM_BOTH) {
	    if (WMIsRegistered(id)) {
		AddStrToWorm("K");
	    } else {
		AddStrToWorm("V");
	    }
	    AddStrToWorm (id);
	    AddStrToWorm (inh);
	    AddStrToWorm (EOO);
	}
        inh[0] = '\0';
	n++;
    }
} /* VarBuild */

/*
 *------------------------------------------------------------------------
 *  Function	: WMBuild
 *  Purpose	: Put the local datastructure WMContext into the worm
 *  Description	:
 *
 *  The function will build a worm buffer using the internal datastructures. 
 *  The client/server flag in the current context is used to determine how 
 *  to build the worm:
 *
 *  Client-mode:
 *
 *  -  the buffer is coded as window
 *  -  commands are always coded
 *  -  only registered, updated variables are coded
 *  -  screenforms are ignored
 *  -  history is ignored
 *  -  messages are ignored
 *
 *  Server-mode:
 *
 *  -  the buffer is coded as buffer
 *  -  commands are ignored
 *  -  only updated variables are coded
 *  -  screenforms are always coded
 *  -  history is always coded
 *  -  messages are always coded
 *
 *  In erroneous situations FALSE will be returned. It indicates that the context 
 *  in invalid, or the worm has become too long. WMErrno is unaffected by this 
 *  routine.
 *
 *  Error values	: none
 *
 *  Example		:
 *
 *    #include <wm.h>
 *    .
 *    (void) WMBuild();
 *    .
 *  Returns	: 
 *
 *  | Value	| Meaning		|
 *  |---------: | :---------------------|
 *  | FALSE	| something fails	|
 *  | TRUE	| succeeded		|
 *
 *------------------------------------------------------------------------
 */
bool
WMBuild(char *Worm)
{
    wm_p = Worm;
    WormP = 0;
    *wm_p = 0;

    if (WMContext[WMCurrentC]->csflag == WM_SERVER || WMContext[WMCurrentC]->csflag == WM_BOTH) {
	HisBuild();
	MesBuild();
	VarBuild();
	DefBuild();
	TemplateBuild();
	if (FrmBuild()) { BufBuild("W"); } 
    } else if (WMContext[WMCurrentC]->csflag == WM_CLIENT) {
	VarBuild();
	ComBuild();
	BufBuild("B");
    } else {
	return FALSE;
    }
    *wm_p++ = 3;
    *wm_p = 0;
    return (WormP != WM_MAXWRMLEN);
} /* WMBuild */

/*------------------------------------------------------------------------
 *  Function	: WMWormToCB
 *  Purpose	: Copy worm parts to CB
 *  Method	: Call of WMGetFieldN, WMGetVarN or WMGetMsg and CBAppAttribute.
 *  Author(s)	: Lolke Boonstra
 *  Creation	: 21-10-1996
 *
 *  History:
 *  21-10-1996	: lolke    : creation
 *
 *  Notes	: Only implemented for Fields, Variables and Messages.
 *
 *  Returns	: FALSE on failure
 *
 *------------------------------------------------------------------------
 */
bool
WMWormToCB(
	char part, 
	CBBuf_t *CBbuf, 
	size_t CBsize)
{
    int i;
    WMIdent_t id;
    char tmp[5000];

    CBClear(CBbuf, CBsize);
    if (part == WM_FIELDS) {
	for (i = 0; WMGetFieldN(i, id, tmp, sizeof(tmp)); i++) {
	    if (!CBAppAttribute(CBbuf, CBsize, id, tmp)) {
		return FALSE;
	    }
	}
    } else if (part == WM_VARIABLES) {
	for (i = 0; WMGetVarN(i, id, tmp, sizeof(tmp)); i++) {
	    if (!CBAppAttribute(CBbuf, CBsize, id, tmp)) {
		return FALSE;
	    }
	}
    } else if (part == WM_MESSAGES) {
	for (i = 0; WMGetMsg(i, id, tmp, sizeof(tmp)); i++) {
	    if (!CBAppAttribute(CBbuf, CBsize, id, tmp)) {
		return FALSE;
	    }
	}
    }
    return TRUE;
} /* WMWormToCB */

/*------------------------------------------------------------------------
 *  Function	: WMWormComplete
 *  Purpose	: Check if supplied worm data is terminated by an ETX
 *  Method	: We check if the last character is an ETX. Alternatively,
 *                we could check for any ETX in the supplied buffer.
 *  Author(s)	: Jan van der Steen
 *  Creation	: 24-08-2001
 *
 *  History:
 *  24-08-2001	: steenj   : creation
 *
 *  Returns	: FALSE on failure
 *
 *------------------------------------------------------------------------
 */
bool
WMWormComplete(const char *worm, size_t len)
{
    if (len && worm[len-1] == *ETX) return TRUE;

    return FALSE;
} /* WMWormComplete */


/*------------------------------------------------------------------------
 *  Function	: WMObjectComplete
 *  Purpose	: Check if supplied worm object is terminated by an EOO
 *  Method	: We check if the last character is an EOO.
 *  Author(s)	: Jan van der Steen
 *  Creation	: 24-08-2001
 *
 *  History:
 *  24-08-2001	: steenj   : creation
 *
 *  Returns	: FALSE on failure
 *
 *------------------------------------------------------------------------
 */
bool
WMObjectComplete(const char *object, size_t len)
{
    if (len && object[len-1] == *EOO) return TRUE;

    return FALSE;
} /* WMObjectComplete */
