/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: ouf.h
 *  Description	: OUF-declarations
 *  Author(s)	: Peter Jan van der Mespel
 *  Creation	: 06-08-1996
 *
 *  History:
 *  06-08-1996	: peter jan      : creation
 *  22-10-1996	: HSi	: Added OUF_EXPANSION_L.
 *  31-10-1997	: HSi	: Increased OUF_REQUEST_L to 14 to accomodate
 *			  formats of length 3 (like MAB).
 *  11-11-1997	: HSi	: Increased OUF_TEXT_L to avoid overflow of
 *			  logstrings.
 *  04-02-1999  : EvD   : Added for new options SEL UPD SED command the lengths
 *			  OUF_LIBRARY_L 40 OUF_BYLIB_L 4 OUF_RECTYPE_L 1
 *                      : Added OUF_ILN_L (4)
 *                      : OUF_REQUEST_L from 14 to 18 positions
 *  12-12-2002  : fb    : Increased OUF_BUFFER to CB_SIZE_MAX.
 *  04-07-2003	: huug	: SRC option in \SEL \UPD command
 *  02-07-2004  : huug  : extra constants defined ofr character negotiation:
 *		:	: OUF_CBUF_L OUF_CHARSET_L
 *  26-08-2004  : huug  : added OUF_UTF8_NOT_SUPPORTED
 *  22-10-2004  :michael: added OUF_*_DIR_NOTDIR error codes for support
 *                        of new directory creation routines in ouf.c
 *  21-l2-2004  :michael: OUF_PPN_L increased from 10 to 16
 *  23-02-2007	: clemens  : fixed problems with/cleaned-up array sizes
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: ouf.h 360943 2018-10-23 16:48:25Z heesakkm $"

#include <stdio.h>
#include <pica.h>
#include <p3.h>

/* Error codes */
#define OUF_NO_ERROR                0
#define OUF_PROFILE_NOT_OPENED      1
#define OUF_PROFILE_NOT_CLOSED      2
#define OUF_TRACE_ERROR             3
#define OUF_CFG_DIR_NOT_SET         4
#define OUF_P3ERROR                 5
#define OUF_NOT_ENOUGH_MEMORY       6
#define OUF_BUFFER_NOT_CLEARED      7
#define OUF_ATTRIBUTE_NOT_LOCATED   8
#define OUF_US_FILE_NOT_OPENED      9
#define OUF_US_FILE_NOT_CLOSED     10
#define OUF_FR_READ_ERROR          11
#define OUF_FILE_STATUS_NOT_RETR   13
#define OUF_FR_FILE_NOT_OPENED     14
#define OUF_FR_FILE_NOT_CLOSED     15
#define OUF_TTL_FILE_NOT_OPENED    16
#define OUF_TTL_FILE_NOT_CLOSED    17
#define OUF_BLOCK_FAILED           18
#define OUF_UNBLOCK_FAILED         19
#define OUF_SIGNAL_SET_NOT_EMPTIED 20
#define OUF_SIGNAL_HANDLER_NOT_SET 21
#define OUF_SIGNAL_NOT_ADDED       22
#define OUF_UNKNOWN_FORMAT         23
#define OUF_FR_SYNTAX_ERROR        24
#define OUF_PROFILE_NOT_UPDATED    25
#define OUF_FETCH_DIR_NOT_FOUND    26 
#define OUF_FETCH_DIR_NOTDIR       27 
#define OUF_FETCH_DIR_PROTECTED    28 
#define OUF_TEMP_DIR_NOTDIR        29 
#define OUF_TEMP_DIR_NOT_FOUND     30 
#define OUF_TEMP_DIR_PROTECTED     31 
#define OUF_TITLE_DIR_NOTDIR       33 
#define OUF_TITLE_DIR_NOT_FOUND    34 
#define OUF_TITLE_DIR_PROTECTED    35 
#define OUF_UPDSIG_DIR_NOTDIR      36 
#define OUF_UPDSIG_DIR_NOT_FOUND   37 
#define OUF_UPDSIG_DIR_PROTECTED   38 
#define OUF_TRACE_DIR_NOTDIR       39 
#define OUF_TRACE_DIR_NOT_FOUND    40 
#define OUF_TRACE_DIR_PROTECTED    41 
#define OUF_UTF8_NOT_SUPPORTED     42

#define OUF_CBUF_L  240
#define OUF_DATABASE_L 30
#define OUF_DATETIME_L 35
#define OUF_DIR_L      80
#define OUF_ERROR_L    80
#define OUF_FIELD_L    80
#define OUF_FILE_L     80
#define OUF_FORMAT_L   10
#define OUF_EXPANSION_L 10
#define OUF_HOST_L     60
#define OUF_OPTIONS_L  100
#define OUF_PASSWORD_L 15
#define OUF_PORT_L     16
#define OUF_PPN_L      16
#define OUF_USER_L     40
#define OUF_SYNTAX_L   31
#define OUF_MORE_L     99
#define OUF_REQUEST_L  (OUF_PPN_L + OUF_ILN_L + OUF_SYNTAX_L + 2)
#define OUF_SOURCE_L  40
#define OUF_LIBRARY_L  40
#define OUF_BYLIB_L	20
#define OUF_RECTYPE_L	10
#define OUF_ILN_L	20

#define OUF_MAX_KEYLEN         32
#define OUF_MAX_PPN            20001
#define OUF_MAX_UPDATES        450
#define OUF_DBS_UPDATE_SIZE    65
#define OUF_BUFFER	       CB_SIZE_MAX
#define OUF_MAX_PPN_RETRIEVAL  3
#define OUF_MAX_LINKTAB	       64

#define OUF_SEP                31

#define OUF_NO_EVENT           0
#define OUF_FETCH_EVENT        1
#define OUF_UPDATE_EVENT       2
#define OUF_INTERRUPT_EVENT    3

#define OUF_FIELD_DELIMITERS   "\r"
#define OUF_UPDATE_SIGNAL_FILE "updsig"
#define OUF_FETCH_FILE         "fetch"
#define OUF_TITLE_FILE         "title"
#define OUF_TRACE_FILE         "trace"
#define OUF_TEMP_FILE          "ouf.tmp"
#define ERR_EXT			".err"
#define OUF_ERR                "OUF"
#define OUF_DUMMY_DOMAIN       "@server.domain"

#define OUF_DEF_LOOP_CHECK_TIMEOUT 300
#define OUF_DEF_MAX_RECORDS    500
#define OUF_DEF_MAX_FETCHRECORDS 50
#define OUF_DEF_MAX_FETCHTIME  120
#define OUF_DEF_MAX_TIME       1
#define OUF_DEF_US_POLL        1
#define OUF_DEF_FORMAT         "P"
#define OUF_DEF_LOOP_CHECK_INT 60
#define OUF_DEF_MAX_RETR_FREQ  3
#define OUF_DEF_FR_POLL        7
#define OUF_DEF_LAST_TIME      "01-01-1980"
#define OUF_DEF_MAX_TITLEBYTES 100000
#define OUF_DEF_MAX_TITLEAGE   10
#define OUF_OK                 0
#define OUF_ERROR              -1
#define OUF_NOT_FOUND          100
#define OUF_AUTHORITY          1
#define OUF_DISCONNECT         2
#define OUF_NO_SUPPORT         3
#define OUF_PROTOCOL           4
#define OUF_SERVER             5
#define OUF_TTLWRONG           6
#define OUF_LOOP_DETECTED      7
#define OUF_MIN_UPDATE_POLL    5
#define OUF_MAX_TITLEPATTERN   320

#define OUF_INI		"ouf.ini"

#define OUF_DEF_EXPANSION	""	/* Default expansion */

#define OUF_DIRMODE		0777

#define OUF_DEBUG(n, expr) if (gDebug>=n) (void) tr_write expr

#define OUF_UPD  "ouf_update"
#define OUF_FETCH  "ouf_fetch"

typedef struct {
    char host[OUF_HOST_L + 1];
    char port[OUF_PORT_L + 1];
    char user[OUF_USER_L + 1];
    char password[OUF_PASSWORD_L + 1];
    char database[OUF_DATABASE_L + 1];
    bool bUseUTF8;
    P3Handle_p P3Handle;
    bool bConnected;
    bool bEncryptPwd;
    bool bCBS4inuse;
} CBS_t;

bool CBS_init (FILE *, CBS_t *);
void close_connection (CBS_t *);
bool  dir_exists (char *, int, int, int);
char * OUFConstructTimeStampedFilename (char *, const char *, const char *);
bool OUFConvertPica3ToADBFormat (char *, char *);
const char *OUFGetResultMessage (int, char *);
bool select_database (CBS_t *);
void OUFSetError (int ouferrno);
bool setup_connection (CBS_t *);
const char *OUFStripFilename (const char *);
char * get_titlefname (char *fmt, char *dir, int iln);
int retrieve_title( CBBuf_t *CBbuf, size_t CBSize, const char *PPN,
    const char *copy, const char *syntax, const char *expansion,
    const char *ILN, CBS_t *cbsp);

int gDebug;
