/*========================================================================
 *  Copyright (c) 1994 by Pica
 *
 *  File	: ouf_pf.c
 *  Description	: a library for reading, updating and deleting tables
 *                of the *.ini type.
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *  20-11-1998	: EvD      : PFRead addapted. Output string initiated \0
 *  20-10-2004	: michael  : fixed "temp" file name overloading "bug"
 *  12-10-2005	: clemens  : added support for comment (silly)
 *
 *  Notes	:
 *
 *========================================================================
 */

#pragma ident "$Id: ouf_pf.c 374014 2020-02-06 11:01:25Z heesakkm $"

#include <string.h>
#include <ctype.h>
#include <stdio.h>

#include <pica.h>
#include <picamisc.h>
#include <env.h>
#include <er.h>
#include "ouf_pf.h"
#include "ouf_trace.h"

/*------------------------------------------------------------------------
 *  Function	: GetRight
 *  Purpose	: get the right part corresponding to lin and left1.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *  12-01-2005	: michael  : now declared static
 *  12-10-2005	: clemens  : fixed bug in loop-handling
 *
 *  Notes	:
 *
 *  Returns	: TRUE if found, else FALSE.
 *                fulfill right1 with right content if found.
 *
 *------------------------------------------------------------------------
 */
static bool
GetRight(
	char *lin,
	const char *left1,
	char *right1)
{
    int i,j;
    char stri[MAXBUFF+1];

    for (i = 0; PM_is_ascii_space(lin[i]); i++) ;
    for (j = 0; (lin[i] != '=') && (lin[i] != '\0'); i++, j++){
	stri[j] = lin[i];
    }
    j--;
    for ( ; PM_is_ascii_space(stri[j]); j--) ;
    stri[j+1] = '\0';
    if (strcmp(stri,left1) == 0){
	if (lin[i] != '\0') i++;
	for ( ; PM_is_ascii_space(lin[i]); i++) ;
	for (j = 0; (lin[i] != '\0') &&
		    !( (lin[i] == '-') && (lin[i+1] == '-') ); i++, j++){
	    right1[j] = lin[i];
	}
	j--;
	for ( ; PM_is_ascii_space(right1[j]); j--) ;
	right1[j+1] = '\0';
	return TRUE;
    }
    return FALSE;
} /* GetRight() */

#if 0	// Unused functions
/*------------------------------------------------------------------------
 *  Function	: GetBoth
 *  Purpose	: Retrieve both right and left parts of lin.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *  12-01-2005	: michael  : now declared static
 *
 *  Notes	:
 *
 *  Returns	: fulfill left1 and right1 with appropriate content.
 *
 *------------------------------------------------------------------------
 */
static void
GetBoth(
	char *lin,
	char *left1,
	char *right1)
{
    int i,j;

    for (i = 0; PM_is_ascii_space(lin[i]); i++) ;
    for (j = 0; lin[i] != '='; i++, j++){
	left1[j] = lin[i];
    }
    i++;
    if (j != 0) j--;
    else left1[0] = '\0';
    for ( ; PM_is_ascii_space(left1[j]); j--) ;
    left1[j+1] = '\0';
    for ( ; PM_is_ascii_space(lin[i]); i++) ;
    for (j = 0; (lin[i] != '\0') &&
		!( (lin[i] == '-') && (lin[i+1] == '-') ); i++, j++){
	right1[j] = lin[i];
    }
    if (j != 0) j--;
    else right1[0] = '\0';
    for ( ; PM_is_ascii_space(right1[j]); j--) ;
    right1[j+1] = '\0';
} /* GetBoth() */
#endif

/*------------------------------------------------------------------------
 *  Function	: SearchHeading
 *  Purpose	: Search a heading
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *  12-01-2005	: michael  : now declared static
 *
 *  Notes	: keep position of the heading in the file.
 *                (in fact just at the end of the heading).
 *
 *  Returns	: TRUE if found else FALSE.
 *
 *------------------------------------------------------------------------
 */
static bool
SearchHeading(
	FILE *ft,
	const char *head)
{
    int i,j;
    char line[MAXBUFF+1];
    char str1[MAXBUFF];

    while (fgets(line,MAXBUFF,ft) != NULL){
	for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	if (line[i] == '['){
	    for (j = 0; !PM_is_ascii_space(line[i]); i++, j++){
	       str1[j] = line[i];
	    }
	    str1[j] = '\0';
	    if (strcmp(str1,head) == 0) return TRUE;
	}
    }
    return FALSE;
} /* SearchHeading() */

#if 0	// Unused function
/*------------------------------------------------------------------------
 *  Function	: PFReadN
 *  Purpose	: Read both part of a record at the N position inside
 *		  a heading.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *
 *  Notes	:
 *
 *  Returns	: TRUE if success else FALSE.
 *		  left and right strings are fulfilled from the record.
 *
 *------------------------------------------------------------------------
 */
bool
PFReadN(
	FILE *fpt,
	const char *heading,
	int position,
	char *left,
	char *right)
{
    int i,j;
    char line[MAXBUFF+1];

    if (position < 1) return FALSE;
    rewind(fpt);
    if (SearchHeading(fpt,heading)){
	for (i = 1; i <= position; i++){
	    if (fgets(line,MAXBUFF,fpt) == NULL) return FALSE;
            for (j = 0; PM_is_ascii_space(line[j]); j++) ;
	    if (line[j] == '[') return FALSE;
	    if (line[j] == '\0') i--;      /* blank line */
	}
	GetBoth(line,left,right);
	return TRUE;
    }
    return FALSE;
} /* PFReadN() */
#endif

/*------------------------------------------------------------------------
 *  Function	: PFRead
 *  Purpose	: Retrieve the right part of a record designated by
 *                the left part and it's heading.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *
 *  Notes	:
 *
 *  Returns	: TRUE if success, else FALSE.
 *                right string is filled with the content of the record.
 *
 *------------------------------------------------------------------------
 */
bool
PFRead(
	FILE *fpt,
	const char *heading,
	const char *left,
	char *right)
{
    int i;
    char line[MAXBUFF+1];

    *right = '\0';
    rewind(fpt);
    if (SearchHeading(fpt,heading)){
	while (fgets(line,MAXBUFF,fpt) != NULL){
	    for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	    if (line[i] == '[') return FALSE;
	    if (line[i] == ';') continue;
	    if (GetRight(line,left,right)) return TRUE;
	}
    }
    return FALSE;
} /* PFRead() */

/*------------------------------------------------------------------------
 *  Function	: PFOpen
 *  Purpose	: Open a file.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *  25-04-1995	: eck      : adapted to ibw PFOpen()
 *
 *  Notes	:
 *
 *  Returns	: File pointer or NULL if wrong.
 *
 *------------------------------------------------------------------------
 */
FILE *
PFOpen(
	const char *file,
	const char *mode)
{
    FILE	*fp;
    char	pathfile[1000];
    const char	*path;

    path = ENVGetVar("DOTINIDIR");
    pathfile[0] = '\0';
    if (path != NULL && file[0]!='/') {
	(void) strcat(pathfile, path);
    }
    (void) strcat(pathfile, file);

    fp = fopen(pathfile, mode);
    if (fp == NULL) {
	(void) ERWrite(APPLE_FILEOPEN, __func__, "%s", pathfile);
    }
    return fp;
} /* PFOpen() */

/*------------------------------------------------------------------------
 *  Function	: PFClose
 *  Purpose	: Close the file associated to the pointer.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 13-09-1994
 *
 *  History:
 *  13-09-1994	: remy     : creation
 *
 *  Notes	:
 *
 *  Returns	: TRUE (success) else FALSE.
 *
 *------------------------------------------------------------------------
 */
bool
PFClose(
	FILE *fptr)
{
    if (fclose(fptr) == EOF) return FALSE;
    return TRUE;
} /* PFClose() */

#if 0	// Unused functions
/*------------------------------------------------------------------------
 *  Function	: PFDelete
 *  Purpose	: Delete a record in a table
 *  Method	: Creates a temporary file, writes the original to it
 *                minus the record to delete.
 *                Then overwrite the original with the temporary one.
 *  Author(s)	: Remy Deschamps
 *  Creation	: 14-09-1994
 *
 *  History:
 *  14-09-1994	: remy     : creation
 *  21-10-2004	: michael  : changed to use tmpnam() to create the temp
 *                           file name, and some associated changes
 *  21-01-2005	: michael  : now using a self-defined array for tmpnam
 *
 *  Notes	: does every open and close itself.
 *
 *  Returns	: TRUE if delete worked, FALSE otherwise
 *
 *------------------------------------------------------------------------
 */
bool
PFDelete(
	const char *file,
	const char *heading,
	const char *left)
{
    FILE *fp, *tmpfp = NULL;
    int i,j;
    bool HEAD;
    char line[MAXBUFF+1];
    char right[MAXBUFF+1];
    char str[MAXBUFF+1];
    char filename[BUFSIZ];
	
    if (PM_file_tmpnam(filename) == NULL) {
	/* well, there's not much that we can do here. */;
	(void)fprintf (stderr, "PM_file_tmpnam() failed in PFDelete!\n");
	return FALSE;
    }

    HEAD = FALSE;
    if ((fp = (PFOpen (file,"r"))) == NULL) return FALSE;
    if (PFRead(fp,heading,left,right)){  /* test if the line exists */
	rewind(fp);

	if (filename != NULL) {
	    tmpfp = fopen (filename, "w+");
	}

	if (tmpfp == NULL) {
	    (void) PFClose(fp);
	    return FALSE;
        }

        while (fgets(line,MAXBUFF,fp) != NULL){
	    for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	    if (line[i] != '['){
		if (!HEAD) {
		    (void) fputs(line,tmpfp);
		} else if (line[i] == ';') {
		    (void) fputs(line,tmpfp);
		} else if (!GetRight(line,left,right)) {
		    (void) fputs(line,tmpfp);
		}
	    } else {
		for (j = 0; !PM_is_ascii_space(line[i]); i++, j++){
		    str[j] = line[i];
		}
		str[j] = '\0';
		HEAD = (strcmp(str,heading) == 0);
		(void) fputs(line,tmpfp);
	    }
	}

	/* we have to do this PFClose/PFOpen pair because */
	/* PFOpen mangles the filename that it receives   */
	/* internally. -michael                           */
	
	(void) PFClose(fp);  
        fp = PFOpen(file,"w");
	(void) rewind (tmpfp); /* rewind suffices here */
        
        while (fgets(line,MAXBUFF,tmpfp) != NULL) (void) fputs(line,fp);
	
	(void) PFClose(fp);
	(void) fclose (tmpfp);
	(void) remove (filename);
	return TRUE;
    }
    (void) PFClose(fp);
    return FALSE;
} /* PFDelete() */

/*------------------------------------------------------------------------
 *  Function	: PFStore
 *  Purpose	: store a new record.
 *  Method	: is using a temporary file.
 *  Author(s)	: Remy Deschamps
 *  Creation	: 14-09-1994
 *
 *  History:
 *  14-09-1994	: remy     : creation
 *  21-10-2004	: michael  : changed to use tmpnam() to create the temp
 *                           file name, and some associated changes
 *  21-01-2004  : michael  : change goofy strcat() sequence to sprintf(),
 *                           and now using a self-defined array for tmpnam()
 *
 *  Notes	: file must not be opened before calling this function
 *
 *  Returns	: TRUE if ok, else FALSE
 *
 *------------------------------------------------------------------------
 */
bool
PFStore(
	const char *file,
	const char *heading,
	const char *left,
	const char *right)
{
    FILE *fp, *tmpfp = NULL;
    int i,j;
    char line[MAXBUFF+1];
    char record[MAXBUFF+1];
    char str[MAXBUFF+1];
    char filename[BUFSIZ];
    
    if (PM_file_tmpnam(filename) == NULL) {
	/* time to go */
	return FALSE;
    }

    (void) sprintf(record, "%s = %s\n", left, right);

    if ((fp = PFOpen(file,"r")) == NULL) return FALSE;
    if (!PFRead(fp,heading,left,line)){
	rewind(fp);

	if (filename != NULL) {
	    tmpfp = fopen (filename, "w+");
	}

	/* tmpfp was initialized to NULL */

	if (tmpfp == NULL){
	    (void) PFClose(fp);
	    return FALSE;
        }

	while (fgets(line,MAXBUFF,fp) != NULL){
	    for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	    if (line[i] != '[') (void) fputs(line,tmpfp);
	    else {
		for (j = 0; !PM_is_ascii_space(line[i]); i++, j++) str[j] = line[i];
		str[j] = '\0';
		(void) fputs(line,tmpfp);
		if (strcmp(str,heading) == 0) (void) fputs(record,tmpfp);
	    }
	}
	(void) PFClose(fp);
	(void) PFClose(tmpfp);
	fp = PFOpen(file,"w");
	(void) rewind (tmpfp);
	while (fgets(line,MAXBUFF,tmpfp) != NULL){
	    (void) fputs(line,fp);
	}
	(void) PFClose(fp);
	(void) fclose(tmpfp);
	(void) remove (filename);
	return TRUE;
    }
    (void) PFClose(fp);
    return FALSE;
} /* PFStore() */
#endif

/*------------------------------------------------------------------------
 *  Function	: PFUpdate
 *  Purpose	: Update a record of the table.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 15-09-1994
 *
 *  History:
 *  15-09-1994	: remy     : creation
 *  21-10-2004	: michael  : changed to use tmpnam() to create the temp
 *                           file name, and some associated changes
 *
 *  21-01-2005	: michael  : replaced strcat sequence with an sprintf, now
 *                           using self-defined array for tmpnam
 *
 *  Notes	: if the record to update doesn't exist then creates it.
 *
 *  Returns	: TRUE (success) else FALSE.
 *
 *------------------------------------------------------------------------
 */
bool
PFUpdate(
	const char *file,
	const char *heading,
	const char *left,
	const char *right)
{
    FILE *fp, *tmpfp = NULL;
    int i,j;
    bool HEAD, CREATE;
    char line[MAXBUFF+1];
    char record[MAXBUFF+1];
    char str[MAXBUFF+1];
    char filename[BUFSIZ];
    
    if (PM_file_tmpnam(filename) == NULL) {
	return FALSE;
    }

    (void) sprintf (record, "%s = %s\n", left, right);
    
    HEAD = FALSE;
    CREATE = TRUE;
    if ((fp = (PFOpen(file,"r"))) == NULL) return FALSE;
    if (!SearchHeading(fp,heading)){
	(void) PFClose(fp);
	return FALSE;
    }
    rewind(fp);

    if (filename != NULL) {
	tmpfp = fopen (filename, "w+");
    }

    if (tmpfp == NULL) {
	(void) PFClose(fp);
	return FALSE;
    }
    
    while (fgets(line,MAXBUFF,fp) != NULL){
        for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	if (line[i] != '['){
	    if (!HEAD)  {
		(void) fputs(line,tmpfp);
	    } else if (line[i] == ';') {
		(void) fputs(line,tmpfp);
	    } else if (!GetRight(line,left,str)) {
		(void) fputs(line,tmpfp);
	    } else {
		(void) fputs(record,tmpfp);
		CREATE = FALSE;
	    }
	} else {
	    if ((HEAD) && (CREATE)) {
		(void) fputs(record,tmpfp);
		CREATE = FALSE;
	    }
	    for (j = 0; !PM_is_ascii_space(line[i]); i++, j++){
		str[j] = line[i];
	    }
	    str[j] = '\0';
	    HEAD = (strcmp(str,heading) == 0);
	    (void) fputs(line,tmpfp);
	}
    }
    (void) PFClose(fp);
    (void) rewind(tmpfp);

    fp = fopen (file, "w"); /* this is a mess */

    if (fp == NULL) {
	(void) tr_write ("fopen of \"%s\" failed!\n", file);
	(void) fclose (tmpfp);
	(void) remove (filename);
	return FALSE;
    }

    while (fgets (line, MAXBUFF, tmpfp) != NULL) {
	(void) fputs (line, fp);
    }
    (void) fclose (fp);
    (void) fclose (tmpfp);
    (void) remove (filename);
    return TRUE;
} /* PFUpdate() */

#if 0	// Unused functions
/*------------------------------------------------------------------------
 *  Function	: PFCreateHeading
 *  Purpose	: Create a new heading in the table.
 *  Method	:
 *  Author(s)	: Remy Deschamps
 *  Creation	: 14-09-1994
 *
 *  History:
 *  14-09-1994	: remy     : creation
 *  21-01-2005	: michael  : replaced strcats with a strncpy and a strncat
 *
 *  Notes	:
 *
 *  Returns	: TRUE (success) else FALSE.
 *
 *------------------------------------------------------------------------
 */
bool
PFCreateHeading(
	const char *file,
	const char *heading)
{
    FILE *fpt;
    char str[MAXBUFF+1];

    (void) strncpy (str, heading, sizeof str);
    (void) strncat (str,"\n", sizeof str);

    if ((fpt = (PFOpen(file,"a+"))) == NULL) return FALSE;
    if (!SearchHeading(fpt,heading)){ /* if heading isn't already there */
	(void) fputs(str,fpt);
	(void) PFClose(fpt);
	return TRUE;
    } else {
	(void) PFClose(fpt);
	return FALSE;
    }
} /* PFCreateHeading() */

/*------------------------------------------------------------------------
 *  Function	: PFClearHeading
 *  Purpose	: Delete the content of an heading plus the heading
 *                if boolean is set to true.
 *  Method	: is using a temporary file.
 *  Author(s)	: Remy Deschamps
 *  Creation	: 14-09-1994
 *
 *  History:
 *  14-09-1994	: remy     : creation
 *  21-10-2004	: michael  : changed to use tmpnam() to create the temp
 *                           file name, and some associated changes
 *  21-01-2005	: michael  : changed tmpfile to an array
 *
 *  Notes	:
 *
 *  Returns	: TRUE (success) else FALSE
 *
 *------------------------------------------------------------------------
 */
bool
PFClearHeading(
	const char *file,
	const char *heading,
	bool HEADALSO)
{
    FILE *fp, *tmpfp = NULL;
    int i,j;
    char line[MAXBUFF+1];
    char str[MAXBUFF+1];
    bool HEAD;
    char filename[BUFSIZ];
    
    if (PM_file_tmpnam(filename) == NULL) {
	return FALSE;
    }

    HEAD = FALSE;
    if ((fp = (PFOpen (file,"r"))) == NULL) return FALSE;
    if (SearchHeading (fp,heading)){   /* does the heading exist? */
	rewind (fp);

	if (filename != NULL) {
	    tmpfp = fopen (filename, "w+");
	}

	if (tmpfp == NULL) {
	    (void) PFClose (fp);
	    return FALSE;
	}

	while (fgets(line,MAXBUFF,fp) != NULL){
	    for (i = 0; PM_is_ascii_space(line[i]); i++) ;
	    if (line[i] != '['){
		if (!HEAD) (void) fputs(line,tmpfp);
	    } else {
		for (j = 0; !PM_is_ascii_space(line[i]); i++, j++){
		    str[j] = line[i];
		}
		str[j] = '\0';
		HEAD = (strcmp(str,heading) == 0);
                if (!HEAD) (void) fputs(line,tmpfp);
		else if(!HEADALSO) (void) fputs(line,tmpfp);
	    }
	}
	(void) PFClose(fp);
	fp = PFOpen (file,"w");

	(void) rewind (tmpfp);

	while (fgets(line, MAXBUFF, tmpfp) != NULL){
	    (void) fputs(line,fp);
	}
	(void) PFClose(fp);
	(void) fclose (tmpfp);
	(void) remove (filename);
	return TRUE;
    }
    (void) PFClose (fp);
    return FALSE;
} /* PFClearHeading() */
#endif
