/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: ouf_trace.h
 *  Description	: Trace declarations
 *  Author(s)	: Peter Jan van der Mespel
 *  Creation	: 23-04-1996
 *
 *  History:
 *  23-04-1996	: peter jan : creation
 *  27-12-2004	: michael   : added bBuffer option to TRSettings struct
 *                            for turning stdio output buffering on/off
 *                            ouf.ini variable for this is "Buffer"
 *
 *  Notes	:
 *
 *========================================================================
 */

#ifndef OUF_TRACE_H_INCLUDED
#define OUF_TRACE_H_INCLUDED

#pragma ident "$Id: ouf_trace.h 368646 2019-07-16 16:25:21Z vaneckh $"

#include <stdio.h>
#include <pica.h>

/* Error codes */
enum {
    TRACE_NO_ERROR,
    TRACE_ERROR,
    TRACE_FILE_NOT_OPENED,
    TRACE_FILE_NOT_CLOSED,
    TRACE_FILE_NOT_CREATED,
    TRACE_NOT_INITIALIZED,
    TRACE_FILENAME_TOO_LONG
};

#define TRACE_DATETIME_L            25
#define TRACE_FILE_L               100
#define TRACE_ERROR_L               80
#define TRACE_DEFAULT_DEBUGLEV       5

/* Type definitions */
typedef struct
{
    bool bInitialized;

    bool bTrace;
    bool bReplace;
    bool bScreen;

    char tracefile[TRACE_FILE_L+1];

    int  nError;
} TRACE_t;

/* Function definitions */
bool tr_date(void);
bool tr_init(FILE *, int *);
/*PRINTFLIKE1*/
bool tr_write(const char *fmt, ...) __attribute__((format(printf, 1, 2)));

#define TRACEWrite(s) (void)tr_write("%s", s)
#define TRACEInit(x,y,z) tr_init(x,z)
#define TRACEDate tr_date

#endif /* OUF_TRACE_H_INCLUDED */
