/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: ouftest.h
 *  Description	: OUFTest-declarations
 *  Author(s)	: Peter Jan van der Mespel
 *  Creation	: 01-11-1996
 *
 *  History:
 *  01-11-1996	: peter jan      : creation
 *  15-02-1999  : els  : added ILN_L (3) INSTITUTION_L (10)
 *  12-12-2002  : fb   : Increased OUFTEST_BUFFER to CB_SIZE_MAX.
 *  16-04-2020	: zwaalm   : CBS-9161 Fixed parsing PPN and EPN (unused).
 *
 *  Notes	:
 *
 *========================================================================
 */

#ifndef OUFTEST_H_INCLUDED
#define OUFTEST_H_INCLUDED

#pragma ident "$Id: ouftest.h 376107 2020-04-17 14:07:46Z zwaalm $"

#include <pica.h>

/* Error codes */
#define OUFTEST_NO_ERROR                0
#define OUFTEST_PROFILE_NOT_OPENED      1
#define OUFTEST_PROFILE_NOT_CLOSED      2
#define OUFTEST_TRACE_ERROR             3
#define OUFTEST_CFG_FILE_NOT_SET        4
#define OUFTEST_P3ERROR                 5
#define OUFTEST_NOT_ENOUGH_MEMORY       6
#define OUFTEST_US_FILE_NOT_OPENED      9
#define OUFTEST_US_FILE_NOT_CLOSED     10
#define OUFTEST_FR_READ_ERROR          11
#define OUFTEST_FETCH_DIR_NOT_FOUND    12
#define OUFTEST_FILE_STATUS_NOT_RETR   13
#define OUFTEST_FR_FILE_NOT_OPENED     14
#define OUFTEST_FR_FILE_NOT_CLOSED     15
#define OUFTEST_TTL_FILE_NOT_OPENED    16
#define OUFTEST_TTL_FILE_NOT_CLOSED    17
#define OUFTEST_BLOCK_FAILED           18
#define OUFTEST_UNBLOCK_FAILED         19
#define OUFTEST_SIGNAL_SET_NOT_EMPTIED 20
#define OUFTEST_SIGNAL_HANDLER_NOT_SET 21
#define OUFTEST_SIGNAL_NOT_ADDED       22
#define OUFTEST_UNKNOWN_FORMAT         23
#define OUFTEST_TITLE_DIR_NOT_FOUND    26
#define OUFTEST_PROC_DIR_NOT_FOUND     27
#define OUFTEST_UPDSIG_DIR_NOT_FOUND   28
#define OUFTEST_TITLE_DIR_PROTECTED    29
#define OUFTEST_PROC_DIR_PROTECTED     30
#define OUFTEST_UPDSIG_DIR_PROTECTED   31
#define OUFTEST_DIR_NOT_FOUND          32
#define OUFTEST_TTL_FILE_NOT_MOVED     33
#define OUFTEST_US_FILE_NOT_MOVED      34
#define OUFTEST_US_READ_ERROR          35
#define OUFTEST_PARSE_ERROR            36
#define OUFTEST_PPLIB_ERROR            37
#define OUFTEST_FETCH_DIR_PROTECTED    38

#define OUFTEST_COMMAND_L  250
#define OUFTEST_DATABASE_L 30
#define OUFTEST_DATETIME_L 35
#define OUFTEST_DIR_L      100
#define OUFTEST_ERROR_L    80
#define OUFTEST_FIELD_L    80
#define OUFTEST_FILE_L     100
#define OUFTEST_FORMAT_L   10
#define OUFTEST_HOST_L     20
#define OUFTEST_OPTIONS_L  100
#define OUFTEST_PASSWORD_L 15
#define OUFTEST_PORT_L     4
#define OUFTEST_PPN_L      16
#define OUFTEST_TEXT_L     250
#define OUFTEST_USER_L     20
#define OUFTEST_SYNTAX_L   31
#define OUFTEST_MORE_L     99
#define OUFTEST_COPY_L     2
#define OUFTEST_MAT_L      2
#define OUFTEST_TYPE_L     2
#define OUFTEST_REQUEST_L  12
#define OUFTEST_ILN_L	   3
#define OUFTEST_INSTITUTION_L	10

#define OUFTEST_MAX_PPN            200
#define OUFTEST_MAX_UPDATES        15
#define OUFTEST_DBS_UPDATE_SIZE    65
#define OUFTEST_BUFFER	           CB_SIZE_MAX
#define OUFTEST_MAX_PPN_RETRIEVAL  3
#define OUFTEST_MAX_REFS           5

#define OUFTEST_SEP                31

#define OUFTEST_NO_EVENT           0
#define OUFTEST_TITLE_EVENT        1
#define OUFTEST_UPDATE_EVENT       2
#define OUFTEST_INTERRUPT_EVENT    3
#define OUFTEST_SLEEP_EVENT        4

#define OUFTEST_FIELD_DELIMITERS   "\r"
#define OUFTEST_UPDATE_SIGNAL_FILE "updsig"
#define OUFTEST_FETCH_REQUEST_FILE "fetch"
#define OUFTEST_TITLE_FILE         "title"
#define OUFTEST_TRACE_FILE         "trace"
#define OUFTEST_TEMP_FILE          "ouftest.tmp"
#define OUFTEST_ERROR_EXTENSION    ".err"
#define OUFTEST_ERR                "OUFTEST"
#define OUFTEST_DUMMY_DOMAIN       "@server.domain"

#define OUFTEST_OK                 0
#define OUFTEST_ERROR              -1
#define OUFTEST_NOT_FOUND          100

#define OUFTEST_NAME		   "ouftest"

/* Function definitions */
bool  OUFTestApplication(void);
char* OUFTestGetLastError(void);

int gDebug;

#endif /* OUFTEST_H_INCLUDED */
