/*========================================================================
 *  Copyright (c) 1996 by Pica
 *
 *  File	: ouftest_parse.h
 *  Description	: Title parsing module declarations
 *  Author(s)	: Peter Jan van der Mespel
 *  Creation	: 04-11-1996
 *
 *  History:
 *  04-11-1996	: peter jan      : creation
 *  16-04-2020	: zwaalm   : CBS-9161 Fixed parsing PPN and EPN (unused).
 *  23-04-2020	: zwaalm   : CBS-9352 Used counted string title buffer.
 *
 *  Notes	:
 *
 *========================================================================
 */

#ifndef OUFTEST_PARSE_H_INCLUDED
#define OUFTEST_PARSE_H_INCLUDED

#pragma ident "$Id: ouftest_parse.h 376284 2020-04-23 10:25:46Z zwaalm $"

#include <pica.h>
#include <cs.h>

/* Error codes */
#define PARSE_NO_ERROR               0
#define PARSE_ERROR                  1
#define PARSE_FILE_NOT_OPENED        2
#define PARSE_FILE_NOT_CLOSED        3
#define PARSE_SYNTAX_ERROR           4
#define PARSE_READ_ERROR             5
#define PARSE_COMPONENT_TOO_LARGE    6

#define PARSE_DATETIME_L            25
#define PARSE_FILE_L               100
#define PARSE_ERROR_L               80
#define PARSE_MSG_L                 80
#define PARSE_SMALL_BUFFER_L      1024
#define PARSE_LARGE_BUFFER_L      8192

#define PARSE_PPN_L                 16
#define PARSE_DBSLVL_L               3
#define PARSE_TIME_L                12
#define PARSE_LENGTH_L               4
#define PARSE_ILN_L		     3

#define PARSE_SPACE                " "
#define PARSE_SPACE_CR             " \r"
#define PARSE_CR_CHAR              '\r'
#define PARSE_CR_STR               "\r"

typedef struct
{
    char strPPN[PARSE_PPN_L+1];
    char strIln[PARSE_ILN_L+1];
    char cFormat;
    int  nResultCode;
    char strMessage[PARSE_MSG_L+1];
    char strDatabaseLevel[PARSE_DBSLVL_L+1];
    int  nLength;
    char strTime[PARSE_TIME_L+1];
    CSBuf_p pchTitleComponent;
} TitleRec;

/* Function definitions */
bool        ParseExit(void);
bool        OUFT_ParseInit(char *strTitleFile);
const char* ParseGetLastError(void);
TitleRec*   ParseGetNextRecord(void);

#endif /* OUFTEST_PARSE_H_INCLUDED */
