#!/usr/bin/perl

# Use this script to quickly add an object to a reference data collection.
# It will add a new object to the array of reference data, assign a UUID to the human readable name.
# The script automatically saves the appended object.
# This file could then be loaded into a Folio service via API.
# This file could also be used as input to the ref2choose.pl script, which creates an xsl:choose element populate with xsl:when elements.

use JSON;
use Data::UUID;
use Data::Dumper;
use utf8;

binmode(STDIN, 'utf8');
binmode(STDOUT, 'utf8');
my $infile = shift or die "Usage: ./append_ref.pl <json_ref_file>";
$/ = "";
open IN, "<:encoding(utf-8)", $infile or die "Can't open input file!";
my $text = <IN>;
my $json = JSON->new->decode($text);
close IN;
print "Enter reference name (followed by Enter, Ctrl-D): ";
my $name = <STDIN>;
chomp $name;
my $root;
foreach (keys $json) {
  if ($_ ne 'totalRecords') {
    $root = $_;
    last;
  }
}
foreach (@{ $json->{$root} }) {
  if ($_->{name} eq $name) {
    die "The name '$name' already exists!"
  }
}

my $ug = Data::UUID->new;
$uuid = $ug->create();
my $id = lc $ug->to_string($uuid);
my $obj = { id => $id, name => $name };
if ($json->{$root}[0]->{source}) {
  $obj->{source} = 'folio';
}
push @{ $json->{$root} }, $obj;
my $tot = @{ $json->{$root} };
$json->{totalRecords} = $tot;
$json_out = JSON->new->pretty->encode($json);
open OUT, ">:utf8", $infile;
print OUT $json_out;
print "Added " . $obj->{name} . " with id " . $obj->{id} . "\n";
