#!/usr/bin/perl

use JSON;
use XML::LibXML;

my $infile = shift or die "Usage: ref2xsl.pl <json_ref_file>";
$/ = "";

open IN, $infile or die "Can't open input file!";
my $text = <IN>;
my $json = decode_json($text);
my $root;
foreach (keys $json) {
  if ($_ ne 'totalRecords') {
    $root = $_;
    last;
  }
}

my $node_map = {
  locations => 'permanentLocationId',
  identifierTypes => 'identifierTypeId',
  mtypes => 'materialTypeId',
  loantypes => 'permanentLoanTypeId'
};

my $fel = $node_map->{$root} || $root;
my $doc = XML::LibXML::Document->new('1.0', 'UTF-8');
my $template = $doc->createElement('xsl:template');
$template->setAttribute(match => $fel);
my $folioNode = $doc->createElement($fel);
my $choose = $doc->createElement('xsl:choose');
my $node = 'name';
if ($root =~ /contributorTypes/) {
  $node = 'code';
}

my $other;
my $final;
foreach (@{ $json->{$root} }) {
  # if ($node eq 'code') {
  #    my $comment = XML::LibXML::Comment->new($_->{name});
  #    $choose->appendChild($comment);
  # }
  if (($node eq 'code' && $_->{$node} eq 'oth') || $_->{$node} =~ /^other|^unknown/i) {
    $other = $doc->createElement('xsl:otherwise');
    $other->appendTextNode($_->{id});
  } else {
    my $when = $doc->createElement('xsl:when');
    $when->setAttribute(test => ".='$_->{$node}'");
    $when->appendTextNode($_->{id});
    $choose->appendChild($when);
    $final = $_->{id};
  }
}
if (!$other) {
    $other = $doc->createElement('xsl:otherwise');
    $other->appendTextNode($final);
}
$choose->appendChild($other);
$folioNode->appendChild($choose);
$template->appendChild($folioNode);
$doc->setDocumentElement($template);
my $output = $doc->toString(2);
$output =~ s/^<\?.+\n//;
print $output;