#!/usr/bin/perl -w

# Reads from pica+ XML file and outputs inventory JSON files.

use strict;
use warnings;
use XML::LibXML;
use XML::LibXSLT;
use JSON;
use File::Copy;
use Config::Tiny;
use Data::Dumper;
use utf8;

binmode(STDOUT, 'utf8');

my ($style_file, $style_file2, @source_files) = @ARGV;
if (! $source_files[0]) {
  die "Usage: ./picaxml2json.pl <stylesheet1> <stylesheet2> <pica_xml_file>";
}

my $json = JSON->new->allow_nonref;
my $parser = XML::LibXML->new();
my $xslt = XML::LibXSLT->new();
my $stylesheet = $xslt->parse_stylesheet_file($style_file);
my $stylesheet2 = $xslt->parse_stylesheet_file($style_file2);
   
foreach my $file ( @source_files ) {
  my $source_doc = $parser->parse_file($file);
  my $output1 = $stylesheet->transform($source_doc);
  my $dom = $stylesheet2->transform($output1);
  foreach my $rec ($dom->findnodes('/collection/record')) {
    my $out = {};
    my $recObj = getElements($rec);
    # print Dumper $recObj;
    if ($recObj->{holdingsRecords}) {
      $out->{holdingsRecords} = $recObj->{holdingsRecords};
      delete $recObj->{holdingsRecords};
    }
    $out->{instance} = $recObj;
    print $json->pretty->encode($out);
  } 
}

sub getElements {
  my $dom = shift;
  my $out = {};
  foreach my $node ($dom->childNodes()) {
    my $name = $node->nodeName;
    my @cn = $node->childNodes();
    if ($cn[0] && $cn[0]->nodeName eq '#text') {
      $out->{$name} = $cn[0]->textContent;
    } elsif ($cn[0] && $cn[0]->nodeName eq 'arr') {
      $out->{$name} = [];
      foreach my $item ($cn[0]->getChildrenByTagName('i')) {
        my @cn = $item->childNodes();
        if ($cn[0] && $cn[0]->nodeName eq '#text') {
          push @{ $out->{$name} }, $cn[0]->textContent;
        } else {
          push @{ $out->{$name} }, getElements($item);
        }
      }
    } elsif ($cn[0]) {
      $out->{$name} = {};
      foreach (@cn) {
        my $cname = $_->nodeName();
        $out->{$name}->{$cname} = $_->textContent;
      }
    }
  }
  return $out;
}