$(document).ready(function () {

  let base;

  const setUrl = () => {
    let cbase = document.cookie.replace(/.*base=([^;]+).*/, '$1');
    base = cbase || 'http://' + location.hostname + ':3030';
  }

  const clear = () => {
    $('#content').empty();
    $('nav ~ form').hide();
    $('#run-status').hide();
  }

  const errAlert = (res, status, err) => {
    if (res.responseJSON) {
      alert(err + ': ' + res.responseJSON.message);
    } else {
      alert(`It looks like there was a connection error.\nMake sure api.pl is running at ${base}`);
    }
  }

  const getKey = () => {
    let cook = document.cookie;
    cook = cook.replace(/.*\bkey=([^;]+).*/, '$1');
    return cook;
  }

  const killAll = (sig) => {
    clear();
    let apiKey = getKey();
    $.ajax({
      url: `${base}/stop?key=${apiKey}&sig=${sig}`,
      cache: false
    })
      .done((res) => {
        let mesg = '<pre>';
        mesg += res.messages.join('\n');
        mesg += '</pre>';
        $('#content').append(mesg)
      })
      .fail(errAlert);
  }

  const getLogs = (ep) => {
    clear();
    $('#run-status').show().text('Working...');
    let apiKey = getKey();
    $.ajax({
      url: `${base}/${ep}?key=${apiKey}`,
      cache: false
    })
      .done(res => {
        $('#run-status').html(`<span>Showing ${res.lines.length} lines</span><button class="btn-sm btn-primary ml-2" id="reload-${ep}">Reload</button>`);
        let content = '<pre>';
        content += res.lines.join('\n');
        content += '<pre>';
        $('#content').append(content);
      })
      .fail(errAlert);
  }

  const getSettings = (type) => {
    clear();
    $('#ini-form').show();
    $(`#${type}`).prop('checked', true);
    let apiKey = getKey();
    $.ajax({
      url: `${base}/settings?key=${apiKey}&type=${type}`,
      cache: false
    })
      .done(res => {
        $('#ini-text').val(res.raw);
        $('#content').append(content);
      })
      .fail(errAlert);
  }

  const getFileList = (type, subdir) => {
    clear();
    $('#run-status').show().text('Working...');
    let apiKey = getKey();
    let url = `${base}/files?key=${apiKey}&type=${type}`;
    if (subdir) url += `&name=${subdir}&sub=1`;
    $.ajax({
      url: url, 
      cache: false
    })
      .done(function (res) {
        let flist = '<table><tbody>';
        let utype = 'unlink-' + type;
        if (subdir) utype += ':' + subdir;
        res.files.forEach(f => {
          let name = f;
          if (subdir) {
            name = `${subdir}/${f}`;
          }
          flist += `<tr><td><a href="viewfile.html?type=${type}&name=${name}" target="_blank">${name}</a></td></tr>`;
        });
        flist += '</tbody></table>';
        $('#run-status').show().html(`Count: ${res.count}`);
        if (res.count > 0) {
          $('#run-status').append(`<button class="btn-sm btn-danger ml-2" id="${utype}">Delete all</button>`);
        }
        $('#content').append(flist);
        if (subdir) {
          let backId = '';
          if (type === 'updsig-done') {
            backId = 'updsig-back';
          } else {
            backId = 'titles-back';
          }
          $('#content').append(`<button class="btn-sm btn-light" id="${backId}">Go back</button>`);
        }
      })
      .fail(errAlert);
  }

  const getDirList = (type) => {
    clear();
    $('#run-status').show().text('Working...');
    let apiKey = getKey();
    $.ajax({
      url: `${base}/files?key=${apiKey}&type=${type}`,
      cache: false
    })
      .done(function (res) {
        let flist = '<table><tbody>'
        res.files.forEach(f => {
          flist += `<tr><td><a href="#" class="subdir" data-dir="${type}">${f}</a></td></tr>`;
        });
        flist += '</tbody></table>';
        $('#run-status').show().html(`Directory count: ${res.count}`);
        $('#content').append(flist);
      })
      .fail(errAlert);
  }

  const getFailedRecords = (jid) => {
    clear();
    let apiKey = getKey();
    $('#run-status').show();
    $('#run-status').text('Working...');
    $.ajax({
      url: `${base}/harvester?key=${apiKey}&command=failed-records&jid=${jid}`,
      cache: false
    })
      .done(function (res) {
        const hits = res.totalRecords;
        const disabled = (hits === 0) ? ' disabled' : '';
        $('#run-status').html(`<span><strong>Host:</strong> ${res.harvesterUrl} <strong>Job:</strong> ${res.jobId} <strong>Count:</strong> ${hits}</span> <button class="btn-sm btn-primary ml-2" id="makefetch-${jid}" ${disabled}>Make fetch</button>`);
        let table = '<table class="table"><thead><tr><th>Date</th><th>Filename</th><th>URL</th></tr></thead><tbody>';
        res.failedRecords.forEach(r => {
          table += `<tr><td>${r.file.date}</td><td>${r.file.name}</td><td><a href="${r.url}" target="_blank">${r.url}</a></td></tr>`;
        });
        table += '</tbody></table>'
        $('#content').append(table);
      })
      .fail(errAlert);
  }

  const getHarvestLog = (jid) => {
    $('#content').empty();
    let apiKey = getKey();
    let fd = $('#from-val').val();
    $('#run-status').show();
    $('#run-status').text('Working...');
    let url = (jid) ? `${base}/harvester?key=${apiKey}&command=log&jid=${jid}` : `${base}/harvester?key=${apiKey}&command=log&from=${fd}` 
    $.ajax({
      url: url,
      cache: false
    })
      .done(function (res) {
        $('#run-status').hide();
        let revLines = res.lines.reverse();
        let log = revLines.join('\n');
        $('#run-status').show().html(`<span><strong>Host:</strong> ${res.harvesterUrl} <strong>Job:</strong> ${res.jobId} <strong>Lines:</strong> ${res.totalRecords}</span>`);
        $('#content').append('<div id="hlog" style="white-space: pre; font-family: monospace" />');
        $('#hlog').text(log);
      })
      .fail(errAlert);
  }

  setUrl();

  if (location.pathname.match(/viewfile.html/)) {
    let apiKey = getKey();
    let name = location.href.replace(/.+?\bname=([^&]+).*/, '$1');
    let type = location.href.replace(/.+?\btype=([^&]+).*/, '$1');
    $.ajax({
      url: `${base}/files?key=${apiKey}&type=${type}&name=${name}`,
      cache: false
    }) 
    .done((res) => {
      $('#content').html(`<pre>${res.contents}</pre>`);
    })
    .fail((res) => { 
      alert(res.responseText);
    });
  }

  $('#apibase').val(base);
  $('#apikey').val(getKey());

  $('#set-apikey').click(() => {
    let apiKey = $('#apikey').val();
    base = $('#apibase').val();
    document.cookie = `key=${apiKey}; expires=Tue, 19 Jan 2038 03:14:07 UTC;`;
    document.cookie = `base=${base}; expires=Tue, 19 Jan 2038 03:14:07 UTC;`;
    setUrl();
    alert(`API Key set to ${apiKey}\nBase address set to ${base}`);
    clear();
  });

  $('#status').click(() => {
    clear();
    let apiKey = getKey();
    $.ajax({
      url: `${base}/status?key=${apiKey}`,
      cache: false
    })
    .done(function (res) {
      let table = '<table class="table"><thead><th>Script</th><th>PID</th><th>Status</th></thead><tbody>';
      let stopped = false;
      res.scripts.forEach(p => {
        p.pid = (p.pid) ? p.pid : '--';
        table += `<tr><td>${p.name}</td><td>${p.pid}</td><td class="align-middle"><span class="${p.status}" title="${p.status}"></tr>`;
        if (p.status === 'stopped') {
          stopped = true;
        }
      });
      table += '</tbody></table>';
      $('#content').append(table);
    })
    .fail(errAlert);
  });

  $('#env').click(() => {
    clear();
    let apiKey = getKey();
    $.ajax({
      url: `${base}/view_env?key=${apiKey}`,
      cache: false
    })
      .done(function (res) {
        let table = '<table class="table"><thead><th>Key</th><th>Value</th></thead><tbody>';
        let stopped = false;
        for (let key in res) {
          table += `<tr><td>${key}</td><td>${res[key]}</td></tr>`;
        }
        table += '</tbody></table>';
        $('#content').append(table);
      })
      .fail(errAlert);
  });

  $('#start-all').click(() => {
    clear();
    $('#ts-form').show();
  });

  $('#set-ts').click(() => {
    clear();
    let apiKey = getKey();
    let ts = $('#ts').val();
    let tsString = '';
    if (ts) {
      tsString = '&ts=' + ts;
    }
    $.ajax({
      url: `${base}/start?key=${apiKey}${tsString}`,
      cache: false
    })
      .done((res) => {
        let mesg = '<pre>';
        if (res.last_time) {
          mesg += `Starting from ${res.last_time}\n\n`;
        }
        mesg += res.messages.join('\n');
        mesg += '</pre>';
        $('#content').append(mesg)
      })
      .fail(errAlert);
  });

  $('#harvest-log').click(() => {
    clear();
    let now = new Date().valueOf();
    let halfDay = now - 43200000;
    let fromDate = new Date(halfDay).toISOString();
    fromDate = fromDate.replace(/\.\d+Z$/, '');
    $('#from-form').show();
    if (!$('#from-val').val()) {
      $('#from-val').val(fromDate);
    }
  });

  $('#from-run').click(() => {
    getHarvestLog();
  });

  $('#failed-records').click(() => {
    getFailedRecords('');
  });

  $('#list-jobs').click(() => {
    clear();
    let apiKey = getKey();
    $('#run-status').show();
    $('#run-status').text('Working...');
    $.ajax({
      url: `${base}/harvester?key=${apiKey}&command=list`,
      cache: false
    })
      .done(function (res) {
        const hits = (res.jobs) ? res.jobs.length : 0;
        $('#run-status').text('Count: ' + hits);
        let table = '<table class="table"><thead><tr><th>ID</th><th>Name</th><th>Status</th><th>Show...</th></tr></thead><tbody>';
        res.jobs.forEach(r => {
          table += `<tr><td>${r.id}</td><td>${r.name}</td><td>${r.status}<td><a href="#" id="failedrecs-${r.id}">Failed Records</a> | <a href="#" id="harvestlog-${r.id}">Log</a></td></tr>`;
        });
        table += '</tbody></table>'
        $('#content').append(table);
      })
      .fail(errAlert);
  });

  $('#start-cancel').click(() => clear());

  $('#stop-all').click(() => killAll('15'));
  $('#kill').click(() => killAll('9'));

  $('#trace').click(() => getLogs('trace'));
  $('#err-log').click(() => getLogs('errlog'));
  $('#decode-log').click(() => getLogs('decode_log'));
  $('#picapull-log').click(() => getLogs('picapull_log'));

  $('#updsig').click(() => getFileList('updsig'));
  $('#fetch').click(() => getFileList('fetch'));
  $('#titles').click(() => getFileList('titles'));
  $('#updsig-done').click(() => getDirList('updsig-done'));
  $('#content').on('click', 'a[data-dir="updsig-done"]', function () { getFileList('updsig-done', $(this).text()) });
  $('#titles-done').click(() => getDirList('titles-done'));
  $('#content').on('click', 'a[data-dir="titles-done"]', function () { getFileList('titles-done', $(this).text()) });
  $('#content').on('click', '#updsig-back', function () { getDirList('updsig-done') });
  $('#content').on('click', '#titles-back', function () { getDirList('titles-done') });

  $('#settings').click(() => getSettings('ouf'));
  $('#ouf').click(() => getSettings('ouf'));
  $('#other').click(() => getSettings('other'));
  $('#ini-cancel').click(() => clear());
  $('#setup-cancel').click(() => clear());
  $('#ini-save').click(() => {
    let apiKey = getKey();
    const data = {};
    data.type = $('input[name=iniType]:checked').val();
    data.text = $('#ini-text').val();
    let payload = JSON.stringify(data);
    $.ajax({
      type: "POST",
      url: `${base}/save_ini?key=${apiKey}`,
      contentType: 'application/json; charset=utf-8',
      data: payload
    })
    .done((res) => {
      alert(res.status);
    });
  });
  
  $('#setup').click(() => {
    clear();
    $('#setup-form').show();
  });

  $('#content').on('click', 'a[id*="failedrecs-"]', (event) => {
    let jid = event.target.id.replace(/^.+-/, '');
    getFailedRecords(jid);
  });
  $('#content').on('click', 'a[id*="harvestlog-"]', (event) => {
    let jid = event.target.id.replace(/^.+-/, '');
    getHarvestLog(jid);
  });
  $('#run-status').on('click', 'button[id*="reload-"]', (event) => {
    let ep = event.target.id.replace(/^.+-/, '');
    getLogs(ep);
  });
  $('#run-status').on('click', 'button[id*="makefetch-"]', (event) => {
    let jobId = event.target.id.replace(/^.+-/, '');
    let apiKey = getKey();
    $.ajax({
      url: `${base}/harvester?key=${apiKey}&command=failed-records-fetch&jid=${jobId}`,
      cache: false
    })
    .done((res) => {
      alert(`${res.totalRecords} fetch files created`);
    })
    .fail(errAlert);
  });
  $('#run-status').on('click', 'button[id^="unlink-"]', (event) => {
    let con = confirm('Are you sure?');
    if (con) {
      let apiKey = getKey();
      let type = event.target.id.replace(/^.+?-/, '');
      let sub = '';
      let name = ''
      if (type.match(/:/)) {
        name = type.replace(/.+?:(.+)/, '&name=$1');
        sub = '&sub=1';
        type = type.replace(/:.*/, '');
      }
      let url = `${base}/files?key=${apiKey}&type=${type}${name}${sub}`;
      clear();
      $.ajax({
        type: 'DELETE',
        url: url,
        cache: false
      })
        .done((res) => {
          alert(res.message);
          $('#run-status').text('Working...');
          if (sub) {
            getDirList(type);
          } else {
            getFileList(type);
          }
        })
        .fail(errAlert);
    }
  });
});


